@@ -5,10 +5,14 @@
  *                         Corporation.  All rights reserved.
  * Copyright (c) 2006      The Technical University of Chemnitz. All
  *                         rights reserved.
- * Copyright (c) 2013-2015 Los Alamos National Security, LLC. All rights
+ * Copyright (c) 2013-2017 Los Alamos National Security, LLC. All rights
  *                         reserved.
- * Copyright (c) 2014-2016 Research Organization for Information Science
+ * Copyright (c) 2014-2017 Research Organization for Information Science
  *                         and Technology (RIST). All rights reserved.
+ * Copyright (c) 2017      IBM Corporation.  All rights reserved.
+ * $COPYRIGHT$
+ *
+ * Additional copyrights may follow
  *
  * Author(s): Torsten Hoefler <htor@cs.indiana.edu>
  *
@@ -50,10 +54,10 @@ int NBC_Allreduce_args_compare(NBC_Allreduce_args *a, NBC_Allreduce_args *b, voi
 
 int ompi_coll_libnbc_iallreduce(const void* sendbuf, void* recvbuf, int count, MPI_Datatype datatype, MPI_Op op,
                                 struct ompi_communicator_t *comm, ompi_request_t ** request,
-                                struct mca_coll_base_module_2_1_0_t *module)
+                                struct mca_coll_base_module_2_2_0_t *module)
 {
   int rank, p, res;
-  OPAL_PTRDIFF_TYPE ext, lb;
+  ptrdiff_t ext, lb;
   NBC_Schedule *schedule;
   size_t size;
 #ifdef NBC_CACHE_SCHEDULE
@@ -61,7 +65,7 @@ int ompi_coll_libnbc_iallreduce(const void* sendbuf, void* recvbuf, int count, M
 #endif
   enum { NBC_ARED_BINOMIAL, NBC_ARED_RING } alg;
   char inplace;
-  NBC_Handle *handle;
+  NBC_Handle *handle = NULL;
   ompi_coll_libnbc_module_t *libnbc_module = (ompi_coll_libnbc_module_t*) module;
   ptrdiff_t span, gap;
 
@@ -82,6 +86,19 @@ int ompi_coll_libnbc_iallreduce(const void* sendbuf, void* recvbuf, int count, M
     return res;
   }
 
+  if (1 == p) {
+    if (!inplace) {
+      /* for a single node - copy data to receivebuf */
+      res = NBC_Copy(sendbuf, count, datatype, recvbuf, count, datatype, comm);
+      if (OPAL_UNLIKELY(OMPI_SUCCESS != res)) {
+        NBC_Return_handle (handle);
+        return res;
+      }
+    }
+    *request = &ompi_request_empty;
+    return OMPI_SUCCESS;
+  }
+
   res = NBC_Init_handle (comm, &handle, libnbc_module);
   if (OPAL_UNLIKELY(OMPI_SUCCESS != res)) {
     return res;
@@ -94,15 +111,6 @@ int ompi_coll_libnbc_iallreduce(const void* sendbuf, void* recvbuf, int count, M
     return OMPI_ERR_OUT_OF_RESOURCE;
   }
 
-  if ((p == 1) && !inplace) {
-    /* for a single node - copy data to receivebuf */
-    res = NBC_Copy(sendbuf, count, datatype, recvbuf, count, datatype, comm);
-    if (OPAL_UNLIKELY(OMPI_SUCCESS != res)) {
-      NBC_Return_handle (handle);
-      return res;
-    }
-  }
-
   /* algorithm selection */
   if(p < 4 || size*count < 65536 || !ompi_op_is_commute(op) || inplace) {
     alg = NBC_ARED_BINOMIAL;
@@ -194,7 +202,7 @@ int ompi_coll_libnbc_iallreduce(const void* sendbuf, void* recvbuf, int count, M
 
 int ompi_coll_libnbc_iallreduce_inter(const void* sendbuf, void* recvbuf, int count, MPI_Datatype datatype, MPI_Op op,
                                       struct ompi_communicator_t *comm, ompi_request_t ** request,
-                                      struct mca_coll_base_module_2_1_0_t *module)
+                                      struct mca_coll_base_module_2_2_0_t *module)
 {
   int rank, res, rsize;
   size_t size;
