@@ -14,6 +14,8 @@
  * Copyright (c) 2012-2015 Los Alamos National Security, LLC.
  *                         All rights reserved.
  * Copyright (c) 2013-2015 Intel, Inc. All rights reserved
+ * Copyright (c) 2017      Research Organization for Information Science
+ *                         and Technology (RIST). All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -32,11 +34,12 @@
 #include "mtl_psm2_types.h"
 #include "mtl_psm2_request.h"
 
-#include "psm.h"
+#include "psm2.h"
 
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <unistd.h>
+#include <glob.h>
 
 static int param_priority;
 
@@ -58,9 +61,8 @@ mca_mtl_psm2_component_t mca_mtl_psm2_component = {
             MCA_MTL_BASE_VERSION_2_0_0,
 
             .mca_component_name = "psm2",
-            .mca_component_major_version = OMPI_MAJOR_VERSION,
-            .mca_component_minor_version = OMPI_MINOR_VERSION,
-            .mca_component_release_version = OMPI_RELEASE_VERSION,
+            MCA_BASE_MAKE_VERSION(component, OMPI_MAJOR_VERSION, OMPI_MINOR_VERSION,
+                                  OMPI_RELEASE_VERSION),
             .mca_open_component = ompi_mtl_psm2_component_open,
             .mca_close_component = ompi_mtl_psm2_component_close,
             .mca_query_component = ompi_mtl_psm2_component_query,
@@ -81,7 +83,7 @@ ompi_mtl_psm2_component_register(void)
     ompi_mtl_psm2.connect_timeout = 180;
     (void) mca_base_component_var_register(&mca_mtl_psm2_component.super.mtl_version,
                                            "connect_timeout",
-                                           "PSM connection timeout value in seconds",
+                                           "PSM2 connection timeout value in seconds",
                                            MCA_BASE_VAR_TYPE_INT, NULL, 0, 0,
                                            OPAL_INFO_LVL_9,
                                            MCA_BASE_VAR_SCOPE_READONLY,
@@ -102,22 +104,61 @@ ompi_mtl_psm2_component_register(void)
 static int
 ompi_mtl_psm2_component_open(void)
 {
-  struct stat st;
+  int res;
+  glob_t globbuf;
+  globbuf.gl_offs = 0;
 
   /* Component available only if Omni-Path hardware is present */
-  if (0 == stat("/dev/hfi1", &st)) {
-    return OMPI_SUCCESS;
+  res = glob("/dev/hfi1_[0-9]", GLOB_DOOFFS, NULL, &globbuf);
+  if (0 == res || GLOB_NOMATCH == res) {
+      globfree(&globbuf);
   }
-  else {
+  if (0 != res) {
+      res = glob("/dev/hfi1_[0-9][0-9]", GLOB_APPEND, NULL, &globbuf);
+      if (0 == res || GLOB_NOMATCH == res) {
+          globfree(&globbuf);
+      }
+      if (0 != res) {
+          return OPAL_ERR_NOT_AVAILABLE;
+      }
+  }
+
+  /* Component available only if at least one hfi1 port is ACTIVE */
+  bool foundOnlineHfi1Port = false;
+  size_t i;
+  char portState[128];
+  FILE *devFile;
+  if (glob("/sys/class/infiniband/hfi1_*/ports/*/state",
+        GLOB_DOOFFS, NULL, &globbuf) != 0) {
     return OPAL_ERR_NOT_AVAILABLE;
   }
+
+  for (i=0;i < globbuf.gl_pathc; i++) {
+    devFile = fopen(globbuf.gl_pathv[i], "r");
+    fgets(portState, sizeof(portState), devFile);
+    fclose(devFile);
+
+    if (strstr(portState, "ACTIVE") != NULL) {
+      /* Found at least one ACTIVE port */
+      foundOnlineHfi1Port = true;
+      break;
+    }
+  }
+
+  globfree(&globbuf);
+
+  if (!foundOnlineHfi1Port) {
+    return OPAL_ERR_NOT_AVAILABLE;
+  }
+
+  return OMPI_SUCCESS;
 }
 
 static int
 ompi_mtl_psm2_component_query(mca_base_module_t **module, int *priority)
 {
     /*
-     * if we get here it means that PSM is available so give high priority
+     * if we get here it means that PSM2 is available so give high priority
      */
 
     *priority = param_priority;
@@ -166,14 +207,14 @@ static mca_mtl_base_module_t *
 ompi_mtl_psm2_component_init(bool enable_progress_threads,
                             bool enable_mpi_threads)
 {
-    psm_error_t	err;
-    int	verno_major = PSM_VERNO_MAJOR;
-    int verno_minor = PSM_VERNO_MINOR;
+    psm2_error_t	err;
+    int	verno_major = PSM2_VERNO_MAJOR;
+    int verno_minor = PSM2_VERNO_MINOR;
     int local_rank = -1, num_local_procs = 0;
     int num_total_procs = 0;
 
     /* Compute the total number of processes on this host and our local rank
-     * on that node. We need to provide PSM with these values so it can
+     * on that node. We need to provide PSM2 with these values so it can
      * allocate hardware contexts appropriately across processes.
      */
     if (OMPI_SUCCESS != get_num_local_procs(&num_local_procs)) {
@@ -191,27 +232,27 @@ ompi_mtl_psm2_component_init(bool enable_progress_threads,
         return NULL;
     }
 
-    err = psm_error_register_handler(NULL /* no ep */,
-			             PSM_ERRHANDLER_NOP);
+    err = psm2_error_register_handler(NULL /* no ep */,
+			             PSM2_ERRHANDLER_NOP);
     if (err) {
-        opal_output(0, "Error in psm_error_register_handler (error %s)\n",
-		    psm_error_get_string(err));
+        opal_output(0, "Error in psm2_error_register_handler (error %s)\n",
+		    psm2_error_get_string(err));
 	return NULL;
     }
 
     if (num_local_procs == num_total_procs) {
-      setenv("PSM_DEVICES", "self,shm", 0);
+      setenv("PSM2_DEVICES", "self,shm", 0);
     }
 
-    err = psm_init(&verno_major, &verno_minor);
+    err = psm2_init(&verno_major, &verno_minor);
     if (err) {
-      opal_show_help("help-mtl-psm.txt",
-		     "psm init", true,
-		     psm_error_get_string(err));
+      opal_show_help("help-mtl-psm2.txt",
+		     "psm2 init", true,
+		     psm2_error_get_string(err));
       return NULL;
     }
 
-    /* Complete PSM initialization */
+    /* Complete PSM2 initialization */
     ompi_mtl_psm2_module_init(local_rank, num_local_procs);
 
     ompi_mtl_psm2.super.mtl_request_size =
