@@ -5,8 +5,11 @@
  *                         reserved.
  * Copyright (c) 2014      Intel, Inc. All rights reserved.
  * Copyright (c) 2015      Cisco Systems, Inc.  All rights reserved.
- * Copyright (c) 2015-2016 Research Organization for Information Science
+ * Copyright (c) 2015-2017 Research Organization for Information Science
  *                         and Technology (RIST). All rights reserved.
+ * Copyright (c) 2017      The University of Tennessee and The University
+ *                         of Tennessee Research Foundation.  All rights
+ *                         reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -21,6 +24,7 @@
 #include "ompi/mca/osc/base/osc_base_obj_convert.h"
 #include "ompi/request/request.h"
 #include "opal/util/sys_limits.h"
+#include "opal/include/opal/align.h"
 
 #include "osc_sm.h"
 
@@ -211,7 +215,7 @@ component_select(struct ompi_win_t *win, void **base, size_t size, int disp_unit
         int i, flag;
         size_t pagesize;
         size_t state_size;
-        int posts_size, post_size = (comm_size + 63) / 64;
+        size_t posts_size, post_size = (comm_size + 63) / 64;
 
         OPAL_OUTPUT_VERBOSE((1, ompi_osc_base_framework.framework_output,
                              "allocating shared memory region of size %ld\n", (long) size));
@@ -233,10 +237,10 @@ component_select(struct ompi_win_t *win, void **base, size_t size, int disp_unit
         } else {
             total = size;
         }
-        ret = module->comm->c_coll.coll_allgather(&total, 1, MPI_UNSIGNED_LONG,
+        ret = module->comm->c_coll->coll_allgather(&total, 1, MPI_UNSIGNED_LONG,
                                                   rbuf, 1, MPI_UNSIGNED_LONG,
                                                   module->comm,
-                                                  module->comm->c_coll.coll_allgather_module);
+                                                  module->comm->c_coll->coll_allgather_module);
         if (OMPI_SUCCESS != ret) return ret;
 
         total = 0;
@@ -246,7 +250,9 @@ component_select(struct ompi_win_t *win, void **base, size_t size, int disp_unit
 
 	/* user opal/shmem directly to create a shared memory segment */
 	state_size = sizeof(ompi_osc_sm_global_state_t) + sizeof(ompi_osc_sm_node_state_t) * comm_size;
+        state_size += OPAL_ALIGN_PAD_AMOUNT(state_size, 64);
         posts_size = comm_size * post_size * sizeof (uint64_t);
+        posts_size += OPAL_ALIGN_PAD_AMOUNT(posts_size, 64);
         if (0 == ompi_comm_rank (module->comm)) {
             char *data_file;
             if (asprintf(&data_file, "%s"OPAL_PATH_SEP"shared_window_%d.%s",
@@ -263,8 +269,8 @@ component_select(struct ompi_win_t *win, void **base, size_t size, int disp_unit
             }
         }
 
-	ret = module->comm->c_coll.coll_bcast (&module->seg_ds, sizeof (module->seg_ds), MPI_BYTE, 0,
-					       module->comm, module->comm->c_coll.coll_bcast_module);
+	ret = module->comm->c_coll->coll_bcast (&module->seg_ds, sizeof (module->seg_ds), MPI_BYTE, 0,
+					       module->comm, module->comm->c_coll->coll_bcast_module);
 	if (OMPI_SUCCESS != ret) {
 	    goto error;
 	}
@@ -313,10 +319,10 @@ component_select(struct ompi_win_t *win, void **base, size_t size, int disp_unit
 
     /* share everyone's displacement units. */
     module->disp_units = malloc(sizeof(int) * comm_size);
-    ret = module->comm->c_coll.coll_allgather(&disp_unit, 1, MPI_INT,
+    ret = module->comm->c_coll->coll_allgather(&disp_unit, 1, MPI_INT,
                                               module->disp_units, 1, MPI_INT,
                                               module->comm,
-                                              module->comm->c_coll.coll_allgather_module);
+                                              module->comm->c_coll->coll_allgather_module);
     if (OMPI_SUCCESS != ret) goto error;
 
     module->start_group = NULL;
@@ -372,8 +378,8 @@ component_select(struct ompi_win_t *win, void **base, size_t size, int disp_unit
 #endif
     }
 
-    ret = module->comm->c_coll.coll_barrier(module->comm,
-                                            module->comm->c_coll.coll_barrier_module);
+    ret = module->comm->c_coll->coll_barrier(module->comm,
+                                            module->comm->c_coll->coll_barrier_module);
     if (OMPI_SUCCESS != ret) goto error;
 
     *model = MPI_WIN_UNIFIED;
@@ -459,8 +465,8 @@ ompi_osc_sm_free(struct ompi_win_t *win)
     /* free memory */
     if (NULL != module->segment_base) {
         /* synchronize */
-        module->comm->c_coll.coll_barrier(module->comm,
-                                          module->comm->c_coll.coll_barrier_module);
+        module->comm->c_coll->coll_barrier(module->comm,
+                                          module->comm->c_coll->coll_barrier_module);
 
         if (0 == ompi_comm_rank (module->comm)) {
             opal_shmem_unlink (&module->seg_ds);
@@ -497,8 +503,8 @@ ompi_osc_sm_set_info(struct ompi_win_t *win, struct ompi_info_t *info)
         (ompi_osc_sm_module_t*) win->w_osc_module;
 
     /* enforce collectiveness... */
-    return module->comm->c_coll.coll_barrier(module->comm,
-                                             module->comm->c_coll.coll_barrier_module);
+    return module->comm->c_coll->coll_barrier(module->comm,
+                                             module->comm->c_coll->coll_barrier_module);
 }
 
 
