@@ -128,7 +128,7 @@ do {                                                                    \
                                             count)                      \
 {                                                                       \
     OBJ_RETAIN(comm);                                                   \
-    OBJ_RETAIN(datatype);                                               \
+    OMPI_DATATYPE_RETAIN(datatype);                                     \
     (req_send)->req_base.req_comm = comm;                               \
     (req_send)->req_base.req_datatype = datatype;                       \
     opal_convertor_copy_and_prepare_for_send(                           \
@@ -157,7 +157,7 @@ do {                                                                    \
                                             count)                      \
 {                                                                       \
     OBJ_RETAIN(comm);                                                   \
-    OBJ_RETAIN(datatype);                                               \
+    OMPI_DATATYPE_RETAIN(datatype);                                     \
     (req_send)->req_base.req_comm = comm;                               \
     (req_send)->req_base.req_datatype = datatype;                       \
     opal_convertor_copy_and_prepare_for_send(                           \
@@ -188,7 +188,7 @@ do {                                                                    \
                                             count)                      \
 {                                                                       \
     OBJ_RETAIN(comm);                                                   \
-    OBJ_RETAIN(datatype);                                               \
+    OMPI_DATATYPE_RETAIN(datatype);                                     \
     (req_send)->req_base.req_comm = comm;                               \
     (req_send)->req_base.req_datatype = datatype;                       \
     opal_convertor_copy_and_prepare_for_send(                           \
@@ -218,7 +218,7 @@ do {                                                                    \
                                             count)                      \
 {                                                                       \
     OBJ_RETAIN(comm);                                                   \
-    OBJ_RETAIN(datatype);                                               \
+    OMPI_DATATYPE_RETAIN(datatype);                                     \
     (req_send)->req_base.req_comm = comm;                               \
     (req_send)->req_base.req_datatype = datatype;                       \
     if (opal_datatype_is_contiguous_memory_layout(&datatype->super, count)) { \
@@ -434,7 +434,7 @@ do {
 #define MCA_PML_CM_HVY_SEND_REQUEST_RETURN(sendreq)                     \
     {                                                                   \
         /*  Let the base handle the reference counts */                 \
-        OBJ_RELEASE(sendreq->req_send.req_base.req_datatype);           \
+        OMPI_DATATYPE_RETAIN(sendreq->req_send.req_base.req_datatype);  \
         OBJ_RELEASE(sendreq->req_send.req_base.req_comm);               \
         OMPI_REQUEST_FINI(&sendreq->req_send.req_base.req_ompi);        \
         opal_convertor_cleanup( &(sendreq->req_send.req_base.req_convertor) ); \
@@ -471,7 +471,7 @@ do {                                                                         \
 #define MCA_PML_CM_THIN_SEND_REQUEST_RETURN(sendreq)                    \
     {                                                                   \
         /*  Let the base handle the reference counts */                 \
-        OBJ_RELEASE(sendreq->req_send.req_base.req_datatype);           \
+        OMPI_DATATYPE_RETAIN(sendreq->req_send.req_base.req_datatype);  \
         OBJ_RELEASE(sendreq->req_send.req_base.req_comm);               \
         OMPI_REQUEST_FINI(&sendreq->req_send.req_base.req_ompi);        \
         opal_convertor_cleanup( &(sendreq->req_send.req_base.req_convertor) ); \
