@@ -3,7 +3,7 @@
  * Copyright (c) 2004-2010 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2009 The University of Tennessee and The University
+ * Copyright (c) 2004-2012 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart,
@@ -14,8 +14,10 @@
  * Copyright (c) 2006-2008 University of Houston.  All rights reserved.
  * Copyright (c) 2009-2010 Oracle and/or its affiliates.  All rights reserved
  * Copyright (c) 2011      Sandia National Laboratories. All rights reserved.
- * Copyright (c) 2011-2012 Los Alamos National Security, LLC. All rights
+ * Copyright (c) 2011-2015 Los Alamos National Security, LLC. All rights
  *                         reserved.
+ * Copyright (c) 2012      Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2015      FUJITSU LIMITED.  All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -31,13 +33,15 @@
 #include "opal/class/opal_bitmap.h"
 #include "opal/util/output.h"
 #include "opal/util/show_help.h"
+#include "opal_stdint.h"
+#include "opal/mca/btl/btl.h"
+#include "opal/mca/btl/base/base.h"
 
 #include "ompi/mca/pml/pml.h"
 #include "ompi/mca/pml/base/base.h"
-#include "ompi/mca/btl/btl.h"
 #include "ompi/mca/pml/base/base.h"
-#include "ompi/mca/btl/base/base.h"
 #include "ompi/mca/bml/base/base.h"
+#include "opal/mca/pmix/pmix.h"
 #include "ompi/runtime/ompi_cr.h"
 
 #include "pml_ob1.h"
@@ -54,7 +58,7 @@ mca_pml_ob1_t mca_pml_ob1 = {
         mca_pml_ob1_add_procs,
         mca_pml_ob1_del_procs,
         mca_pml_ob1_enable,
-        mca_pml_ob1_progress,
+        NULL,  /* mca_pml_ob1_progress, */
         mca_pml_ob1_add_comm,
         mca_pml_ob1_del_comm,
         mca_pml_ob1_irecv_init,
@@ -78,13 +82,13 @@ mca_pml_ob1_t mca_pml_ob1 = {
 };
 
 #if OPAL_CUDA_SUPPORT
-void mca_pml_ob1_cuda_add_ipc_support(struct mca_btl_base_module_t* btl,
-                                      int32_t flags, ompi_proc_t* errproc,
-                                      char* btlinfo);
+extern void mca_pml_ob1_cuda_add_ipc_support(struct mca_btl_base_module_t* btl,
+                                             int32_t flags, ompi_proc_t* errproc,
+                                             char* btlinfo);
 #endif /* OPAL_CUDA_SUPPORT */
 
 void mca_pml_ob1_error_handler( struct mca_btl_base_module_t* btl,
-                                int32_t flags, ompi_proc_t* errproc,
+                                int32_t flags, opal_proc_t* errproc,
                                 char* btlinfo );
 
 int mca_pml_ob1_enable(bool enable)
@@ -96,53 +100,53 @@ int mca_pml_ob1_enable(bool enable)
     OBJ_CONSTRUCT(&mca_pml_ob1.lock, opal_mutex_t);
 
     /* fragments */
-    OBJ_CONSTRUCT(&mca_pml_ob1.rdma_frags, ompi_free_list_t);
-    ompi_free_list_init_new( &mca_pml_ob1.rdma_frags,
-                         sizeof(mca_pml_ob1_rdma_frag_t),
-                         opal_cache_line_size,
-                         OBJ_CLASS(mca_pml_ob1_rdma_frag_t),
-                         0,opal_cache_line_size,
-                         mca_pml_ob1.free_list_num,
-                         mca_pml_ob1.free_list_max,
-                         mca_pml_ob1.free_list_inc,
-                         NULL );
-                                                                                                            
-    OBJ_CONSTRUCT(&mca_pml_ob1.recv_frags, ompi_free_list_t);
+    OBJ_CONSTRUCT(&mca_pml_ob1.rdma_frags, opal_free_list_t);
+    opal_free_list_init ( &mca_pml_ob1.rdma_frags,
+                          sizeof(mca_pml_ob1_rdma_frag_t),
+                          opal_cache_line_size,
+                          OBJ_CLASS(mca_pml_ob1_rdma_frag_t),
+                          0,opal_cache_line_size,
+                          mca_pml_ob1.free_list_num,
+                          mca_pml_ob1.free_list_max,
+                          mca_pml_ob1.free_list_inc,
+                          NULL, 0, NULL, NULL, NULL);
 
-    ompi_free_list_init_new( &mca_pml_ob1.recv_frags,
-                         sizeof(mca_pml_ob1_recv_frag_t) + mca_pml_ob1.unexpected_limit,
-                         opal_cache_line_size,
-                         OBJ_CLASS(mca_pml_ob1_recv_frag_t),
-                         0,opal_cache_line_size,
-                         mca_pml_ob1.free_list_num,
-                         mca_pml_ob1.free_list_max,
-                         mca_pml_ob1.free_list_inc,
-                         NULL );
-                                                                                                            
-    OBJ_CONSTRUCT(&mca_pml_ob1.pending_pckts, ompi_free_list_t);
-    ompi_free_list_init_new( &mca_pml_ob1.pending_pckts,
-                         sizeof(mca_pml_ob1_pckt_pending_t),
-                         opal_cache_line_size,
-                         OBJ_CLASS(mca_pml_ob1_pckt_pending_t),
-                         0,opal_cache_line_size,
-                         mca_pml_ob1.free_list_num,
-                         mca_pml_ob1.free_list_max,
-                         mca_pml_ob1.free_list_inc,
-                         NULL );
+    OBJ_CONSTRUCT(&mca_pml_ob1.recv_frags, opal_free_list_t);
+
+    opal_free_list_init ( &mca_pml_ob1.recv_frags,
+                          sizeof(mca_pml_ob1_recv_frag_t) + mca_pml_ob1.unexpected_limit,
+                          opal_cache_line_size,
+                          OBJ_CLASS(mca_pml_ob1_recv_frag_t),
+                          0,opal_cache_line_size,
+                          mca_pml_ob1.free_list_num,
+                          mca_pml_ob1.free_list_max,
+                          mca_pml_ob1.free_list_inc,
+                          NULL, 0, NULL, NULL, NULL);
+
+    OBJ_CONSTRUCT(&mca_pml_ob1.pending_pckts, opal_free_list_t);
+    opal_free_list_init ( &mca_pml_ob1.pending_pckts,
+                          sizeof(mca_pml_ob1_pckt_pending_t),
+                          opal_cache_line_size,
+                          OBJ_CLASS(mca_pml_ob1_pckt_pending_t),
+                          0,opal_cache_line_size,
+                          mca_pml_ob1.free_list_num,
+                          mca_pml_ob1.free_list_max,
+                          mca_pml_ob1.free_list_inc,
+                          NULL, 0, NULL, NULL, NULL);
 
 
-    OBJ_CONSTRUCT(&mca_pml_ob1.buffers, ompi_free_list_t);
-    OBJ_CONSTRUCT(&mca_pml_ob1.send_ranges, ompi_free_list_t);
-    ompi_free_list_init_new( &mca_pml_ob1.send_ranges,
-                         sizeof(mca_pml_ob1_send_range_t) +
-                         (mca_pml_ob1.max_send_per_range - 1) * sizeof(mca_pml_ob1_com_btl_t),
-                         opal_cache_line_size,
-                         OBJ_CLASS(mca_pml_ob1_send_range_t),
-                         0,opal_cache_line_size,
-                         mca_pml_ob1.free_list_num,
-                         mca_pml_ob1.free_list_max,
-                         mca_pml_ob1.free_list_inc,
-                         NULL );
+    OBJ_CONSTRUCT(&mca_pml_ob1.buffers, opal_free_list_t);
+    OBJ_CONSTRUCT(&mca_pml_ob1.send_ranges, opal_free_list_t);
+    opal_free_list_init ( &mca_pml_ob1.send_ranges,
+                          sizeof(mca_pml_ob1_send_range_t) +
+                          sizeof(mca_pml_ob1_com_btl_t[mca_pml_ob1.max_send_per_range]),
+                          opal_cache_line_size,
+                          OBJ_CLASS(mca_pml_ob1_send_range_t),
+                          0,opal_cache_line_size,
+                          mca_pml_ob1.free_list_num,
+                          mca_pml_ob1.free_list_max,
+                          mca_pml_ob1.free_list_inc,
+                          NULL, 0, NULL, NULL, NULL);
 
     /* pending operations */
     OBJ_CONSTRUCT(&mca_pml_ob1.send_pending, opal_list_t);
@@ -158,29 +162,27 @@ int mca_pml_ob1_enable(bool enable)
      * should get ownership for the send and receive requests list, and
      * initialize them with the size of our own requests.
      */
-    ompi_free_list_init_new( &mca_pml_base_send_requests,
-                         sizeof(mca_pml_ob1_send_request_t) +
-                         (mca_pml_ob1.max_rdma_per_request - 1) *
-                         sizeof(mca_pml_ob1_com_btl_t),
-                         opal_cache_line_size,
-                         OBJ_CLASS(mca_pml_ob1_send_request_t),
-                         0,opal_cache_line_size,
-                         mca_pml_ob1.free_list_num,
-                         mca_pml_ob1.free_list_max,
-                         mca_pml_ob1.free_list_inc,
-                         NULL );
+    opal_free_list_init ( &mca_pml_base_send_requests,
+                          sizeof(mca_pml_ob1_send_request_t) +
+                          sizeof(mca_pml_ob1_com_btl_t[mca_pml_ob1.max_rdma_per_request]),
+                          opal_cache_line_size,
+                          OBJ_CLASS(mca_pml_ob1_send_request_t),
+                          0,opal_cache_line_size,
+                          mca_pml_ob1.free_list_num,
+                          mca_pml_ob1.free_list_max,
+                          mca_pml_ob1.free_list_inc,
+                          NULL, 0, NULL, NULL, NULL);
 
-    ompi_free_list_init_new( &mca_pml_base_recv_requests,
-                         sizeof(mca_pml_ob1_recv_request_t) +
-                         (mca_pml_ob1.max_rdma_per_request - 1) *
-                         sizeof(mca_pml_ob1_com_btl_t),
-                         opal_cache_line_size,
-                         OBJ_CLASS(mca_pml_ob1_recv_request_t),
-                         0,opal_cache_line_size,
-                         mca_pml_ob1.free_list_num,
-                         mca_pml_ob1.free_list_max,
-                         mca_pml_ob1.free_list_inc,
-                         NULL );
+    opal_free_list_init ( &mca_pml_base_recv_requests,
+                          sizeof(mca_pml_ob1_recv_request_t) +
+                          sizeof(mca_pml_ob1_com_btl_t[mca_pml_ob1.max_rdma_per_request]),
+                          opal_cache_line_size,
+                          OBJ_CLASS(mca_pml_ob1_recv_request_t),
+                          0,opal_cache_line_size,
+                          mca_pml_ob1.free_list_num,
+                          mca_pml_ob1.free_list_max,
+                          mca_pml_ob1.free_list_inc,
+                          NULL, 0, NULL, NULL, NULL);
 
     mca_pml_ob1.enabled = true;
     return OMPI_SUCCESS;
@@ -190,11 +192,9 @@ int mca_pml_ob1_add_comm(ompi_communicator_t* comm)
 {
     /* allocate pml specific comm data */
     mca_pml_ob1_comm_t* pml_comm = OBJ_NEW(mca_pml_ob1_comm_t);
-    opal_list_item_t *item, *next_item;
-    mca_pml_ob1_recv_frag_t* frag;
+    mca_pml_ob1_recv_frag_t *frag, *next_frag;
     mca_pml_ob1_comm_proc_t* pml_proc;
     mca_pml_ob1_match_hdr_t* hdr;
-    int i;
 
     if (NULL == pml_comm) {
         return OMPI_ERR_OUT_OF_RESOURCE;
@@ -209,16 +209,8 @@ int mca_pml_ob1_add_comm(ompi_communicator_t* comm)
     mca_pml_ob1_comm_init_size(pml_comm, comm->c_remote_group->grp_proc_count);
     comm->c_pml_comm = pml_comm;
 
-    for( i = 0; i < comm->c_remote_group->grp_proc_count; i++ ) {
-        pml_comm->procs[i].ompi_proc = ompi_group_peer_lookup(comm->c_remote_group,i);
-        OBJ_RETAIN(pml_comm->procs[i].ompi_proc);
-    }
     /* Grab all related messages from the non_existing_communicator pending queue */
-    for( item = opal_list_get_first(&mca_pml_ob1.non_existing_communicator_pending);
-         item != opal_list_get_end(&mca_pml_ob1.non_existing_communicator_pending);
-         item = next_item ) {
-        frag = (mca_pml_ob1_recv_frag_t*)item;
-        next_item = opal_list_get_next(item);
+    OPAL_LIST_FOREACH_SAFE(frag, next_frag, &mca_pml_ob1.non_existing_communicator_pending, mca_pml_ob1_recv_frag_t) {
         hdr = &frag->hdr.hdr_match;
 
         /* Is this fragment for the current communicator ? */
@@ -228,8 +220,8 @@ int mca_pml_ob1_add_comm(ompi_communicator_t* comm)
         /* As we now know we work on a fragment for this communicator
          * we should remove it from the
          * non_existing_communicator_pending list. */
-        opal_list_remove_item( &mca_pml_ob1.non_existing_communicator_pending, 
-                               item );
+        opal_list_remove_item (&mca_pml_ob1.non_existing_communicator_pending,
+                               (opal_list_item_t *) frag);
 
       add_fragment_to_unexpected:
 
@@ -248,7 +240,7 @@ int mca_pml_ob1_add_comm(ompi_communicator_t* comm)
          * We just have to push the fragment into the unexpected list of the corresponding
          * proc, or into the out-of-order (cant_match) list.
          */
-        pml_proc = &(pml_comm->procs[hdr->hdr_src]);
+        pml_proc = mca_pml_ob1_peer_lookup(comm, hdr->hdr_src);
 
         if( ((uint16_t)hdr->hdr_seq) == ((uint16_t)pml_proc->expected_sequence) ) {
             /* We're now expecting the next sequence number. */
@@ -282,12 +274,6 @@ int mca_pml_ob1_add_comm(ompi_communicator_t* comm)
 
 int mca_pml_ob1_del_comm(ompi_communicator_t* comm)
 {
-    mca_pml_ob1_comm_t* pml_comm = comm->c_pml_comm;
-    int i;
-
-    for( i = 0; i < comm->c_remote_group->grp_proc_count; i++ ) {
-        OBJ_RELEASE(pml_comm->procs[i].ompi_proc);
-    }
     OBJ_RELEASE(comm->c_pml_comm);
     comm->c_pml_comm = NULL;
     return OMPI_SUCCESS;
@@ -302,9 +288,9 @@ int mca_pml_ob1_del_comm(ompi_communicator_t* comm)
 
 int mca_pml_ob1_add_procs(ompi_proc_t** procs, size_t nprocs)
 {
+    mca_btl_base_selected_module_t *sm;
     opal_bitmap_t reachable;
     int rc;
-    opal_list_item_t *item;
 
     if(nprocs == 0)
         return OMPI_SUCCESS;
@@ -316,7 +302,7 @@ int mca_pml_ob1_add_procs(ompi_proc_t** procs, size_t nprocs)
 
     /*
      * JJH: Disable this in FT enabled builds since
-     * we use a wrapper PML. It will cause this check to 
+     * we use a wrapper PML. It will cause this check to
      * return failure as all processes will return the wrapper PML
      * component in use instead of the wrapped PML component underneath.
      */
@@ -346,14 +332,10 @@ int mca_pml_ob1_add_procs(ompi_proc_t** procs, size_t nprocs)
        BTLs requires iterating over the procs, as the BML does not
        expose all currently in use btls. */
 
-    for (item = opal_list_get_first(&mca_btl_base_modules_initialized) ;
-         item != opal_list_get_end(&mca_btl_base_modules_initialized) ;
-         item = opal_list_get_next(item)) {
-        mca_btl_base_selected_module_t *sm = 
-            (mca_btl_base_selected_module_t*) item;
+    OPAL_LIST_FOREACH(sm, &mca_btl_base_modules_initialized, mca_btl_base_selected_module_t) {
         if (sm->btl_module->btl_eager_limit < sizeof(mca_pml_ob1_hdr_t)) {
             opal_show_help("help-mpi-pml-ob1.txt", "eager_limit_too_small",
-                           true, 
+                           true,
                            sm->btl_component->btl_version.mca_component_name,
                            ompi_process_info.nodename,
                            sm->btl_component->btl_version.mca_component_name,
@@ -375,7 +357,7 @@ int mca_pml_ob1_add_procs(ompi_proc_t** procs, size_t nprocs)
         if (0 != sm->btl_module->btl_cuda_eager_limit) {
             if (sm->btl_module->btl_cuda_eager_limit < sizeof(mca_pml_ob1_hdr_t)) {
                 opal_show_help("help-mpi-pml-ob1.txt", "cuda_eager_limit_too_small",
-                               true, 
+                               true,
                                sm->btl_component->btl_version.mca_component_name,
                                ompi_process_info.nodename,
                                sm->btl_component->btl_version.mca_component_name,
@@ -393,7 +375,7 @@ int mca_pml_ob1_add_procs(ompi_proc_t** procs, size_t nprocs)
         } else {
             if (sm->btl_module->btl_cuda_rdma_limit < sm->btl_module->btl_cuda_eager_limit) {
                 opal_show_help("help-mpi-pml-ob1.txt", "cuda_rdma_limit_too_small",
-                               true, 
+                               true,
                                sm->btl_component->btl_version.mca_component_name,
                                ompi_process_info.nodename,
                                sm->btl_component->btl_version.mca_component_name,
@@ -415,7 +397,7 @@ int mca_pml_ob1_add_procs(ompi_proc_t** procs, size_t nprocs)
                                NULL );
     if(OMPI_SUCCESS != rc)
         goto cleanup_and_return;
-    
+
     rc = mca_bml.bml_register( MCA_PML_OB1_HDR_TYPE_RNDV,
                                mca_pml_ob1_recv_frag_callback_rndv,
                                NULL );
@@ -427,19 +409,19 @@ int mca_pml_ob1_add_procs(ompi_proc_t** procs, size_t nprocs)
                                NULL );
     if(OMPI_SUCCESS != rc)
         goto cleanup_and_return;
-    
+
     rc = mca_bml.bml_register( MCA_PML_OB1_HDR_TYPE_ACK,
                                mca_pml_ob1_recv_frag_callback_ack,
                                NULL );
     if(OMPI_SUCCESS != rc)
         goto cleanup_and_return;
-    
+
     rc = mca_bml.bml_register( MCA_PML_OB1_HDR_TYPE_FRAG,
                                mca_pml_ob1_recv_frag_callback_frag,
                                NULL );
     if(OMPI_SUCCESS != rc)
         goto cleanup_and_return;
-    
+
     rc = mca_bml.bml_register( MCA_PML_OB1_HDR_TYPE_PUT,
                                mca_pml_ob1_recv_frag_callback_put,
                                NULL );
@@ -451,12 +433,12 @@ int mca_pml_ob1_add_procs(ompi_proc_t** procs, size_t nprocs)
                                NULL );
     if(OMPI_SUCCESS != rc)
         goto cleanup_and_return;
-    
+
     /* register error handlers */
     rc = mca_bml.bml_register_error(mca_pml_ob1_error_handler);
     if(OMPI_SUCCESS != rc)
         goto cleanup_and_return;
-    
+
   cleanup_and_return:
     OBJ_DESTRUCT(&reachable);
 
@@ -477,22 +459,145 @@ int mca_pml_ob1_del_procs(ompi_proc_t** procs, size_t nprocs)
  * diagnostics
  */
 
+static void mca_pml_ob1_dump_hdr(mca_pml_ob1_hdr_t* hdr)
+{
+    char *type, header[128];
+
+    switch(hdr->hdr_common.hdr_type) {
+    case MCA_PML_OB1_HDR_TYPE_MATCH:
+        type = "MATCH";
+        snprintf( header, 128, "ctx %5d src %d tag %d seq %d",
+                  hdr->hdr_match.hdr_ctx, hdr->hdr_match.hdr_src,
+                  hdr->hdr_match.hdr_tag, hdr->hdr_match.hdr_seq);
+        break;
+    case MCA_PML_OB1_HDR_TYPE_RNDV:
+        type = "RNDV";
+        snprintf( header, 128, "ctx %5d src %d tag %d seq %d msg_length %" PRIu64,
+                  hdr->hdr_rndv.hdr_match.hdr_ctx, hdr->hdr_rndv.hdr_match.hdr_src,
+                  hdr->hdr_rndv.hdr_match.hdr_tag, hdr->hdr_rndv.hdr_match.hdr_seq,
+                  hdr->hdr_rndv.hdr_msg_length);
+        break;
+    case MCA_PML_OB1_HDR_TYPE_RGET:
+        type = "RGET";
+        snprintf( header, 128, "ctx %5d src %d tag %d seq %d msg_length %" PRIu64
+                  "frag %" PRIu64 " src_ptr %" PRIu64,
+                  hdr->hdr_rndv.hdr_match.hdr_ctx, hdr->hdr_rndv.hdr_match.hdr_src,
+                  hdr->hdr_rndv.hdr_match.hdr_tag, hdr->hdr_rndv.hdr_match.hdr_seq,
+                  hdr->hdr_rndv.hdr_msg_length, hdr->hdr_rget.hdr_frag.lval,
+                  hdr->hdr_rget.hdr_src_ptr);
+        break;
+    case MCA_PML_OB1_HDR_TYPE_ACK:
+        type = "ACK";
+        snprintf( header, 128, "src_req %p dst_req %p offset %" PRIu64 " size %" PRIu64,
+                  hdr->hdr_ack.hdr_src_req.pval, hdr->hdr_ack.hdr_dst_req.pval,
+                  hdr->hdr_ack.hdr_send_offset, hdr->hdr_ack.hdr_send_size);
+        break;
+    case MCA_PML_OB1_HDR_TYPE_FRAG:
+        type = "FRAG";
+        snprintf( header, 128, "offset %" PRIu64 " src_req %p dst_req %p",
+                  hdr->hdr_frag.hdr_frag_offset,
+                  hdr->hdr_frag.hdr_src_req.pval, hdr->hdr_frag.hdr_dst_req.pval);
+        break;
+    case MCA_PML_OB1_HDR_TYPE_PUT:
+        type = "PUT";
+        snprintf( header, 128, "dst_req %p src_frag %p recv_req %p offset %" PRIu64
+                  " dst_ptr %" PRIu64 " dst_size %" PRIu64,
+                  hdr->hdr_rdma.hdr_req.pval, hdr->hdr_rdma.hdr_frag.pval,
+                  hdr->hdr_rdma.hdr_recv_req.pval, hdr->hdr_rdma.hdr_rdma_offset,
+                  hdr->hdr_rdma.hdr_dst_ptr, hdr->hdr_rdma.hdr_dst_size);
+        break;
+    case MCA_PML_OB1_HDR_TYPE_FIN:
+        type = "FIN";
+        header[0] = '\0';
+        break;
+    default:
+        type = "UNKWN";
+        header[0] = '\0';
+        break;
+    }
+    opal_output(0,"hdr %s [%s] %s", type,
+                (hdr->hdr_common.hdr_flags & MCA_PML_OB1_HDR_FLAGS_NBO ? "nbo" : "   "),
+                header);
+}
+
+static void mca_pml_ob1_dump_frag_list(opal_list_t* queue, bool is_req)
+{
+    opal_list_item_t* item;
+    char cpeer[64], ctag[64];
+
+    for( item = opal_list_get_first(queue);
+         item != opal_list_get_end(queue);
+         item =  opal_list_get_next(item) ) {
+
+        if( is_req ) {
+            mca_pml_base_request_t *req = &(((mca_pml_ob1_recv_request_t*)item)->req_recv.req_base);
+
+            if( OMPI_ANY_SOURCE == req->req_peer ) snprintf(cpeer, 64, "%s", "ANY_SOURCE");
+            else snprintf(cpeer, 64, "%d", req->req_peer);
+
+            if( OMPI_ANY_TAG == req->req_tag ) snprintf(ctag, 64, "%s", "ANY_TAG");
+            else snprintf(ctag, 64, "%d", req->req_tag);
+
+            opal_output(0, "req %p peer %s tag %s addr %p count %lu datatype %s [%p] [%s %s] req_seq %" PRIu64,
+                        (void*) req, cpeer, ctag,
+                        (void*) req->req_addr, req->req_count,
+                        (0 != req->req_count ? req->req_datatype->name : "N/A"),
+                        (void*) req->req_datatype,
+                        (req->req_pml_complete ? "pml_complete" : ""),
+                        (req->req_free_called ? "freed" : ""),
+                        req->req_sequence);
+        } else {
+            mca_pml_ob1_recv_frag_t* frag = (mca_pml_ob1_recv_frag_t*)item;
+            mca_pml_ob1_dump_hdr( &frag->hdr );
+        }
+    }
+}
+
 int mca_pml_ob1_dump(struct ompi_communicator_t* comm, int verbose)
 {
     struct mca_pml_comm_t* pml_comm = comm->c_pml_comm;
     int i;
 
+    /* TODO: don't forget to dump mca_pml_ob1.non_existing_communicator_pending */
+
+    opal_output(0, "Communicator %s [%p](%d) rank %d recv_seq %d num_procs %lu last_probed %lu\n",
+                comm->c_name, (void*) comm, comm->c_contextid, comm->c_my_rank,
+                pml_comm->recv_sequence, pml_comm->num_procs, pml_comm->last_probed);
+    if( opal_list_get_size(&pml_comm->wild_receives) ) {
+        opal_output(0, "expected MPI_ANY_SOURCE fragments\n");
+        mca_pml_ob1_dump_frag_list(&pml_comm->wild_receives, true);
+    }
+
     /* iterate through all procs on communicator */
     for( i = 0; i < (int)pml_comm->num_procs; i++ ) {
-        mca_pml_ob1_comm_proc_t* proc = &pml_comm->procs[i];
-        mca_bml_base_endpoint_t* ep = (mca_bml_base_endpoint_t*)proc->ompi_proc->proc_endpoints[OMPI_PROC_ENDPOINT_TAG_BML];
+        mca_pml_ob1_comm_proc_t* proc = pml_comm->procs[i];
+
+        if (NULL == proc) {
+            continue;
+        }
+
+        mca_bml_base_endpoint_t* ep = mca_bml_base_get_endpoint(proc->ompi_proc);
         size_t n;
 
-        opal_output(0, "[Rank %d]\n", i);
-        /* dump all receive queues */
+        opal_output(0, "[Rank %d] expected_seq %d ompi_proc %p send_seq %d\n",
+                    i, proc->expected_sequence, (void*) proc->ompi_proc,
+                    proc->send_sequence);
 
-        /* dump all btls */
-        for(n=0; n<ep->btl_eager.arr_size; n++) {
+        /* dump all receive queues */
+        if( opal_list_get_size(&proc->specific_receives) ) {
+            opal_output(0, "expected specific receives\n");
+            mca_pml_ob1_dump_frag_list(&proc->specific_receives, true);
+        }
+        if( opal_list_get_size(&proc->frags_cant_match) ) {
+            opal_output(0, "out of sequence\n");
+            mca_pml_ob1_dump_frag_list(&proc->frags_cant_match, false);
+        }
+        if( opal_list_get_size(&proc->unexpected_frags) ) {
+            opal_output(0, "unexpected frag\n");
+            mca_pml_ob1_dump_frag_list(&proc->unexpected_frags, false);
+        }
+        /* dump all btls used for eager messages */
+        for( n = 0; n < ep->btl_eager.arr_size; n++ ) {
             mca_bml_base_btl_t* bml_btl = &ep->btl_eager.bml_btls[n];
             bml_btl->btl->btl_dump(bml_btl->btl, bml_btl->btl_endpoint, verbose);
         }
@@ -505,8 +610,8 @@ static void mca_pml_ob1_fin_completion( mca_btl_base_module_t* btl,
                                         struct mca_btl_base_descriptor_t* des,
                                         int status )
 {
-    
-    mca_bml_base_btl_t* bml_btl = (mca_bml_base_btl_t*) des->des_context; 
+
+    mca_bml_base_btl_t* bml_btl = (mca_bml_base_btl_t*) des->des_context;
 
     /* check for pending requests */
     MCA_PML_OB1_PROGRESS_PENDING(bml_btl);
@@ -520,37 +625,32 @@ static void mca_pml_ob1_fin_completion( mca_btl_base_module_t* btl,
  */
 int mca_pml_ob1_send_fin( ompi_proc_t* proc,
                           mca_bml_base_btl_t* bml_btl,
-                          ompi_ptr_t hdr_des,
+                          opal_ptr_t hdr_frag,
+                          uint64_t rdma_size,
                           uint8_t order,
-                          uint32_t status )
+                          int status )
 {
     mca_btl_base_descriptor_t* fin;
-    mca_pml_ob1_fin_hdr_t* hdr;
     int rc;
 
     mca_bml_base_alloc(bml_btl, &fin, order, sizeof(mca_pml_ob1_fin_hdr_t),
-                       MCA_BTL_DES_FLAGS_PRIORITY | MCA_BTL_DES_FLAGS_BTL_OWNERSHIP);
+                       MCA_BTL_DES_FLAGS_PRIORITY | MCA_BTL_DES_FLAGS_BTL_OWNERSHIP | MCA_BTL_DES_FLAGS_SIGNAL);
 
     if(NULL == fin) {
-        MCA_PML_OB1_ADD_FIN_TO_PENDING(proc, hdr_des, bml_btl, order, status);
+        MCA_PML_OB1_ADD_FIN_TO_PENDING(proc, hdr_frag, rdma_size, bml_btl, order, status);
         return OMPI_ERR_OUT_OF_RESOURCE;
     }
     fin->des_cbfunc = mca_pml_ob1_fin_completion;
     fin->des_cbdata = NULL;
 
     /* fill in header */
-    hdr = (mca_pml_ob1_fin_hdr_t*)fin->des_src->seg_addr.pval;
-    hdr->hdr_common.hdr_flags = 0;
-    hdr->hdr_common.hdr_type = MCA_PML_OB1_HDR_TYPE_FIN;
-    hdr->hdr_des = hdr_des;
-    hdr->hdr_fail = status;
+    mca_pml_ob1_fin_hdr_prepare ((mca_pml_ob1_fin_hdr_t *) fin->des_segments->seg_addr.pval,
+                                 0, hdr_frag.lval, status ? status : (int64_t) rdma_size);
 
-    ob1_hdr_hton(hdr, MCA_PML_OB1_HDR_TYPE_FIN, proc);
+    ob1_hdr_hton((mca_pml_ob1_hdr_t *) fin->des_segments->seg_addr.pval, MCA_PML_OB1_HDR_TYPE_FIN, proc);
 
     /* queue request */
-    rc = mca_bml_base_send( bml_btl,
-                            fin,
-                            MCA_PML_OB1_HDR_TYPE_FIN );
+    rc = mca_bml_base_send( bml_btl, fin, MCA_PML_OB1_HDR_TYPE_FIN );
     if( OPAL_LIKELY( rc >= 0 ) ) {
         if( OPAL_LIKELY( 1 == rc ) ) {
             MCA_PML_OB1_PROGRESS_PENDING(bml_btl);
@@ -558,7 +658,7 @@ int mca_pml_ob1_send_fin( ompi_proc_t* proc,
         return OMPI_SUCCESS;
     }
     mca_bml_base_free(bml_btl, fin);
-    MCA_PML_OB1_ADD_FIN_TO_PENDING(proc, hdr_des, bml_btl, order, status);
+    MCA_PML_OB1_ADD_FIN_TO_PENDING(proc, hdr_frag, rdma_size, bml_btl, order, status);
     return OMPI_ERR_OUT_OF_RESOURCE;
 }
 
@@ -575,11 +675,11 @@ void mca_pml_ob1_process_pending_packets(mca_bml_base_btl_t* bml_btl)
         OPAL_THREAD_UNLOCK(&mca_pml_ob1.lock);
         if(NULL == pckt)
             break;
-        if(pckt->bml_btl != NULL && 
+        if(pckt->bml_btl != NULL &&
                 pckt->bml_btl->btl == bml_btl->btl) {
             send_dst = pckt->bml_btl;
         } else {
-            mca_bml_base_endpoint_t* endpoint = 
+            mca_bml_base_endpoint_t* endpoint =
                 (mca_bml_base_endpoint_t*) pckt->proc->proc_endpoints[OMPI_PROC_ENDPOINT_TAG_BML];
             send_dst = mca_bml_base_btl_array_find(
                     &endpoint->btl_eager, bml_btl->btl);
@@ -599,6 +699,7 @@ void mca_pml_ob1_process_pending_packets(mca_bml_base_btl_t* bml_btl)
                         pckt->hdr.hdr_ack.hdr_src_req.lval,
                         pckt->hdr.hdr_ack.hdr_dst_req.pval,
                         pckt->hdr.hdr_ack.hdr_send_offset,
+                        pckt->hdr.hdr_ack.hdr_send_size,
                         pckt->hdr.hdr_common.hdr_flags & MCA_PML_OB1_HDR_FLAGS_NORDMA);
                 if( OPAL_UNLIKELY(OMPI_ERR_OUT_OF_RESOURCE == rc) ) {
                     OPAL_THREAD_LOCK(&mca_pml_ob1.lock);
@@ -610,10 +711,12 @@ void mca_pml_ob1_process_pending_packets(mca_bml_base_btl_t* bml_btl)
                 break;
             case MCA_PML_OB1_HDR_TYPE_FIN:
                 rc = mca_pml_ob1_send_fin(pckt->proc, send_dst,
-                                          pckt->hdr.hdr_fin.hdr_des,
+                                          pckt->hdr.hdr_fin.hdr_frag,
+                                          pckt->hdr.hdr_fin.hdr_size,
                                           pckt->order,
-                                          pckt->hdr.hdr_fin.hdr_fail);
+                                          pckt->status);
                 if( OPAL_UNLIKELY(OMPI_ERR_OUT_OF_RESOURCE == rc) ) {
+                    MCA_PML_OB1_PCKT_PENDING_RETURN(pckt);
                     return;
                 }
                 break;
@@ -655,14 +758,14 @@ void mca_pml_ob1_process_pending_rdma(void)
 
 void mca_pml_ob1_error_handler(
         struct mca_btl_base_module_t* btl, int32_t flags,
-        ompi_proc_t* errproc, char* btlinfo ) { 
+        opal_proc_t* errproc, char* btlinfo ) {
 #if OPAL_CUDA_SUPPORT
     if (flags & MCA_BTL_ERROR_FLAGS_ADD_CUDA_IPC) {
-        mca_pml_ob1_cuda_add_ipc_support(btl, flags, errproc, btlinfo);
+        mca_pml_ob1_cuda_add_ipc_support(btl, flags, (struct ompi_proc_t*)errproc, btlinfo);
         return;
     }
 #endif /* OPAL_CUDA_SUPPORT */
-    ompi_rte_abort(-1, NULL);
+    ompi_rte_abort(-1, btlinfo);
 }
 
 #if OPAL_ENABLE_FT_CR    == 0
@@ -680,8 +783,7 @@ int mca_pml_ob1_ft_event( int state )
     if(OPAL_CRS_CHECKPOINT == state) {
         if( opal_cr_timing_barrier_enabled ) {
             OPAL_CR_SET_TIMER(OPAL_CR_TIMER_CRCPBR1);
-            ompi_rte_barrier(coll);
-            OMPI_WAIT_FOR_COMPLETION(coll->active);
+            opal_pmix.fence(NULL, 0);
         }
 
         OPAL_CR_SET_TIMER(OPAL_CR_TIMER_P2P0);
@@ -689,16 +791,15 @@ int mca_pml_ob1_ft_event( int state )
     else if(OPAL_CRS_CONTINUE == state) {
         first_continue_pass = !first_continue_pass;
 
-        if( !first_continue_pass ) { 
+        if( !first_continue_pass ) {
             if( opal_cr_timing_barrier_enabled ) {
                 OPAL_CR_SET_TIMER(OPAL_CR_TIMER_COREBR0);
-                ompi_rte_barrier(coll);
-                OMPI_WAIT_FOR_COMPLETION(coll->active);
+                opal_pmix.fence(NULL, 0);
             }
             OPAL_CR_SET_TIMER(OPAL_CR_TIMER_P2P2);
         }
 
-        if( orte_cr_continue_like_restart && !first_continue_pass ) {
+        if (opal_cr_continue_like_restart && !first_continue_pass) {
             /*
              * Get a list of processes
              */
@@ -711,7 +812,7 @@ int mca_pml_ob1_ft_event( int state )
              * Refresh the proc structure, and publish our proc info in the modex.
              * NOTE: Do *not* call ompi_proc_finalize as there are many places in
              *       the code that point to indv. procs in this strucutre. For our
-             *       needs here we only need to fix up the modex, bml and pml 
+             *       needs here we only need to fix up the modex, bml and pml
              *       references.
              */
             if (OMPI_SUCCESS != (ret = ompi_proc_refresh())) {
@@ -740,7 +841,7 @@ int mca_pml_ob1_ft_event( int state )
 
         /*
          * Clean out the modex information since it is invalid now.
-         *    orte_grpcomm.purge_proc_attrs();
+         *    ompi_rte_purge_proc_attrs();
          * This happens at the ORTE level, so doing it again here will cause
          * some issues with socket caching.
          */
@@ -750,7 +851,7 @@ int mca_pml_ob1_ft_event( int state )
          * Refresh the proc structure, and publish our proc info in the modex.
          * NOTE: Do *not* call ompi_proc_finalize as there are many places in
          *       the code that point to indv. procs in this strucutre. For our
-         *       needs here we only need to fix up the modex, bml and pml 
+         *       needs here we only need to fix up the modex, bml and pml
          *       references.
          */
         if (OMPI_SUCCESS != (ret = ompi_proc_refresh())) {
@@ -780,36 +881,26 @@ int mca_pml_ob1_ft_event( int state )
         opal_output(0, "pml:base: ft_event: BML ft_event function failed: %d\n",
                     ret);
     }
-    
+
     if(OPAL_CRS_CHECKPOINT == state) {
         OPAL_CR_SET_TIMER(OPAL_CR_TIMER_P2P1);
 
         if( opal_cr_timing_barrier_enabled ) {
             OPAL_CR_SET_TIMER(OPAL_CR_TIMER_P2PBR0);
-            /* JJH Cannot barrier here due to progress engine -- orte_grpcomm.barrier();*/
+            /* JJH Cannot barrier here due to progress engine -- ompi_rte_barrier();*/
         }
     }
     else if(OPAL_CRS_CONTINUE == state) {
         if( !first_continue_pass ) {
             if( opal_cr_timing_barrier_enabled ) {
                 OPAL_CR_SET_TIMER(OPAL_CR_TIMER_P2PBR1);
-                ompi_rte_barrier(coll);
-                OMPI_WAIT_FOR_COMPLETION(coll->active);
+                opal_pmix.fence(NULL, 0);
             }
             OPAL_CR_SET_TIMER(OPAL_CR_TIMER_P2P3);
         }
 
-        if( orte_cr_continue_like_restart && !first_continue_pass ) {
-            /*
-             * Exchange the modex information once again.
-             * BTLs will have republished their modex information.
-             */
-            if (OMPI_SUCCESS != (ret = orte_grpcomm.modex(NULL))) {
-                opal_output(0,
-                            "pml:ob1: ft_event(Restart): Failed orte_grpcomm.modex() = %d",
-                            ret);
-                return ret;
-            }
+        if (opal_cr_continue_like_restart && !first_continue_pass) {
+            opal_pmix.fence(NULL, 0);
 
             /*
              * Startup the PML stack now that the modex is running again
@@ -821,10 +912,7 @@ int mca_pml_ob1_ft_event( int state )
             }
 
             /* Is this barrier necessary ? JJH */
-            if (OMPI_SUCCESS != (ret = ompi_rte_barrier())) {
-                opal_output(0, "pml:ob1: ft_event(Restart): Failed in orte_grpcomm.barrier (%d)", ret);
-                return ret;
-            }
+            opal_pmix.fence(NULL, 0);
 
             if( NULL != procs ) {
                 for(p = 0; p < (int)num_procs; ++p) {
@@ -837,8 +925,7 @@ int mca_pml_ob1_ft_event( int state )
         if( !first_continue_pass ) {
             if( opal_cr_timing_barrier_enabled ) {
                 OPAL_CR_SET_TIMER(OPAL_CR_TIMER_P2PBR2);
-                ompi_rte_barrier(coll);
-                OMPI_WAIT_FOR_COMPLETION(coll->active);
+                opal_pmix.fence(NULL, 0);
             }
             OPAL_CR_SET_TIMER(OPAL_CR_TIMER_CRCP1);
         }
@@ -851,12 +938,7 @@ int mca_pml_ob1_ft_event( int state )
          * Exchange the modex information once again.
          * BTLs will have republished their modex information.
          */
-        if (OMPI_SUCCESS != (ret = orte_grpcomm.modex(NULL))) {
-            opal_output(0,
-                        "pml:ob1: ft_event(Restart): Failed orte_grpcomm.modex() = %d",
-                        ret);
-            return ret;
-        }
+        opal_pmix.fence(NULL, 0);
 
         /*
          * Startup the PML stack now that the modex is running again
@@ -868,10 +950,7 @@ int mca_pml_ob1_ft_event( int state )
         }
 
         /* Is this barrier necessary ? JJH */
-        if (OMPI_SUCCESS != (ret = orte_grpcomm.barrier())) {
-            opal_output(0, "pml:ob1: ft_event(Restart): Failed in orte_grpcomm.barrier (%d)", ret);
-            return ret;
-        }
+        opal_pmix.fence(NULL, 0);
 
         if( NULL != procs ) {
             for(p = 0; p < (int)num_procs; ++p) {
