@@ -2,8 +2,6 @@
  * Copyright (C) 2001-2011 Mellanox Technologies Ltd. ALL RIGHTS RESERVED.
  * Copyright (c) 2015      Research Organization for Information Science
  *                         and Technology (RIST). All rights reserved.
- * Copyright (c) 2015      Los Alamos National Security, LLC. All rights
- *                         reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -20,7 +18,7 @@
 
 #include "opal/runtime/opal.h"
 #include "opal/memoryhooks/memory.h"
-#include "opal/util/opal_environ.h"
+#include "opal/mca/memory/base/base.h"
 #include "opal/mca/pmix/pmix.h"
 #include "ompi/mca/pml/base/pml_base_bsend.h"
 #include "ompi/message/message.h"
@@ -73,14 +71,14 @@ static int send_ep_address(void)
     address = alloca(addrlen);
     error = mxm_ep_get_address(ompi_pml_yalla.mxm_ep, address, &addrlen);
     if (MXM_OK != error) {
-        PML_YALLA_ERROR("Failed to get EP address");
+        PML_YALLA_ERROR("%s", "Failed to get EP address");
         return OMPI_ERROR;
     }
 
     OPAL_MODEX_SEND(rc, OPAL_PMIX_GLOBAL,
                     &mca_pml_yalla_component.pmlm_version, address, addrlen);
     if (OMPI_SUCCESS != rc) {
-        PML_YALLA_ERROR("Open MPI couldn't distribute EP connection details");
+        PML_YALLA_ERROR("%s", "Open MPI couldn't distribute EP connection details");
         return OMPI_ERROR;
     }
 
@@ -94,7 +92,7 @@ static int recv_ep_address(ompi_proc_t *proc, void **address_p, size_t *addrlen_
     OPAL_MODEX_RECV(rc, &mca_pml_yalla_component.pmlm_version, &proc->super.proc_name,
                     address_p, addrlen_p);
     if (rc < 0) {
-        PML_YALLA_ERROR("Failed to receive EP address");
+        PML_YALLA_ERROR("%s", "Failed to receive EP address");
     }
     return rc;
 }
@@ -110,18 +108,20 @@ int mca_pml_yalla_open(void)
 {
     mxm_error_t error;
 
-    PML_YALLA_VERBOSE(1, "mca_pml_yalla_open");
+    PML_YALLA_VERBOSE(1, "%s", "mca_pml_yalla_open");
+
+    (void)mca_base_framework_open(&opal_memory_base_framework, 0);
 
     /* Set memory hooks */
     if ((OPAL_MEMORY_FREE_SUPPORT | OPAL_MEMORY_MUNMAP_SUPPORT) ==
         ((OPAL_MEMORY_FREE_SUPPORT | OPAL_MEMORY_MUNMAP_SUPPORT) &
          opal_mem_hooks_support_level()))
     {
-        PML_YALLA_VERBOSE(1, "enabling on-demand memory mapping");
+        PML_YALLA_VERBOSE(1, "%s", "enabling on-demand memory mapping");
         opal_setenv("MXM_MPI_MEM_ON_DEMAND_MAP", "y", false, &environ);
         ompi_pml_yalla.using_mem_hooks = 1;
     } else {
-        PML_YALLA_VERBOSE(1, "disabling on-demand memory mapping");
+        PML_YALLA_VERBOSE(1, "%s", "disabling on-demand memory mapping");
         ompi_pml_yalla.using_mem_hooks = 0;
     }
     opal_setenv("MXM_MPI_SINGLE_THREAD", ompi_mpi_thread_multiple ? "n" : "y",
@@ -144,7 +144,7 @@ int mca_pml_yalla_open(void)
 
 int mca_pml_yalla_close(void)
 {
-    PML_YALLA_VERBOSE(1, "mca_pml_yalla_close");
+    PML_YALLA_VERBOSE(1, "%s", "mca_pml_yalla_close");
 
     if (ompi_pml_yalla.ctx_opts != NULL) {
         mxm_config_free_context_opts(ompi_pml_yalla.ctx_opts);
@@ -156,6 +156,7 @@ int mca_pml_yalla_close(void)
         mxm_cleanup(ompi_pml_yalla.mxm_context);
         ompi_pml_yalla.mxm_context = NULL;
     }
+    mca_base_framework_close(&opal_memory_base_framework);
     return 0;
 }
 
@@ -164,7 +165,7 @@ int mca_pml_yalla_init(void)
     mxm_error_t error;
     int rc;
 
-    PML_YALLA_VERBOSE(1, "mca_pml_yalla_init");
+    PML_YALLA_VERBOSE(1, "%s", "mca_pml_yalla_init");
 
     if (ompi_pml_yalla.using_mem_hooks) {
         opal_mem_hooks_register_release(mca_pml_yalla_mem_release_cb, NULL);
@@ -197,7 +198,7 @@ int mca_pml_yalla_init(void)
 
 int mca_pml_yalla_cleanup(void)
 {
-    PML_YALLA_VERBOSE(1, "mca_pml_yalla_cleanup");
+    PML_YALLA_VERBOSE(1, "%s", "mca_pml_yalla_cleanup");
 
     opal_progress_unregister(mca_pml_yalla_progress);
 
@@ -250,7 +251,7 @@ int mca_pml_yalla_add_procs(struct ompi_proc_t **procs, size_t nprocs)
         free(address);
 
         if (MXM_OK != error) {
-            PML_YALLA_ERROR("Failed to connect");
+            PML_YALLA_ERROR("%s", "Failed to connect");
             return OMPI_ERROR;
         }
 
@@ -265,7 +266,7 @@ int mca_pml_yalla_del_procs(struct ompi_proc_t **procs, size_t nprocs)
     size_t i;
 
     if (ompi_mpi_finalized) {
-        PML_YALLA_VERBOSE(3, "using bulk powerdown");
+        PML_YALLA_VERBOSE(3, "%s", "using bulk powerdown");
         mxm_ep_powerdown(ompi_pml_yalla.mxm_ep);
     }
 
@@ -312,7 +313,7 @@ int mca_pml_yalla_del_comm(struct ompi_communicator_t* comm)
     mxm_mq_h mq = (void*)comm->c_pml_comm;
 
     if (ompi_pml_yalla.mxm_context == NULL) {
-        PML_YALLA_ERROR("Destroying communicator after MXM context is destroyed");
+        PML_YALLA_ERROR("%s", "Destroying communicator after MXM context is destroyed");
         return OMPI_ERROR;
     }
 
@@ -368,7 +369,6 @@ int mca_pml_yalla_recv(void *buf, size_t count, ompi_datatype_t *datatype, int s
 {
     mxm_recv_req_t rreq;
     mxm_error_t error;
-    int rc;
 
     PML_YALLA_INIT_MXM_RECV_REQ(&rreq, buf, count, datatype, src, tag, comm, recv);
     PML_YALLA_INIT_BLOCKING_MXM_RECV_REQ(&rreq);
@@ -387,10 +387,10 @@ int mca_pml_yalla_recv(void *buf, size_t count, ompi_datatype_t *datatype, int s
                       rreq.completion.sender_imm, rreq.completion.sender_tag,
                       rreq.tag, rreq.tag_mask,
                       rreq.completion.actual_len);
-    rc = PML_YALLA_SET_RECV_STATUS(&rreq, rreq.completion.actual_len, status);
+    PML_YALLA_SET_RECV_STATUS(&rreq, rreq.completion.actual_len, status);
     PML_YALLA_FREE_BLOCKING_MXM_REQ(&rreq.base);
 
-    return rc;
+    return OMPI_SUCCESS;
 }
 
 int mca_pml_yalla_isend_init(const void *buf, size_t count, ompi_datatype_t *datatype,
@@ -400,7 +400,7 @@ int mca_pml_yalla_isend_init(const void *buf, size_t count, ompi_datatype_t *dat
 {
     mca_pml_yalla_send_request_t *sreq;
 
-    sreq = MCA_PML_YALLA_SREQ_INIT(buf, count, datatype, dst, tag, mode, comm,
+    sreq = MCA_PML_YALLA_SREQ_INIT((void *)buf, count, datatype, dst, tag, mode, comm,
                                    OMPI_REQUEST_INACTIVE);
     sreq->super.ompi.req_persistent = true;
     sreq->super.flags = MCA_PML_YALLA_REQUEST_FLAG_SEND;
@@ -469,7 +469,7 @@ int mca_pml_yalla_isend(const void *buf, size_t count, ompi_datatype_t *datatype
     mxm_error_t error;
     int rc;
 
-    sreq = MCA_PML_YALLA_SREQ_INIT(buf, count, datatype, dst, tag, mode, comm,
+    sreq = MCA_PML_YALLA_SREQ_INIT((void *)buf, count, datatype, dst, tag, mode, comm,
                                    OMPI_REQUEST_ACTIVE);
     sreq->super.ompi.req_persistent = false;
     sreq->super.flags = 0;
@@ -501,7 +501,7 @@ int mca_pml_yalla_send(const void *buf, size_t count, ompi_datatype_t *datatype,
     mxm_send_req_t sreq;
     mxm_error_t error;
 
-    PML_YALLA_INIT_MXM_SEND_REQ(&sreq, buf, count, datatype, dst, tag, mode, comm, send);
+    PML_YALLA_INIT_MXM_SEND_REQ(&sreq, (void *)buf, count, datatype, dst, tag, mode, comm, send);
     PML_YALLA_INIT_BLOCKING_MXM_SEND_REQ(&sreq);
 
     PML_YALLA_VERBOSE(8, "send to %d tag %d dtype %s count %zu", dst, tag,
@@ -678,13 +678,13 @@ int mca_pml_yalla_mrecv(void *buf, size_t count, ompi_datatype_t *datatype,
                       rreq.completion.sender_imm, rreq.completion.sender_tag,
                       rreq.tag, rreq.tag_mask,
                       rreq.completion.actual_len);
-    return PML_YALLA_SET_RECV_STATUS(&rreq, rreq.completion.actual_len, status);
+    PML_YALLA_SET_RECV_STATUS(&rreq, rreq.completion.actual_len, status);
+    return OMPI_SUCCESS;
 }
 
 int mca_pml_yalla_start(size_t count, ompi_request_t** requests)
 {
     mca_pml_yalla_base_request_t *req;
-    mca_pml_yalla_send_request_t *sreq;
     mxm_error_t error;
     size_t i;
     int rc;
@@ -699,10 +699,12 @@ int mca_pml_yalla_start(size_t count, ompi_request_t** requests)
 
         PML_YALLA_ASSERT(req->ompi.req_state != OMPI_REQUEST_INVALID);
         PML_YALLA_RESET_OMPI_REQ(&req->ompi, OMPI_REQUEST_ACTIVE);
-        PML_YALLA_RESET_PML_REQ(req);
 
         if (req->flags & MCA_PML_YALLA_REQUEST_FLAG_SEND) {
+            mca_pml_yalla_send_request_t *sreq;
             sreq = (mca_pml_yalla_send_request_t *)req;
+            PML_YALLA_RESET_PML_REQ(req, PML_YALLA_MXM_REQBASE(sreq));
+
             if (req->flags & MCA_PML_YALLA_REQUEST_FLAG_BSEND) {
                 PML_YALLA_VERBOSE(8, "start bsend request %p", (void *)sreq);
                 rc = mca_pml_yalla_bsend(&sreq->mxm);
@@ -719,8 +721,12 @@ int mca_pml_yalla_start(size_t count, ompi_request_t** requests)
                 }
             }
         } else {
+            mca_pml_yalla_recv_request_t *rreq;
+            rreq = (mca_pml_yalla_recv_request_t *)req;
+            PML_YALLA_RESET_PML_REQ(req, PML_YALLA_MXM_REQBASE(rreq));
+
             PML_YALLA_VERBOSE(8, "start recv request %p", (void *)req);
-            error = mxm_req_recv(&((mca_pml_yalla_recv_request_t *)req)->mxm);
+            error = mxm_req_recv(&rreq->mxm);
             if (MXM_OK != error) {
                 return OMPI_ERROR;
             }
