@@ -3,11 +3,14 @@
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
  * Copyright (c) 2009      Cisco Systems, Inc.  All rights reserved.
- * Copyright (c) 2011-2013 The University of Tennessee and The University
+ * Copyright (c) 2011-2017 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
- * Copyright (c) 2011-2013 INRIA.  All rights reserved.
+ * Copyright (c) 2011-2013 Inria.  All rights reserved.
  * Copyright (c) 2011-2013 Université Bordeaux 1
+ * Copyright (c) 2014-2015 Research Organization for Information Science
+ *                         and Technology (RIST). All rights reserved.
+ * Copyright (c) 2016      IBM Corporation.  All rights reserved.
  */
 
 #include "ompi_config.h"
@@ -30,11 +33,11 @@ typedef struct _dist_graph_elem {
 } mca_topo_base_dist_graph_elem_t;
 
 int mca_topo_base_dist_graph_distribute(mca_topo_base_module_t* module,
-                                        ompi_communicator_t *comm, 
-                                        int n, int nodes[],
-                                        int degrees[], int targets[], 
-                                        int weights[],
-                                        mca_topo_base_comm_dist_graph_2_1_0_t** ptopo)
+                                        ompi_communicator_t *comm,
+                                        int n, const int nodes[],
+                                        const int degrees[], const int targets[],
+                                        const int weights[],
+                                        mca_topo_base_comm_dist_graph_2_2_0_t** ptopo)
 {
     int i, j, err, count, left_over, pending_reqs, current_pos, index, csize;
     int *rin = NULL, *rout, *temp = NULL;
@@ -42,7 +45,7 @@ int mca_topo_base_dist_graph_distribute(mca_topo_base_module_t* module,
     size_t int_size, how_much;
     ompi_status_public_t status;
     ompi_request_t **reqs = NULL;
-    mca_topo_base_comm_dist_graph_2_1_0_t* topo=NULL;
+    mca_topo_base_comm_dist_graph_2_2_0_t* topo=NULL;
 
     ompi_datatype_type_size( (ompi_datatype_t*)&ompi_mpi_int, &int_size);
 
@@ -52,7 +55,7 @@ int mca_topo_base_dist_graph_distribute(mca_topo_base_module_t* module,
      * In addition we compute 3 arrays (that are allocated in one go):
      * - cnt: the number of elements for a peer
      * - pos: the position of the first element for a peer
-     * - idx: temporaru indexes and message count after the reduce.
+     * - idx: temporary indexes and message count after the reduce.
      */
     cnt = (mca_topo_base_dist_graph_elem_t*)calloc(3 * csize, sizeof(mca_topo_base_dist_graph_elem_t));
     if( NULL == cnt ) {
@@ -111,21 +114,19 @@ int mca_topo_base_dist_graph_distribute(mca_topo_base_module_t* module,
         }
     }
 
-    err = comm->c_coll.coll_reduce_scatter_block( MPI_IN_PLACE, idx, 2,
+    err = comm->c_coll->coll_reduce_scatter_block( MPI_IN_PLACE, idx, 2,
                                                   (ompi_datatype_t*)&ompi_mpi_int, MPI_SUM, comm,
-                                                  comm->c_coll.coll_allreduce_module);
+                                                  comm->c_coll->coll_reduce_scatter_block_module);
     /**
      * At this point in the indexes array we have:
-     * - indexes[0] total number of in edges
-     * - indexes[1] total number of out edges
+     * - idx[0].in  total number of IN  edges
+     * - idx[0].out total number of OUT edges
      */
-    topo = (mca_topo_base_comm_dist_graph_2_1_0_t*)malloc(sizeof(mca_topo_base_comm_dist_graph_2_1_0_t));
+    topo = OBJ_NEW(mca_topo_base_comm_dist_graph_2_2_0_t);
     if( NULL == topo ) {
         err = OMPI_ERR_OUT_OF_RESOURCE;
         goto bail_out;
     }
-    topo->in = topo->inw = NULL;
-    topo->out = topo->outw = NULL;
     topo->indegree  = idx[0].in;
     topo->outdegree = idx[0].out;
     topo->weighted = (weights != MPI_UNWEIGHTED);
@@ -134,7 +135,7 @@ int mca_topo_base_dist_graph_distribute(mca_topo_base_module_t* module,
         if (NULL == topo->in) {
             err = OMPI_ERR_OUT_OF_RESOURCE;
             goto bail_out;
-        }    
+        }
         if (MPI_UNWEIGHTED != weights) {
             topo->inw = (int*)malloc(sizeof(int) * topo->indegree);
             if (NULL == topo->inw) {
@@ -148,7 +149,7 @@ int mca_topo_base_dist_graph_distribute(mca_topo_base_module_t* module,
         if (NULL == topo->out) {
             err = OMPI_ERR_OUT_OF_RESOURCE;
             goto bail_out;
-        }    
+        }
         if (MPI_UNWEIGHTED != weights) {
             topo->outw = (int*)malloc(sizeof(int) * topo->outdegree);
             if (NULL == topo->outw) {
@@ -273,29 +274,17 @@ int mca_topo_base_dist_graph_distribute(mca_topo_base_module_t* module,
         free(cnt);
     }
     if( NULL != topo ) {
-        if ( NULL != topo->in ) {
-            free(topo->in);
-        }
-        if ( NULL != topo->out ) {
-            free(topo->out);
-        }
-        if ( NULL != topo->inw ) {
-            free(topo->inw);
-        }
-        if ( NULL != topo->outw ) {
-            free(topo->outw);
-        }
-        free(topo);
+        OBJ_RELEASE(topo);
     }
     return err;
 }
 
 int mca_topo_base_dist_graph_create(mca_topo_base_module_t* module,
-                                    ompi_communicator_t *comm_old, 
-                                    int n, int nodes[],
-                                    int degrees[], int targets[], 
-                                    int weights[],
-                                    ompi_info_t *info, int reorder, 
+                                    ompi_communicator_t *comm_old,
+                                    int n, const int nodes[],
+                                    const int degrees[], const int targets[],
+                                    const int weights[],
+                                    ompi_info_t *info, int reorder,
                                     ompi_communicator_t **newcomm)
 {
     int err;
@@ -315,7 +304,7 @@ int mca_topo_base_dist_graph_create(mca_topo_base_module_t* module,
     err = mca_topo_base_dist_graph_distribute(module,
                                               *newcomm, 
                                               n, nodes,
-                                              degrees, targets, 
+                                              degrees, targets,
                                               weights,
                                               &((*newcomm)->c_topo->mtc.dist_graph));
     if( OMPI_SUCCESS != err ) {
@@ -323,3 +312,32 @@ int mca_topo_base_dist_graph_create(mca_topo_base_module_t* module,
     }
     return err;
 }
+
+static void mca_topo_base_comm_dist_graph_2_2_0_construct(mca_topo_base_comm_dist_graph_2_2_0_t * dist_graph) {
+    dist_graph->in = NULL;
+    dist_graph->inw = NULL;
+    dist_graph->out = NULL;
+    dist_graph->outw = NULL;
+    dist_graph->indegree = 0;
+    dist_graph->outdegree = 0;
+    dist_graph->weighted = false;
+}
+
+static void mca_topo_base_comm_dist_graph_2_2_0_destruct(mca_topo_base_comm_dist_graph_2_2_0_t * dist_graph) {
+    if (NULL != dist_graph->in) {
+        free(dist_graph->in);
+    }
+    if (NULL != dist_graph->inw) {
+        free(dist_graph->inw);
+    }
+    if (NULL != dist_graph->out) {
+        free(dist_graph->out);
+    }
+    if (NULL != dist_graph->outw) {
+        free(dist_graph->outw);
+    }
+}
+
+OBJ_CLASS_INSTANCE(mca_topo_base_comm_dist_graph_2_2_0_t, opal_object_t,
+                   mca_topo_base_comm_dist_graph_2_2_0_construct,
+                   mca_topo_base_comm_dist_graph_2_2_0_destruct);
