@@ -12,7 +12,7 @@
  *                         All rights reserved.
  * Copyright (c) 2014      Los Alamos National Security, LLC. All rights
  *                         reserved.
- * Copyright (c) 2015-2017 Research Organization for Information Science
+ * Copyright (c) 2015      Research Organization for Information Science
  *                         and Technology (RIST). All rights reserved.
  * $COPYRIGHT$
  *
@@ -61,6 +61,11 @@ int MPI_Win_lock(int lock_type, int rank, int assert, MPI_Win win)
         }
     }
 
+    /* NTH: do not bother keeping track of locking MPI_PROC_NULL. */
+    if (MPI_PROC_NULL == rank) return MPI_SUCCESS;
+
+    OPAL_CR_ENTER_LIBRARY();
+
     rc = win->w_osc_module->osc_lock(lock_type, rank, assert, win);
     OMPI_ERRHANDLER_RETURN(rc, win, rc, FUNC_NAME);
 }
