@@ -1,3 +1,23 @@
+/*
+ * Copyright (c) 2004-2007 The Trustees of Indiana University and Indiana
+ *                         University Research and Technology
+ *                         Corporation.  All rights reserved.
+ * Copyright (c) 2004-2005 The University of Tennessee and The University
+ *                         of Tennessee Research Foundation.  All rights
+ *                         reserved.
+ * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart,
+ *                         University of Stuttgart.  All rights reserved.
+ * Copyright (c) 2004-2005 The Regents of the University of California.
+ *                         All rights reserved.
+ * Copyright (c) 2016      Los Alamos National Security, LLC. All rights
+ *                         reserved.
+ * $COPYRIGHT$
+ *
+ * Additional copyrights may follow
+ *
+ * $HEADER$
+ */
+
 #include "ompi_config.h"
 
 #include <stdlib.h>
@@ -116,70 +136,90 @@ JNIEXPORT void JNICALL Java_mpi_File_setView(
 
 JNIEXPORT void JNICALL Java_mpi_File_readAt(
         JNIEnv *env, jobject jthis, jlong fh, jlong fileOffset,
-        jobject buf, jboolean db, jint offset, jint count,
-        jlong type, jint bType, jlongArray stat)
+        jobject buf, jboolean db, jint off, jint count,
+        jlong jType, jint bType, jlongArray stat)
 {
-    void *ptr, *base;
-    ptr = ompi_java_getBufPtr(&base, env, buf, db, bType, offset);
+    jboolean exception;
+    MPI_Datatype type = (MPI_Datatype)jType;
+    void *ptr;
+    ompi_java_buffer_t *item;
+    ompi_java_getWritePtr(&ptr, &item, env, buf, db, count, type);
     MPI_Status status;
 
     int rc = MPI_File_read_at((MPI_File)fh, (MPI_Offset)fileOffset,
-                              ptr, count, (MPI_Datatype)type, &status);
+                              ptr, count, type, &status);
 
-    ompi_java_exceptionCheck(env, rc);
-    ompi_java_releaseBufPtr(env, buf, db, base, bType);
-    ompi_java_status_set(env, stat, &status);
+    exception = ompi_java_exceptionCheck(env, rc);
+    ompi_java_releaseWritePtr(ptr, item, env, buf, db, off, count, type, bType);
+    
+    if(!exception)
+        ompi_java_status_set(env, stat, &status);
 }
 
 JNIEXPORT void JNICALL Java_mpi_File_readAtAll(
         JNIEnv *env, jobject jthis, jlong fh, jlong fileOffset,
-        jobject buf, jboolean db, jint offset, jint count,
-        jlong type, jint bType, jlongArray stat)
+        jobject buf, jboolean db, jint off, jint count,
+        jlong jType, jint bType, jlongArray stat)
 {
-    void *ptr, *base;
-    ptr = ompi_java_getBufPtr(&base, env, buf, db, bType, offset);
+    jboolean exception;
+    MPI_Datatype type = (MPI_Datatype)jType;
+    void *ptr;
+    ompi_java_buffer_t *item;
+    ompi_java_getWritePtr(&ptr, &item, env, buf, db, count, type);
     MPI_Status status;
 
     int rc = MPI_File_read_at_all((MPI_File)fh, (MPI_Offset)fileOffset,
-                                  ptr, count, (MPI_Datatype)type, &status);
+                                  ptr, count, type, &status);
 
-    ompi_java_exceptionCheck(env, rc);
-    ompi_java_releaseBufPtr(env, buf, db, base, bType);
-    ompi_java_status_set(env, stat, &status);
+    exception = ompi_java_exceptionCheck(env, rc);
+    ompi_java_releaseWritePtr(ptr, item, env, buf, db, off, count, type, bType);
+    
+    if(!exception)
+        ompi_java_status_set(env, stat, &status);
 }
 
 JNIEXPORT void JNICALL Java_mpi_File_writeAt(
         JNIEnv *env, jobject jthis, jlong fh, jlong fileOffset,
-        jobject buf, jboolean db, jint offset, jint count,
-        jlong type, jint bType, jlongArray stat)
+        jobject buf, jboolean db, jint off, jint count,
+        jlong jType, jint bType, jlongArray stat)
 {
-    void *ptr, *base;
-    ptr = ompi_java_getBufPtr(&base, env, buf, db, bType, offset);
+    jboolean exception;
+    MPI_Datatype type = (MPI_Datatype)jType;
+    void *ptr;
+    ompi_java_buffer_t *item;
+    ompi_java_getReadPtr(&ptr, &item, env, buf, db, off, count, type, bType);
     MPI_Status status;
 
     int rc = MPI_File_write_at((MPI_File)fh, (MPI_Offset)fileOffset,
-                               ptr, count, (MPI_Datatype)type, &status);
+                               ptr, count, type, &status);
 
-    ompi_java_exceptionCheck(env, rc);
-    ompi_java_releaseReadBufPtr(env, buf, db, base, bType);
-    ompi_java_status_set(env, stat, &status);
+    exception = ompi_java_exceptionCheck(env, rc);
+    ompi_java_releaseReadPtr(ptr, item, buf, db);
+    
+    if(!exception)
+        ompi_java_status_set(env, stat, &status);
 }
 
 JNIEXPORT void JNICALL Java_mpi_File_writeAtAll(
         JNIEnv *env, jobject jthis, jlong fh, jlong fileOffset,
-        jobject buf, jboolean db, jint offset, jint count,
-        jlong type, jint bType, jlongArray stat)
+        jobject buf, jboolean db, jint off, jint count,
+        jlong jType, jint bType, jlongArray stat)
 {
-    void *ptr, *base;
-    ptr = ompi_java_getBufPtr(&base, env, buf, db, bType, offset);
+    jboolean exception;
+    MPI_Datatype type = (MPI_Datatype)jType;
+    void *ptr;
+    ompi_java_buffer_t *item;
+    ompi_java_getReadPtr(&ptr, &item, env, buf, db, off, count, type, bType);
     MPI_Status status;
 
     int rc = MPI_File_write_at_all((MPI_File)fh, (MPI_Offset)fileOffset,
                                    ptr, count, (MPI_Datatype)type, &status);
 
-    ompi_java_exceptionCheck(env, rc);
-    ompi_java_releaseReadBufPtr(env, buf, db, base, bType);
-    ompi_java_status_set(env, stat, &status);
+    exception = ompi_java_exceptionCheck(env, rc);
+    ompi_java_releaseReadPtr(ptr, item, buf, db);
+    
+    if(!exception)
+        ompi_java_status_set(env, stat, &status);
 }
 
 JNIEXPORT jlong JNICALL Java_mpi_File_iReadAt(
@@ -212,66 +252,74 @@ JNIEXPORT jlong JNICALL Java_mpi_File_iWriteAt(
 
 JNIEXPORT void JNICALL Java_mpi_File_read(
         JNIEnv *env, jobject jthis, jlong fh, jobject buf, jboolean db,
-        jint offset, jint count, jlong type, jint bType, jlongArray stat)
+        jint off, jint count, jlong jType, jint bType, jlongArray stat)
 {
-    void *ptr, *base;
-    ptr = ompi_java_getBufPtr(&base, env, buf, db, bType, offset);
+    jboolean exception;
+    MPI_Datatype type = (MPI_Datatype)jType;
+    void *ptr;
+    ompi_java_buffer_t *item;
+    ompi_java_getWritePtr(&ptr, &item, env, buf, db, count, type);
     MPI_Status status;
-
-    int rc = MPI_File_read((MPI_File)fh, ptr, count,
-                           (MPI_Datatype)type, &status);
-
-    ompi_java_exceptionCheck(env, rc);
-    ompi_java_releaseBufPtr(env, buf, db, base, bType);
-    ompi_java_status_set(env, stat, &status);
+    int rc = MPI_File_read((MPI_File)fh, ptr, count, type, &status);
+    exception = ompi_java_exceptionCheck(env, rc);
+    ompi_java_releaseWritePtr(ptr, item, env, buf, db, off, count, type, bType);
+    
+    if(!exception)
+        ompi_java_status_set(env, stat, &status);
 }
 
 JNIEXPORT void JNICALL Java_mpi_File_readAll(
         JNIEnv *env, jobject jthis, jlong fh, jobject buf, jboolean db,
-        jint offset, jint count, jlong type, jint bType, jlongArray stat)
+        jint off, jint count, jlong jType, jint bType, jlongArray stat)
 {
-    void *ptr, *base;
-    ptr = ompi_java_getBufPtr(&base, env, buf, db, bType, offset);
+    jboolean exception;
+    MPI_Datatype type = (MPI_Datatype)jType;
+    void *ptr;
+    ompi_java_buffer_t *item;
+    ompi_java_getWritePtr(&ptr, &item, env, buf, db, count, type);
     MPI_Status status;
-
-    int rc = MPI_File_read_all((MPI_File)fh, ptr, count,
-                               (MPI_Datatype)type, &status);
-
-    ompi_java_exceptionCheck(env, rc);
-    ompi_java_releaseBufPtr(env, buf, db, base, bType);
-    ompi_java_status_set(env, stat, &status);
+    int rc = MPI_File_read_all((MPI_File)fh, ptr, count, type, &status);
+    exception = ompi_java_exceptionCheck(env, rc);
+    ompi_java_releaseWritePtr(ptr, item, env, buf, db, off, count, type, bType);
+    
+    if(!exception)
+        ompi_java_status_set(env, stat, &status);
 }
 
 JNIEXPORT void JNICALL Java_mpi_File_write(
         JNIEnv *env, jobject jthis, jlong fh, jobject buf, jboolean db,
-        jint offset, jint count, jlong type, jint bType, jlongArray stat)
+        jint off, jint count, jlong jType, jint bType, jlongArray stat)
 {
-    void *ptr, *base;
-    ptr = ompi_java_getBufPtr(&base, env, buf, db, bType, offset);
+    jboolean exception;
+    MPI_Datatype type = (MPI_Datatype)jType;
+    void *ptr;
+    ompi_java_buffer_t *item;
+    ompi_java_getReadPtr(&ptr, &item, env, buf, db, off, count, type, bType);
     MPI_Status status;
-
-    int rc = MPI_File_write((MPI_File)fh, ptr, count,
-                            (MPI_Datatype)type, &status);
-
-    ompi_java_exceptionCheck(env, rc);
-    ompi_java_releaseReadBufPtr(env, buf, db, base, bType);
-    ompi_java_status_set(env, stat, &status);
+    int rc = MPI_File_write((MPI_File)fh, ptr, count, type, &status);
+    exception = ompi_java_exceptionCheck(env, rc);
+    ompi_java_releaseReadPtr(ptr, item, buf, db);
+    
+    if(!exception)
+        ompi_java_status_set(env, stat, &status);
 }
 
 JNIEXPORT void JNICALL Java_mpi_File_writeAll(
         JNIEnv *env, jobject jthis, jlong fh, jobject buf, jboolean db,
-        jint offset, jint count, jlong type, jint bType, jlongArray stat)
+        jint off, jint count, jlong jType, jint bType, jlongArray stat)
 {
-    void *ptr, *base;
-    ptr = ompi_java_getBufPtr(&base, env, buf, db, bType, offset);
+    jboolean exception;
+    MPI_Datatype type = (MPI_Datatype)jType;
+    void *ptr;
+    ompi_java_buffer_t *item;
+    ompi_java_getReadPtr(&ptr, &item, env, buf, db, off, count, type, bType);
     MPI_Status status;
-
-    int rc = MPI_File_write_all((MPI_File)fh, ptr, count,
-                                (MPI_Datatype)type, &status);
-
-    ompi_java_exceptionCheck(env, rc);
-    ompi_java_releaseReadBufPtr(env, buf, db, base, bType);
-    ompi_java_status_set(env, stat, &status);
+    int rc = MPI_File_write_all((MPI_File)fh, ptr, count, type, &status);
+    exception = ompi_java_exceptionCheck(env, rc);
+    ompi_java_releaseReadPtr(ptr, item, buf, db);
+    
+    if(!exception)
+        ompi_java_status_set(env, stat, &status);
 }
 
 JNIEXPORT jlong JNICALL Java_mpi_File_iRead(
@@ -329,34 +377,38 @@ JNIEXPORT jlong JNICALL Java_mpi_File_getByteOffset(
 
 JNIEXPORT void JNICALL Java_mpi_File_readShared(
         JNIEnv *env, jobject jthis, jlong fh, jobject buf, jboolean db,
-        jint offset, jint count, jlong type, jint bType, jlongArray stat)
+        jint off, jint count, jlong jType, jint bType, jlongArray stat)
 {
-    void *ptr, *base;
-    ptr = ompi_java_getBufPtr(&base, env, buf, db, bType, offset);
+    jboolean exception;
+    MPI_Datatype type = (MPI_Datatype)jType;
+    void *ptr;
+    ompi_java_buffer_t *item;
+    ompi_java_getWritePtr(&ptr, &item, env, buf, db, count, type);
     MPI_Status status;
-
-    int rc = MPI_File_read_shared((MPI_File)fh, ptr, count,
-                                  (MPI_Datatype)type, &status);
-
-    ompi_java_exceptionCheck(env, rc);
-    ompi_java_releaseBufPtr(env, buf, db, base, bType);
-    ompi_java_status_set(env, stat, &status);
+    int rc = MPI_File_read_shared((MPI_File)fh, ptr, count, type, &status);
+    exception = ompi_java_exceptionCheck(env, rc);
+    ompi_java_releaseWritePtr(ptr, item, env, buf, db, off, count, type, bType);
+    
+    if(!exception)
+        ompi_java_status_set(env, stat, &status);
 }
 
 JNIEXPORT void JNICALL Java_mpi_File_writeShared(
         JNIEnv *env, jobject jthis, jlong fh, jobject buf, jboolean db,
-        jint offset, jint count, jlong type, jint bType, jlongArray stat)
+        jint off, jint count, jlong jType, jint bType, jlongArray stat)
 {
-    void *ptr, *base;
-    ptr = ompi_java_getBufPtr(&base, env, buf, db, bType, offset);
+    jboolean exception;
+    MPI_Datatype type = (MPI_Datatype)jType;
+    void *ptr;
+    ompi_java_buffer_t *item;
+    ompi_java_getReadPtr(&ptr, &item, env, buf, db, off, count, type, bType);
     MPI_Status status;
-
-    int rc = MPI_File_write_shared((MPI_File)fh, ptr, count,
-                                   (MPI_Datatype)type, &status);
-
-    ompi_java_exceptionCheck(env, rc);
-    ompi_java_releaseReadBufPtr(env, buf, db, base, bType);
-    ompi_java_status_set(env, stat, &status);
+    int rc = MPI_File_write_shared((MPI_File)fh, ptr, count, type, &status);
+    exception = ompi_java_exceptionCheck(env, rc);
+    ompi_java_releaseReadPtr(ptr, item, buf, db);
+    
+    if(!exception)
+        ompi_java_status_set(env, stat, &status);
 }
 
 JNIEXPORT jlong JNICALL Java_mpi_File_iReadShared(
@@ -389,34 +441,38 @@ JNIEXPORT jlong JNICALL Java_mpi_File_iWriteShared(
 
 JNIEXPORT void JNICALL Java_mpi_File_readOrdered(
         JNIEnv *env, jobject jthis, jlong fh, jobject buf, jboolean db,
-        jint offset, jint count, jlong type, jint bType, jlongArray stat)
+        jint off, jint count, jlong jType, jint bType, jlongArray stat)
 {
-    void *ptr, *base;
-    ptr = ompi_java_getBufPtr(&base, env, buf, db, bType, offset);
+    jboolean exception;
+    MPI_Datatype type = (MPI_Datatype)jType;
+    void *ptr;
+    ompi_java_buffer_t *item;
+    ompi_java_getWritePtr(&ptr, &item, env, buf, db, count, type);
     MPI_Status status;
-
-    int rc = MPI_File_read_ordered((MPI_File)fh, ptr, count,
-                                   (MPI_Datatype)type, &status);
-
-    ompi_java_exceptionCheck(env, rc);
-    ompi_java_releaseBufPtr(env, buf, db, base, bType);
-    ompi_java_status_set(env, stat, &status);
+    int rc = MPI_File_read_ordered((MPI_File)fh, ptr, count, type, &status);
+    exception = ompi_java_exceptionCheck(env, rc);
+    ompi_java_releaseWritePtr(ptr, item, env, buf, db, off, count, type, bType);
+    
+    if(!exception)
+        ompi_java_status_set(env, stat, &status);
 }
 
 JNIEXPORT void JNICALL Java_mpi_File_writeOrdered(
         JNIEnv *env, jobject jthis, jlong fh, jobject buf, jboolean db,
-        jint offset, jint count, jlong type, jint bType, jlongArray stat)
+        jint off, jint count, jlong jType, jint bType, jlongArray stat)
 {
-    void *ptr, *base;
-    ptr = ompi_java_getBufPtr(&base, env, buf, db, bType, offset);
+    jboolean exception;
+    MPI_Datatype type = (MPI_Datatype)jType;
+    void *ptr;
+    ompi_java_buffer_t *item;
+    ompi_java_getReadPtr(&ptr, &item, env, buf, db, off, count, type, bType);
     MPI_Status status;
-
-    int rc = MPI_File_write_ordered((MPI_File)fh, ptr, count,
-                                    (MPI_Datatype)type, &status);
-
-    ompi_java_exceptionCheck(env, rc);
-    ompi_java_releaseReadBufPtr(env, buf, db, base, bType);
-    ompi_java_status_set(env, stat, &status);
+    int rc = MPI_File_write_ordered((MPI_File)fh, ptr, count, type, &status);
+    exception = ompi_java_exceptionCheck(env, rc);
+    ompi_java_releaseReadPtr(ptr, item, buf, db);
+    
+    if(!exception)
+        ompi_java_status_set(env, stat, &status);
 }
 
 JNIEXPORT void JNICALL Java_mpi_File_seekShared(
@@ -452,8 +508,9 @@ JNIEXPORT void JNICALL Java_mpi_File_readAtAllEnd(
     MPI_Status status;
     void *ptr = (*env)->GetDirectBufferAddress(env, buf);
     int rc = MPI_File_read_at_all_end((MPI_File)fh, ptr, &status);
-    ompi_java_exceptionCheck(env, rc);
-    ompi_java_status_set(env, stat, &status);
+    
+    if(!ompi_java_exceptionCheck(env, rc))
+        ompi_java_status_set(env, stat, &status);
 }
 
 JNIEXPORT void JNICALL Java_mpi_File_writeAtAllBegin(
@@ -473,8 +530,9 @@ JNIEXPORT void JNICALL Java_mpi_File_writeAtAllEnd(
     MPI_Status status;
     void *ptr = (*env)->GetDirectBufferAddress(env, buf);
     int rc = MPI_File_write_at_all_end((MPI_File)fh, ptr, &status);
-    ompi_java_exceptionCheck(env, rc);
-    ompi_java_status_set(env, stat, &status);
+    
+    if(!ompi_java_exceptionCheck(env, rc))
+        ompi_java_status_set(env, stat, &status);
 }
 
 JNIEXPORT void JNICALL Java_mpi_File_readAllBegin(
@@ -495,8 +553,9 @@ JNIEXPORT void JNICALL Java_mpi_File_readAllEnd(
     MPI_Status status;
     void *ptr = (*env)->GetDirectBufferAddress(env, buf);
     int rc = MPI_File_read_all_end((MPI_File)fh, ptr, &status);
-    ompi_java_exceptionCheck(env, rc);
-    ompi_java_status_set(env, stat, &status);
+    
+    if(!ompi_java_exceptionCheck(env, rc))
+        ompi_java_status_set(env, stat, &status);
 }
 
 JNIEXPORT void JNICALL Java_mpi_File_writeAllBegin(
@@ -517,8 +576,9 @@ JNIEXPORT void JNICALL Java_mpi_File_writeAllEnd(
     MPI_Status status;
     void *ptr = (*env)->GetDirectBufferAddress(env, buf);
     int rc = MPI_File_write_all_end((MPI_File)fh, ptr, &status);
-    ompi_java_exceptionCheck(env, rc);
-    ompi_java_status_set(env, stat, &status);
+    
+    if(!ompi_java_exceptionCheck(env, rc))
+        ompi_java_status_set(env, stat, &status);
 }
 
 JNIEXPORT void JNICALL Java_mpi_File_readOrderedBegin(
@@ -539,8 +599,9 @@ JNIEXPORT void JNICALL Java_mpi_File_readOrderedEnd(
     MPI_Status status;
     void *ptr = (*env)->GetDirectBufferAddress(env, buf);
     int rc = MPI_File_read_ordered_end((MPI_File)fh, ptr, &status);
-    ompi_java_exceptionCheck(env, rc);
-    ompi_java_status_set(env, stat, &status);
+    
+    if(!ompi_java_exceptionCheck(env, rc))
+        ompi_java_status_set(env, stat, &status);
 }
 
 JNIEXPORT void JNICALL Java_mpi_File_writeOrderedBegin(
@@ -561,8 +622,9 @@ JNIEXPORT void JNICALL Java_mpi_File_writeOrderedEnd(
     MPI_Status status;
     void *ptr = (*env)->GetDirectBufferAddress(env, buf);
     int rc = MPI_File_write_ordered_end((MPI_File)fh, ptr, &status);
-    ompi_java_exceptionCheck(env, rc);
-    ompi_java_status_set(env, stat, &status);
+    
+    if(!ompi_java_exceptionCheck(env, rc))
+        ompi_java_status_set(env, stat, &status);
 }
 
 JNIEXPORT jint JNICALL Java_mpi_File_getTypeExtent(
