@@ -16,6 +16,8 @@
  * Copyright (c) 2016      Los Alamos National Security, LLC. All rights
  *                         reserved.
  * Copyright (c) 2016      Mellanox Technologies. All rights reserved.
+ * Copyright (c) 2016      Research Organization for Information Science
+ *                         and Technology (RIST). All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -29,8 +31,7 @@
 #include "ompi/request/request_default.h"
 #include "ompi/request/grequest.h"
 
-#include "ompi/mca/pml/base/pml_base_request.h"
-
+#include "ompi/mca/crcp/crcp.h"
 
 int ompi_request_default_wait(
     ompi_request_t ** req_ptr,
@@ -40,6 +41,7 @@ int ompi_request_default_wait(
 
     ompi_request_wait_completion(req);
 
+    OMPI_CRCP_REQUEST_COMPLETE(req);
 
     /* return status.  If it's a generalized request, we *have* to
        invoke the query_fn, even if the user procided STATUS_IGNORE.
@@ -89,6 +91,11 @@ int ompi_request_default_wait_any(size_t count,
     ompi_request_t *request=NULL;
     ompi_wait_sync_t sync;
 
+    if (OPAL_UNLIKELY(0 == count)) {
+        *index = MPI_UNDEFINED;
+        return OMPI_SUCCESS;
+    }
+
     WAIT_SYNC_INIT(&sync, 1);
 
     num_requests_null_inactive = 0;
@@ -154,6 +161,11 @@ int ompi_request_default_wait_any(size_t count,
 
     request = requests[*index];
     assert( REQUEST_COMPLETE(request) );
+#if OPAL_ENABLE_FT_CR == 1
+    if( opal_cr_is_enabled ) {
+        OMPI_CRCP_REQUEST_COMPLETE(request);
+    }
+#endif
     /* Per note above, we have to call gen request query_fn even
        if STATUS_IGNORE was provided */
     if (OMPI_REQUEST_GEN == request->req_type) {
@@ -192,11 +204,15 @@ int ompi_request_default_wait_all( size_t count,
     int mpi_error = OMPI_SUCCESS;
     ompi_wait_sync_t sync;
 
+    if (OPAL_UNLIKELY(0 == count)) {
+        return OMPI_SUCCESS;
+    }
+
     WAIT_SYNC_INIT(&sync, count);
     rptr = requests;
     for (i = 0; i < count; i++) {
         request = *rptr++;
-        
+
         if( request->req_state == OMPI_REQUEST_INACTIVE ) {
             completed++;
             continue;
@@ -258,6 +274,10 @@ int ompi_request_default_wait_all( size_t count,
             }
             assert( REQUEST_COMPLETE(request) );
 
+            if( opal_cr_is_enabled) {
+                OMPI_CRCP_REQUEST_COMPLETE(request);
+            }
+
             if (OMPI_REQUEST_GEN == request->req_type) {
                 ompi_grequest_invoke_query(request, &request->req_status);
             }
@@ -313,6 +333,10 @@ int ompi_request_default_wait_all( size_t count,
             }
             assert( REQUEST_COMPLETE(request) );
 
+            if( opal_cr_is_enabled) {
+                OMPI_CRCP_REQUEST_COMPLETE(request);
+            }
+
             /* Per note above, we have to call gen request query_fn
                even if STATUSES_IGNORE was provided */
             if (OMPI_REQUEST_GEN == request->req_type) {
@@ -361,6 +385,11 @@ int ompi_request_default_wait_some(size_t count,
     ompi_wait_sync_t sync;
     size_t sync_sets = 0, sync_unsets = 0;
     
+    if (OPAL_UNLIKELY(0 == count)) {
+        *outcount = MPI_UNDEFINED;
+        return OMPI_SUCCESS;
+    }
+
     WAIT_SYNC_INIT(&sync, 1);
 
     *outcount = 0;
@@ -378,8 +407,8 @@ int ompi_request_default_wait_some(size_t count,
             num_requests_null_inactive++;
             continue;
         }
-
-        if( !OPAL_ATOMIC_CMPSET_PTR(&request->req_complete, REQUEST_PENDING, &sync) ) {
+        indices[i] = OPAL_ATOMIC_CMPSET_PTR(&request->req_complete, REQUEST_PENDING, &sync);
+        if( !indices[i] ) {
             /* If the request is completed go ahead and mark it as such */
             assert( REQUEST_COMPLETE(request) );
             num_requests_done++;
@@ -410,15 +439,23 @@ int ompi_request_default_wait_some(size_t count,
         if( request->req_state == OMPI_REQUEST_INACTIVE ) {
             continue;
         }
-        /* Atomically mark the request as pending. If this succeed
-         * then the request was not completed, and it is now marked as
-         * pending. Otherwise, the request is complete )either it was
-         * before or it has been meanwhile). The major drawback here
-         * is that we will do all the atomics operations in all cases.
+        /* Here we have 3 possibilities:
+         * a) request was found completed in the first loop
+         *    => ( indices[i] == 0 )
+         * b) request was completed between first loop and this check
+         *    => ( indices[i] == 1 ) and we can NOT atomically mark the 
+         *    request as pending.
+         * c) request wasn't finished yet
+         *    => ( indices[i] == 1 ) and we CAN  atomically mark the 
+         *    request as pending.
+         * NOTE that in any case (i >= num_requests_done) as latter grows
+         * either slowly (in case of partial completion)
+         * OR in parallel with `i` (in case of full set completion)  
          */
-        if( !OPAL_ATOMIC_CMPSET_PTR(&request->req_complete, &sync, REQUEST_PENDING) ) {
-            indices[num_requests_done] = i;
-            num_requests_done++;
+        if( !indices[i] ){
+            indices[num_requests_done++] = i;
+        } else if( !OPAL_ATOMIC_CMPSET_PTR(&request->req_complete, &sync, REQUEST_PENDING) ) {
+            indices[num_requests_done++] = i;
         }
     }
     sync_unsets = count - num_requests_null_inactive - num_requests_done;
@@ -437,6 +474,13 @@ int ompi_request_default_wait_some(size_t count,
     for (size_t i = 0; i < num_requests_done; i++) {
         request = requests[indices[i]];
         assert( REQUEST_COMPLETE(request) );
+
+#if OPAL_ENABLE_FT_CR == 1
+        if( opal_cr_is_enabled) {
+            OMPI_CRCP_REQUEST_COMPLETE(request);
+        }
+#endif
+
         /* Per note above, we have to call gen request query_fn even
            if STATUS_IGNORE was provided */
         if (OMPI_REQUEST_GEN == request->req_type) {
