@@ -14,7 +14,7 @@
  * Copyright (c) 2009-2012 Cisco Systems, Inc.  All rights reserved.
  * Copyright (c) 2013-2015 Los Alamos National Security, LLC.  All rights
  *                         reserved.
- * Copyright (c) 2015-2016 Research Organization for Information Science
+ * Copyright (c) 2015-2017 Research Organization for Information Science
  *                         and Technology (RIST). All rights reserved.
  * $COPYRIGHT$
  *
@@ -46,6 +46,7 @@ opal_pointer_array_t ompi_mpi_windows = {{0}};
 ompi_predefined_win_t ompi_mpi_win_null = {{{0}}};
 ompi_predefined_win_t *ompi_mpi_win_null_addr = &ompi_mpi_win_null;
 mca_base_var_enum_t *ompi_win_accumulate_ops = NULL;
+mca_base_var_enum_flag_t *ompi_win_accumulate_order = NULL;
 
 static mca_base_var_enum_value_t accumulate_ops_values[] = {
     {.value = OMPI_WIN_ACCUMULATE_OPS_SAME_OP_NO_OP, .string = "same_op_no_op",},
@@ -53,6 +54,16 @@ static mca_base_var_enum_value_t accumulate_ops_values[] = {
     {.value = -1, .string = NULL},
 };
 
+static mca_base_var_enum_value_flag_t accumulate_order_flags[] = {
+    {.flag = OMPI_WIN_ACC_ORDER_NONE, .string = "none", .conflicting_flag = OMPI_WIN_ACC_ORDER_RAR |
+     OMPI_WIN_ACC_ORDER_WAR | OMPI_WIN_ACC_ORDER_RAW | OMPI_WIN_ACC_ORDER_WAW},
+    {.flag = OMPI_WIN_ACC_ORDER_RAR, .string = "rar", .conflicting_flag = OMPI_WIN_ACC_ORDER_NONE},
+    {.flag = OMPI_WIN_ACC_ORDER_WAR, .string = "war", .conflicting_flag = OMPI_WIN_ACC_ORDER_NONE},
+    {.flag = OMPI_WIN_ACC_ORDER_RAW, .string = "raw", .conflicting_flag = OMPI_WIN_ACC_ORDER_NONE},
+    {.flag = OMPI_WIN_ACC_ORDER_WAW, .string = "waw", .conflicting_flag = OMPI_WIN_ACC_ORDER_NONE},
+    {0},
+};
+
 static void ompi_win_construct(ompi_win_t *win);
 static void ompi_win_destruct(ompi_win_t *win);
 
@@ -86,6 +97,11 @@ ompi_win_init(void)
         return ret;
     }
 
+    ret = mca_base_var_enum_create_flag ("accumulate_order", accumulate_order_flags, &ompi_win_accumulate_order);
+    if (OPAL_SUCCESS != ret) {
+        return ret;
+    }
+
     return OMPI_SUCCESS;
 }
 
@@ -115,6 +131,7 @@ int ompi_win_finalize(void)
     OBJ_DESTRUCT(&ompi_mpi_win_null.win);
     OBJ_DESTRUCT(&ompi_mpi_windows);
     OBJ_RELEASE(ompi_win_accumulate_ops);
+    OBJ_RELEASE(ompi_win_accumulate_order);
 
     return OMPI_SUCCESS;
 }
@@ -123,7 +140,7 @@ static int alloc_window(struct ompi_communicator_t *comm, ompi_info_t *info, int
 {
     ompi_win_t *win;
     ompi_group_t *group;
-    int acc_ops, flag, ret;
+    int acc_ops, acc_order, flag, ret;
 
     /* create the object */
     win = OBJ_NEW(ompi_win_t);
@@ -139,7 +156,19 @@ static int alloc_window(struct ompi_communicator_t *comm, ompi_info_t *info, int
         return ret;
     }
 
-    win->w_acc_ops = acc_ops;
+    win->w_acc_ops = (ompi_win_accumulate_ops_t)acc_ops;
+
+    ret = ompi_info_get_value_enum (info, "accumulate_order", &acc_order,
+                                    OMPI_WIN_ACC_ORDER_RAR | OMPI_WIN_ACC_ORDER_WAR |
+                                    OMPI_WIN_ACC_ORDER_RAW | OMPI_WIN_ACC_ORDER_WAW,
+                                    &(ompi_win_accumulate_order->super), &flag);
+    if (OMPI_SUCCESS != ret) {
+        OBJ_RELEASE(win);
+        return ret;
+    }
+
+    win->w_acc_order = acc_order;
+
     win->w_flavor = flavor;
 
     /* setup data that is independent of osc component */
@@ -162,25 +191,25 @@ config_window(void *base, size_t size, int disp_unit,
                           MPI_WIN_BASE, base, true);
     if (OMPI_SUCCESS != ret) return ret;
 
-    ret = ompi_attr_set_fortran_mpi2(WIN_ATTR, win,
-                                     &win->w_keyhash,
-                                     MPI_WIN_SIZE, size, true);
+    ret = ompi_attr_set_aint(WIN_ATTR, win,
+                             &win->w_keyhash,
+                             MPI_WIN_SIZE, size, true);
     if (OMPI_SUCCESS != ret) return ret;
 
-    ret = ompi_attr_set_fortran_mpi1(WIN_ATTR, win,
-                                     &win->w_keyhash,
-                                     MPI_WIN_DISP_UNIT, disp_unit,
-                                     true);
+    ret = ompi_attr_set_int(WIN_ATTR, win,
+                            &win->w_keyhash,
+                            MPI_WIN_DISP_UNIT, disp_unit,
+                            true);
     if (OMPI_SUCCESS != ret) return ret;
 
-    ret = ompi_attr_set_fortran_mpi1(WIN_ATTR, win,
-                                     &win->w_keyhash,
-                                     MPI_WIN_CREATE_FLAVOR, flavor, true);
+    ret = ompi_attr_set_int(WIN_ATTR, win,
+                            &win->w_keyhash,
+                            MPI_WIN_CREATE_FLAVOR, flavor, true);
     if (OMPI_SUCCESS != ret) return ret;
 
-    ret = ompi_attr_set_fortran_mpi1(WIN_ATTR, win,
-                                     &win->w_keyhash,
-                                     MPI_WIN_MODEL, model, true);
+    ret = ompi_attr_set_int(WIN_ATTR, win,
+                            &win->w_keyhash,
+                            MPI_WIN_MODEL, model, true);
     if (OMPI_SUCCESS != ret) return ret;
 
     win->w_f_to_c_index = opal_pointer_array_add(&ompi_mpi_windows, win);
