@@ -1,3 +1,4 @@
+/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil -*- */
 /*
  * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
@@ -5,20 +6,22 @@
  * Copyright (c) 2004-2005 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
- * Copyright (c) 2004-2007 High Performance Computing Center Stuttgart, 
+ * Copyright (c) 2004-2007 High Performance Computing Center Stuttgart,
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
+ * Copyright (c) 2015      Los Alamos National Security, LLC. All rights
+ *                         reserved.
  * $COPYRIGHT$
- * 
+ *
  * Additional copyrights may follow
- * 
+ *
  * $HEADER$
  */
 
 /**
  * @file
- * 
+ *
  * Implementation of opal_object_t, the base opal foundation class
  */
 
@@ -47,6 +50,8 @@ opal_class_t opal_object_t_class = {
     sizeof(opal_object_t) /* size of the opal object */
 };
 
+int opal_class_init_epoch = 1;
+
 /*
  * Local variables
  */
@@ -81,7 +86,7 @@ void opal_class_initialize(opal_class_t *cls)
     /* Check to see if any other thread got in here and initialized
        this class before we got a chance to */
 
-    if (1 == cls->cls_initialized) {
+    if (opal_class_init_epoch == cls->cls_initialized) {
         return;
     }
     opal_atomic_lock(&class_lock);
@@ -90,7 +95,7 @@ void opal_class_initialize(opal_class_t *cls)
        roughly the same time, it may have gotten the lock and
        initialized.  So check again. */
 
-    if (1 == cls->cls_initialized) {
+    if (opal_class_init_epoch == cls->cls_initialized) {
         opal_atomic_unlock(&class_lock);
         return;
     }
@@ -118,7 +123,7 @@ void opal_class_initialize(opal_class_t *cls)
      * plus for each a NULL-sentinel
      */
 
-    cls->cls_construct_array = 
+    cls->cls_construct_array =
         (void (**)(opal_object_t*))malloc((cls_construct_array_count +
                                            cls_destruct_array_count + 2) *
                                           sizeof(opal_construct_t) );
@@ -151,7 +156,7 @@ void opal_class_initialize(opal_class_t *cls)
     }
     *cls_destruct_array = NULL;  /* end marker for the destructors */
 
-    cls->cls_initialized = 1;
+    cls->cls_initialized = opal_class_init_epoch;
     save_class(cls);
 
     /* All done */
@@ -167,6 +172,12 @@ int opal_class_finalize(void)
 {
     int i;
 
+    if (INT_MAX == opal_class_init_epoch) {
+        opal_class_init_epoch = 1;
+    } else {
+        opal_class_init_epoch++;
+    }
+
     if (NULL != classes) {
         for (i = 0; i < num_classes; ++i) {
             if (NULL != classes[i]) {
@@ -199,7 +210,7 @@ static void expand_array(void)
     int i;
 
     max_classes += increment;
-    classes = (void**)realloc(classes, sizeof(void *) * max_classes);
+    classes = (void**)realloc(classes, sizeof(opal_class_t*) * max_classes);
     if (NULL == classes) {
         perror("class malloc failed");
         exit(-1);
