@@ -3,7 +3,7 @@
  * Copyright (c) 2004-2006 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2010 The University of Tennessee and The University
+ * Copyright (c) 2004-2017 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2006 High Performance Computing Center Stuttgart,
@@ -14,6 +14,8 @@
  *                         reserved.
  * Copyright (c) 2009      Sun Microsystems, Inc.  All rights reserved.
  * Copyright (c) 2009      Oak Ridge National Labs.  All rights reserved.
+ * Copyright (c) 2017      Research Organization for Information Science
+ *                         and Technology (RIST). All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -35,9 +37,6 @@
 #include "opal_config.h"
 
 #include <stddef.h>
-#ifdef HAVE_STDINT_H
-#include <stdint.h>
-#endif
 
 #include "opal/class/opal_object.h"
 
@@ -54,9 +53,10 @@ BEGIN_C_DECLS
 #endif
 /*
  * No more than this number of _Basic_ datatypes in C/CPP or Fortran
- * are supported (in order to not change setup and usage of btypes).
+ * are supported (in order to not change setup and usage of the predefined
+ * datatypes).
  *
- * XXX TODO Adapt to whatever the OMPI-layer needs
+ * BEWARE: This constant should reflect whatever the OMPI-layer needs.
  */
 #define OPAL_DATATYPE_MAX_SUPPORTED  47
 
@@ -64,7 +64,7 @@ BEGIN_C_DECLS
 /* flags for the datatypes. */
 #define OPAL_DATATYPE_FLAG_UNAVAILABLE   0x0001  /**< datatypes unavailable on the build (OS or compiler dependant) */
 #define OPAL_DATATYPE_FLAG_PREDEFINED    0x0002  /**< cannot be removed: initial and predefined datatypes */
-#define OPAL_DATATYPE_FLAG_COMMITED      0x0004  /**< ready to be used for a send/recv operation */
+#define OPAL_DATATYPE_FLAG_COMMITTED      0x0004  /**< ready to be used for a send/recv operation */
 #define OPAL_DATATYPE_FLAG_OVERLAP       0x0008  /**< datatype is unpropper for a recv operation */
 #define OPAL_DATATYPE_FLAG_CONTIGUOUS    0x0010  /**< contiguous datatype */
 #define OPAL_DATATYPE_FLAG_NO_GAPS       0x0020  /**< no gaps around the datatype, aka OPAL_DATATYPE_FLAG_CONTIGUOUS and extent == size */
@@ -79,7 +79,7 @@ BEGIN_C_DECLS
                                           OPAL_DATATYPE_FLAG_CONTIGUOUS | \
                                           OPAL_DATATYPE_FLAG_NO_GAPS |    \
                                           OPAL_DATATYPE_FLAG_DATA |       \
-                                          OPAL_DATATYPE_FLAG_COMMITED)
+                                          OPAL_DATATYPE_FLAG_COMMITTED)
 
 
 /**
@@ -109,13 +109,14 @@ struct opal_datatype_t {
     uint32_t           bdt_used; /**< bitset of which basic datatypes are used in the data description */
     size_t             size;     /**< total size in bytes of the memory used by the data if
                                       the data is put on a contiguous buffer */
-    OPAL_PTRDIFF_TYPE  true_lb;  /**< the true lb of the data without user defined lb and ub */
-    OPAL_PTRDIFF_TYPE  true_ub;  /**< the true ub of the data without user defined lb and ub */
-    OPAL_PTRDIFF_TYPE  lb;       /**< lower bound in memory */
-    OPAL_PTRDIFF_TYPE  ub;       /**< upper bound in memory */
+    ptrdiff_t          true_lb;  /**< the true lb of the data without user defined lb and ub */
+    ptrdiff_t          true_ub;  /**< the true ub of the data without user defined lb and ub */
+    ptrdiff_t          lb;       /**< lower bound in memory */
+    ptrdiff_t          ub;       /**< upper bound in memory */
     /* --- cacheline 1 boundary (64 bytes) --- */
     size_t             nbElems;  /**< total number of elements inside the datatype */
     uint32_t           align;    /**< data should be aligned to */
+    uint32_t           loops;    /**< number of loops on the iternal type stack */
 
     /* Attribute fields */
     char               name[OPAL_MAX_OBJECT_NAME];  /**< name of the datatype */
@@ -124,11 +125,12 @@ struct opal_datatype_t {
     dt_type_desc_t     opt_desc; /**< short description of the data used when conversion is useless
                                       or in the send case (without conversion) */
 
-    uint32_t           btypes[OPAL_DATATYPE_MAX_SUPPORTED];
-                                 /**< basic elements count used to compute the size of the
-                                      datatype for remote nodes. The length of the array is dependent on
-                                      the maximum number of datatypes of all top layers.
-                                      Reason being is that Fortran is not at the OPAL layer. */
+    size_t             *ptypes;  /**< array of basic predefined types that facilitate the computing
+                                      of the remote size in heterogeneous environments. The length of the
+                                      array is dependent on the maximum number of predefined datatypes of
+                                      all language interfaces (because Fortran is not known at the OPAL
+                                      layer). This field should never be initialized in homogeneous
+                                      environments */
     /* --- cacheline 5 boundary (320 bytes) was 32-36 bytes ago --- */
 
     /* size: 352, cachelines: 6, members: 15 */
@@ -189,7 +191,7 @@ OPAL_DECLSPEC int32_t opal_datatype_destroy( opal_datatype_t** );
 static inline int32_t
 opal_datatype_is_committed( const opal_datatype_t* type )
 {
-    return ((type->flags & OPAL_DATATYPE_FLAG_COMMITED) == OPAL_DATATYPE_FLAG_COMMITED);
+    return ((type->flags & OPAL_DATATYPE_FLAG_COMMITTED) == OPAL_DATATYPE_FLAG_COMMITTED);
 }
 
 static inline int32_t
@@ -227,19 +229,19 @@ OPAL_DECLSPEC void opal_datatype_dump( const opal_datatype_t* pData );
 /* data creation functions */
 OPAL_DECLSPEC int32_t opal_datatype_clone( const opal_datatype_t * src_type, opal_datatype_t * dest_type );
 OPAL_DECLSPEC int32_t opal_datatype_create_contiguous( int count, const opal_datatype_t* oldType, opal_datatype_t** newType );
-OPAL_DECLSPEC int32_t opal_datatype_resize( opal_datatype_t* type, OPAL_PTRDIFF_TYPE lb, OPAL_PTRDIFF_TYPE extent );
+OPAL_DECLSPEC int32_t opal_datatype_resize( opal_datatype_t* type, ptrdiff_t lb, ptrdiff_t extent );
 OPAL_DECLSPEC int32_t opal_datatype_add( opal_datatype_t* pdtBase, const opal_datatype_t* pdtAdd, uint32_t count,
-                                         OPAL_PTRDIFF_TYPE disp, OPAL_PTRDIFF_TYPE extent );
+                                         ptrdiff_t disp, ptrdiff_t extent );
 
 static inline int32_t
-opal_datatype_type_lb( const opal_datatype_t* pData, OPAL_PTRDIFF_TYPE* disp )
+opal_datatype_type_lb( const opal_datatype_t* pData, ptrdiff_t* disp )
 {
     *disp = pData->lb;
     return 0;
 }
 
 static inline int32_t
-opal_datatype_type_ub( const opal_datatype_t* pData, OPAL_PTRDIFF_TYPE* disp )
+opal_datatype_type_ub( const opal_datatype_t* pData, ptrdiff_t* disp )
 {
     *disp = pData->ub;
     return 0;
@@ -253,21 +255,21 @@ opal_datatype_type_size( const opal_datatype_t* pData, size_t *size )
 }
 
 static inline int32_t
-opal_datatype_type_extent( const opal_datatype_t* pData, OPAL_PTRDIFF_TYPE* extent )
+opal_datatype_type_extent( const opal_datatype_t* pData, ptrdiff_t* extent )
 {
     *extent = pData->ub - pData->lb;
     return 0;
 }
 
 static inline int32_t
-opal_datatype_get_extent( const opal_datatype_t* pData, OPAL_PTRDIFF_TYPE* lb, OPAL_PTRDIFF_TYPE* extent)
+opal_datatype_get_extent( const opal_datatype_t* pData, ptrdiff_t* lb, ptrdiff_t* extent)
 {
     *lb = pData->lb; *extent = pData->ub - pData->lb;
     return 0;
 }
 
 static inline int32_t
-opal_datatype_get_true_extent( const opal_datatype_t* pData, OPAL_PTRDIFF_TYPE* true_lb, OPAL_PTRDIFF_TYPE* true_extent)
+opal_datatype_get_true_extent( const opal_datatype_t* pData, ptrdiff_t* true_lb, ptrdiff_t* true_extent)
 {
     *true_lb = pData->true_lb;
     *true_extent = (pData->true_ub - pData->true_lb);
@@ -282,6 +284,8 @@ OPAL_DECLSPEC int32_t
 opal_datatype_copy_content_same_ddt( const opal_datatype_t* pData, int32_t count,
                                      char* pDestBuf, char* pSrcBuf );
 
+OPAL_DECLSPEC int opal_datatype_compute_ptypes( opal_datatype_t* datatype );
+
 OPAL_DECLSPEC const opal_datatype_t*
 opal_datatype_match_size( int size, uint16_t datakind, uint16_t datalang );
 
@@ -298,12 +302,12 @@ opal_datatype_sndrcv( void *sbuf, int32_t scount, const opal_datatype_t* sdtype,
 OPAL_DECLSPEC int32_t
 opal_datatype_get_args( const opal_datatype_t* pData, int32_t which,
                         int32_t * ci, int32_t * i,
-                        int32_t * ca, OPAL_PTRDIFF_TYPE* a,
+                        int32_t * ca, ptrdiff_t* a,
                         int32_t * cd, opal_datatype_t** d, int32_t * type);
 OPAL_DECLSPEC int32_t
 opal_datatype_set_args( opal_datatype_t* pData,
                         int32_t ci, int32_t ** i,
-                        int32_t ca, OPAL_PTRDIFF_TYPE* a,
+                        int32_t ca, ptrdiff_t* a,
                         int32_t cd, opal_datatype_t** d,int32_t type);
 OPAL_DECLSPEC int32_t
 opal_datatype_copy_args( const opal_datatype_t* source_data,
@@ -332,6 +336,28 @@ OPAL_DECLSPEC opal_datatype_t*
 opal_datatype_create_from_packed_description( void** packed_buffer,
                                               struct opal_proc_t* remote_processor );
 
+/* Compute the span in memory of count datatypes. This function help with temporary
+ * memory allocations for receiving already typed data (such as those used for reduce
+ * operations). This span is the distance between the minimum and the maximum byte
+ * in the memory layout of count datatypes, or in other terms the memory needed to
+ * allocate count times the datatype without the gap in the beginning and at the end.
+ *
+ * Returns: the memory span of count repetition of the datatype, and in the gap
+ *          argument, the number of bytes of the gap at the beginning.
+ */
+static inline ptrdiff_t
+opal_datatype_span( const opal_datatype_t* pData, int64_t count,
+                    ptrdiff_t* gap)
+{
+    ptrdiff_t extent = (pData->ub - pData->lb);
+    ptrdiff_t true_extent = (pData->true_ub - pData->true_lb);
+    if (OPAL_UNLIKELY(0 == pData->size) || (0 == count)) {
+        return 0;
+    }
+    *gap = pData->true_lb;
+    return true_extent + (count - 1) * extent;
+}
+
 #if OPAL_ENABLE_DEBUG
 /*
  * Set a breakpoint to this function in your favorite debugger
