@@ -10,6 +10,9 @@
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
  * Copyright (c) 2011-2013 Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2014-2016 Intel, Inc. All rights reserved.
+ * Copyright (c) 2014      Research Organization for Information Science
+ *                         and Technology (RIST). All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -24,7 +27,8 @@
 #include "opal/util/output.h"
 #include "opal/dss/dss_internal.h"
 
-int opal_dss_pack(opal_buffer_t *buffer, const void *src, int32_t num_vals,
+int opal_dss_pack(opal_buffer_t *buffer,
+                  const void *src, int32_t num_vals,
                   opal_data_type_t type)
 {
     int rc;
@@ -48,8 +52,9 @@ int opal_dss_pack(opal_buffer_t *buffer, const void *src, int32_t num_vals,
     return opal_dss_pack_buffer(buffer, src, num_vals, type);
 }
 
-int opal_dss_pack_buffer(opal_buffer_t *buffer, const void *src, int32_t num_vals,
-                  opal_data_type_t type)
+int opal_dss_pack_buffer(opal_buffer_t *buffer,
+                         const void *src, int32_t num_vals,
+                         opal_data_type_t type)
 {
     int rc;
     opal_dss_type_info_t *info;
@@ -328,17 +333,39 @@ int opal_dss_pack_string(opal_buffer_t *buffer, const void *src,
 int opal_dss_pack_float(opal_buffer_t *buffer, const void *src,
                         int32_t num_vals, opal_data_type_t type)
 {
-    int64_t tmp[2];
     int ret = OPAL_SUCCESS;
     int32_t i;
     float *ssrc = (float*)src;
+    char *convert;
 
     for (i = 0; i < num_vals; ++i) {
-        tmp[0] = (int64_t)ssrc[i];
-        tmp[1] = (int64_t)(1000000.0 * (ssrc[i] - tmp[0]));
-        if (OPAL_SUCCESS != (ret = opal_dss_pack_int64(buffer, tmp, 2, OPAL_INT64))) {
+        asprintf(&convert, "%f", ssrc[i]);
+        if (OPAL_SUCCESS != (ret = opal_dss_pack_string(buffer, &convert, 1, OPAL_STRING))) {
+            free(convert);
             return ret;
         }
+        free(convert);
+    }
+
+    return OPAL_SUCCESS;
+}
+
+/* DOUBLE */
+int opal_dss_pack_double(opal_buffer_t *buffer, const void *src,
+                         int32_t num_vals, opal_data_type_t type)
+{
+    int ret = OPAL_SUCCESS;
+    int32_t i;
+    double *ssrc = (double*)src;
+    char *convert;
+
+    for (i = 0; i < num_vals; ++i) {
+        asprintf(&convert, "%f", ssrc[i]);
+        if (OPAL_SUCCESS != (ret = opal_dss_pack_string(buffer, &convert, 1, OPAL_STRING))) {
+            free(convert);
+            return ret;
+        }
+        free(convert);
     }
 
     return OPAL_SUCCESS;
@@ -364,6 +391,28 @@ int opal_dss_pack_timeval(opal_buffer_t *buffer, const void *src,
     return OPAL_SUCCESS;
 }
 
+/* TIME */
+int opal_dss_pack_time(opal_buffer_t *buffer, const void *src,
+                       int32_t num_vals, opal_data_type_t type)
+{
+    int ret = OPAL_SUCCESS;
+    int32_t i;
+    time_t *ssrc = (time_t *)src;
+    uint64_t ui64;
+
+    /* time_t is a system-dependent size, so cast it
+     * to uint64_t as a generic safe size
+     */
+    for (i = 0; i < num_vals; ++i) {
+        ui64 = (uint64_t)ssrc[i];
+        if (OPAL_SUCCESS != (ret = opal_dss_pack_int64(buffer, &ui64, 1, OPAL_UINT64))) {
+            return ret;
+        }
+    }
+
+    return OPAL_SUCCESS;
+}
+
 
 /* PACK FUNCTIONS FOR GENERIC OPAL TYPES */
 
@@ -374,11 +423,11 @@ int opal_dss_pack_data_type(opal_buffer_t *buffer, const void *src, int32_t num_
                             opal_data_type_t type)
 {
     int ret;
-    
+
     /* Turn around and pack the real type */
     if (OPAL_SUCCESS != (ret = opal_dss_pack_buffer(buffer, src, num_vals, OPAL_DATA_TYPE_T))) {
     }
-    
+
     return ret;
 }
 
@@ -420,9 +469,9 @@ int opal_dss_pack_pstat(opal_buffer_t *buffer, const void *src,
     int32_t i;
     int ret;
     char *cptr;
-    
+
     ptr = (opal_pstats_t **) src;
-    
+
     for (i = 0; i < num_vals; ++i) {
         cptr = ptr[i]->node;
         if (OPAL_SUCCESS != (ret = opal_dss_pack_buffer(buffer, &cptr, 1, OPAL_STRING))) {
@@ -450,6 +499,9 @@ int opal_dss_pack_pstat(opal_buffer_t *buffer, const void *src,
         if (OPAL_SUCCESS != (ret = opal_dss_pack_buffer(buffer, &ptr[i]->num_threads, 1, OPAL_INT16))) {
             return ret;
         }
+        if (OPAL_SUCCESS != (ret = opal_dss_pack_float(buffer, &ptr[i]->pss, 1, OPAL_FLOAT))) {
+            return ret;
+        }
         if (OPAL_SUCCESS != (ret = opal_dss_pack_float(buffer, &ptr[i]->vsize, 1, OPAL_FLOAT))) {
             return ret;
         }
@@ -569,12 +621,11 @@ int opal_dss_pack_node_stat(opal_buffer_t *buffer, const void *src,
     opal_node_stats_t **ptr;
     int32_t i, j;
     int ret;
-    opal_list_item_t *item;
     opal_diskstats_t *ds;
     opal_netstats_t *ns;
 
     ptr = (opal_node_stats_t **) src;
-    
+
     for (i = 0; i < num_vals; ++i) {
         if (OPAL_SUCCESS != (ret = opal_dss_pack_float(buffer, &ptr[i]->la, 1, OPAL_FLOAT))) {
             return ret;
@@ -617,13 +668,12 @@ int opal_dss_pack_node_stat(opal_buffer_t *buffer, const void *src,
         if (OPAL_SUCCESS != (ret = opal_dss_pack_buffer(buffer, &j, 1, OPAL_INT32))) {
             return ret;
         }
-        /* pack them */
-        for (item = opal_list_get_first(&ptr[i]->diskstats);
-             item != opal_list_get_end(&ptr[i]->diskstats);
-             item = opal_list_get_next(item)) {
-            ds = (opal_diskstats_t*)item;
-            if (OPAL_SUCCESS != (ret = pack_disk_stats(buffer, ds))) {
-                return ret;
+        if (0 < j) {
+            /* pack them */
+            OPAL_LIST_FOREACH(ds, &ptr[i]->diskstats, opal_diskstats_t) {
+                if (OPAL_SUCCESS != (ret = pack_disk_stats(buffer, ds))) {
+                    return ret;
+                }
             }
         }
         /* pack the number of net stat objects on the list */
@@ -631,13 +681,12 @@ int opal_dss_pack_node_stat(opal_buffer_t *buffer, const void *src,
         if (OPAL_SUCCESS != (ret = opal_dss_pack_buffer(buffer, &j, 1, OPAL_INT32))) {
             return ret;
         }
-        /* pack them */
-        for (item = opal_list_get_first(&ptr[i]->netstats);
-             item != opal_list_get_end(&ptr[i]->netstats);
-             item = opal_list_get_next(item)) {
-            ns = (opal_netstats_t*)item;
-            if (OPAL_SUCCESS != (ret = pack_net_stats(buffer, ns))) {
-                return ret;
+        if (0 < j) {
+            /* pack them */
+            OPAL_LIST_FOREACH(ns, &ptr[i]->netstats, opal_netstats_t) {
+                if (OPAL_SUCCESS != (ret = pack_net_stats(buffer, ns))) {
+                    return ret;
+                }
             }
         }
     }
@@ -656,20 +705,22 @@ int opal_dss_pack_value(opal_buffer_t *buffer, const void *src,
     int ret;
 
     ptr = (opal_value_t **) src;
-    
+
     for (i = 0; i < num_vals; ++i) {
         /* pack the key and type */
         if (OPAL_SUCCESS != (ret = opal_dss_pack_string(buffer, &ptr[i]->key, 1, OPAL_STRING))) {
             return ret;
         }
-        if (OPAL_SUCCESS != (ret = opal_dss_pack_data_type(buffer, &ptr[i]->scope, 1, OPAL_DATA_SCOPE_T))) {
-            return ret;
-        }
         if (OPAL_SUCCESS != (ret = opal_dss_pack_data_type(buffer, &ptr[i]->type, 1, OPAL_DATA_TYPE))) {
             return ret;
         }
         /* now pack the right field */
         switch (ptr[i]->type) {
+        case OPAL_BOOL:
+            if (OPAL_SUCCESS != (ret = opal_dss_pack_buffer(buffer, &ptr[i]->data.flag, 1, OPAL_BOOL))) {
+                return ret;
+            }
+            break;
         case OPAL_BYTE:
             if (OPAL_SUCCESS != (ret = opal_dss_pack_buffer(buffer, &ptr[i]->data.byte, 1, OPAL_BYTE))) {
                 return ret;
@@ -680,6 +731,11 @@ int opal_dss_pack_value(opal_buffer_t *buffer, const void *src,
                 return ret;
             }
             break;
+        case OPAL_SIZE:
+            if (OPAL_SUCCESS != (ret = opal_dss_pack_buffer(buffer, &ptr[i]->data.size, 1, OPAL_SIZE))) {
+                return ret;
+            }
+            break;
         case OPAL_PID:
             if (OPAL_SUCCESS != (ret = opal_dss_pack_buffer(buffer, &ptr[i]->data.pid, 1, OPAL_PID))) {
                 return ret;
@@ -716,6 +772,10 @@ int opal_dss_pack_value(opal_buffer_t *buffer, const void *src,
             }
             break;
         case OPAL_UINT8:
+        case OPAL_PERSIST:
+        case OPAL_SCOPE:
+        case OPAL_DATA_RANGE:
+        case OPAL_PROC_STATE:
             if (OPAL_SUCCESS != (ret = opal_dss_pack_buffer(buffer, &ptr[i]->data.uint8, 1, OPAL_UINT8))) {
                 return ret;
             }
@@ -726,6 +786,7 @@ int opal_dss_pack_value(opal_buffer_t *buffer, const void *src,
             }
             break;
         case OPAL_UINT32:
+        case OPAL_INFO_DIRECTIVES:
             if (OPAL_SUCCESS != (ret = opal_dss_pack_buffer(buffer, &ptr[i]->data.uint32, 1, OPAL_UINT32))) {
                 return ret;
             }
@@ -752,11 +813,29 @@ int opal_dss_pack_value(opal_buffer_t *buffer, const void *src,
                 return ret;
             }
             break;
+        case OPAL_DOUBLE:
+            if (OPAL_SUCCESS != (ret = opal_dss_pack_buffer(buffer, &ptr[i]->data.dval, 1, OPAL_DOUBLE))) {
+                return ret;
+            }
+            break;
         case OPAL_TIMEVAL:
             if (OPAL_SUCCESS != (ret = opal_dss_pack_buffer(buffer, &ptr[i]->data.tv, 1, OPAL_TIMEVAL))) {
                 return ret;
             }
             break;
+        case OPAL_PTR:
+            /* just ignore these values */
+            break;
+        case OPAL_NAME:
+            if (OPAL_SUCCESS != (ret = opal_dss_pack_buffer(buffer, &ptr[i]->data.name, 1, OPAL_NAME))) {
+                return ret;
+            }
+            break;
+        case OPAL_STATUS:
+            if (OPAL_SUCCESS != (ret = opal_dss_pack_buffer(buffer, &ptr[i]->data.status, 1, OPAL_INT))) {
+                return ret;
+            }
+            break;
         default:
             opal_output(0, "PACK-OPAL-VALUE: UNSUPPORTED TYPE %d", (int)ptr[i]->type);
             return OPAL_ERROR;
@@ -778,7 +857,7 @@ int opal_dss_pack_buffer_contents(opal_buffer_t *buffer, const void *src,
     int ret;
 
     ptr = (opal_buffer_t **) src;
-    
+
     for (i = 0; i < num_vals; ++i) {
         /* pack the number of bytes */
         OPAL_OUTPUT((opal_dss_verbose, "opal_dss_pack_buffer_contents: bytes_used %u\n", (unsigned)ptr[i]->bytes_used));
@@ -786,11 +865,120 @@ int opal_dss_pack_buffer_contents(opal_buffer_t *buffer, const void *src,
             return ret;
         }
         /* pack the bytes */
-        if (OPAL_SUCCESS != (ret = opal_dss_pack_byte(buffer, ptr[i]->base_ptr, ptr[i]->bytes_used, OPAL_BYTE))) {
-            return ret;
+        if (0 < ptr[i]->bytes_used) {
+            if (OPAL_SUCCESS != (ret = opal_dss_pack_byte(buffer, ptr[i]->base_ptr, ptr[i]->bytes_used, OPAL_BYTE))) {
+                return ret;
+            }
+        } else {
+            ptr[i]->base_ptr = NULL;
         }
     }
     return OPAL_SUCCESS;
 }
 
+/*
+ * NAME
+ */
+int opal_dss_pack_name(opal_buffer_t *buffer, const void *src,
+                           int32_t num_vals, opal_data_type_t type)
+{
+    int rc;
+    int32_t i;
+    opal_process_name_t* proc;
+    opal_jobid_t *jobid;
+    opal_vpid_t *vpid;
+
+    /* collect all the jobids in a contiguous array */
+    jobid = (opal_jobid_t*)malloc(num_vals * sizeof(opal_jobid_t));
+    if (NULL == jobid) {
+        OPAL_ERROR_LOG(OPAL_ERR_OUT_OF_RESOURCE);
+        return OPAL_ERR_OUT_OF_RESOURCE;
+    }
+    proc = (opal_process_name_t*)src;
+    for (i=0; i < num_vals; i++) {
+        jobid[i] = proc->jobid;
+        proc++;
+    }
+    /* now pack them in one shot */
+    if (OPAL_SUCCESS != (rc =
+                         opal_dss_pack_jobid(buffer, jobid, num_vals, OPAL_JOBID))) {
+        OPAL_ERROR_LOG(rc);
+        free(jobid);
+        return rc;
+    }
+    free(jobid);
+
+    /* collect all the vpids in a contiguous array */
+    vpid = (opal_vpid_t*)malloc(num_vals * sizeof(opal_vpid_t));
+    if (NULL == vpid) {
+        OPAL_ERROR_LOG(OPAL_ERR_OUT_OF_RESOURCE);
+        return OPAL_ERR_OUT_OF_RESOURCE;
+    }
+    proc = (opal_process_name_t*)src;
+    for (i=0; i < num_vals; i++) {
+        vpid[i] = proc->vpid;
+        proc++;
+    }
+    /* now pack them in one shot */
+    if (OPAL_SUCCESS != (rc =
+                         opal_dss_pack_vpid(buffer, vpid, num_vals, OPAL_VPID))) {
+        OPAL_ERROR_LOG(rc);
+        free(vpid);
+        return rc;
+    }
+    free(vpid);
+
+    return OPAL_SUCCESS;
+}
+
+/*
+ * JOBID
+ */
+int opal_dss_pack_jobid(opal_buffer_t *buffer, const void *src,
+                            int32_t num_vals, opal_data_type_t type)
+{
+    int ret;
+
+    /* Turn around and pack the real type */
+    if (OPAL_SUCCESS != (
+                         ret = opal_dss_pack_buffer(buffer, src, num_vals, OPAL_JOBID_T))) {
+        OPAL_ERROR_LOG(ret);
+    }
+
+    return ret;
+}
+
+/*
+ * VPID
+ */
+int opal_dss_pack_vpid(opal_buffer_t *buffer, const void *src,
+                           int32_t num_vals, opal_data_type_t type)
+{
+    int ret;
+
+    /* Turn around and pack the real type */
+    if (OPAL_SUCCESS != (
+                         ret = opal_dss_pack_buffer(buffer, src, num_vals, OPAL_VPID_T))) {
+        OPAL_ERROR_LOG(ret);
+    }
+
+    return ret;
+}
+
+/*
+ * STATUS
+ */
+int opal_dss_pack_status(opal_buffer_t *buffer, const void *src,
+                         int32_t num_vals, opal_data_type_t type)
+{
+    int ret;
+
+    /* Turn around and pack the real type */
+    ret = opal_dss_pack_buffer(buffer, src, num_vals, OPAL_INT);
+    if (OPAL_SUCCESS != ret) {
+        OPAL_ERROR_LOG(ret);
+    }
+
+    return ret;
+}
 
