@@ -10,7 +10,7 @@
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
  * Copyright (c) 2012      Los Alamos National Security, Inc.  All rights reserved.
- * Copyright (c) 2014      Intel, Inc. All rights reserved.
+ * Copyright (c) 2014-2016 Intel, Inc. All rights reserved.
  * Copyright (c) 2014      Research Organization for Information Science
  *                         and Technology (RIST). All rights reserved.
  * $COPYRIGHT$
@@ -25,6 +25,7 @@
 #include "opal_stdint.h"
 #include <stdio.h>
 
+#include "opal/util/error.h"
 #include "opal/dss/dss_internal.h"
 
 int opal_dss_print(char **output, char *prefix, void *src, opal_data_type_t type)
@@ -653,10 +654,10 @@ int opal_dss_print_pstat(char **output, char *prefix, opal_pstats_t *src, opal_d
         return OPAL_SUCCESS;
     }
     asprintf(output, "%sOPAL_PSTATS SAMPLED AT: %ld.%06ld\n%snode: %s rank: %d pid: %d cmd: %s state: %c pri: %d #threads: %d Processor: %d\n"
-             "%s\ttime: %ld.%06ld cpu: %5.2f VMsize: %8.2f PeakVMSize: %8.2f RSS: %8.2f\n",
+             "%s\ttime: %ld.%06ld cpu: %5.2f  PSS: %8.2f  VMsize: %8.2f PeakVMSize: %8.2f RSS: %8.2f\n",
              prefx, (long)src->sample_time.tv_sec, (long)src->sample_time.tv_usec,
              prefx, src->node, src->rank, src->pid, src->cmd, src->state[0], src->priority, src->num_threads, src->processor,
-             prefx, (long)src->time.tv_sec, (long)src->time.tv_usec, src->percent_cpu, src->vsize, src->peak_vsize, src->rss);
+             prefx, (long)src->time.tv_sec, (long)src->time.tv_usec, src->percent_cpu, src->pss, src->vsize, src->peak_vsize, src->rss);
     if (prefx != prefix) {
         free(prefx);
     }
@@ -702,8 +703,7 @@ int opal_dss_print_node_stat(char **output, char *prefix, opal_node_stats_t *src
  */
 int opal_dss_print_value(char **output, char *prefix, opal_value_t *src, opal_data_type_t type)
 {
-    char *prefx, *t2;
-    int i;
+    char *prefx;
 
     /* deal with NULL prefix */
     if (NULL == prefix) asprintf(&prefx, " ");
@@ -795,184 +795,17 @@ int opal_dss_print_value(char **output, char *prefix, opal_value_t *src, opal_da
         asprintf(output, "%sOPAL_VALUE: Data type: OPAL_TIMEVAL\tKey: %s\tValue: %ld.%06ld", prefx,
                  src->key, (long)src->data.tv.tv_sec, (long)src->data.tv.tv_usec);
         break;
+    case OPAL_TIME:
+        asprintf(output, "%sOPAL_VALUE: Data type: OPAL_TIME\tKey: %s\tValue: %s", prefx,
+                 src->key, ctime(&src->data.time));
+        break;
+    case OPAL_NAME:
+        asprintf(output, "%sOPAL_VALUE: Data type: OPAL_NAME\tKey: %s\tValue: %s", prefx,
+                 src->key, OPAL_NAME_PRINT(src->data.name));
+        break;
     case OPAL_PTR:
         asprintf(output, "%sOPAL_VALUE: Data type: OPAL_PTR\tKey: %s", prefx, src->key);
         break;
-    case OPAL_FLOAT_ARRAY:
-        asprintf(output, "%sOPAL_VALUE: Data type: OPAL_FLOAT_ARRAY\tKey: %s \tSIZE: %d \tDATA: ",
-                 prefx, src->key, src->data.fval_array.size);
-        for (i = 0; i < src->data.fval_array.size; i++) {
-            asprintf(&t2, "%s\n%s\t%f", *output, prefx, src->data.fval_array.data[i]);
-            free(*output);
-            *output = t2;
-        }
-        break;
-    case OPAL_DOUBLE_ARRAY:
-        asprintf(output, "%sOPAL_VALUE: Data type: OPAL_DOUBLE_ARRAY\tKey: %s \tSIZE: %d \tDATA: ",
-                 prefx, src->key, src->data.dval_array.size);
-        for (i = 0; i < src->data.dval_array.size; i++) {
-            asprintf(&t2, "%s\n%s\t%f", *output, prefx, src->data.dval_array.data[i]);
-            free(*output);
-            *output = t2;
-        }
-        break;
-    case OPAL_STRING_ARRAY:
-        asprintf(output, "%sOPAL_VALUE: Data type: OPAL_STRING_ARRAY\tKey: %s \tSIZE: %d \tDATA: ",
-                 prefx, src->key, src->data.string_array.size);
-        for (i = 0; i < src->data.string_array.size; i++) {
-            asprintf(&t2, "%s\n%s\t%s", *output, prefx, src->data.string_array.data[i]);
-            free(*output);
-            *output = t2;
-        }
-        break;
-    case OPAL_BOOL_ARRAY:
-        asprintf(output, "%sOPAL_VALUE: Data type: OPAL_BOOL_ARRAY\tKey: %s \tSIZE: %d \tDATA: ",
-                 prefx, src->key, src->data.flag_array.size);
-        for (i = 0; i < src->data.flag_array.size; i++) {
-            asprintf(&t2, "%s\n%s\t%s", *output, prefx, src->data.flag_array.data[i] ? "true" : "false");
-            free(*output);
-            *output = t2;
-        }
-        break;
-    case OPAL_SIZE_ARRAY:
-        asprintf(output, "%sOPAL_VALUE: Data type: OPAL_SIZE_ARRAY\tKey: %s \tSIZE: %d \tDATA: ",
-                 prefx, src->key, src->data.size_array.size);
-        for (i = 0; i < src->data.size_array.size; i++) {
-            asprintf(&t2, "%s\n%s\t%lu", *output, prefx, (unsigned long)src->data.size_array.data[i]);
-            free(*output);
-            *output = t2;
-        }
-        break;
-    case OPAL_BYTE_ARRAY:
-        asprintf(output, "%sOPAL_VALUE: Data type: OPAL_BYTE_ARRAY\tKey: %s \tSIZE: %d \tDATA: ",
-                 prefx, src->key, src->data.byte_array.size);
-        for (i = 0; i < src->data.byte_array.size; i++) {
-            asprintf(&t2, "%s\n%s\t%x", *output, prefx, (uint8_t)src->data.byte_array.data[i]);
-            free(*output);
-            *output = t2;
-        }
-        break;
-    case OPAL_INT_ARRAY:
-        asprintf(output, "%sOPAL_VALUE: Data type: OPAL_INT_ARRAY\tKey: %s \tSIZE: %d \tDATA: ",
-                 prefx, src->key, src->data.integer_array.size);
-        for (i = 0; i < src->data.integer_array.size; i++) {
-            asprintf(&t2, "%s\n%s\t%d", *output, prefx, src->data.integer_array.data[i]);
-            free(*output);
-            *output = t2;
-        }
-        break;
-    case OPAL_INT8_ARRAY:
-        asprintf(output, "%sOPAL_VALUE: Data type: OPAL_INT8_ARRAY\tKey: %s \tSIZE: %d \tDATA: ",
-                 prefx, src->key, src->data.int8_array.size);
-        for (i = 0; i < src->data.int8_array.size; i++) {
-            asprintf(&t2, "%s\n%s\t%d", *output, prefx, (int)src->data.int8_array.data[i]);
-            free(*output);
-            *output = t2;
-        }
-        break;
-    case OPAL_INT16_ARRAY:
-        asprintf(output, "%sOPAL_VALUE: Data type: OPAL_INT16_ARRAY\tKey: %s \tSIZE: %d \tDATA: ",
-                 prefx, src->key, src->data.int16_array.size);
-        for (i = 0; i < src->data.int16_array.size; i++) {
-            asprintf(&t2, "%s\n%s\t%d", *output, prefx, (int)src->data.int16_array.data[i]);
-            free(*output);
-            *output = t2;
-        }
-        break;
-    case OPAL_INT32_ARRAY:
-        asprintf(output, "%sOPAL_VALUE: Data type: OPAL_INT32_ARRAY\tKey: %s \tSIZE: %d \tDATA: ",
-                 prefx, src->key, src->data.int32_array.size);
-        for (i = 0; i < src->data.int32_array.size; i++) {
-            asprintf(&t2, "%s\n%s\t%d", *output, prefx, src->data.int32_array.data[i]);
-            free(*output);
-            *output = t2;
-        }
-        break;
-    case OPAL_INT64_ARRAY:
-        asprintf(output, "%sOPAL_VALUE: Data type: OPAL_INT64_ARRAY\tKey: %s \tSIZE: %d \tDATA: ",
-                 prefx, src->key, src->data.int64_array.size);
-        for (i = 0; i < src->data.int64_array.size; i++) {
-            asprintf(&t2, "%s\n%s\t%ld", *output, prefx, (long)src->data.int64_array.data[i]);
-            free(*output);
-            *output = t2;
-        }
-        break;
-    case OPAL_UINT_ARRAY:
-        asprintf(output, "%sOPAL_VALUE: Data type: OPAL_UINT_ARRAY\tKey: %s \tSIZE: %d \tDATA: ",
-                 prefx, src->key, src->data.uint_array.size);
-        for (i = 0; i < src->data.uint_array.size; i++) {
-            asprintf(&t2, "%s\n%s\t%u", *output, prefx, src->data.uint_array.data[i]);
-            free(*output);
-            *output = t2;
-        }
-        break;
-    case OPAL_UINT8_ARRAY:
-        asprintf(output, "%sOPAL_VALUE: Data type: OPAL_UINT8_ARRAY\tKey: %s \tSIZE: %d \tDATA: ",
-                 prefx, src->key, src->data.uint8_array.size);
-        for (i = 0; i < src->data.uint8_array.size; i++) {
-            asprintf(&t2, "%s\n%s\t%u", *output, prefx, (unsigned int)src->data.uint8_array.data[i]);
-            free(*output);
-            *output = t2;
-        }
-        break;
-    case OPAL_UINT16_ARRAY:
-        asprintf(output, "%sOPAL_VALUE: Data type: OPAL_UINT16_ARRAY\tKey: %s \tSIZE: %d \tDATA: ",
-                 prefx, src->key, src->data.uint16_array.size);
-        for (i = 0; i < src->data.uint16_array.size; i++) {
-            asprintf(&t2, "%s\n%s\t%u", *output, prefx, (unsigned int)src->data.uint16_array.data[i]);
-            free(*output);
-            *output = t2;
-        }
-        break;
-    case OPAL_UINT32_ARRAY:
-        asprintf(output, "%sOPAL_VALUE: Data type: OPAL_UINT32_ARRAY\tKey: %s \tSIZE: %d \tDATA: ",
-                 prefx, src->key, src->data.uint32_array.size);
-        for (i = 0; i < src->data.uint32_array.size; i++) {
-            asprintf(&t2, "%s\n%s\t%u", *output, prefx, (unsigned int)src->data.uint32_array.data[i]);
-            free(*output);
-            *output = t2;
-        }
-        break;
-    case OPAL_UINT64_ARRAY:
-        asprintf(output, "%sOPAL_VALUE: Data type: OPAL_UINT64_ARRAY\tKey: %s \tSIZE: %d \tDATA: ",
-                 prefx, src->key, src->data.uint64_array.size);
-        for (i = 0; i < src->data.uint64_array.size; i++) {
-            asprintf(&t2, "%s\n%s\t%lu", *output, prefx, (unsigned long)src->data.uint64_array.data[i]);
-            free(*output);
-            *output = t2;
-        }
-        break;
-    case OPAL_BYTE_OBJECT_ARRAY:
-        asprintf(output, "%sOPAL_VALUE: Data type: OPAL_BYTE_OBJECT_ARRAY\tKey: %s \tSIZE: %d \tDATA: ",
-                 prefx, src->key, src->data.bo_array.size);
-        for (i = 0; i < src->data.bo_array.size; i++) {
-            asprintf(&t2, "%s\n%s\tData: %s\tSize: %lu ", *output, prefx,
-                     (NULL == src->data.bo_array.data[i].bytes) ? "NULL" : "NON-NULL",
-                     (unsigned long)src->data.bo_array.data[i].size);
-            free(*output);
-            *output = t2;
-        }
-        break;
-    case OPAL_PID_ARRAY:
-        asprintf(output, "%sOPAL_VALUE: Data type: OPAL_PID_ARRAY\tKey: %s \tSIZE: %d \tDATA: ",
-                 prefx, src->key, src->data.pid_array.size);
-        for (i = 0; i < src->data.pid_array.size; i++) {
-            asprintf(&t2, "%s\n%s\t%u", *output, prefx, (unsigned int)src->data.pid_array.data[i]);
-            free(*output);
-            *output = t2;
-        }
-        break;
-    case OPAL_TIMEVAL_ARRAY:
-        asprintf(output, "%sOPAL_VALUE: Data type: OPAL_TIMEVAL_ARRAY\tKey: %s \tSIZE: %d \tDATA: ",
-                 prefx, src->key, src->data.tv_array.size);
-        for (i = 0; i < src->data.tv_array.size; i++) {
-            asprintf(&t2, "%s\n%s\t%ld.%06ld", *output, prefx,
-                     (long)src->data.tv_array.data[i].tv_sec,
-                     (long)src->data.tv_array.data[i].tv_usec);
-            free(*output);
-            *output = t2;
-        }
-        break;
     default:
         asprintf(output, "%sOPAL_VALUE: Data type: UNKNOWN\tKey: %s\tValue: UNPRINTABLE",
                  prefx, src->key);
@@ -1060,3 +893,29 @@ int opal_dss_print_vpid(char **output, char *prefix,
 
     return OPAL_SUCCESS;
 }
+
+int opal_dss_print_status(char **output, char *prefix,
+                          int *src, opal_data_type_t type)
+{
+    char *prefx;
+
+    /* deal with NULL prefix */
+    if (NULL == prefix) asprintf(&prefx, " ");
+    else prefx = prefix;
+
+    /* if src is NULL, just print data type and return */
+    if (NULL == src) {
+        asprintf(output, "%sData type: OPAL_STATUS\tValue: NULL pointer", prefx);
+        if (prefx != prefix) {
+            free(prefx);
+        }
+        return OPAL_SUCCESS;
+    }
+
+    asprintf(output, "%sData type: OPAL_STATUS\tValue: %s", prefx, opal_strerror(*src));
+    if (prefx != prefix) {
+        free(prefx);
+    }
+
+    return OPAL_SUCCESS;
+}
