@@ -1,3 +1,4 @@
+/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil -*- */
 /*
  * Copyright (c) 2004-2007 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
@@ -9,7 +10,10 @@
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
- * Copyright (c) 2012      Los Alamos National Security, Inc.  All rights reserved. 
+ * Copyright (c) 2012-2015 Los Alamos National Security, Inc.  All rights reserved.
+ * Copyright (c) 2014-2016 Intel, Inc. All rights reserved.
+ * Copyright (c) 2014-2015 Research Organization for Information Science
+ *                         and Technology (RIST). All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -190,7 +194,7 @@ int opal_dss_unpack_int(opal_buffer_t *buffer, void *dest,
         /* slow path - types are different sizes */
         UNPACK_SIZE_MISMATCH(int, remote_type, ret);
     }
-    
+
     return ret;
 }
 
@@ -223,7 +227,7 @@ int opal_dss_unpack_sizet(opal_buffer_t *buffer, void *dest,
         /* slow path - types are different sizes */
         UNPACK_SIZE_MISMATCH(size_t, remote_type, ret);
     }
-    
+
     return ret;
 }
 
@@ -256,7 +260,7 @@ int opal_dss_unpack_pid(opal_buffer_t *buffer, void *dest,
         /* slow path - types are different sizes */
         UNPACK_SIZE_MISMATCH(pid_t, remote_type, ret);
     }
-    
+
     return ret;
 }
 
@@ -320,7 +324,8 @@ int opal_dss_unpack_int16(opal_buffer_t *buffer, void *dest,
     /* unpack the data */
     for (i = 0; i < (*num_vals); ++i) {
         memcpy( &(tmp), buffer->unpack_ptr, sizeof(tmp) );
-        desttmp[i] = ntohs(tmp);
+        tmp = ntohs(tmp);
+        memcpy(&desttmp[i], &tmp, sizeof(tmp));
         buffer->unpack_ptr += sizeof(tmp);
     }
 
@@ -342,7 +347,8 @@ int opal_dss_unpack_int32(opal_buffer_t *buffer, void *dest,
     /* unpack the data */
     for (i = 0; i < (*num_vals); ++i) {
         memcpy( &(tmp), buffer->unpack_ptr, sizeof(tmp) );
-        desttmp[i] = ntohl(tmp);
+        tmp = ntohl(tmp);
+        memcpy(&desttmp[i], &tmp, sizeof(tmp));
         buffer->unpack_ptr += sizeof(tmp);
     }
 
@@ -364,7 +370,8 @@ int opal_dss_unpack_int64(opal_buffer_t *buffer, void *dest,
     /* unpack the data */
     for (i = 0; i < (*num_vals); ++i) {
         memcpy( &(tmp), buffer->unpack_ptr, sizeof(tmp) );
-        desttmp[i] = ntoh64(tmp);
+        tmp = ntoh64(tmp);
+        memcpy(&desttmp[i], &tmp, sizeof(tmp));
         buffer->unpack_ptr += sizeof(tmp);
     }
 
@@ -402,9 +409,9 @@ int opal_dss_unpack_float(opal_buffer_t *buffer, void *dest,
                           int32_t *num_vals, opal_data_type_t type)
 {
     int32_t i, n;
-    int64_t tmp[2];
-    float *desttmp = (float*) dest;
+    float *desttmp = (float*) dest, tmp;
     int ret;
+    char *convert;
 
    OPAL_OUTPUT( ( opal_dss_verbose, "opal_dss_unpack_float * %d\n", (int)*num_vals ) );
     /* check to see if there's enough data in buffer */
@@ -414,11 +421,48 @@ int opal_dss_unpack_float(opal_buffer_t *buffer, void *dest,
 
     /* unpack the data */
     for (i = 0; i < (*num_vals); ++i) {
-        n=2;
-        if (OPAL_SUCCESS != (ret = opal_dss_unpack_int64(buffer, tmp, &n, OPAL_INT64))) {
+        n=1;
+        if (OPAL_SUCCESS != (ret = opal_dss_unpack_string(buffer, &convert, &n, OPAL_STRING))) {
             return ret;
         }
-        desttmp[i] = (float)tmp[0] + (float)tmp[1]/1000000.0;
+        if (NULL == convert) {
+            return OPAL_ERR_UNPACK_FAILURE;
+        }
+        tmp = strtof(convert, NULL);
+        memcpy(&desttmp[i], &tmp, sizeof(tmp));
+        free(convert);
+        convert = NULL;
+    }
+    return OPAL_SUCCESS;
+}
+
+int opal_dss_unpack_double(opal_buffer_t *buffer, void *dest,
+                           int32_t *num_vals, opal_data_type_t type)
+{
+    int32_t i, n;
+    double *desttmp = (double*) dest, tmp;
+    int ret;
+    char *convert;
+
+   OPAL_OUTPUT( ( opal_dss_verbose, "opal_dss_unpack_double * %d\n", (int)*num_vals ) );
+    /* check to see if there's enough data in buffer */
+    if (opal_dss_too_small(buffer, (*num_vals)*sizeof(double))) {
+        return OPAL_ERR_UNPACK_READ_PAST_END_OF_BUFFER;
+    }
+
+    /* unpack the data */
+    for (i = 0; i < (*num_vals); ++i) {
+        n=1;
+        if (OPAL_SUCCESS != (ret = opal_dss_unpack_string(buffer, &convert, &n, OPAL_STRING))) {
+            return ret;
+        }
+        if (NULL == convert) {
+            return OPAL_ERR_UNPACK_FAILURE;
+        }
+        tmp = strtod(convert, NULL);
+        memcpy(&desttmp[i], &tmp, sizeof(tmp));
+        free(convert);
+        convert = NULL;
     }
     return OPAL_SUCCESS;
 }
@@ -428,7 +472,7 @@ int opal_dss_unpack_timeval(opal_buffer_t *buffer, void *dest,
 {
     int32_t i, n;
     int64_t tmp[2];
-    struct timeval *desttmp = (struct timeval *) dest;
+    struct timeval *desttmp = (struct timeval *) dest, tt;
     int ret;
 
    OPAL_OUTPUT( ( opal_dss_verbose, "opal_dss_unpack_timeval * %d\n", (int)*num_vals ) );
@@ -443,12 +487,42 @@ int opal_dss_unpack_timeval(opal_buffer_t *buffer, void *dest,
         if (OPAL_SUCCESS != (ret = opal_dss_unpack_int64(buffer, tmp, &n, OPAL_INT64))) {
             return ret;
         }
-        desttmp[i].tv_sec = tmp[0];
-        desttmp[i].tv_usec = tmp[1];
+        tt.tv_sec = tmp[0];
+        tt.tv_usec = tmp[1];
+        memcpy(&desttmp[i], &tt, sizeof(tt));
     }
     return OPAL_SUCCESS;
 }
 
+int opal_dss_unpack_time(opal_buffer_t *buffer, void *dest,
+                         int32_t *num_vals, opal_data_type_t type)
+{
+    int32_t i, n;
+    time_t *desttmp = (time_t *) dest, tmp;
+    int ret;
+    uint64_t ui64;
+
+    /* time_t is a system-dependent size, so cast it
+     * to uint64_t as a generic safe size
+     */
+
+   OPAL_OUTPUT( ( opal_dss_verbose, "opal_dss_unpack_time * %d\n", (int)*num_vals ) );
+    /* check to see if there's enough data in buffer */
+   if (opal_dss_too_small(buffer, (*num_vals)*(sizeof(uint64_t)))) {
+        return OPAL_ERR_UNPACK_READ_PAST_END_OF_BUFFER;
+    }
+
+    /* unpack the data */
+    for (i = 0; i < (*num_vals); ++i) {
+        n=1;
+        if (OPAL_SUCCESS != (ret = opal_dss_unpack_int64(buffer, &ui64, &n, OPAL_UINT64))) {
+            return ret;
+        }
+        tmp = (time_t)ui64;
+        memcpy(&desttmp[i], &tmp, sizeof(tmp));
+    }
+    return OPAL_SUCCESS;
+}
 
 
 /* UNPACK FUNCTIONS FOR GENERIC OPAL TYPES */
@@ -514,10 +588,10 @@ int opal_dss_unpack_pstat(opal_buffer_t *buffer, void *dest,
     int32_t i, n, m;
     int ret;
     char *cptr;
-    
+
     ptr = (opal_pstats_t **) dest;
     n = *num_vals;
-    
+
     for (i = 0; i < n; ++i) {
         /* allocate the new object */
         ptr[i] = OBJ_NEW(opal_pstats_t);
@@ -569,6 +643,11 @@ int opal_dss_unpack_pstat(opal_buffer_t *buffer, void *dest,
             return ret;
         }
         m=1;
+        if (OPAL_SUCCESS != (ret = opal_dss_unpack_float(buffer, &ptr[i]->pss, &m, OPAL_FLOAT))) {
+            OPAL_ERROR_LOG(ret);
+            return ret;
+        }
+        m=1;
         if (OPAL_SUCCESS != (ret = opal_dss_unpack_float(buffer, &ptr[i]->vsize, &m, OPAL_FLOAT))) {
             OPAL_ERROR_LOG(ret);
             return ret;
@@ -594,7 +673,7 @@ int opal_dss_unpack_pstat(opal_buffer_t *buffer, void *dest,
             return ret;
         }
     }
-    
+
     return OPAL_SUCCESS;
 }
 
@@ -614,6 +693,7 @@ static int unpack_disk_stats(opal_buffer_t *buffer, opal_node_stats_t *ns)
     /* unpack them */
     for (i=0; i < n; i++) {
         dk = OBJ_NEW(opal_diskstats_t);
+        assert(dk);
         m=1;
         if (OPAL_SUCCESS != (ret = opal_dss_unpack_buffer(buffer, &dk->disk, &m, OPAL_STRING))) {
             OPAL_ERROR_LOG(ret);
@@ -718,6 +798,7 @@ static int unpack_net_stats(opal_buffer_t *buffer, opal_node_stats_t *ns)
     /* unpack them */
     for (i=0; i < n; i++) {
         net = OBJ_NEW(opal_netstats_t);
+        assert(net);
         m=1;
         if (OPAL_SUCCESS != (ret = opal_dss_unpack_buffer(buffer, &net->net_interface, &m, OPAL_STRING))) {
             OPAL_ERROR_LOG(ret);
@@ -780,10 +861,10 @@ int opal_dss_unpack_node_stat(opal_buffer_t *buffer, void *dest,
     opal_node_stats_t **ptr;
     int32_t i, n, m;
     int ret;
-    
+
     ptr = (opal_node_stats_t **) dest;
     n = *num_vals;
-    
+
     for (i = 0; i < n; ++i) {
         /* allocate the new object */
         ptr[i] = OBJ_NEW(opal_node_stats_t);
@@ -860,8 +941,9 @@ int opal_dss_unpack_node_stat(opal_buffer_t *buffer, void *dest,
             OPAL_ERROR_LOG(ret);
             return ret;
         }
+        OBJ_RELEASE(ptr[i]);
     }
-    
+
     return OPAL_SUCCESS;
 }
 
@@ -877,7 +959,7 @@ int opal_dss_unpack_value(opal_buffer_t *buffer, void *dest,
 
     ptr = (opal_value_t **) dest;
     n = *num_vals;
-    
+
     for (i = 0; i < n; ++i) {
         /* allocate the new object */
         ptr[i] = OBJ_NEW(opal_value_t);
@@ -890,16 +972,17 @@ int opal_dss_unpack_value(opal_buffer_t *buffer, void *dest,
             return ret;
         }
         m=1;
-        if (OPAL_SUCCESS != (ret = opal_dss_unpack_data_type(buffer, &ptr[i]->scope, &m, OPAL_DATA_SCOPE_T))) {
-            return ret;
-        }
-        m=1;
         if (OPAL_SUCCESS != (ret = opal_dss_unpack_data_type(buffer, &ptr[i]->type, &m, OPAL_DATA_TYPE))) {
             return ret;
         }
         /* now unpack the right field */
         m=1;
         switch (ptr[i]->type) {
+        case OPAL_BOOL:
+            if (OPAL_SUCCESS != (ret = opal_dss_unpack_buffer(buffer, &ptr[i]->data.flag, &m, OPAL_BOOL))) {
+                return ret;
+            }
+            break;
         case OPAL_BYTE:
             if (OPAL_SUCCESS != (ret = opal_dss_unpack_buffer(buffer, &ptr[i]->data.byte, &m, OPAL_BYTE))) {
                 return ret;
@@ -910,6 +993,11 @@ int opal_dss_unpack_value(opal_buffer_t *buffer, void *dest,
                 return ret;
             }
             break;
+        case OPAL_SIZE:
+            if (OPAL_SUCCESS != (ret = opal_dss_unpack_buffer(buffer, &ptr[i]->data.size, &m, OPAL_SIZE))) {
+                return ret;
+            }
+            break;
         case OPAL_PID:
             if (OPAL_SUCCESS != (ret = opal_dss_unpack_buffer(buffer, &ptr[i]->data.pid, &m, OPAL_PID))) {
                 return ret;
@@ -988,11 +1076,21 @@ int opal_dss_unpack_value(opal_buffer_t *buffer, void *dest,
                 return ret;
             }
             break;
+        case OPAL_DOUBLE:
+            if (OPAL_SUCCESS != (ret = opal_dss_unpack_buffer(buffer, &ptr[i]->data.dval, &m, OPAL_DOUBLE))) {
+                return ret;
+            }
+            break;
         case OPAL_TIMEVAL:
             if (OPAL_SUCCESS != (ret = opal_dss_unpack_buffer(buffer, &ptr[i]->data.tv, &m, OPAL_TIMEVAL))) {
                 return ret;
             }
             break;
+        case OPAL_NAME:
+            if (OPAL_SUCCESS != (ret = opal_dss_unpack_buffer(buffer, &ptr[i]->data.name, &m, OPAL_NAME))) {
+                return ret;
+            }
+            break;
         default:
             opal_output(0, "PACK-OPAL-VALUE: UNSUPPORTED TYPE");
             return OPAL_ERROR;
@@ -1015,7 +1113,7 @@ int opal_dss_unpack_buffer_contents(opal_buffer_t *buffer, void *dest,
 
     ptr = (opal_buffer_t **) dest;
     n = *num_vals;
-    
+
     for (i = 0; i < n; ++i) {
         /* allocate the new object */
         ptr[i] = OBJ_NEW(opal_buffer_t);
@@ -1027,16 +1125,15 @@ int opal_dss_unpack_buffer_contents(opal_buffer_t *buffer, void *dest,
         if (OPAL_SUCCESS != (ret = opal_dss_unpack_sizet(buffer, &nbytes, &m, OPAL_SIZE))) {
             return ret;
         }
+        m = nbytes;
         /* setup the buffer's data region */
         if (0 < nbytes) {
             ptr[i]->base_ptr = (char*)malloc(nbytes);
+            /* unpack the bytes */
+            if (OPAL_SUCCESS != (ret = opal_dss_unpack_byte(buffer, ptr[i]->base_ptr, &m, OPAL_BYTE))) {
+                return ret;
+            }
         }
-        /* unpack the bytes */
-        m=nbytes;
-        if (OPAL_SUCCESS != (ret = opal_dss_unpack_byte(buffer, ptr[i]->base_ptr, &m, OPAL_BYTE))) {
-            return ret;
-        }
-        /* fill-in the metadata */
         ptr[i]->pack_ptr = ptr[i]->base_ptr + m;
         ptr[i]->unpack_ptr = ptr[i]->base_ptr;
         ptr[i]->bytes_allocated = nbytes;
@@ -1044,3 +1141,115 @@ int opal_dss_unpack_buffer_contents(opal_buffer_t *buffer, void *dest,
     }
     return OPAL_SUCCESS;
 }
+
+/*
+ * NAME
+ */
+int opal_dss_unpack_name(opal_buffer_t *buffer, void *dest,
+                        int32_t *num_vals, opal_data_type_t type)
+{
+    int rc;
+    int32_t i, num;
+    opal_process_name_t* proc;
+    opal_jobid_t *jobid;
+    opal_vpid_t *vpid;
+
+    num = *num_vals;
+
+    /* allocate space for all the jobids in a contiguous array */
+    jobid = (opal_jobid_t*)malloc(num * sizeof(opal_jobid_t));
+    if (NULL == jobid) {
+        OPAL_ERROR_LOG(OPAL_ERR_OUT_OF_RESOURCE);
+        *num_vals = 0;
+        return OPAL_ERR_OUT_OF_RESOURCE;
+    }
+    /* now unpack them in one shot */
+    if (OPAL_SUCCESS != (rc =
+                         opal_dss_unpack_jobid(buffer, jobid, num_vals, OPAL_JOBID))) {
+        OPAL_ERROR_LOG(rc);
+        *num_vals = 0;
+        free(jobid);
+        return rc;
+    }
+
+    /* collect all the vpids in a contiguous array */
+    vpid = (opal_vpid_t*)malloc(num * sizeof(opal_vpid_t));
+    if (NULL == vpid) {
+        OPAL_ERROR_LOG(OPAL_ERR_OUT_OF_RESOURCE);
+        *num_vals = 0;
+        free(jobid);
+        return OPAL_ERR_OUT_OF_RESOURCE;
+    }
+    /* now unpack them in one shot */
+    if (OPAL_SUCCESS != (rc =
+                         opal_dss_unpack_vpid(buffer, vpid, num_vals, OPAL_VPID))) {
+        OPAL_ERROR_LOG(rc);
+        *num_vals = 0;
+        free(vpid);
+        free(jobid);
+        return rc;
+    }
+
+    /* build the names from the jobid/vpid arrays */
+    proc = (opal_process_name_t*)dest;
+    for (i=0; i < num; i++) {
+        proc->jobid = jobid[i];
+        proc->vpid = vpid[i];
+        proc++;
+    }
+
+    /* cleanup */
+    free(vpid);
+    free(jobid);
+
+    return OPAL_SUCCESS;
+}
+
+/*
+ * JOBID
+ */
+int opal_dss_unpack_jobid(opal_buffer_t *buffer, void *dest,
+                         int32_t *num_vals, opal_data_type_t type)
+{
+    int ret;
+
+    /* Turn around and unpack the real type */
+    if (OPAL_SUCCESS != (ret = opal_dss_unpack_buffer(buffer, dest, num_vals, OPAL_JOBID_T))) {
+        OPAL_ERROR_LOG(ret);
+    }
+
+    return ret;
+}
+
+/*
+ * VPID
+ */
+int opal_dss_unpack_vpid(opal_buffer_t *buffer, void *dest,
+                        int32_t *num_vals, opal_data_type_t type)
+{
+    int ret;
+
+    /* Turn around and unpack the real type */
+    if (OPAL_SUCCESS != (ret = opal_dss_unpack_buffer(buffer, dest, num_vals, OPAL_VPID_T))) {
+        OPAL_ERROR_LOG(ret);
+    }
+
+    return ret;
+}
+
+/*
+ * STATUS
+ */
+int opal_dss_unpack_status(opal_buffer_t *buffer, void *dest,
+                           int32_t *num_vals, opal_data_type_t type)
+{
+    int ret;
+
+    /* Turn around and unpack the real type */
+    ret = opal_dss_unpack_buffer(buffer, dest, num_vals, OPAL_INT);
+    if (OPAL_SUCCESS != ret) {
+        OPAL_ERROR_LOG(ret);
+    }
+
+    return ret;
+}
