@@ -5,16 +5,21 @@
  * Copyright (c) 2004-2005 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
- * Copyright (c) 2004-2010 High Performance Computing Center Stuttgart, 
+ * Copyright (c) 2004-2010 High Performance Computing Center Stuttgart,
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
  * Copyright (c) 2009      Sun Microsystems, Inc.  All rights reserved.
  * Copyright (c) 2009-2013 Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2013      Mellanox Technologies, Inc.
+ *                         All rights reserved.
+ * Copyright (c) 2015-2017 Research Organization for Information Science
+ *                         and Technology (RIST). All rights reserved.
+ * Copyright (c) 2015      Intel, Inc. All rights reserved.
  * $COPYRIGHT$
- * 
+ *
  * Additional copyrights may follow
- * 
+ *
  * $HEADER$
  *
  * This file is included at the bottom of opal_config.h, and is
@@ -26,7 +31,7 @@
  * need to #ifndef/#endif protection here.
  */
 
-#ifndef OPAL_CONFIG_H 
+#ifndef OPAL_CONFIG_H
 #error "opal_config_bottom.h should only be included from opal_config.h"
 #endif
 
@@ -34,15 +39,15 @@
  * If we build a static library, Visual C define the _LIB symbol. In the
  * case of a shared library _USERDLL get defined.
  *
- * OMPI_BUILDING and _LIB define how ompi_config.h
+ * OMPI_BUILDING and _LIB define how opal_config.h
  * handles configuring all of Open MPI's "compatibility" code.  Both
- * constants will always be defined by the end of ompi_config.h.
+ * constants will always be defined by the end of opal_config.h.
  *
  * OMPI_BUILDING affects how much compatibility code is included by
- * ompi_config.h.  It will always be 1 or 0.  The user can set the
- * value before including either mpi.h or ompi_config.h and it will be
- * respected.  If ompi_config.h is included before mpi.h, it will
- * default to 1.  If mpi.h is included before ompi_config.h, it will
+ * opal_config.h.  It will always be 1 or 0.  The user can set the
+ * value before including either mpi.h or opal_config.h and it will be
+ * respected.  If opal_config.h is included before mpi.h, it will
+ * default to 1.  If mpi.h is included before opal_config.h, it will
  * default to 0.
  */
 #ifndef OMPI_BUILDING
@@ -52,7 +57,7 @@
 /*
  * Flex is trying to include the unistd.h file. As there is no configure
  * option or this, the flex generated files will try to include the file
- * even on platforms without unistd.h (such as Windows). Therefore, if we
+ * even on platforms without unistd.h. Therefore, if we
  * know this file is not available, we can prevent flex from including it.
  */
 #ifndef HAVE_UNISTD_H
@@ -66,9 +71,6 @@
  *
  **********************************************************************/
 
-/* Do we have posix or solaris thread lib */
-#define OPAL_HAVE_THREADS (OPAL_HAVE_POSIX_THREADS)
-
 /*
  * BEGIN_C_DECLS should be used at the beginning of your declarations,
  * so that C++ compilers don't mangle their names.  Use END_C_DECLS at
@@ -226,45 +228,12 @@
 #    define __opal_attribute_weak_alias__(a)
 #endif
 
-/***********************************************************************
- *
- * Windows library interface declaration code
- *
- **********************************************************************/
-#if !defined(__WINDOWS__)
-#  if defined(_WIN32) || defined(WIN32) || defined(WIN64)
-#    define __WINDOWS__
-#  endif
-#endif  /* !defined(__WINDOWS__) */
-
-#if defined(__WINDOWS__)
-
-#  if defined(_USRDLL)    /* building shared libraries (.DLL) */
-#    if defined(OPAL_EXPORTS)
-#      define OPAL_DECLSPEC        __declspec(dllexport)
-#      define OPAL_MODULE_DECLSPEC
-#    else
-#      if defined(OPAL_IMPORTS)
-#        define OPAL_DECLSPEC      __declspec(dllimport)
-#      else
-#        define OPAL_DECLSPEC
-#      endif  /*defined(OPAL_IMPORTS)*/
-#      if defined(OPAL_MODULE_EXPORTS)
-#        define OPAL_MODULE_DECLSPEC __declspec(dllexport)
-#      else
-#        define OPAL_MODULE_DECLSPEC __declspec(dllimport)
-#      endif  /* defined(OPAL_MODULE_EXPORTS) */
-#    endif  /* defined(OPAL_EXPORTS) */
-#  else          /* building static library */
-#    if defined(OPAL_IMPORTS)
-#      define OPAL_DECLSPEC        __declspec(dllimport)
-#    else
-#      define OPAL_DECLSPEC
-#    endif  /* defined(OPAL_IMPORTS) */
-#    define OPAL_MODULE_DECLSPEC
-#  endif  /* defined(_USRDLL) */
-#  include "opal/win32/win_compat.h"
+#if OPAL_HAVE_ATTRIBUTE_DESTRUCTOR
+#    define __opal_attribute_destructor__    __attribute__((__destructor__))
 #else
+#    define __opal_attribute_destructor__
+#endif
+
 #  if OPAL_C_HAVE_VISIBILITY
 #    define OPAL_DECLSPEC           __opal_attribute_visibility__("default")
 #    define OPAL_MODULE_DECLSPEC    __opal_attribute_visibility__("default")
@@ -272,12 +241,7 @@
 #    define OPAL_DECLSPEC
 #    define OPAL_MODULE_DECLSPEC
 #  endif
-#endif  /* defined(__WINDOWS__) */
 
-/*
- * Do we have <stdint.h>?
- */
-#ifdef HAVE_STDINT_H
 #if !defined(__STDC_LIMIT_MACROS) && (defined(c_plusplus) || defined (__cplusplus))
 /* When using a C++ compiler, the max / min value #defines for std
    types are only included if __STDC_LIMIT_MACROS is set before
@@ -285,10 +249,7 @@
 #define __STDC_LIMIT_MACROS
 #endif
 #include "opal_config.h"
-#include <stdint.h>
-#else
 #include "opal_stdint.h"
-#endif
 
 /***********************************************************************
  *
@@ -299,65 +260,6 @@
  **********************************************************************/
 #if OMPI_BUILDING
 
-#ifndef HAVE_PTRDIFF_T
-typedef OPAL_PTRDIFF_TYPE ptrdiff_t;
-#endif
-
-/*
- * If we're in C, we may need to bring in the bool type and true/false
- * constants.  OPAL_NEED_C_BOOL will be true if the compiler either
- * needs <stdbool.h> or doesn't define the bool type at all.
- */
-#if !(defined(c_plusplus) || defined(__cplusplus))
-#    if OPAL_NEED_C_BOOL
-#        if OPAL_USE_STDBOOL_H
-             /* If we're using <stdbool.h>, there is an implicit
-                assumption that the C++ bool is the same size and has
-                the same alignment.  However, configure may have
-                disabled the MPI C++ bindings, so if "_Bool" exists,
-                then use that sizeof. */
-#            include <stdbool.h>
-             /* This section exists because AC_SIZEOF(bool) may not be
-                run in configure if we're not building the MPI C++
-                bindings. */
-#            undef SIZEOF_BOOL
-#            if SIZEOF__BOOL > 0
-#                define SIZEOF_BOOL SIZEOF__BOOL
-#            else
-                 /* If all else fails, assume it's 1 */
-#                define SIZEOF_BOOL 1
-#            endif
-#        else
-             /* We need to create a bool type and ensure that it's the
-                same size / alignment as the C++ bool size /
-                alignment */
-#            define false 0
-#            define true 1
-#            if SIZEOF_BOOL == SIZEOF_CHAR && OPAL_ALIGNMENT_CXX_BOOL == OPAL_ALIGNMENT_CHAR
-typedef unsigned char bool;
-#            elif SIZEOF_BOOL == SIZEOF_SHORT && OPAL_ALIGNMENT_CXX_BOOL == OPAL_ALIGNMENT_SHORT
-typedef short bool;
-#            elif SIZEOF_BOOL == SIZEOF_INT && OPAL_ALIGNMENT_CXX_BOOL == OPAL_ALIGNMENT_INT
-typedef int bool;
-#            elif SIZEOF_BOOL == SIZEOF_LONG && OPAL_ALIGNMENT_CXX_BOOL == OPAL_ALIGNMENT_LONG
-typedef long bool;
-#            elif defined(SIZEOF_LONG_LONG) && defined(OPAL_ALIGNMENT_LONG) && SIZEOF_BOOL == SIZEOF_LONG && OPAL_ALIGNMENT_CXX_BOOL == OPAL_ALIGNMENT_LONG
-typedef long long bool;
-             /* If we have _Bool, use that */
-#            elif SIZEOF__BOOL > 0
-#                undef SIZEOF_BOOL
-#                define bool _Bool
-#                define SIZEOF_BOOL SIZEOF__BOOL
-#            else
-             /* If all else fails, just make bool be an unsigned char
-                and size of 1 */
-typedef unsigned char bool;
-#                define SIZEOF_BOOL 1
-#            endif
-#        endif  /* OPAL_USE_STDBOOL_H */
-#    endif  /* OPAL_NEED_C_BOOL */
-#endif
-
 /*
  * Maximum size of a filename path.
  */
@@ -376,14 +278,17 @@ typedef unsigned char bool;
 /*
  * Set the compile-time path-separator on this system and variable separator
  */
-#ifdef __WINDOWS__
-#define OPAL_PATH_SEP "\\"
-#define OPAL_ENV_SEP  ';'
-#else
 #define OPAL_PATH_SEP "/"
 #define OPAL_ENV_SEP  ':'
-#endif
 
+#if defined(MAXHOSTNAMELEN)
+#define OPAL_MAXHOSTNAMELEN (MAXHOSTNAMELEN + 1)
+#elif defined(HOST_NAME_MAX)
+#define OPAL_MAXHOSTNAMELEN (HOST_NAME_MAX + 1)
+#else
+/* SUSv2 guarantees that "Host names are limited to 255 bytes". */
+#define OPAL_MAXHOSTNAMELEN (255 + 1)
+#endif
 
 /*
  * Do we want memory debugging?
@@ -394,13 +299,13 @@ typedef unsigned char bool;
  * 2. In the OMPI C++ bindings: we do not want them
  * 3. In the OMPI C++ executables: we do want them
  *
- * So for 1, everyone must include <ompi_config.h> first.  For 2, the
- * C++ bindings will never include <ompi_config.h> -- they will only
- * include <mpi.h>, which includes <ompi_config.h>, but after
+ * So for 1, everyone must include <opal_config.h> first.  For 2, the
+ * C++ bindings will never include <opal_config.h> -- they will only
+ * include <mpi.h>, which includes <opal_config.h>, but after
  * setting OMPI_BUILDING to 0  For 3, it's the same as 1 -- just include
- * <ompi_config.h> first.
+ * <opal_config.h> first.
  *
- * Give code that needs to include ompi_config.h but really can't have
+ * Give code that needs to include opal_config.h but really can't have
  * this stuff enabled (like the memory manager code) a way to turn us
  * off
  */
@@ -484,9 +389,9 @@ typedef unsigned char bool;
  * are no htonl and friends.  If that's the case, provide stubs.  I
  * would hope we never find a platform that doesn't have these macros
  * and would want to talk to the outside world... On other platforms
- * (like Windows) we fail to detect them correctly.
+ * we fail to detect them correctly.
  */
-#if !defined(__WINDOWS__) && !defined(HAVE_UNIX_BYTESWAP)
+#if !defined(HAVE_UNIX_BYTESWAP)
 static inline uint32_t htonl(uint32_t hostvar) { return hostvar; }
 static inline uint32_t ntohl(uint32_t netvar) { return netvar; }
 static inline uint16_t htons(uint16_t hostvar) { return hostvar; }
@@ -504,35 +409,16 @@ static inline uint16_t ntohs(uint16_t netvar) { return netvar; }
 #define __func__ __FILE__
 #endif
 
-/**
- * Because of the way we're using the opal_object inside Open MPI (i.e.
- * dynamic resolution at run-time to derive all objects from the basic
- * type), on Windows we have to build everything on C++ mode, simply
- * because the C mode does not support dynamic resolution in DLL. Therefore,
- * no automatic conversion is allowed. All types have to be explicitly casted
- * or the compiler generate an error. This is true even for the void* type. As
- * we use void* to silence others compilers in the resolution of the addr member
- * of the iovec structure, we have now to find a way around. The simplest solution
- * is to define a special type for this field (just for casting). It can be
- * set to void* on all platforms with the exception of windows where it has to be
- * char*.
- */
-#if defined(__WINDOWS__)
-#define IOVBASE_TYPE  char
-#else
 #define IOVBASE_TYPE  void
-#endif  /* defined(__WINDOWS__) */
+
+/* ensure the bool type is defined as it is used everywhere */
+#include <stdbool.h>
 
 /**
  * If we generate our own bool type, we need a special way to cast the result
- * in such a way to keep the compilers silent. Right now, th only compiler who
- * complain about int to bool conversions is the Microsoft compiler.
+ * in such a way to keep the compilers silent.
  */
-#if defined(__WINDOWS__)
-#  define OPAL_INT_TO_BOOL(VALUE)  ((VALUE) != 0 ? true : false)
-#else
 #  define OPAL_INT_TO_BOOL(VALUE)  (bool)(VALUE)
-#endif  /* defined(__WINDOWS__) */
 
 /**
  * Top level define to check 2 things: a) if we want ipv6 support, and
@@ -607,14 +493,9 @@ static inline uint16_t ntohs(uint16_t netvar) { return netvar; }
 #ifdef HAVE_HOSTLIB_H
 /* gethostname() */
 #include <hostLib.h>
-
-#ifndef MAXHOSTNAMELEN
-#define MAXHOSTNAMELEN 64
 #endif
 #endif
 
-#endif
-
 /* If we're in C++, then just undefine restrict and then define it to
    nothing.  "restrict" is not part of the C++ language, and we don't
    have a corresponding AC_CXX_RESTRICT to figure out what the C++
@@ -630,7 +511,7 @@ static inline uint16_t ntohs(uint16_t netvar) { return netvar; }
    want to protect others from the autoconf/automake-generated
    PACKAGE_<foo> macros in opal_config.h.  We can't put these undef's
    directly in opal_config.h because they'll be turned into #defines'
-   via autoconf.  
+   via autoconf.
 
    So put them here in case any only else includes OMPI/ORTE/OPAL's
    config.h files. */
