@@ -2,7 +2,11 @@
 /*
  * Copyright (c) 2013-2017 Los Alamos National Security, LLC. All rights
  *                         reserved.
- * Copyright (c) 2015 Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2015      Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2015      Bull SAS.  All rights reserved.
+ * Copyright (c) 2015      The University of Tennessee and The University
+ *                         of Tennessee Research Foundation.  All rights
+ *                         reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -298,6 +302,7 @@ int mca_base_pvar_register (const char *project, const char *framework, const ch
                 }
             }
 
+            pvar->pvar_index = pvar_count;
             opal_hash_table_set_value_ptr (&mca_base_pvar_index_hash, pvar->name, strlen (pvar->name),
                                            (void *)(uintptr_t) pvar->pvar_index);
 
@@ -332,7 +337,6 @@ int mca_base_pvar_register (const char *project, const char *framework, const ch
     }
 
     pvar->ctx        = ctx;
-    pvar->pvar_index = pvar_count;
 
     return pvar->pvar_index;
 }
@@ -705,7 +709,8 @@ int mca_base_pvar_handle_write_value (mca_base_pvar_handle_t *handle, const void
         return OPAL_ERR_PERM;
     }
 
-    /* TODO -- actually write the variable. this will likely require a pvar lock */
+    /* write the value directly from the variable. */
+    ret = handle->pvar->set_value (handle->pvar, value, handle->obj_handle);
 
     ret = mca_base_pvar_handle_update (handle);
     if (OPAL_SUCCESS != ret) {
