@@ -150,7 +150,6 @@ mca_btl_tcp_endpoint_dump(int level,
     opal_socklen_t obtlen;
     opal_socklen_t addrlen = sizeof(inaddr);
     mca_btl_tcp_frag_t* item;
-    mca_btl_tcp_proc_t* this_proc = mca_btl_tcp_proc_local();
 
     used += snprintf(&outmsg[used], DEBUG_LENGTH - used, "%s: ", msg);
     if (used >= DEBUG_LENGTH) goto out;
@@ -251,7 +250,7 @@ mca_btl_tcp_endpoint_dump(int level,
         if (used >= DEBUG_LENGTH) goto out;
 #if MCA_BTL_TCP_ENDPOINT_CACHE
         used += snprintf(&outmsg[used], DEBUG_LENGTH - used, "\n\t[cache %p used %lu/%lu]",
-                         btl_endpoint->endpoint_cache, btl_endpoint->endpoint_cache_pos - btl_endpoint->endpoint_cache,
+                         (void*)btl_endpoint->endpoint_cache, btl_endpoint->endpoint_cache_pos - btl_endpoint->endpoint_cache,
                          btl_endpoint->endpoint_cache_length);
         if (used >= DEBUG_LENGTH) goto out;
 #endif  /* MCA_BTL_TCP_ENDPOINT_CACHE */
@@ -280,7 +279,7 @@ out:
     opal_output_verbose(level, opal_btl_base_framework.framework_output,
                         "[%s:%d:%s][%s -> %s] %s",
                         fname, lineno, funcname,
-                        (NULL != this_proc ? OPAL_NAME_PRINT(mca_btl_tcp_proc_local()->proc_opal->proc_name) : "unknown"),
+                        OPAL_NAME_PRINT(opal_proc_local_get()->proc_name),
                         (NULL != btl_endpoint->endpoint_proc ? OPAL_NAME_PRINT(btl_endpoint->endpoint_proc->proc_opal->proc_name) : "unknown remote"),
                         outmsg);
 }
@@ -300,18 +299,17 @@ static inline void mca_btl_tcp_endpoint_event_init(mca_btl_base_endpoint_t* btl_
 
     opal_event_set(mca_btl_tcp_event_base, &btl_endpoint->endpoint_recv_event,
                     btl_endpoint->endpoint_sd,
-                    OPAL_EV_READ|OPAL_EV_PERSIST,
+                    OPAL_EV_READ | OPAL_EV_PERSIST,
                     mca_btl_tcp_endpoint_recv_handler,
                     btl_endpoint );
     /**
-     * The send event should be non persistent until the endpoint is
-     * completely connected. This means, when the event is created it
-     * will be fired only once, and when the endpoint is marked as
-     * CONNECTED the event should be recreated with the correct flags.
+     * In the multi-threaded case, the send event must be persistent in order
+     * to avoid missing the connection notification in send_handler due to
+     * a local handling of the peer process (which holds the lock).
      */
     opal_event_set(mca_btl_tcp_event_base, &btl_endpoint->endpoint_send_event,
                     btl_endpoint->endpoint_sd,
-                    OPAL_EV_WRITE,
+                    OPAL_EV_WRITE | OPAL_EV_PERSIST,
                     mca_btl_tcp_endpoint_send_handler,
                     btl_endpoint);
 }
@@ -409,8 +407,7 @@ mca_btl_tcp_endpoint_send_blocking(mca_btl_base_endpoint_t* btl_endpoint,
 static int mca_btl_tcp_endpoint_send_connect_ack(mca_btl_base_endpoint_t* btl_endpoint)
 {
     /* send process identifier to remote endpoint */
-    mca_btl_tcp_proc_t* btl_proc = mca_btl_tcp_proc_local();
-    opal_process_name_t guid = btl_proc->proc_opal->proc_name;
+    opal_process_name_t guid = opal_proc_local_get()->proc_name;
 
     OPAL_PROCESS_NAME_HTON(guid);
     if(mca_btl_tcp_endpoint_send_blocking(btl_endpoint, &guid, sizeof(guid)) !=
@@ -423,7 +420,6 @@ static int mca_btl_tcp_endpoint_send_connect_ack(mca_btl_base_endpoint_t* btl_en
 static void *mca_btl_tcp_endpoint_complete_accept(int fd, int flags, void *context)
 {
     mca_btl_base_endpoint_t* btl_endpoint = (mca_btl_base_endpoint_t*)context;
-    mca_btl_tcp_proc_t* this_proc = mca_btl_tcp_proc_local();
     struct timeval now = {0, 0};
     int cmpval;
 
@@ -452,7 +448,7 @@ static void *mca_btl_tcp_endpoint_complete_accept(int fd, int flags, void *conte
     }
 
     cmpval = opal_compare_proc(btl_endpoint->endpoint_proc->proc_opal->proc_name,
-                               this_proc->proc_opal->proc_name);
+                               opal_proc_local_get()->proc_name);
     if((btl_endpoint->endpoint_sd < 0) ||
        (btl_endpoint->endpoint_state != MCA_BTL_TCP_CONNECTED &&
         cmpval < 0)) {
@@ -511,6 +507,7 @@ void mca_btl_tcp_endpoint_accept(mca_btl_base_endpoint_t* btl_endpoint,
  */
 void mca_btl_tcp_endpoint_close(mca_btl_base_endpoint_t* btl_endpoint)
 {
+    MCA_BTL_TCP_ENDPOINT_DUMP(1, btl_endpoint, false, "[close]");
     if(btl_endpoint->endpoint_sd < 0)
         return;
     btl_endpoint->endpoint_retries++;
@@ -518,14 +515,16 @@ void mca_btl_tcp_endpoint_close(mca_btl_base_endpoint_t* btl_endpoint)
     opal_event_del(&btl_endpoint->endpoint_recv_event);
     MCA_BTL_TCP_ENDPOINT_DUMP(1, btl_endpoint, false, "event_del(send) [close]");
     opal_event_del(&btl_endpoint->endpoint_send_event);
-    CLOSE_THE_SOCKET(btl_endpoint->endpoint_sd);
-    btl_endpoint->endpoint_sd = -1;
+
 #if MCA_BTL_TCP_ENDPOINT_CACHE
     free( btl_endpoint->endpoint_cache );
     btl_endpoint->endpoint_cache        = NULL;
     btl_endpoint->endpoint_cache_pos    = NULL;
     btl_endpoint->endpoint_cache_length = 0;
 #endif  /* MCA_BTL_TCP_ENDPOINT_CACHE */
+
+    CLOSE_THE_SOCKET(btl_endpoint->endpoint_sd);
+    btl_endpoint->endpoint_sd = -1;
     /**
      * If we keep failing to connect to the peer let the caller know about
      * this situation by triggering all the pending fragments callback and
@@ -558,13 +557,6 @@ static void mca_btl_tcp_endpoint_connected(mca_btl_base_endpoint_t* btl_endpoint
     btl_endpoint->endpoint_retries = 0;
     MCA_BTL_TCP_ENDPOINT_DUMP(1, btl_endpoint, true, "READY [endpoint_connected]");
 
-    /* Create the send event in a persistent manner. */
-    opal_event_set(mca_btl_tcp_event_base, &btl_endpoint->endpoint_send_event,
-                    btl_endpoint->endpoint_sd,
-                    OPAL_EV_WRITE | OPAL_EV_PERSIST,
-                    mca_btl_tcp_endpoint_send_handler,
-                    btl_endpoint );
-
     if(opal_list_get_size(&btl_endpoint->endpoint_frags) > 0) {
         if(NULL == btl_endpoint->endpoint_send_frag)
             btl_endpoint->endpoint_send_frag = (mca_btl_tcp_frag_t*)
@@ -683,9 +675,9 @@ void mca_btl_tcp_set_socket_options(int sd)
 /*
  *  Start a connection to the endpoint. This will likely not complete,
  *  as the socket is set to non-blocking, so register for event
- *  notification of connect completion. On connection we send
- *  our globally unique process identifier to the endpoint and wait for
- *  the endpoints response.
+ *  notification of connect completion. On connection we send our
+ *  globally unique process identifier to the endpoint and wait for
+ *  the endpoint response.
  */
 static int mca_btl_tcp_endpoint_start_connect(mca_btl_base_endpoint_t* btl_endpoint)
 {
@@ -779,6 +771,10 @@ static void mca_btl_tcp_endpoint_complete_connect(mca_btl_base_endpoint_t* btl_e
     opal_socklen_t so_length = sizeof(so_error);
     struct sockaddr_storage endpoint_addr;
 
+    /* Delete the send event notification, as the next step is waiting for the ack
+     * from the peer. Once this ack is received we will deal with the send notification
+     * accordingly.
+     */
     opal_event_del(&btl_endpoint->endpoint_send_event);
 
     mca_btl_tcp_proc_tosocks(btl_endpoint->endpoint_addr, &endpoint_addr);
@@ -802,12 +798,7 @@ static void mca_btl_tcp_endpoint_complete_connect(mca_btl_base_endpoint_t* btl_e
         return;
     }
 
-    /* Do not unregister from receiving send event notifications, instead
-     * leave the event to trigger once more, and then it will get automatically
-     * deleted as no send fragments are available.
-     */
-
-     if(mca_btl_tcp_endpoint_send_connect_ack(btl_endpoint) == OPAL_SUCCESS) {
+    if(mca_btl_tcp_endpoint_send_connect_ack(btl_endpoint) == OPAL_SUCCESS) {
         btl_endpoint->endpoint_state = MCA_BTL_TCP_CONNECT_ACK;
         opal_event_add(&btl_endpoint->endpoint_recv_event, 0);
         MCA_BTL_TCP_ENDPOINT_DUMP(10, btl_endpoint, false, "event_add(recv) [complete_connect]");
