@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2011-2018 Cisco Systems, Inc.  All rights reserved
+ * Copyright (c) 2011-2015 Cisco Systems, Inc.  All rights reserved.
  * Copyright (c) 2013      Los Alamos National Security, LLC.  All rights reserved.
  * Copyright (c) 2015      Intel, Inc. All rights reserved.
- * Copyright (c) 2015      Research Organization for Information Science
+ * Copyright (c) 2015-2017 Research Organization for Information Science
  *                         and Technology (RIST). All rights reserved.
  * Copyright (c) 2017      IBM Corporation.  All rights reserved.
  *
@@ -22,10 +22,6 @@
 
 #include "opal_config.h"
 
-#if defined(__CYGWIN__) && defined(WIN32)
-#undef WIN32
-#endif
-
 #include "event.h"
 #include "event2/event.h"
 #include "event2/thread.h"
@@ -56,6 +52,8 @@ OPAL_DECLSPEC opal_event_base_t* opal_event_base_create(void);
 
 OPAL_DECLSPEC int opal_event_init(void);
 
+OPAL_DECLSPEC int opal_event_finalize(void);
+
 #define opal_event_reinit(b) event_reinit((b))
 
 #define opal_event_base_init_common_timeout (b, t) event_base_init_common_timeout((b), (t))
