@@ -1,12 +1,10 @@
 /*
- * Copyright (c) 2011-2012 Cisco Systems, Inc.  All rights reserved.
- * Copyright (c) 2013-2014 Intel, Inc. All rights reserved.
- * Copyright (c) 2014      Research Organization for Information Science
- *                         and Technology (RIST). All rights reserved.
+ * Copyright (c) 2011-2017 Cisco Systems, Inc.  All rights reserved
+ * Copyright (c) 2013-2017 Intel, Inc.  All rights reserved.
  * $COPYRIGHT$
- * 
+ *
  * Additional copyrights may follow
- * 
+ *
  * $HEADER$
  */
 
@@ -17,7 +15,7 @@
 
 #include "opal/dss/dss_types.h"
 
-#include "opal/mca/hwloc/hwloc.h"
+#include "opal/mca/hwloc/hwloc-internal.h"
 
 /*
  * Global functions for MCA overall hwloc open and close
@@ -59,9 +57,7 @@ opal_hwloc_print_buffers_t *opal_hwloc_get_print_buffer(void);
 extern char* opal_hwloc_print_null;
 OPAL_DECLSPEC char* opal_hwloc_base_print_locality(opal_hwloc_locality_t locality);
 
-#if OPAL_HAVE_HWLOC
-OPAL_DECLSPEC extern char *opal_hwloc_base_slot_list;
-OPAL_DECLSPEC extern char *opal_hwloc_base_cpu_set;
+OPAL_DECLSPEC extern char *opal_hwloc_base_cpu_list;
 OPAL_DECLSPEC extern hwloc_cpuset_t opal_hwloc_base_given_cpus;
 OPAL_DECLSPEC extern char *opal_hwloc_base_topo_file;
 
@@ -92,20 +88,20 @@ OPAL_DECLSPEC int opal_hwloc_base_set_binding_policy(opal_binding_policy_t *poli
 
 /**
  * Loads opal_hwloc_my_cpuset (global variable in
- * opal/mca/hwloc/hwloc.h) for this process.  opal_hwloc_my_cpuset
+ * opal/mca/hwloc/hwloc-internal.h) for this process.  opal_hwloc_my_cpuset
  * will be loaded with this process' binding, or, if the process is
  * not bound, use the hwloc root object's (available and online)
  * cpuset.
  */
 OPAL_DECLSPEC void opal_hwloc_base_get_local_cpuset(void);
 
-struct orte_rmaps_numa_node_t {
+struct opal_rmaps_numa_node_t {
     opal_list_item_t super;
     int index;
     float dist_from_closed;
 };
-typedef struct orte_rmaps_numa_node_t orte_rmaps_numa_node_t;
-OBJ_CLASS_DECLARATION(orte_rmaps_numa_node_t);
+typedef struct opal_rmaps_numa_node_t opal_rmaps_numa_node_t;
+OBJ_CLASS_DECLARATION(opal_rmaps_numa_node_t);
 
 /**
  * Enum for what memory allocation policy we want for user allocations.
@@ -175,8 +171,8 @@ OPAL_DECLSPEC unsigned int opal_hwloc_base_get_obj_idx(hwloc_topology_t topo,
                                                        hwloc_obj_t obj,
                                                        opal_hwloc_resource_type_t rtype);
 
-OPAL_DECLSPEC int opal_hwloc_get_sorted_numa_list(hwloc_topology_t topo, 
-                                    char* device_name, 
+OPAL_DECLSPEC int opal_hwloc_get_sorted_numa_list(hwloc_topology_t topo,
+                                    char* device_name,
                                     opal_list_t *sorted_list);
 
 /**
@@ -195,7 +191,7 @@ OPAL_DECLSPEC bool opal_hwloc_base_single_cpu(hwloc_cpuset_t cpuset);
  * Provide a utility to parse a slot list against the local
  * cpus of given type, and produce a cpuset for the described binding
  */
-OPAL_DECLSPEC int opal_hwloc_base_slot_list_parse(const char *slot_str,
+OPAL_DECLSPEC int opal_hwloc_base_cpu_list_parse(const char *slot_str,
                                                   hwloc_topology_t topo,
                                                   opal_hwloc_resource_type_t rtype,
                                                   hwloc_cpuset_t cpumask);
@@ -211,7 +207,7 @@ OPAL_DECLSPEC char* opal_hwloc_base_check_on_coprocessor(void);
  */
 OPAL_DECLSPEC int opal_hwloc_base_report_bind_failure(const char *file,
                                                       int line,
-                                                      const char *msg, 
+                                                      const char *msg,
                                                       int rc);
 
 /**
@@ -258,7 +254,7 @@ OPAL_DECLSPEC int opal_hwloc_base_cset2str(char *str, int len,
                                            hwloc_cpuset_t cpuset);
 
 /**
- * Make a prettyprint string for a cset in a map format.  
+ * Make a prettyprint string for a cset in a map format.
  * Example: [B./..]
  * Key:  [] - signifies socket
  *        / - divider between cores
@@ -278,7 +274,16 @@ OPAL_DECLSPEC hwloc_obj_t opal_hwloc_base_get_pu(hwloc_topology_t topo,
  * if responsible for freeing the returned string */
 OPAL_DECLSPEC char* opal_hwloc_base_get_topo_signature(hwloc_topology_t topo);
 
-#endif
+
+/* get a string describing the locality of a given process */
+OPAL_DECLSPEC char* opal_hwloc_base_get_locality_string(hwloc_topology_t topo, char *bitmap);
+
+/* extract a location from the locality string */
+OPAL_DECLSPEC char* opal_hwloc_base_get_location(char *locality,
+                                                 hwloc_obj_type_t type,
+                                                 unsigned index);
+
+OPAL_DECLSPEC opal_hwloc_locality_t opal_hwloc_compute_relative_locality(char *loc1, char *loc2);
 
 END_C_DECLS
 
