@@ -12,7 +12,9 @@
  * Copyright (c) 2008-2015 Cisco Systems, Inc.  All rights reserved.
  * Copyright (c) 2010-2015 Los Alamos National Security, LLC.
  *                         All rights reserved.
- * Copyright (c) 2013-2015 Intel, Inc. All rights reserved
+ * Copyright (c) 2013-2017 Intel, Inc. All rights reserved.
+ * Copyright (c) 2016-2017 Research Organization for Information Science
+ *                         and Technology (RIST). All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -29,6 +31,7 @@
 #include "opal/util/output.h"
 #include "opal/util/malloc.h"
 #include "opal/util/net.h"
+#include "opal/util/proc.h"
 #include "opal/util/keyval_parse.h"
 #include "opal/util/show_help.h"
 #include "opal/memoryhooks/memory.h"
@@ -40,30 +43,24 @@
 #include "opal/mca/installdirs/base/base.h"
 #include "opal/mca/memchecker/base/base.h"
 #include "opal/mca/memcpy/base/base.h"
-#include "opal/mca/memory/base/base.h"
 #include "opal/mca/backtrace/base/base.h"
-#include "opal/mca/sec/base/base.h"
 #include "opal/mca/timer/base/base.h"
 #include "opal/mca/hwloc/base/base.h"
 #include "opal/mca/event/base/base.h"
 #include "opal/runtime/opal_progress.h"
 #include "opal/mca/shmem/base/base.h"
+#if OPAL_ENABLE_FT_CR    == 1
+#include "opal/mca/compress/base/base.h"
+#endif
+
+#include "opal/runtime/opal_cr.h"
+#include "opal/mca/crs/base/base.h"
+#include "opal/threads/tsd.h"
 
 extern int opal_initialized;
 extern int opal_util_initialized;
 extern bool opal_init_called;
 
-static void __opal_attribute_destructor__ opal_cleanup (void)
-{
-    if (!opal_initialized) {
-        /* nothing to do */
-        return;
-    }
-
-    /* finalize the class/object system */
-    opal_class_finalize();
-}
-
 int
 opal_finalize_util(void)
 {
@@ -90,6 +87,8 @@ opal_finalize_util(void)
 
     (void) mca_base_framework_close(&opal_installdirs_base_framework);
 
+    mca_base_close();
+
     /* finalize the memory allocator */
     opal_malloc_finalize();
 
@@ -107,9 +106,8 @@ opal_finalize_util(void)
 
     opal_datatype_finalize();
 
-#if OPAL_NO_LIB_DESTRUCTOR
-    opal_cleanup ();
-#endif
+    /* finalize the class/object system */
+    opal_class_finalize();
 
     free (opal_process_info.nodename);
     opal_process_info.nodename = NULL;
@@ -130,8 +128,12 @@ opal_finalize(void)
 
     opal_progress_finalize();
 
-    /* close the security framework */
-    (void) mca_base_framework_close(&opal_sec_base_framework);
+    /* close the checkpoint and restart service */
+    opal_cr_finalize();
+
+#if OPAL_ENABLE_FT_CR    == 1
+    (void) mca_base_framework_close(&opal_compress_base_framework);
+#endif
 
     (void) mca_base_framework_close(&opal_event_base_framework);
 
@@ -141,13 +143,6 @@ opal_finalize(void)
     (void) mca_base_framework_close(&opal_backtrace_base_framework);
     (void) mca_base_framework_close(&opal_memchecker_base_framework);
 
-    /* close the memory manager components.  Registered hooks can
-       still be fired any time between now and the call to
-       opal_mem_free_finalize(), and callbacks from the memory manager
-       hooks to the bowels of the mem_free code can still occur any
-       time between now and end of application (even post main()!) */
-    (void) mca_base_framework_close(&opal_memory_base_framework);
-
     /* close the memcpy framework */
     (void) mca_base_framework_close(&opal_memcpy_base_framework);
 
@@ -160,8 +155,8 @@ opal_finalize(void)
     /* close the shmem framework */
     (void) mca_base_framework_close(&opal_shmem_base_framework);
 
-    /* close the sec framework */
-    (void) mca_base_framework_close(&opal_sec_base_framework);
+    /* cleanup the main thread specific stuff */
+    opal_tsd_keys_destruct();
 
     /* finalize util code */
     opal_finalize_util();
