@@ -6,18 +6,19 @@
  * Copyright (c) 2004-2011 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
- * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
+ * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart,
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
- * Copyright (c) 2006-2012 Cisco Systems, Inc.  All rights reserved.
- * Copyright (c) 2010-2013 Los Alamos National Security, LLC.
- *                         All rights reserved.
+ * Copyright (c) 2006-2015 Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2010-2016 Los Alamos National Security, LLC. All rights
+ *                         reserved.
  * Copyright (c) 2011-2012 University of Houston. All rights reserved.
+ * Copyright (c) 2016      Intel, Inc.  All rights reserved.
  * $COPYRIGHT$
- * 
+ *
  * Additional copyrights may follow
- * 
+ *
  * $HEADER$
  */
 
@@ -88,7 +89,7 @@ const char *opal_info_ver_mca = "mca";
 const char *opal_info_ver_type = "type";
 const char *opal_info_ver_component = "component";
 
-static bool opal_info_registered = false;
+static int opal_info_registered = 0;
 
 static void component_map_construct(opal_info_component_map_t *map)
 {
@@ -118,37 +119,39 @@ int opal_info_init(int argc, char **argv,
 
     /* Initialize the argv parsing handle */
     if (OPAL_SUCCESS != (ret = opal_init_util(&argc, &argv))) {
-        opal_show_help("help-opal_info.txt", "lib-call-fail", true, 
+        opal_show_help("help-opal_info.txt", "lib-call-fail", true,
                        "opal_init_util", __FILE__, __LINE__, NULL);
         exit(ret);
     }
-    
+
     /* add the cmd line options */
-    opal_cmd_line_make_opt3(opal_info_cmd_line, 'V', NULL, "version", 0, 
+    opal_cmd_line_make_opt3(opal_info_cmd_line, 'V', NULL, "version", 0,
                             "Show version of Open MPI");
-    opal_cmd_line_make_opt3(opal_info_cmd_line, '\0', NULL, "param", 2, 
+    opal_cmd_line_make_opt3(opal_info_cmd_line, '\0', NULL, "param", 2,
                             "Show MCA parameters.  The first parameter is the framework (or the keyword \"all\"); the second parameter is the specific component name (or the keyword \"all\").");
-    opal_cmd_line_make_opt3(opal_info_cmd_line, '\0', NULL, "params", 2, 
+    opal_cmd_line_make_opt3(opal_info_cmd_line, '\0', NULL, "params", 2,
                             "Synonym for --param");
-    opal_cmd_line_make_opt3(opal_info_cmd_line, '\0', NULL, "internal", 0, 
+    opal_cmd_line_make_opt3(opal_info_cmd_line, '\0', NULL, "internal", 0,
                             "Show internal MCA parameters (not meant to be modified by users)");
-    opal_cmd_line_make_opt3(opal_info_cmd_line, '\0', NULL, "path", 1, 
+    opal_cmd_line_make_opt3(opal_info_cmd_line, '\0', NULL, "path", 1,
                             "Show paths that Open MPI was configured with.  Accepts the following parameters: prefix, bindir, libdir, incdir, mandir, pkglibdir, sysconfdir, all");
-    opal_cmd_line_make_opt3(opal_info_cmd_line, '\0', NULL, "arch", 0, 
+    opal_cmd_line_make_opt3(opal_info_cmd_line, '\0', NULL, "arch", 0,
                             "Show architecture Open MPI was compiled on");
-    opal_cmd_line_make_opt3(opal_info_cmd_line, 'c', NULL, "config", 0, 
+    opal_cmd_line_make_opt3(opal_info_cmd_line, 'c', NULL, "config", 0,
                             "Show configuration options");
-    opal_cmd_line_make_opt3(opal_info_cmd_line, 'h', NULL, "help", 0, 
+    opal_cmd_line_make_opt3(opal_info_cmd_line, 't', NULL, "type", 1,
+                            "Show internal MCA parameters with the type specified in parameter.");
+    opal_cmd_line_make_opt3(opal_info_cmd_line, 'h', NULL, "help", 0,
                             "Show this help message");
-    opal_cmd_line_make_opt3(opal_info_cmd_line, '\0', NULL, "pretty-print", 0, 
+    opal_cmd_line_make_opt3(opal_info_cmd_line, '\0', NULL, "pretty-print", 0,
                             "When used in conjunction with other parameters, the output is displayed in 'pretty-print' format (default)");
-    opal_cmd_line_make_opt3(opal_info_cmd_line, '\0', NULL, "parsable", 0, 
+    opal_cmd_line_make_opt3(opal_info_cmd_line, '\0', NULL, "parsable", 0,
                             "When used in conjunction with other parameters, the output is displayed in a machine-parsable format");
-    opal_cmd_line_make_opt3(opal_info_cmd_line, '\0', NULL, "parseable", 0, 
+    opal_cmd_line_make_opt3(opal_info_cmd_line, '\0', NULL, "parseable", 0,
                             "Synonym for --parsable");
-    opal_cmd_line_make_opt3(opal_info_cmd_line, '\0', NULL, "hostname", 0, 
+    opal_cmd_line_make_opt3(opal_info_cmd_line, '\0', NULL, "hostname", 0,
                             "Show the hostname that Open MPI was configured and built on");
-    opal_cmd_line_make_opt3(opal_info_cmd_line, 'a', NULL, "all", 0, 
+    opal_cmd_line_make_opt3(opal_info_cmd_line, 'a', NULL, "all", 0,
                             "Show all configuration options and MCA parameters");
     opal_cmd_line_make_opt3(opal_info_cmd_line, 'l', NULL, "level", 1,
                             "Show only variables with at most this level (1-9)");
@@ -157,7 +160,7 @@ int opal_info_init(int argc, char **argv,
 
     /* set our threading level */
     opal_set_using_threads(false);
-    
+
     /* Get MCA parameters, if any */
     if( OPAL_SUCCESS != mca_base_open() ) {
         opal_show_help("help-opal_info.txt", "lib-call-fail", true, "mca_base_open", __FILE__, __LINE__ );
@@ -165,14 +168,14 @@ int opal_info_init(int argc, char **argv,
         return OPAL_ERROR;
     }
     mca_base_cmd_line_setup(opal_info_cmd_line);
-    
+
     /* Initialize the opal_output system */
     if (!opal_output_init()) {
         return OPAL_ERROR;
     }
-    
+
     /* Do the parsing */
-    ret = opal_cmd_line_parse(opal_info_cmd_line, false, argc, argv);
+    ret = opal_cmd_line_parse(opal_info_cmd_line, false, false, argc, argv);
     if (OPAL_SUCCESS != ret) {
         cmd_error = true;
         if (OPAL_ERR_SILENT != ret) {
@@ -180,14 +183,14 @@ int opal_info_init(int argc, char **argv,
                     opal_strerror(ret));
         }
     }
-    if (!cmd_error && 
-        (opal_cmd_line_is_taken(opal_info_cmd_line, "help") || 
+    if (!cmd_error &&
+        (opal_cmd_line_is_taken(opal_info_cmd_line, "help") ||
          opal_cmd_line_is_taken(opal_info_cmd_line, "h"))) {
         char *str, *usage;
 
         want_help = true;
         usage = opal_cmd_line_get_usage_msg(opal_info_cmd_line);
-        str = opal_show_help_string("help-opal_info.txt", "usage", 
+        str = opal_show_help_string("help-opal_info.txt", "usage",
                                     true, usage);
         if (NULL != str) {
             printf("%s", str);
@@ -204,9 +207,9 @@ int opal_info_init(int argc, char **argv,
         opal_finalize_util();
         exit(cmd_error ? 1 : 0);
     }
-    
+
     mca_base_cmd_line_process_args(opal_info_cmd_line, &app_env, &global_env);
-    
+
 
     /* set the flags */
     if (opal_cmd_line_is_taken(opal_info_cmd_line, "pretty-print")) {
@@ -219,14 +222,12 @@ int opal_info_init(int argc, char **argv,
         /* register only selected components */
         opal_info_register_flags = MCA_BASE_REGISTER_DEFAULT;
     }
-    
+
     return OPAL_SUCCESS;
 }
 
 void opal_info_finalize(void)
 {
-    opal_info_close_components ();
-    mca_base_close();
     opal_finalize_util();
 }
 
@@ -293,12 +294,10 @@ int opal_info_register_framework_params(opal_pointer_array_t *component_map)
 {
     int rc;
 
-    if (opal_info_registered) {
+    if (opal_info_registered++) {
         return OPAL_SUCCESS;
     }
 
-    opal_info_registered = true;
-
     /* Register mca/base parameters */
     if( OPAL_SUCCESS != mca_base_open() ) {
         opal_show_help("help-opal_info.txt", "lib-call-fail", true, "mca_base_open", __FILE__, __LINE__ );
@@ -319,19 +318,27 @@ void opal_info_close_components(void)
 {
     int i;
 
+    assert(opal_info_registered);
+    if (--opal_info_registered) {
+        return;
+    }
+
     for (i=0; NULL != opal_frameworks[i]; i++) {
         (void) mca_base_framework_close(opal_frameworks[i]);
     }
+
+    /* release our reference to MCA */
+    mca_base_close ();
 }
 
 
 void opal_info_show_path(const char *type, const char *value)
 {
     char *pretty, *path;
-    
+
     pretty = strdup(type);
     pretty[0] = toupper(pretty[0]);
-    
+
     asprintf(&path, "path:%s", type);
     opal_info_out(pretty, path, value);
     free(pretty);
@@ -342,7 +349,7 @@ void opal_info_do_path(bool want_all, opal_cmd_line_t *cmd_line)
 {
     int i, count;
     char *scope;
-    
+
     /* Check bozo case */
     count = opal_cmd_line_get_ninsts(cmd_line, "path");
     for (i = 0; i < count; ++i) {
@@ -352,7 +359,7 @@ void opal_info_do_path(bool want_all, opal_cmd_line_t *cmd_line)
             break;
         }
     }
-    
+
     if (want_all) {
         opal_info_show_path(opal_info_path_prefix, opal_install_dirs.prefix);
         opal_info_show_path(opal_info_path_exec_prefix, opal_install_dirs.exec_prefix);
@@ -361,7 +368,7 @@ void opal_info_do_path(bool want_all, opal_cmd_line_t *cmd_line)
         opal_info_show_path(opal_info_path_libdir, opal_install_dirs.libdir);
         opal_info_show_path(opal_info_path_incdir, opal_install_dirs.includedir);
         opal_info_show_path(opal_info_path_mandir, opal_install_dirs.mandir);
-        opal_info_show_path(opal_info_path_pkglibdir, opal_install_dirs.ompilibdir);
+        opal_info_show_path(opal_info_path_pkglibdir, opal_install_dirs.opallibdir);
         opal_info_show_path(opal_info_path_libexecdir, opal_install_dirs.libexecdir);
         opal_info_show_path(opal_info_path_datarootdir, opal_install_dirs.datarootdir);
         opal_info_show_path(opal_info_path_datadir, opal_install_dirs.datadir);
@@ -369,14 +376,14 @@ void opal_info_do_path(bool want_all, opal_cmd_line_t *cmd_line)
         opal_info_show_path(opal_info_path_sharedstatedir, opal_install_dirs.sharedstatedir);
         opal_info_show_path(opal_info_path_localstatedir, opal_install_dirs.localstatedir);
         opal_info_show_path(opal_info_path_infodir, opal_install_dirs.infodir);
-        opal_info_show_path(opal_info_path_pkgdatadir, opal_install_dirs.ompidatadir);
-        opal_info_show_path(opal_info_path_pkglibdir, opal_install_dirs.ompilibdir);
-        opal_info_show_path(opal_info_path_pkgincludedir, opal_install_dirs.ompiincludedir);
+        opal_info_show_path(opal_info_path_pkgdatadir, opal_install_dirs.opaldatadir);
+        opal_info_show_path(opal_info_path_pkglibdir, opal_install_dirs.opallibdir);
+        opal_info_show_path(opal_info_path_pkgincludedir, opal_install_dirs.opalincludedir);
     } else {
         count = opal_cmd_line_get_ninsts(cmd_line, "path");
         for (i = 0; i < count; ++i) {
             scope = opal_cmd_line_get_param(cmd_line, "path", i, 0);
-            
+
             if (0 == strcmp(opal_info_path_prefix, scope)) {
                 opal_info_show_path(opal_info_path_prefix, opal_install_dirs.prefix);
             } else if (0 == strcmp(opal_info_path_bindir, scope)) {
@@ -388,7 +395,7 @@ void opal_info_do_path(bool want_all, opal_cmd_line_t *cmd_line)
             } else if (0 == strcmp(opal_info_path_mandir, scope)) {
                 opal_info_show_path(opal_info_path_mandir, opal_install_dirs.mandir);
             } else if (0 == strcmp(opal_info_path_pkglibdir, scope)) {
-                opal_info_show_path(opal_info_path_pkglibdir, opal_install_dirs.ompilibdir);
+                opal_info_show_path(opal_info_path_pkglibdir, opal_install_dirs.opallibdir);
             } else if (0 == strcmp(opal_info_path_sysconfdir, scope)) {
                 opal_info_show_path(opal_info_path_sysconfdir, opal_install_dirs.sysconfdir);
             } else if (0 == strcmp(opal_info_path_exec_prefix, scope)) {
@@ -408,9 +415,9 @@ void opal_info_do_path(bool want_all, opal_cmd_line_t *cmd_line)
             } else if (0 == strcmp(opal_info_path_infodir, scope)) {
                 opal_info_show_path(opal_info_path_infodir, opal_install_dirs.infodir);
             } else if (0 == strcmp(opal_info_path_pkgdatadir, scope)) {
-                opal_info_show_path(opal_info_path_pkgdatadir, opal_install_dirs.ompidatadir);
+                opal_info_show_path(opal_info_path_pkgdatadir, opal_install_dirs.opaldatadir);
             } else if (0 == strcmp(opal_info_path_pkgincludedir, scope)) {
-                opal_info_show_path(opal_info_path_pkgincludedir, opal_install_dirs.ompiincludedir);
+                opal_info_show_path(opal_info_path_pkgincludedir, opal_install_dirs.opalincludedir);
             } else {
                 char *usage = opal_cmd_line_get_usage_msg(cmd_line);
                 opal_show_help("help-opal_info.txt", "usage", true, usage);
@@ -423,6 +430,7 @@ void opal_info_do_path(bool want_all, opal_cmd_line_t *cmd_line)
 
 void opal_info_do_params(bool want_all_in, bool want_internal,
                          opal_pointer_array_t *mca_types,
+                         opal_pointer_array_t *component_map,
                          opal_cmd_line_t *opal_info_cmd_line)
 {
     mca_base_var_info_lvl_t max_level = OPAL_INFO_LVL_1;
@@ -472,10 +480,13 @@ void opal_info_do_params(bool want_all_in, bool want_internal,
             }
         }
     }
-    
+
     /* Show the params */
-    
+
     if (want_all) {
+        opal_info_show_component_version(mca_types, component_map, opal_info_type_all,
+                                         opal_info_component_all, opal_info_ver_full,
+                                         opal_info_ver_all);
         for (i = 0; i < mca_types->size; ++i) {
             if (NULL == (type = (char *)opal_pointer_array_get_item(mca_types, i))) {
                 continue;
@@ -486,7 +497,7 @@ void opal_info_do_params(bool want_all_in, bool want_internal,
         for (i = 0; i < count; ++i) {
             type = opal_cmd_line_get_param(opal_info_cmd_line, p, (int)i, 0);
             component = opal_cmd_line_get_param(opal_info_cmd_line, p, (int)i, 1);
-            
+
             for (found = false, i = 0; i < mca_types->size; ++i) {
                 if (NULL == (str = (char *)opal_pointer_array_get_item(mca_types, i))) {
                     continue;
@@ -496,14 +507,17 @@ void opal_info_do_params(bool want_all_in, bool want_internal,
                     break;
                 }
             }
-            
+
             if (!found) {
                 char *usage = opal_cmd_line_get_usage_msg(opal_info_cmd_line);
                 opal_show_help("help-opal_info.txt", "not-found", true, type);
                 free(usage);
                 exit(1);
             }
-            
+
+            opal_info_show_component_version(mca_types, component_map, type,
+                                             component, opal_info_ver_full,
+                                             opal_info_ver_all);
             opal_info_show_mca_params(type, component, max_level, want_internal);
         }
     }
@@ -532,6 +546,62 @@ void opal_info_err_params(opal_pointer_array_t *component_map)
     return;
 }
 
+void opal_info_do_type(opal_cmd_line_t *opal_info_cmd_line)
+{
+    mca_base_var_info_lvl_t max_level = OPAL_INFO_LVL_1;
+    int count;
+    char *type, *str;
+    int i, j, k, len, ret;
+    char *p;
+    const mca_base_var_t *var;
+    char** strings, *message;
+    const mca_base_var_group_t *group;
+    p = "type";
+
+    if (NULL != (str = opal_cmd_line_get_param (opal_info_cmd_line, "level", 0, 0))) {
+        char *tmp;
+
+        errno = 0;
+        max_level = strtol (str, &tmp, 10) + OPAL_INFO_LVL_1 - 1;
+        if (0 != errno || '\0' != tmp[0] || max_level < OPAL_INFO_LVL_1 || max_level > OPAL_INFO_LVL_9) {
+            char *usage = opal_cmd_line_get_usage_msg(opal_info_cmd_line);
+            opal_show_help("help-opal_info.txt", "invalid-level", true, str);
+            free(usage);
+            exit(1);
+        }
+    }
+
+    count = opal_cmd_line_get_ninsts(opal_info_cmd_line, p);
+    len = mca_base_var_get_count ();
+
+    for (k = 0; k < count; ++k) {
+        type = opal_cmd_line_get_param(opal_info_cmd_line, p, k, 0);
+        for (i = 0; i < len; ++i) {
+            ret = mca_base_var_get (i, &var);
+            if (OPAL_SUCCESS != ret) {
+                continue;
+            }
+            if (0 == strcmp(type, var_type_names[var->mbv_type]) && (var->mbv_info_lvl <= max_level)) {
+                ret = mca_base_var_dump(var->mbv_index, &strings, !opal_info_pretty ? MCA_BASE_VAR_DUMP_PARSABLE : MCA_BASE_VAR_DUMP_READABLE);
+                if (OPAL_SUCCESS != ret) {
+                    continue;
+                }
+                (void) mca_base_var_group_get(var->mbv_group_index, &group);
+                for (j = 0 ; strings[j] ; ++j) {
+                    if (0 == j && opal_info_pretty) {
+                        asprintf (&message, "MCA %s", group->group_framework);
+                        opal_info_out(message, message, strings[j]);
+                        free(message);
+                    } else {
+                        opal_info_out("", "", strings[j]);
+                    }
+                    free(strings[j]);
+                }
+                free(strings);
+            }
+        }
+    }
+}
 
 static void opal_info_show_mca_group_params(const mca_base_var_group_t *group, mca_base_var_info_lvl_t max_level, bool want_internal)
 {
@@ -557,16 +627,16 @@ static void opal_info_show_mca_group_params(const mca_base_var_group_t *group, m
         /* read the selection parameter */
         var_id = mca_base_var_find (group->group_project, group->group_framework, NULL, NULL);
         if (0 <= var_id) {
-            const mca_base_var_storage_t *value;
+            const mca_base_var_storage_t *value=NULL;
             char **requested_components;
             bool include_mode;
 
             mca_base_var_get_value (var_id, &value, NULL, NULL);
-            if (NULL != value->stringval && '\0' != value->stringval[0]) {
+            if (NULL != value && NULL != value->stringval && '\0' != value->stringval[0]) {
                 mca_base_component_parse_requested (value->stringval, &include_mode, &requested_components);
 
                 for (i = 0, requested = !include_mode ; requested_components[i] ; ++i) {
-                    if (0 == strcmp (requested_components[i], group->group_component)) {
+                    if (0 == strcmp (requested_components[i], group_component)) {
                         requested = include_mode;
                         break;
                     }
@@ -577,6 +647,10 @@ static void opal_info_show_mca_group_params(const mca_base_var_group_t *group, m
         }
     }
 
+    const mca_base_var_group_t *curr_group = NULL;
+    char *component_msg = NULL;
+    asprintf(&component_msg, " %s", group_component);
+
     for (i = 0 ; i < count ; ++i) {
         ret = mca_base_var_get(variables[i], &var);
         if (OPAL_SUCCESS != ret || ((var->mbv_flags & MCA_BASE_VAR_FLAG_INTERNAL) &&
@@ -585,6 +659,15 @@ static void opal_info_show_mca_group_params(const mca_base_var_group_t *group, m
             continue;
         }
 
+        if (opal_info_pretty && curr_group != group) {
+            asprintf(&message, "MCA%s %s%s", requested ? "" : " (disabled)",
+                     group->group_framework,
+                     component_msg ? component_msg : "");
+            opal_info_out(message, message, "---------------------------------------------------");
+            free(message);
+            curr_group = group;
+        }
+
         ret = mca_base_var_dump(variables[i], &strings, !opal_info_pretty ? MCA_BASE_VAR_DUMP_PARSABLE : MCA_BASE_VAR_DUMP_READABLE);
         if (OPAL_SUCCESS != ret) {
             continue;
@@ -592,7 +675,9 @@ static void opal_info_show_mca_group_params(const mca_base_var_group_t *group, m
 
         for (j = 0 ; strings[j] ; ++j) {
             if (0 == j && opal_info_pretty) {
-                asprintf (&message, "MCA%s %s", requested ? "" : " (disabled)", group->group_framework);
+                asprintf (&message, "MCA%s %s%s", requested ? "" : " (disabled)",
+                          group->group_framework,
+                          component_msg ? component_msg : "");
                 opal_info_out(message, message, strings[j]);
                 free(message);
             } else {
@@ -620,6 +705,15 @@ static void opal_info_show_mca_group_params(const mca_base_var_group_t *group, m
             continue;
         }
 
+        if (opal_info_pretty && curr_group != group) {
+            asprintf(&message, "MCA%s %s%s", requested ? "" : " (disabled)",
+                     group->group_framework,
+                     component_msg ? component_msg : "");
+            opal_info_out(message, message, "---------------------------------------------------");
+            free(message);
+            curr_group = group;
+        }
+
         ret = mca_base_pvar_dump (variables[i], &strings, !opal_info_pretty ? MCA_BASE_VAR_DUMP_PARSABLE : MCA_BASE_VAR_DUMP_READABLE);
         if (OPAL_SUCCESS != ret) {
             continue;
@@ -627,7 +721,9 @@ static void opal_info_show_mca_group_params(const mca_base_var_group_t *group, m
 
         for (j = 0 ; strings[j] ; ++j) {
             if (0 == j && opal_info_pretty) {
-                asprintf (&message, "MCA%s %s", requested ? "" : " (disabled)", group->group_framework);
+                asprintf (&message, "MCA%s %s%s", requested ? "" : " (disabled)",
+                          group->group_framework,
+                          component_msg ? component_msg : "");
                 opal_info_out(message, message, strings[j]);
                 free(message);
             } else {
@@ -643,7 +739,7 @@ static void opal_info_show_mca_group_params(const mca_base_var_group_t *group, m
             opal_info_out("", "", message);
             free (message);
         }
-        free(strings);      
+        free(strings);
     }
 
     groups = OPAL_VALUE_ARRAY_GET_BASE(&group->group_subgroups, const int);
@@ -656,9 +752,10 @@ static void opal_info_show_mca_group_params(const mca_base_var_group_t *group, m
         }
         opal_info_show_mca_group_params(group, max_level, want_internal);
     }
+    free(component_msg);
 }
 
-void opal_info_show_mca_params(const char *type, const char *component, 
+void opal_info_show_mca_params(const char *type, const char *component,
                                mca_base_var_info_lvl_t max_level, bool want_internal)
 {
     const mca_base_var_group_t *group;
@@ -698,6 +795,43 @@ void opal_info_do_hostname()
 }
 
 
+static char *escape_quotes(const char *value)
+{
+    const char *src;
+    int num_quotes = 0;
+    for (src = value; src != NULL && *src != '\0'; ++src) {
+        if ('"' == *src) {
+            ++num_quotes;
+        }
+    }
+
+    // If there are no quotes in the string, there's nothing to do
+    if (0 == num_quotes) {
+        return NULL;
+    }
+
+    // If we have quotes, make a new string.  Copy over the old
+    // string, escaping the quotes along the way.  This is simple and
+    // clear to read; it's not particularly efficient (performance is
+    // definitely not important here).
+    char *quoted_value;
+    quoted_value = calloc(1, strlen(value) + num_quotes + 1);
+    if (NULL == quoted_value) {
+        return NULL;
+    }
+
+    char *dest;
+    for (src = value, dest = quoted_value; *src != '\0'; ++src, ++dest) {
+        if ('"' == *src) {
+            *dest++ = '\\';
+        }
+        *dest = *src;
+    }
+
+    return quoted_value;
+}
+
+
 /*
  * Private variables - set some reasonable screen size defaults
  */
@@ -705,8 +839,8 @@ void opal_info_do_hostname()
 static int centerpoint = 24;
 static int screen_width = 78;
 
-/* 
- * Prints the passed integer in a pretty or parsable format.
+/*
+ * Prints the passed message in a pretty or parsable format.
  */
 void opal_info_out(const char *pretty_message, const char *plain_message, const char *value)
 {
@@ -714,7 +848,7 @@ void opal_info_out(const char *pretty_message, const char *plain_message, const
     char *spaces = NULL;
     char *filler = NULL;
     char *pos, *v, savev, *v_to_free;
-    
+
 #ifdef HAVE_ISATTY
     /* If we have isatty(), if this is not a tty, then disable
      * wrapping for grep-friendly behavior
@@ -723,7 +857,7 @@ void opal_info_out(const char *pretty_message, const char *plain_message, const
         screen_width = INT_MAX;
     }
 #endif
-    
+
 #ifdef TIOCGWINSZ
     if (screen_width < INT_MAX) {
         struct winsize size;
@@ -733,6 +867,11 @@ void opal_info_out(const char *pretty_message, const char *plain_message, const
     }
 #endif
 
+    /* Sanity check (allow NULL to mean "") */
+    if (NULL == value) {
+        value = "";
+    }
+
     /* Strip leading and trailing whitespace from the string value */
     value_offset = strspn(value, " ");
 
@@ -746,13 +885,13 @@ void opal_info_out(const char *pretty_message, const char *plain_message, const
 
     if (opal_info_pretty && NULL != pretty_message) {
         if (centerpoint > (int)strlen(pretty_message)) {
-            asprintf(&spaces, "%*s", centerpoint - 
+            asprintf(&spaces, "%*s", centerpoint -
                      (int)strlen(pretty_message), " ");
         } else {
             spaces = strdup("");
 #if OPAL_ENABLE_DEBUG
             if (centerpoint < (int)strlen(pretty_message)) {
-                opal_show_help("help-opal_info.txt", 
+                opal_show_help("help-opal_info.txt",
                                "developer warning: field too long", false,
                                pretty_message, centerpoint);
             }
@@ -766,14 +905,14 @@ void opal_info_out(const char *pretty_message, const char *plain_message, const
         }
         free(spaces);
         spaces = NULL;
-        
+
         while (true) {
             if (strlen(v) < max_value_width) {
                 printf("%s%s\n", filler, v);
                 break;
             } else {
                 asprintf(&spaces, "%*s", centerpoint + 2, " ");
-                
+
                 /* Work backwards to find the first space before
                  * max_value_width
                  */
@@ -786,11 +925,11 @@ void opal_info_out(const char *pretty_message, const char *plain_message, const
                      * space after max_value_width.
                      */
                     pos = strchr(&v[max_value_width], ' ');
-                    
+
                     if (NULL == pos) {
-                        
+
                         /* There's just no spaces.  So just print it and be done. */
-                        
+
                         printf("%s%s\n", filler, v);
                         break;
                     } else {
@@ -803,7 +942,7 @@ void opal_info_out(const char *pretty_message, const char *plain_message, const
                     printf("%s%s\n", filler, v);
                     v = pos + 1;
                 }
-                
+
                 /* Reset for the next iteration */
                 free(filler);
                 filler = strdup(spaces);
@@ -819,7 +958,23 @@ void opal_info_out(const char *pretty_message, const char *plain_message, const
         }
     } else {
         if (NULL != plain_message && 0 < strlen(plain_message)) {
-            printf("%s:%s\n", plain_message, value);
+            // Escape any double quotes in the value.
+            char *quoted_value;
+            quoted_value = escape_quotes(value);
+            if (NULL != quoted_value) {
+                value = quoted_value;
+            }
+
+            char *colon = strchr(value, ':');
+            if (NULL != colon) {
+                printf("%s:\"%s\"\n", plain_message, value);
+            } else {
+                printf("%s:%s\n", plain_message, value);
+            }
+
+            if (NULL != quoted_value) {
+                free(quoted_value);
+            }
         } else {
             printf("%s\n", value);
         }
@@ -829,12 +984,15 @@ void opal_info_out(const char *pretty_message, const char *plain_message, const
     }
 }
 
-void opal_info_out_int(const char *pretty_message, 
-                       const char *plain_message, 
+/*
+ * Prints the passed integer in a pretty or parsable format.
+ */
+void opal_info_out_int(const char *pretty_message,
+                       const char *plain_message,
                        int value)
 {
     char *valstr;
-    
+
     asprintf(&valstr, "%d", (int)value);
     opal_info_out(pretty_message, plain_message, valstr);
     free(valstr);
@@ -846,7 +1004,7 @@ void opal_info_out_int(const char *pretty_message,
  */
 void opal_info_show_component_version(opal_pointer_array_t *mca_types,
                                       opal_pointer_array_t *component_map,
-                                      const char *type_name, 
+                                      const char *type_name,
                                       const char *component_name,
                                       const char *scope, const char *ver_type)
 {
@@ -857,7 +1015,7 @@ void opal_info_show_component_version(opal_pointer_array_t *mca_types,
     int j;
     char *pos;
     opal_info_component_map_t *map;
-    
+
     /* see if all components wanted */
     if (0 == strcmp(opal_info_component_all, component_name)) {
         want_all_components = true;
@@ -866,7 +1024,7 @@ void opal_info_show_component_version(opal_pointer_array_t *mca_types,
     /* see if all types wanted */
     if (0 != strcmp(opal_info_type_all, type_name)) {
         /* Check to see if the type is valid */
-    
+
         for (found = false, j = 0; j < mca_types->size; ++j) {
             if (NULL == (pos = (char*)opal_pointer_array_get_item(mca_types, j))) {
                 continue;
@@ -876,14 +1034,14 @@ void opal_info_show_component_version(opal_pointer_array_t *mca_types,
                 break;
             }
         }
-    
+
         if (!found) {
             return;
         }
     } else {
         want_all_types = true;
     }
-    
+
     /* Now that we have a valid type, find the right components */
     for (j=0; j < component_map->size; j++) {
         if (NULL == (map = (opal_info_component_map_t*)opal_pointer_array_get_item(component_map, j))) {
@@ -893,7 +1051,7 @@ void opal_info_show_component_version(opal_pointer_array_t *mca_types,
             /* found it! */
             OPAL_LIST_FOREACH(cli, map->components, mca_base_component_list_item_t) {
                 const mca_base_component_t *component = cli->cli_component;
-                if (want_all_components || 
+                if (want_all_components ||
                     0 == strcmp(component->mca_component_name, component_name)) {
                     opal_info_show_mca_version(component, scope, ver_type);
                 }
@@ -922,39 +1080,39 @@ void opal_info_show_mca_version(const mca_base_component_t* component,
     char *api_version;
     char *component_version;
     char *tmp;
-    
+
     if (0 == strcmp(ver_type, opal_info_ver_all) ||
         0 == strcmp(ver_type, opal_info_ver_mca)) {
         want_mca = true;
     }
-    
+
     if (0 == strcmp(ver_type, opal_info_ver_all) ||
         0 == strcmp(ver_type, opal_info_ver_type)) {
         want_type = true;
     }
-    
+
     if (0 == strcmp(ver_type, opal_info_ver_all) ||
         0 == strcmp(ver_type, opal_info_ver_component)) {
         want_component = true;
     }
-    
+
     mca_version = opal_info_make_version_str(scope, component->mca_major_version,
                                              component->mca_minor_version,
                                              component->mca_release_version, "",
-                                             false, "");
+                                             "");
     api_version = opal_info_make_version_str(scope, component->mca_type_major_version,
                                              component->mca_type_minor_version,
                                              component->mca_type_release_version, "",
-                                             false, "");
+                                             "");
     component_version = opal_info_make_version_str(scope, component->mca_component_major_version,
                                                    component->mca_component_minor_version,
-                                                   component->mca_component_release_version, 
-                                                   "", false, "");
+                                                   component->mca_component_release_version,
+                                                   "", "");
     if (opal_info_pretty) {
         asprintf(&message, "MCA %s", component->mca_type_name);
         printed = false;
         asprintf(&content, "%s (", component->mca_component_name);
-        
+
         if (want_mca) {
             asprintf(&tmp, "%sMCA v%s", content, mca_version);
             free(content);
@@ -991,13 +1149,13 @@ void opal_info_show_mca_version(const mca_base_component_t* component,
         } else {
             tmp = NULL;
         }
-        
+
         opal_info_out(message, NULL, tmp);
         free(message);
         if (NULL != tmp) {
             free(tmp);
         }
-        
+
     } else {
         asprintf(&message, "mca:%s:%s:version", component->mca_type_name, component->mca_component_name);
         if (want_mca) {
@@ -1032,33 +1190,22 @@ void opal_info_show_mca_version(const mca_base_component_t* component,
 
 char *opal_info_make_version_str(const char *scope,
                                  int major, int minor, int release,
-                                 const char *greek, 
-                                 bool want_repo_rev, const char *repo)
+                                 const char *greek,
+                                 const char *repo)
 {
     char *str = NULL, *tmp;
     char temp[BUFSIZ];
-    
+
     temp[BUFSIZ - 1] = '\0';
     if (0 == strcmp(scope, opal_info_ver_full) ||
         0 == strcmp(scope, opal_info_ver_all)) {
-        snprintf(temp, BUFSIZ - 1, "%d.%d", major, minor);
+        snprintf(temp, BUFSIZ - 1, "%d.%d.%d", major, minor, release);
         str = strdup(temp);
-        if (release > 0) {
-            snprintf(temp, BUFSIZ - 1, ".%d", release);
-            asprintf(&tmp, "%s%s", str, temp);
-            free(str);
-            str = tmp;
-        }
         if (NULL != greek) {
             asprintf(&tmp, "%s%s", str, greek);
             free(str);
             str = tmp;
         }
-        if (want_repo_rev && NULL != repo) {
-            asprintf(&tmp, "%s%s", str, repo);
-            free(str);
-            str = tmp;
-        }
     } else if (0 == strcmp(scope, opal_info_ver_major)) {
         snprintf(temp, BUFSIZ - 1, "%d", major);
     } else if (0 == strcmp(scope, opal_info_ver_minor)) {
@@ -1070,11 +1217,11 @@ char *opal_info_make_version_str(const char *scope,
     } else if (0 == strcmp(scope, opal_info_ver_repo)) {
         str = strdup(repo);
     }
-    
+
     if (NULL == str) {
         str = strdup(temp);
     }
-    
+
     return str;
 }
 
@@ -1083,11 +1230,11 @@ void opal_info_show_opal_version(const char *scope)
     char *tmp, *tmp2;
 
     asprintf(&tmp, "%s:version:full", opal_info_type_opal);
-    tmp2 = opal_info_make_version_str(scope, 
-                                      OPAL_MAJOR_VERSION, OPAL_MINOR_VERSION, 
-                                      OPAL_RELEASE_VERSION, 
+    tmp2 = opal_info_make_version_str(scope,
+                                      OPAL_MAJOR_VERSION, OPAL_MINOR_VERSION,
+                                      OPAL_RELEASE_VERSION,
                                       OPAL_GREEK_VERSION,
-                                      OPAL_WANT_REPO_REV, OPAL_REPO_REV);
+                                      OPAL_REPO_REV);
     opal_info_out("OPAL", tmp, tmp2);
     free(tmp);
     free(tmp2);
