@@ -1,3 +1,4 @@
+/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil -*- */
 /*
  * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
@@ -5,17 +6,19 @@
  * Copyright (c) 2004-2005 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
- * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
+ * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart,
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
- * Copyright (c) 2006-2013 Los Alamos National Security, LLC.  All rights
- *                         reserved. 
+ * Copyright (c) 2006-2016 Los Alamos National Security, LLC.  All rights
+ *                         reserved.
+ * Copyright (c) 2015-2016 Research Organization for Information Science
+ *                         and Technology (RIST). All rights reserved.
  *
  * $COPYRIGHT$
- * 
+ *
  * Additional copyrights may follow
- * 
+ *
  * $HEADER$
  */
 
@@ -34,13 +37,14 @@
 #include "opal/runtime/opal_params.h"
 
 #define OPAL_PROGRESS_USE_TIMERS (OPAL_TIMER_CYCLE_SUPPORTED || OPAL_TIMER_USEC_SUPPORTED)
+#define OPAL_PROGRESS_ONLY_USEC_NATIVE (OPAL_TIMER_USEC_NATIVE && !OPAL_TIMER_CYCLE_NATIVE)
 
 #if OPAL_ENABLE_DEBUG
 bool opal_progress_debug = false;
 #endif
 
-/* 
- * default parameters 
+/*
+ * default parameters
  */
 static int opal_progress_event_flag = OPAL_EVLOOP_ONCE | OPAL_EVLOOP_NONBLOCK;
 int opal_progress_spin_count = 10000;
@@ -52,12 +56,16 @@ int opal_progress_spin_count = 10000;
 static opal_atomic_lock_t progress_lock;
 
 /* callbacks to progress */
-static opal_progress_callback_t *callbacks = NULL;
+static volatile opal_progress_callback_t *callbacks = NULL;
 static size_t callbacks_len = 0;
 static size_t callbacks_size = 0;
 
+static volatile opal_progress_callback_t *callbacks_lp = NULL;
+static size_t callbacks_lp_len = 0;
+static size_t callbacks_lp_size = 0;
+
 /* do we want to call sched_yield() if nothing happened */
-static int call_yield = 1;
+bool opal_progress_yield_when_idle = false;
 
 #if OPAL_PROGRESS_USE_TIMERS
 static opal_timer_t event_progress_last_time = 0;
@@ -68,7 +76,7 @@ static int32_t event_progress_counter = 0;
 /* reset value for counter when it hits 0 */
 static int32_t event_progress_delta = 0;
 #endif
-/* users of the event library from MPI cause the tick rate to 
+/* users of the event library from MPI cause the tick rate to
    be every time */
 static int32_t num_event_users = 0;
 
@@ -86,6 +94,9 @@ static int debug_output = -1;
  */
 static int fake_cb(void) { return 0; }
 
+static int _opal_progress_unregister (opal_progress_callback_t cb, volatile opal_progress_callback_t *callback_array,
+                                      size_t *callback_array_len);
+
 /* init the progress engine - called from orte_init */
 int
 opal_progress_init(void)
@@ -102,10 +113,31 @@ opal_progress_init(void)
     }
 #endif
 
+    callbacks_size = callbacks_lp_size = 8;
+
+    callbacks = malloc (callbacks_size * sizeof (callbacks[0]));
+    callbacks_lp = malloc (callbacks_lp_size * sizeof (callbacks_lp[0]));
+
+    if (NULL == callbacks || NULL == callbacks_lp) {
+        free ((void *) callbacks);
+        free ((void *) callbacks_lp);
+        callbacks_size = callbacks_lp_size = 0;
+        callbacks = callbacks_lp = NULL;
+        return OPAL_ERR_OUT_OF_RESOURCE;
+    }
+
+    for (size_t i = 0 ; i < callbacks_size ; ++i) {
+        callbacks[i] = fake_cb;
+    }
+
+    for (size_t i = 0 ; i < callbacks_lp_size ; ++i) {
+        callbacks_lp[i] = fake_cb;
+    }
+
     OPAL_OUTPUT((debug_output, "progress: initialized event flag to: %x",
-                 opal_progress_event_flag));                 
+                 opal_progress_event_flag));
     OPAL_OUTPUT((debug_output, "progress: initialized yield_when_idle to: %s",
-                 call_yield == 0 ? "false" : "true"));
+                 opal_progress_yield_when_idle ? "true" : "false"));
     OPAL_OUTPUT((debug_output, "progress: initialized num users to: %d",
                  num_event_users));
     OPAL_OUTPUT((debug_output, "progress: initialized poll rate to: %ld",
@@ -123,10 +155,13 @@ opal_progress_finalize(void)
 
     callbacks_len = 0;
     callbacks_size = 0;
-    if (NULL != callbacks) {
-        free(callbacks);
-        callbacks = NULL;
-    }
+    free ((void *) callbacks);
+    callbacks = NULL;
+
+    callbacks_lp_len = 0;
+    callbacks_lp_size = 0;
+    free ((void *) callbacks_lp);
+    callbacks_lp = NULL;
 
     opal_atomic_unlock(&progress_lock);
 
@@ -135,7 +170,7 @@ opal_progress_finalize(void)
 
 
 /*
- * Progress the event library and any functions that have registered to 
+ * Progress the event library and any functions that have registered to
  * be called.  We don't propogate errors from the progress functions,
  * so no action is taken if they return failures.  The functions are
  * expected to return the number of events progressed, to determine
@@ -148,33 +183,34 @@ opal_progress_finalize(void)
 void
 opal_progress(void)
 {
+    static volatile uint32_t num_calls = 0;
     size_t i;
     int events = 0;
 
     if( opal_progress_event_flag != 0 ) {
 #if OPAL_HAVE_WORKING_EVENTOPS
 #if OPAL_PROGRESS_USE_TIMERS
-#if OPAL_TIMER_USEC_NATIVE
+#if OPAL_PROGRESS_ONLY_USEC_NATIVE
         opal_timer_t now = opal_timer_base_get_usec();
 #else
         opal_timer_t now = opal_timer_base_get_cycles();
-#endif  /* OPAL_TIMER_USEC_NATIVE */
+#endif  /* OPAL_PROGRESS_ONLY_USEC_NATIVE */
     /* trip the event library if we've reached our tick rate and we are
        enabled */
         if (now - event_progress_last_time > event_progress_delta ) {
-                event_progress_last_time = (num_event_users > 0) ? 
+                event_progress_last_time = (num_event_users > 0) ?
                     now - event_progress_delta : now;
 
-                events += opal_event_loop(opal_event_base, opal_progress_event_flag);
+                events += opal_event_loop(opal_sync_event_base, opal_progress_event_flag);
         }
 
 #else /* OPAL_PROGRESS_USE_TIMERS */
     /* trip the event library if we've reached our tick rate and we are
        enabled */
         if (OPAL_THREAD_ADD32(&event_progress_counter, -1) <= 0 ) {
-                event_progress_counter = 
+                event_progress_counter =
                     (num_event_users > 0) ? 0 : event_progress_delta;
-                events += opal_event_loop(opal_event_base, opal_progress_event_flag);
+                events += opal_event_loop(opal_sync_event_base, opal_progress_event_flag);
         }
 #endif /* OPAL_PROGRESS_USE_TIMERS */
 
@@ -186,8 +222,15 @@ opal_progress(void)
         events += (callbacks[i])();
     }
 
-#if defined(HAVE_SCHED_YIELD)
-    if (call_yield && events <= 0) {
+    if (callbacks_lp_len > 0 && (OPAL_THREAD_ADD32((volatile int32_t *) &num_calls, 1) & 0x7) == 0) {
+        /* run low priority callbacks once every 8 calls to opal_progress() */
+        for (i = 0 ; i < callbacks_lp_len ; ++i) {
+            events += (callbacks_lp[i])();
+        }
+    }
+
+#if OPAL_HAVE_SCHED_YIELD
+    if (opal_progress_yield_when_idle && events <= 0) {
         /* If there is nothing to do - yield the processor - otherwise
          * we could consume the processor for the entire time slice. If
          * the processor is oversubscribed - this will result in a best-case
@@ -214,10 +257,14 @@ opal_progress_set_event_flag(int flag)
 void
 opal_progress_event_users_increment(void)
 {
+#if OPAL_ENABLE_DEBUG
     int32_t val;
     val = opal_atomic_add_32(&num_event_users, 1);
 
     OPAL_OUTPUT((debug_output, "progress: event_users_increment setting count to %d", val));
+#else
+    (void)opal_atomic_add_32(&num_event_users, 1);
+#endif
 
 #if OPAL_PROGRESS_USE_TIMERS
     /* force an update next round (we'll be past the delta) */
@@ -232,10 +279,14 @@ opal_progress_event_users_increment(void)
 void
 opal_progress_event_users_decrement(void)
 {
+#if OPAL_ENABLE_DEBUG || ! OPAL_PROGRESS_USE_TIMERS
     int32_t val;
     val = opal_atomic_sub_32(&num_event_users, 1);
 
     OPAL_OUTPUT((debug_output, "progress: event_users_decrement setting count to %d", val));
+#else
+    (void)opal_atomic_sub_32(&num_event_users, 1);
+#endif
 
 #if !OPAL_PROGRESS_USE_TIMERS
    /* start now in delaying if it's easy */
@@ -249,11 +300,11 @@ opal_progress_event_users_decrement(void)
 bool
 opal_progress_set_yield_when_idle(bool yieldopt)
 {
-    bool tmp = (call_yield == 0) ? false : true;
-    call_yield = (yieldopt) ? 1 : 0;
+    bool tmp = opal_progress_yield_when_idle;
+    opal_progress_yield_when_idle = (yieldopt) ? 1 : 0;
 
     OPAL_OUTPUT((debug_output, "progress: progress_set_yield_when_idle to %s",
-                                    call_yield == 0 ? "false" : "true"));
+                                    opal_progress_yield_when_idle ? "true" : "false"));
 
     return tmp;
 }
@@ -266,7 +317,7 @@ opal_progress_set_event_poll_rate(int polltime)
 
 #if OPAL_PROGRESS_USE_TIMERS
     event_progress_delta = 0;
-#  if OPAL_TIMER_USEC_NATIVE
+#  if OPAL_PROGRESS_ONLY_USEC_NATIVE
     event_progress_last_time = opal_timer_base_get_usec();
 #  else
     event_progress_last_time = opal_timer_base_get_cycles();
@@ -293,77 +344,136 @@ opal_progress_set_event_poll_rate(int polltime)
 #endif
     }
 
-#if OPAL_PROGRESS_USE_TIMERS && !OPAL_TIMER_USEC_NATIVE
+#if OPAL_PROGRESS_USE_TIMERS && !OPAL_PROGRESS_ONLY_USEC_NATIVE
     /*  going to use cycles for counter.  Adjust specified usec into cycles */
     event_progress_delta = event_progress_delta * opal_timer_base_get_freq() / 1000000;
 #endif
 }
 
+static int opal_progress_find_cb (opal_progress_callback_t cb, volatile opal_progress_callback_t *cbs,
+                                     size_t cbs_len)
+{
+    for (size_t i = 0 ; i < cbs_len ; ++i) {
+        if (cbs[i] == cb) {
+            return (int) i;
+        }
+    }
 
-int
-opal_progress_register(opal_progress_callback_t cb)
+    return OPAL_ERR_NOT_FOUND;
+}
+
+static int _opal_progress_register (opal_progress_callback_t cb, volatile opal_progress_callback_t **cbs,
+                                    size_t *cbs_size, size_t *cbs_len)
 {
     int ret = OPAL_SUCCESS;
-    size_t index;
+
+    if (OPAL_ERR_NOT_FOUND != opal_progress_find_cb (cb, *cbs, *cbs_len)) {
+        return OPAL_SUCCESS;
+    }
+
+    /* see if we need to allocate more space */
+    if (*cbs_len + 1 > *cbs_size) {
+        opal_progress_callback_t *tmp, *old;
+
+        tmp = (opal_progress_callback_t *) malloc (sizeof (tmp[0]) * 2 * *cbs_size);
+        if (tmp == NULL) {
+            return OPAL_ERR_TEMP_OUT_OF_RESOURCE;
+        }
+
+        if (*cbs) {
+            /* copy old callbacks */
+            memcpy (tmp, (void *) *cbs, sizeof(tmp[0]) * *cbs_size);
+        }
+
+        for (size_t i = *cbs_len ; i < 2 * *cbs_size ; ++i) {
+            tmp[i] = fake_cb;
+        }
+
+        opal_atomic_wmb ();
+
+        /* swap out callback array */
+        old = opal_atomic_swap_ptr (cbs, tmp);
+
+        opal_atomic_wmb ();
+
+        free (old);
+        *cbs_size *= 2;
+    }
+
+    cbs[0][*cbs_len] = cb;
+    ++*cbs_len;
+
+    opal_atomic_wmb ();
+
+    return ret;
+}
+
+int opal_progress_register (opal_progress_callback_t cb)
+{
+    int ret;
 
     opal_atomic_lock(&progress_lock);
 
-    /* see if we need to allocate more space */
-    if (callbacks_len + 1 > callbacks_size) {
-        opal_progress_callback_t *tmp;
-        tmp = (opal_progress_callback_t*)realloc(callbacks, sizeof(opal_progress_callback_t) * (callbacks_size + 4));
-        if (tmp == NULL) {
-            ret = OPAL_ERR_TEMP_OUT_OF_RESOURCE;
-            goto cleanup;
-        }
-        /* registering fake callbacks to fill callbacks[] */
-        for( index = callbacks_len + 1 ;  index < callbacks_size + 4 ; index++) {
-            tmp[index] = &fake_cb;
-        }
+    (void) _opal_progress_unregister (cb, callbacks_lp, &callbacks_lp_len);
 
-        callbacks = tmp;
-        callbacks_size += 4;
-    }
-
-    callbacks[callbacks_len++] = cb;
-
- cleanup:
+    ret = _opal_progress_register (cb, &callbacks, &callbacks_size, &callbacks_len);
 
     opal_atomic_unlock(&progress_lock);
 
     return ret;
 }
 
-int
-opal_progress_unregister(opal_progress_callback_t cb)
+int opal_progress_register_lp (opal_progress_callback_t cb)
 {
-    size_t i;
-    int ret = OPAL_ERR_NOT_FOUND;
+    int ret;
 
     opal_atomic_lock(&progress_lock);
 
-    for (i = 0 ; i < callbacks_len ; ++i) {
-        if (cb == callbacks[i]) {
-            callbacks[i] = &fake_cb;
-            ret = OPAL_SUCCESS;
-            break;
-        }
+    (void) _opal_progress_unregister (cb, callbacks, &callbacks_len);
+
+    ret = _opal_progress_register (cb, &callbacks_lp, &callbacks_lp_size, &callbacks_lp_len);
+
+    opal_atomic_unlock(&progress_lock);
+
+    return ret;
+}
+
+static int _opal_progress_unregister (opal_progress_callback_t cb, volatile opal_progress_callback_t *callback_array,
+                                      size_t *callback_array_len)
+{
+    int ret = opal_progress_find_cb (cb, callback_array, *callback_array_len);
+    if (OPAL_ERR_NOT_FOUND == ret) {
+        return ret;
     }
-    
+
     /* If we found the function we're unregistering: If callbacks_len
        is 0, we're not goig to do anything interesting anyway, so
        skip.  If callbacks_len is 1, it will soon be 0, so no need to
-       do any repacking.  size_t can be unsigned, so 0 - 1 is bad for
-       a loop condition :). */
-    if (OPAL_SUCCESS == ret) {
-        if (callbacks_len > 1 ) {
-            /* now tightly pack the array */
-            for ( ; i < callbacks_len - 1 ; ++i) {
-                callbacks[i] = callbacks[i + 1];
-            }
-        }
-        callbacks[callbacks_len - 1] = &fake_cb;
-        callbacks_len--;
+       do any repacking. */
+    for (size_t i = (size_t) ret ; i < *callback_array_len - 1 ; ++i) {
+        /* copy callbacks atomically since another thread may be in
+         * opal_progress(). */
+        (void) opal_atomic_swap_ptr (callback_array + i, callback_array[i+1]);
+    }
+
+    callback_array[*callback_array_len] = fake_cb;
+    --*callback_array_len;
+
+    return OPAL_SUCCESS;
+}
+
+int opal_progress_unregister (opal_progress_callback_t cb)
+{
+    int ret;
+
+    opal_atomic_lock(&progress_lock);
+
+    ret = _opal_progress_unregister (cb, callbacks, &callbacks_len);
+
+    if (OPAL_SUCCESS != ret) {
+        /* if not in the high-priority array try to remove from the lp array.
+         * a callback will never be in both. */
+        ret = _opal_progress_unregister (cb, callbacks_lp, &callbacks_lp_len);
     }
 
     opal_atomic_unlock(&progress_lock);
