@@ -14,6 +14,8 @@
  * Copyright (c) 2010-2015 Cisco Systems, Inc.  All rights reserved.
  * Copyright (c) 2014      Los Alamos National Security, LLC. All rights
  *                         reserved.
+ * Copyright (c) 2015-2016 Research Organization for Information Science
+ *                         and Technology (RIST). All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -98,10 +100,6 @@ int opal_ifnametoaddr(const char* if_name, struct sockaddr* addr, int length)
 {
     opal_if_t* intf;
 
-    if (OPAL_SUCCESS != mca_base_framework_open(&opal_if_base_framework, 0)) {
-        return OPAL_ERROR;
-    }
-
     for (intf =  (opal_if_t*)opal_list_get_first(&opal_if_list);
         intf != (opal_if_t*)opal_list_get_end(&opal_if_list);
         intf =  (opal_if_t*)opal_list_get_next(intf)) {
@@ -123,10 +121,6 @@ int opal_ifnametoindex(const char* if_name)
 {
     opal_if_t* intf;
 
-    if (OPAL_SUCCESS != mca_base_framework_open(&opal_if_base_framework, 0)) {
-        return -1;
-    }
-
     for (intf =  (opal_if_t*)opal_list_get_first(&opal_if_list);
         intf != (opal_if_t*)opal_list_get_end(&opal_if_list);
         intf =  (opal_if_t*)opal_list_get_next(intf)) {
@@ -147,10 +141,6 @@ int16_t opal_ifnametokindex(const char* if_name)
 {
     opal_if_t* intf;
 
-    if (OPAL_SUCCESS != mca_base_framework_open(&opal_if_base_framework, 0)) {
-        return -1;
-    }
-
     for (intf =  (opal_if_t*)opal_list_get_first(&opal_if_list);
         intf != (opal_if_t*)opal_list_get_end(&opal_if_list);
         intf =  (opal_if_t*)opal_list_get_next(intf)) {
@@ -171,10 +161,6 @@ int opal_ifindextokindex(int if_index)
 {
     opal_if_t* intf;
 
-    if (OPAL_SUCCESS != mca_base_framework_open(&opal_if_base_framework, 0)) {
-        return -1;
-    }
-
     for (intf =  (opal_if_t*)opal_list_get_first(&opal_if_list);
         intf != (opal_if_t*)opal_list_get_end(&opal_if_list);
         intf =  (opal_if_t*)opal_list_get_next(intf)) {
@@ -205,10 +191,6 @@ int opal_ifaddrtoname(const char* if_addr, char* if_name, int length)
         return OPAL_ERR_NOT_FOUND;
     }
 
-    if (OPAL_SUCCESS != mca_base_framework_open(&opal_if_base_framework, 0)) {
-        return OPAL_ERROR;
-    }
-
     memset(&hints, 0, sizeof(hints));
     hints.ai_family = PF_UNSPEC;
     hints.ai_socktype = SOCK_STREAM;
@@ -272,10 +254,6 @@ int16_t opal_ifaddrtokindex(const char* if_addr)
     int if_kernel_index;
     size_t len;
 
-    if (OPAL_SUCCESS != mca_base_framework_open(&opal_if_base_framework, 0)) {
-        return OPAL_ERROR;
-    }
-
     memset(&hints, 0, sizeof(hints));
     hints.ai_family = PF_UNSPEC;
     hints.ai_socktype = SOCK_STREAM;
@@ -327,10 +305,6 @@ int16_t opal_ifaddrtokindex(const char* if_addr)
 
 int opal_ifcount(void)
 {
-    if (OPAL_SUCCESS != mca_base_framework_open(&opal_if_base_framework, 0)) {
-        return 0;
-    }
-
     return opal_list_get_size(&opal_if_list);
 }
 
@@ -344,10 +318,6 @@ int opal_ifbegin(void)
 {
     opal_if_t *intf;
 
-    if (OPAL_SUCCESS != mca_base_framework_open(&opal_if_base_framework, 0)) {
-        return -1;
-    }
-
     intf = (opal_if_t*)opal_list_get_first(&opal_if_list);
     if (NULL != intf)
         return intf->if_index;
@@ -365,10 +335,6 @@ int opal_ifnext(int if_index)
 {
     opal_if_t *intf;
 
-    if (OPAL_SUCCESS != mca_base_framework_open(&opal_if_base_framework, 0)) {
-        return -1;
-    }
-
     for (intf =  (opal_if_t*)opal_list_get_first(&opal_if_list);
         intf != (opal_if_t*)opal_list_get_end(&opal_if_list);
         intf =  (opal_if_t*)opal_list_get_next(intf)) {
@@ -397,10 +363,6 @@ int opal_ifindextoaddr(int if_index, struct sockaddr* if_addr, unsigned int leng
 {
     opal_if_t* intf;
 
-    if (OPAL_SUCCESS != mca_base_framework_open(&opal_if_base_framework, 0)) {
-        return OPAL_ERROR;
-    }
-
     for (intf =  (opal_if_t*)opal_list_get_first(&opal_if_list);
          intf != (opal_if_t*)opal_list_get_end(&opal_if_list);
          intf =  (opal_if_t*)opal_list_get_next(intf)) {
@@ -421,10 +383,6 @@ int opal_ifkindextoaddr(int if_kindex, struct sockaddr* if_addr, unsigned int le
 {
     opal_if_t* intf;
 
-    if (OPAL_SUCCESS != mca_base_framework_open(&opal_if_base_framework, 0)) {
-        return OPAL_ERROR;
-    }
-
     for (intf =  (opal_if_t*)opal_list_get_first(&opal_if_list);
          intf != (opal_if_t*)opal_list_get_end(&opal_if_list);
          intf =  (opal_if_t*)opal_list_get_next(intf)) {
@@ -446,10 +404,6 @@ int opal_ifindextomask(int if_index, uint32_t* if_mask, int length)
 {
     opal_if_t* intf;
 
-    if (OPAL_SUCCESS != mca_base_framework_open(&opal_if_base_framework, 0)) {
-        return OPAL_ERROR;
-    }
-
     for (intf =  (opal_if_t*)opal_list_get_first(&opal_if_list);
         intf != (opal_if_t*)opal_list_get_end(&opal_if_list);
         intf =  (opal_if_t*)opal_list_get_next(intf)) {
@@ -510,10 +464,6 @@ int opal_ifindextoflags(int if_index, uint32_t* if_flags)
 {
     opal_if_t* intf;
 
-    if (OPAL_SUCCESS != mca_base_framework_open(&opal_if_base_framework, 0)) {
-        return OPAL_ERROR;
-    }
-
     for (intf =  (opal_if_t*)opal_list_get_first(&opal_if_list);
         intf != (opal_if_t*)opal_list_get_end(&opal_if_list);
         intf =  (opal_if_t*)opal_list_get_next(intf)) {
@@ -536,10 +486,6 @@ int opal_ifindextoname(int if_index, char* if_name, int length)
 {
     opal_if_t *intf;
 
-    if (OPAL_SUCCESS != mca_base_framework_open(&opal_if_base_framework, 0)) {
-        return OPAL_ERROR;
-    }
-
     for (intf =  (opal_if_t*)opal_list_get_first(&opal_if_list);
         intf != (opal_if_t*)opal_list_get_end(&opal_if_list);
         intf =  (opal_if_t*)opal_list_get_next(intf)) {
@@ -561,10 +507,6 @@ int opal_ifkindextoname(int if_kindex, char* if_name, int length)
 {
     opal_if_t *intf;
 
-    if (OPAL_SUCCESS != mca_base_framework_open(&opal_if_base_framework, 0)) {
-        return OPAL_ERROR;
-    }
-
     for (intf =  (opal_if_t*)opal_list_get_first(&opal_if_list);
         intf != (opal_if_t*)opal_list_get_end(&opal_if_list);
         intf =  (opal_if_t*)opal_list_get_next(intf)) {
@@ -595,7 +537,7 @@ opal_ifislocal(const char *hostname)
     return false;
 }
 
-static uint32_t parse_ipv4_dots(const char *addr, uint32_t* net, int* dots)
+static int parse_ipv4_dots(const char *addr, uint32_t* net, int* dots)
 {
     const char *start = addr, *end;
     uint32_t n[]={0,0,0,0};
@@ -697,10 +639,6 @@ bool opal_ifisloopback(int if_index)
 {
     opal_if_t* intf;
 
-    if (OPAL_SUCCESS != mca_base_framework_open(&opal_if_base_framework, 0)) {
-        return OPAL_ERROR;
-    }
-
     for (intf =  (opal_if_t*)opal_list_get_first(&opal_if_list);
         intf != (opal_if_t*)opal_list_get_end(&opal_if_list);
         intf =  (opal_if_t*)opal_list_get_next(intf)) {
@@ -777,10 +715,6 @@ void opal_ifgetaliases(char ***aliases)
     /* set default answer */
     *aliases = NULL;
 
-    if (OPAL_SUCCESS != mca_base_framework_open(&opal_if_base_framework, 0)) {
-        return;
-    }
-
     for (intf =  (opal_if_t*)opal_list_get_first(&opal_if_list);
         intf != (opal_if_t*)opal_list_get_end(&opal_if_list);
         intf =  (opal_if_t*)opal_list_get_next(intf)) {
