@@ -8,10 +8,11 @@
  *                         reserved.
  * Copyright (c) 2011-2013 Los Alamos National Security, LLC.
  *                         All rights reserved.
+ * Copyright (c) 2014-2017 Intel, Inc.  All rights reserved.
  * $COPYRIGHT$
- * 
+ *
  * Additional copyrights may follow
- * 
+ *
  * $HEADER$
  */
 
@@ -21,9 +22,7 @@
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif  /* HAVE_UNISTD_H */
-#ifdef HAVE_STRING_H
 #include <string.h>
-#endif
 
 #include "opal/util/output.h"
 #include "opal/dss/dss.h"
@@ -34,14 +33,15 @@
 #include "orte/util/session_dir.h"
 #include "orte/util/show_help.h"
 #include "orte/util/nidmap.h"
+#include "orte/util/threads.h"
 
+#include "orte/mca/iof/base/base.h"
 #include "orte/mca/rml/rml.h"
 #include "orte/mca/odls/odls.h"
 #include "orte/mca/odls/base/base.h"
 #include "orte/mca/odls/base/odls_private.h"
 #include "orte/mca/plm/plm_types.h"
 #include "orte/mca/routed/routed.h"
-#include "orte/mca/sensor/sensor.h"
 #include "orte/mca/ess/ess.h"
 #include "orte/mca/state/state.h"
 
@@ -60,45 +60,24 @@
  */
 static int init(void);
 static int finalize(void);
-
-static int predicted_fault(opal_list_t *proc_list,
-                           opal_list_t *node_list,
-                           opal_list_t *suggested_map);
-
-static int suggest_map_targets(orte_proc_t *proc,
-                               orte_node_t *oldnode,
-                               opal_list_t *node_list);
-
-static int ft_event(int state);
-
+static void orted_abort(int error_code, char *fmt, ...);
 
 /******************
  * default_orted module
  ******************/
 orte_errmgr_base_module_t orte_errmgr_default_orted_module = {
-    init,
-    finalize,
-    orte_errmgr_base_log,
-    orte_errmgr_base_abort,
-    orte_errmgr_base_abort_peers,
-    predicted_fault,
-    suggest_map_targets,
-    ft_event,
-    orte_errmgr_base_register_migration_warning,
-    NULL,
-    orte_errmgr_base_execute_error_callbacks
+    .init = init,
+    .finalize = finalize,
+    .logfn = orte_errmgr_base_log,
+    .abort = orted_abort,
+    .abort_peers = orte_errmgr_base_abort_peers
 };
 
 /* Local functions */
 static bool any_live_children(orte_jobid_t job);
 static int pack_state_update(opal_buffer_t *alert, orte_job_t *jobdat);
 static int pack_state_for_proc(opal_buffer_t *alert, orte_proc_t *child);
-static bool all_children_registered(orte_jobid_t job);
-static int pack_child_contact_info(orte_jobid_t job, opal_buffer_t *buf);
 static void failed_start(orte_job_t *jobdat);
-static void update_local_children(orte_job_t *jobdat,
-                                  orte_job_state_t jobstate,
-                                  orte_proc_state_t state);
 static void killprocs(orte_jobid_t job, orte_vpid_t vpid);
 
 static void job_errors(int fd, short args, void *cbdata);
@@ -128,6 +107,129 @@ static int finalize(void)
     return ORTE_SUCCESS;
 }
 
+static void wakeup(int sd, short args, void *cbdata)
+{
+    /* nothing more we can do */
+    ORTE_ACQUIRE_OBJECT(cbdata);
+    orte_quit(0, 0, NULL);
+}
+
+/* this function only gets called when FORCED_TERMINATE
+ * has been invoked, which means that there is some
+ * internal failure (e.g., to pack/unpack a correct value).
+ * We could just exit, but that doesn't result in any
+ * meaningful error message to the user. Likewise, just
+ * printing something to stdout/stderr won't necessarily
+ * get back to the user. Instead, we will send an error
+ * report to mpirun and give it a chance to order our
+ * termination. In order to ensure we _do_ terminate,
+ * we set a timer - if it fires before we receive the
+ * termination command, then we will exit on our own. This
+ * protects us in the case that the failure is in the
+ * messaging system itself */
+static void orted_abort(int error_code, char *fmt, ...)
+{
+    va_list arglist;
+    char *outmsg = NULL;
+    orte_plm_cmd_flag_t cmd;
+    opal_buffer_t *alert;
+    orte_vpid_t null=ORTE_VPID_INVALID;
+    orte_proc_state_t state = ORTE_PROC_STATE_CALLED_ABORT;
+    orte_timer_t *timer;
+    int rc;
+
+    /* only do this once */
+    if (orte_abnormal_term_ordered) {
+        return;
+    }
+
+    /* set the aborting flag */
+    orte_abnormal_term_ordered = true;
+
+    /* If there was a message, construct it */
+    va_start(arglist, fmt);
+    if (NULL != fmt) {
+        vasprintf(&outmsg, fmt, arglist);
+    }
+    va_end(arglist);
+
+    /* use the show-help system to get the message out */
+    orte_show_help("help-errmgr-base.txt", "simple-message", true, outmsg);
+
+    /* tell the HNP we are in distress */
+    alert = OBJ_NEW(opal_buffer_t);
+    /* pack update state command */
+    cmd = ORTE_PLM_UPDATE_PROC_STATE;
+    if (ORTE_SUCCESS != (rc = opal_dss.pack(alert, &cmd, 1, ORTE_PLM_CMD))) {
+        ORTE_ERROR_LOG(rc);
+        OBJ_RELEASE(alert);
+        goto cleanup;
+    }
+    /* pack the jobid */
+    if (ORTE_SUCCESS != (rc = opal_dss.pack(alert, &ORTE_PROC_MY_NAME->jobid, 1, ORTE_JOBID))) {
+        ORTE_ERROR_LOG(rc);
+        OBJ_RELEASE(alert);
+        goto cleanup;
+    }
+    /* pack our vpid */
+    if (ORTE_SUCCESS != (rc = opal_dss.pack(alert, &ORTE_PROC_MY_NAME->vpid, 1, ORTE_VPID))) {
+        ORTE_ERROR_LOG(rc);
+        OBJ_RELEASE(alert);
+        goto cleanup;
+    }
+    /* pack our pid */
+    if (ORTE_SUCCESS != (rc = opal_dss.pack(alert, &orte_process_info.pid, 1, OPAL_PID))) {
+        ORTE_ERROR_LOG(rc);
+        OBJ_RELEASE(alert);
+        goto cleanup;
+    }
+    /* pack our state */
+    if (ORTE_SUCCESS != (rc = opal_dss.pack(alert, &state, 1, ORTE_PROC_STATE))) {
+        ORTE_ERROR_LOG(rc);
+        OBJ_RELEASE(alert);
+        goto cleanup;
+    }
+    /* pack our exit code */
+    if (ORTE_SUCCESS != (rc = opal_dss.pack(alert, &error_code, 1, ORTE_EXIT_CODE))) {
+        ORTE_ERROR_LOG(rc);
+        OBJ_RELEASE(alert);
+        goto cleanup;
+    }
+    /* flag that this job is complete so the receiver can know */
+    if (ORTE_SUCCESS != (rc = opal_dss.pack(alert, &null, 1, ORTE_VPID))) {
+        ORTE_ERROR_LOG(rc);
+        OBJ_RELEASE(alert);
+        goto cleanup;
+    }
+
+    /* send it */
+    if (0 > (rc = orte_rml.send_buffer_nb(orte_mgmt_conduit,
+                                          ORTE_PROC_MY_HNP, alert,
+                                          ORTE_RML_TAG_PLM,
+                                          orte_rml_send_callback, NULL))) {
+        ORTE_ERROR_LOG(rc);
+        OBJ_RELEASE(alert);
+        /* we can't communicate, so give up */
+        orte_quit(0, 0, NULL);
+        return;
+    }
+
+  cleanup:
+    /* set a timer for exiting - this also gives the message a chance
+     * to get out! */
+    if (NULL == (timer = OBJ_NEW(orte_timer_t))) {
+        ORTE_ERROR_LOG(ORTE_ERR_OUT_OF_RESOURCE);
+        return;
+    }
+    timer->tv.tv_sec = 5;
+    timer->tv.tv_usec = 0;
+    opal_event_evtimer_set(orte_event_base, timer->ev, wakeup, NULL);
+    opal_event_set_priority(timer->ev, ORTE_ERROR_PRI);
+    ORTE_POST_OBJECT(timer);
+    opal_event_evtimer_add(timer->ev, &timer->tv);
+
+}
+
 static void job_errors(int fd, short args, void *cbdata)
 {
     orte_state_caddy_t *caddy = (orte_state_caddy_t*)cbdata;
@@ -137,6 +239,8 @@ static void job_errors(int fd, short args, void *cbdata)
     orte_plm_cmd_flag_t cmd;
     opal_buffer_t *alert;
 
+    ORTE_ACQUIRE_OBJECT(caddy);
+
     /*
      * if orte is trying to shutdown, just let it
      */
@@ -168,12 +272,6 @@ static void job_errors(int fd, short args, void *cbdata)
     case ORTE_JOB_STATE_FAILED_TO_START:
         failed_start(jdata);
         break;
-    case ORTE_JOB_STATE_SENSOR_BOUND_EXCEEDED:
-        /* update all procs in job */
-        update_local_children(jdata, jobstate, ORTE_PROC_STATE_SENSOR_BOUND_EXCEEDED);
-        /* order all local procs for this job to be killed */
-        killprocs(jdata->jobid, ORTE_VPID_WILDCARD);
-        break;
     case ORTE_JOB_STATE_COMM_FAILED:
         /* kill all local procs */
         killprocs(ORTE_JOBID_WILDCARD, ORTE_VPID_WILDCARD);
@@ -204,7 +302,8 @@ static void job_errors(int fd, short args, void *cbdata)
         goto cleanup;
     }
     /* send it */
-    if (0 > (rc = orte_rml.send_buffer_nb(ORTE_PROC_MY_HNP, alert,
+    if (0 > (rc = orte_rml.send_buffer_nb(orte_mgmt_conduit,
+                                          ORTE_PROC_MY_HNP, alert,
                                           ORTE_RML_TAG_PLM,
                                           orte_rml_send_callback, NULL))) {
         ORTE_ERROR_LOG(rc);
@@ -221,21 +320,14 @@ static void proc_errors(int fd, short args, void *cbdata)
     orte_job_t *jdata;
     orte_process_name_t *proc = &caddy->name;
     orte_proc_state_t state = caddy->proc_state;
-
+    char *rtmod;
     orte_proc_t *child, *ptr;
     opal_buffer_t *alert;
     orte_plm_cmd_flag_t cmd;
     int rc=ORTE_SUCCESS;
-    orte_vpid_t null=ORTE_VPID_INVALID;
-    orte_ns_cmp_bitmask_t mask=ORTE_NS_CMP_ALL;
     int i;
 
-    /*
-     * if orte is trying to shutdown, just let it
-     */
-    if (orte_finalizing) {
-        goto cleanup;
-    }
+    ORTE_ACQUIRE_OBJECT(caddy);
 
     OPAL_OUTPUT_VERBOSE((2, orte_errmgr_base_framework.framework_output,
                          "%s errmgr:default_orted:proc_errors process %s error state %s",
@@ -243,8 +335,21 @@ static void proc_errors(int fd, short args, void *cbdata)
                          ORTE_NAME_PRINT(proc),
                          orte_proc_state_to_str(state)));
 
+    /*
+     * if orte is trying to shutdown, just let it
+     */
+    if (orte_finalizing) {
+        OPAL_OUTPUT_VERBOSE((2, orte_errmgr_base_framework.framework_output,
+                             "%s errmgr:default_orted:proc_errors finalizing - ignoring error",
+                             ORTE_NAME_PRINT(ORTE_PROC_MY_NAME)));
+        goto cleanup;
+    }
+
     /* if this is a heartbeat failure, let the HNP handle it */
     if (ORTE_PROC_STATE_HEARTBEAT_FAILED == state) {
+        OPAL_OUTPUT_VERBOSE((2, orte_errmgr_base_framework.framework_output,
+                             "%s errmgr:default_orted:proc_errors heartbeat failed - ignoring error",
+                             ORTE_NAME_PRINT(ORTE_PROC_MY_NAME)));
         goto cleanup;
     }
 
@@ -252,9 +357,12 @@ static void proc_errors(int fd, short args, void *cbdata)
      * lifeline
      */
     if (ORTE_PROC_STATE_LIFELINE_LOST == state ||
-        ORTE_PROC_STATE_UNABLE_TO_SEND_MSG == state) {
+        ORTE_PROC_STATE_UNABLE_TO_SEND_MSG == state ||
+        ORTE_PROC_STATE_NO_PATH_TO_TARGET == state ||
+        ORTE_PROC_STATE_PEER_UNKNOWN == state ||
+        ORTE_PROC_STATE_FAILED_TO_CONNECT == state) {
         OPAL_OUTPUT_VERBOSE((2, orte_errmgr_base_framework.framework_output,
-                             "%s errmgr:orted lifeline lost - exiting",
+                             "%s errmgr:orted lifeline lost or unable to communicate - exiting",
                              ORTE_NAME_PRINT(ORTE_PROC_MY_NAME)));
         /* set our exit status */
         ORTE_UPDATE_EXIT_STATUS(ORTE_ERROR_DEFAULT_EXIT_CODE);
@@ -263,24 +371,47 @@ static void proc_errors(int fd, short args, void *cbdata)
         /* terminate - our routed children will see
          * us leave and automatically die
          */
-        ORTE_FORCED_TERMINATE(ORTE_ERROR_DEFAULT_EXIT_CODE);
+        orte_quit(0, 0, NULL);
         goto cleanup;
     }
 
-        /* get the job object */
+    /* get the job object */
     if (NULL == (jdata = orte_get_job_data_object(proc->jobid))) {
         /* must already be complete */
+        OPAL_OUTPUT_VERBOSE((2, orte_errmgr_base_framework.framework_output,
+                             "%s errmgr:default_orted:proc_errors NULL jdata - ignoring error",
+                             ORTE_NAME_PRINT(ORTE_PROC_MY_NAME)));
         goto cleanup;
     }
 
+    /* get our management conduit's routed module name */
+    rtmod = orte_rml.get_routed(orte_mgmt_conduit);
+
     if (ORTE_PROC_STATE_COMM_FAILED == state) {
         /* if it is our own connection, ignore it */
         if (OPAL_EQUAL == orte_util_compare_name_fields(ORTE_NS_CMP_ALL, ORTE_PROC_MY_NAME, proc)) {
+            OPAL_OUTPUT_VERBOSE((2, orte_errmgr_base_framework.framework_output,
+                                 "%s errmgr:default_orted:proc_errors comm_failed to self - ignoring error",
+                                 ORTE_NAME_PRINT(ORTE_PROC_MY_NAME)));
             goto cleanup;
         }
         /* was it a daemon? */
         if (proc->jobid != ORTE_PROC_MY_NAME->jobid) {
-            /* nope - ignore */
+            /* nope - we can't seem to trust that we will catch the waitpid
+             * in this situation, so push this over to be handled as if
+             * it were a waitpid trigger so we don't create a bunch of
+             * duplicate code */
+            OPAL_OUTPUT_VERBOSE((2, orte_errmgr_base_framework.framework_output,
+                                 "%s errmgr:default_orted:proc_errors comm_failed to non-daemon - handling as waitpid",
+                                 ORTE_NAME_PRINT(ORTE_PROC_MY_NAME)));
+            /* get the proc_t */
+            if (NULL == (child = (orte_proc_t*)opal_pointer_array_get_item(jdata->procs, proc->vpid))) {
+                ORTE_ERROR_LOG(ORTE_ERR_NOT_FOUND);
+                ORTE_FORCED_TERMINATE(ORTE_ERROR_DEFAULT_EXIT_CODE);
+                goto cleanup;
+            }
+            /* leave the exit code alone - process this as a waitpid */
+            odls_base_default_wait_local_proc(child, NULL);
             goto cleanup;
         }
         OPAL_OUTPUT_VERBOSE((2, orte_errmgr_base_framework.framework_output,
@@ -325,12 +456,15 @@ static void proc_errors(int fd, short args, void *cbdata)
                                  "%s errmgr:default_orted reporting lost connection to daemon %s",
                                  ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
                                  ORTE_NAME_PRINT(proc)));
-            if (0 > (rc = orte_rml.send_buffer_nb(ORTE_PROC_MY_HNP, alert,
+            if (0 > (rc = orte_rml.send_buffer_nb(orte_mgmt_conduit,
+                                                  ORTE_PROC_MY_HNP, alert,
                                                   ORTE_RML_TAG_PLM,
                                                   orte_rml_send_callback, NULL))) {
                 ORTE_ERROR_LOG(rc);
                 OBJ_RELEASE(alert);
             }
+            /* mark that we notified the HNP for this job so we don't do it again */
+            orte_set_attribute(&jdata->attributes, ORTE_JOB_FAIL_NOTIFIED, ORTE_ATTR_LOCAL, NULL, OPAL_BOOL);
             /* continue on */
             goto cleanup;
         }
@@ -339,7 +473,7 @@ static void proc_errors(int fd, short args, void *cbdata)
             /* are any of my children still alive */
             for (i=0; i < orte_local_children->size; i++) {
                 if (NULL != (child = (orte_proc_t*)opal_pointer_array_get_item(orte_local_children, i))) {
-                    if (child->alive && child->state < ORTE_PROC_STATE_UNTERMINATED) {
+                    if (ORTE_FLAG_TEST(child, ORTE_PROC_FLAG_ALIVE)) {
                         OPAL_OUTPUT_VERBOSE((5, orte_state_base_framework.framework_output,
                                              "%s errmgr:default:orted[%s(%d)] proc %s is alive",
                                              ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
@@ -351,7 +485,7 @@ static void proc_errors(int fd, short args, void *cbdata)
             }
             /* if all my routes and children are gone, then terminate
                ourselves nicely (i.e., this is a normal termination) */
-            if (0 == orte_routed.num_routes()) {
+            if (0 == orte_routed.num_routes(rtmod)) {
                 OPAL_OUTPUT_VERBOSE((2, orte_errmgr_base_framework.framework_output,
                                      "%s errmgr:default:orted all routes gone - exiting",
                                      ORTE_NAME_PRINT(ORTE_PROC_MY_NAME)));
@@ -360,33 +494,25 @@ static void proc_errors(int fd, short args, void *cbdata)
                 OPAL_OUTPUT_VERBOSE((2, orte_errmgr_base_framework.framework_output,
                                      "%s errmgr:default:orted not exiting, num_routes() == %d",
                                      ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
-                                     (int)orte_routed.num_routes()));
+                                     (int)orte_routed.num_routes(rtmod)));
             }
         }
         /* if not, then we can continue */
         goto cleanup;
     }
 
-    /* if there are no local procs for this job, we can
-     * ignore this call
-     */
-    if (0 == jdata->num_local_procs) {
+    if (NULL == (child = (orte_proc_t*)opal_pointer_array_get_item(jdata->procs, proc->vpid))) {
+        ORTE_ERROR_LOG(ORTE_ERR_NOT_FOUND);
+        ORTE_FORCED_TERMINATE(ORTE_ERROR_DEFAULT_EXIT_CODE);
         goto cleanup;
     }
-
-    /* find this proc in the local children */
-    child = NULL;
-    for (i=0; i < orte_local_children->size; i++) {
-        if (NULL == (ptr = (orte_proc_t*)opal_pointer_array_get_item(orte_local_children, i))) {
-            continue;
-        }
-        if (OPAL_EQUAL == orte_util_compare_name_fields(mask, &ptr->name, proc)) {
-            child = ptr;
-            break;
-        }
-    }
-    if (NULL == child) {
-        ORTE_ERROR_LOG(ORTE_ERR_NOT_FOUND);
+    /* if this is not a local proc for this job, we can
+     * ignore this call
+     */
+    if (!ORTE_FLAG_TEST(child, ORTE_PROC_FLAG_LOCAL)) {
+        OPAL_OUTPUT_VERBOSE((2, orte_errmgr_base_framework.framework_output,
+                             "%s errmgr:default_orted:proc_errors proc is not local - ignoring error",
+                             ORTE_NAME_PRINT(ORTE_PROC_MY_NAME)));
         goto cleanup;
     }
 
@@ -395,81 +521,56 @@ static void proc_errors(int fd, short args, void *cbdata)
                          ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
                          orte_proc_state_to_str(state),
                          ORTE_NAME_PRINT(proc)));
- 
-    if (ORTE_PROC_STATE_SENSOR_BOUND_EXCEEDED == state) {
-        child->state = state;
-        /* Decrement the number of local procs */
-        jdata->num_local_procs--;
-        /* kill this proc */
-        killprocs(proc->jobid, proc->vpid);
-        goto cleanup;
-    }
 
     if (ORTE_PROC_STATE_TERM_NON_ZERO == state) {
-        if (!orte_abort_non_zero_exit) {
-            /* leave the child in orte_local_children so we can
-             * later send the state info after full job termination
-             */
-            child->state = state;
-            child->waitpid_recvd = true;
-            if (child->iof_complete) {
-                /* the proc has terminated */
-                child->alive = false;
-                /* Clean up the session directory as if we were the process
-                 * itself.  This covers the case where the process died abnormally
-                 * and didn't cleanup its own session directory.
-                 */
-                orte_session_dir_finalize(&child->name);
-                /* track job status */
-                jdata->num_terminated++;
-            }
-            /* treat this as normal termination */
-            goto REPORT_STATE;
-        }
-        /* report this as abnormal termination to the HNP */
-        alert = OBJ_NEW(opal_buffer_t);
-        /* pack update state command */
-        cmd = ORTE_PLM_UPDATE_PROC_STATE;
-        if (ORTE_SUCCESS != (rc = opal_dss.pack(alert, &cmd, 1, ORTE_PLM_CMD))) {
-            ORTE_ERROR_LOG(rc);
-            return;
-        }
-        /* pack only the data for this proc - have to start with the jobid
-         * so the receiver can unpack it correctly
-         */
-        if (ORTE_SUCCESS != (rc = opal_dss.pack(alert, &proc->jobid, 1, ORTE_JOBID))) {
-            ORTE_ERROR_LOG(rc);
-            return;
-        }
-
+        /* update the state */
         child->state = state;
-        /* now pack the child's info */
-        if (ORTE_SUCCESS != (rc = pack_state_for_proc(alert, child))) {
-            ORTE_ERROR_LOG(rc);
-            return;
-        }
-        /* remove the child from our local array as it is no longer alive */
-        opal_pointer_array_set_item(orte_local_children, i, NULL);
-        /* Decrement the number of local procs */
-        jdata->num_local_procs--;
+        /* report this as abnormal termination to the HNP, unless we already have
+         * done so for this job */
+        if (!orte_get_attribute(&jdata->attributes, ORTE_JOB_FAIL_NOTIFIED, NULL, OPAL_BOOL)) {
+            alert = OBJ_NEW(opal_buffer_t);
+            /* pack update state command */
+            cmd = ORTE_PLM_UPDATE_PROC_STATE;
+            if (ORTE_SUCCESS != (rc = opal_dss.pack(alert, &cmd, 1, ORTE_PLM_CMD))) {
+                ORTE_ERROR_LOG(rc);
+                return;
+            }
+            /* pack only the data for this proc - have to start with the jobid
+             * so the receiver can unpack it correctly
+             */
+            if (ORTE_SUCCESS != (rc = opal_dss.pack(alert, &proc->jobid, 1, ORTE_JOBID))) {
+                ORTE_ERROR_LOG(rc);
+                return;
+            }
 
-        OPAL_OUTPUT_VERBOSE((5, orte_errmgr_base_framework.framework_output,
-                             "%s errmgr:default_orted reporting proc %s abnormally terminated with non-zero status (local procs = %d)",
-                             ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
-                             ORTE_NAME_PRINT(&child->name),
-                             jdata->num_local_procs));
-        
-        /* release the child object */
-        OBJ_RELEASE(child);
-
-        /* send it */
-        if (0 > (rc = orte_rml.send_buffer_nb(ORTE_PROC_MY_HNP, alert,
-                                              ORTE_RML_TAG_PLM,
-                                              orte_rml_send_callback, NULL))) {
-            ORTE_ERROR_LOG(rc);
-            OBJ_RELEASE(alert);
+            /* now pack the child's info */
+            if (ORTE_SUCCESS != (rc = pack_state_for_proc(alert, child))) {
+                ORTE_ERROR_LOG(rc);
+                return;
+            }
+            /* send it */
+            OPAL_OUTPUT_VERBOSE((5, orte_errmgr_base_framework.framework_output,
+                                 "%s errmgr:default_orted reporting proc %s abnormally terminated with non-zero status (local procs = %d)",
+                                 ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
+                                 ORTE_NAME_PRINT(&child->name),
+                                 jdata->num_local_procs));
+            if (0 > (rc = orte_rml.send_buffer_nb(orte_mgmt_conduit,
+                                                  ORTE_PROC_MY_HNP, alert,
+                                                  ORTE_RML_TAG_PLM,
+                                                  orte_rml_send_callback, NULL))) {
+                ORTE_ERROR_LOG(rc);
+                OBJ_RELEASE(alert);
+            }
+            /* mark that we notified the HNP for this job so we don't do it again */
+            orte_set_attribute(&jdata->attributes, ORTE_JOB_FAIL_NOTIFIED, ORTE_ATTR_LOCAL, NULL, OPAL_BOOL);
         }
-        return;
+        /* if the proc has terminated, notify the state machine */
+        if (ORTE_FLAG_TEST(child, ORTE_PROC_FLAG_IOF_COMPLETE) &&
+            ORTE_FLAG_TEST(child, ORTE_PROC_FLAG_WAITPID) &&
+            !ORTE_FLAG_TEST(child, ORTE_PROC_FLAG_RECORDED)) {
+            ORTE_ACTIVATE_PROC_STATE(proc, ORTE_PROC_STATE_TERMINATED);
+        }
+        goto cleanup;
     }
 
     if (ORTE_PROC_STATE_FAILED_TO_START == state ||
@@ -488,107 +589,87 @@ static void proc_errors(int fd, short args, void *cbdata)
     }
 
     if (ORTE_PROC_STATE_TERMINATED < state) {
+        /* if we were ordered to terminate, see if
+         * any of our routes or local children remain alive - if not, then
+         * terminate ourselves. */
+        if (orte_orteds_term_ordered) {
+            /* mark the child as no longer alive and update the counters, if necessary.
+             * we have to do this here as we aren't going to send this to the state
+             * machine, and we want to keep the bookkeeping accurate just in case */
+            if (ORTE_FLAG_TEST(child, ORTE_PROC_FLAG_ALIVE)) {
+                ORTE_FLAG_UNSET(child, ORTE_PROC_FLAG_ALIVE);
+            }
+            if (!ORTE_FLAG_TEST(child, ORTE_PROC_FLAG_RECORDED)) {
+                ORTE_FLAG_SET(child, ORTE_PROC_FLAG_RECORDED);
+                jdata->num_terminated++;
+            }
+            for (i=0; i < orte_local_children->size; i++) {
+                if (NULL != (child = (orte_proc_t*)opal_pointer_array_get_item(orte_local_children, i))) {
+                    if (ORTE_FLAG_TEST(child, ORTE_PROC_FLAG_ALIVE)) {
+                        goto keep_going;
+                    }
+                }
+            }
+            /* if all my routes and children are gone, then terminate
+               ourselves nicely (i.e., this is a normal termination) */
+            if (0 == orte_routed.num_routes(rtmod)) {
+                OPAL_OUTPUT_VERBOSE((2, orte_errmgr_base_framework.framework_output,
+                                     "%s errmgr:default:orted all routes gone - exiting",
+                                     ORTE_NAME_PRINT(ORTE_PROC_MY_NAME)));
+                ORTE_ACTIVATE_JOB_STATE(NULL, ORTE_JOB_STATE_DAEMONS_TERMINATED);
+            }
+            /* no need to alert the HNP - we are already on our way out */
+            goto cleanup;
+        }
+
+    keep_going:
         /* if the job hasn't completed and the state is abnormally
-         * terminated, then we need to alert the HNP right away
+         * terminated, then we need to alert the HNP right away - but
+         * only do this once!
          */
-        alert = OBJ_NEW(opal_buffer_t);
-        /* pack update state command */
-        cmd = ORTE_PLM_UPDATE_PROC_STATE;
-        if (ORTE_SUCCESS != (rc = opal_dss.pack(alert, &cmd, 1, ORTE_PLM_CMD))) {
-            ORTE_ERROR_LOG(rc);
-            return;
-        }
-        /* pack only the data for this proc - have to start with the jobid
-         * so the receiver can unpack it correctly
-         */
-        if (ORTE_SUCCESS != (rc = opal_dss.pack(alert, &proc->jobid, 1, ORTE_JOBID))) {
-            ORTE_ERROR_LOG(rc);
-            return;
-        }
-
-        child->state = state;
-        /* now pack the child's info */
-        if (ORTE_SUCCESS != (rc = pack_state_for_proc(alert, child))) {
-            ORTE_ERROR_LOG(rc);
-            return;
-        }
-
-        OPAL_OUTPUT_VERBOSE((5, orte_errmgr_base_framework.framework_output,
-                             "%s errmgr:default_orted reporting proc %s aborted to HNP (local procs = %d)",
-                             ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
-                             ORTE_NAME_PRINT(&child->name),
-                             jdata->num_local_procs));
-        
-        /* send it */
-        if (0 > (rc = orte_rml.send_buffer_nb(ORTE_PROC_MY_HNP, alert,
-                                              ORTE_RML_TAG_PLM,
-                                              orte_rml_send_callback, NULL))) {
-            ORTE_ERROR_LOG(rc);
-        }
-        /* if the proc still reports itself as alive, then we need
-         * to ensure the state machine sees it */
-        if (child->alive && child->waitpid_recvd) {
-            ORTE_ACTIVATE_PROC_STATE(&child->name, ORTE_PROC_STATE_WAITPID_FIRED);
-        }
-        return;
-    }
-
- REPORT_STATE:
-    if (ORTE_PROC_STATE_REGISTERED == state) {
-        /* see if everyone in this job has registered */
-        if (all_children_registered(proc->jobid)) {
-            /* once everyone registers, send their contact info to
-             * the HNP so it is available to debuggers and anyone
-             * else that needs it
-             */
-
-            OPAL_OUTPUT_VERBOSE((5, orte_errmgr_base_framework.framework_output,
-                                 "%s errmgr:default_orted: sending contact info to HNP",
-                                 ORTE_NAME_PRINT(ORTE_PROC_MY_NAME)));
-            
+        if (!orte_get_attribute(&jdata->attributes, ORTE_JOB_FAIL_NOTIFIED, NULL, OPAL_BOOL)) {
             alert = OBJ_NEW(opal_buffer_t);
-            /* pack init routes command */
-            cmd = ORTE_PLM_INIT_ROUTES_CMD;
+            /* pack update state command */
+            cmd = ORTE_PLM_UPDATE_PROC_STATE;
             if (ORTE_SUCCESS != (rc = opal_dss.pack(alert, &cmd, 1, ORTE_PLM_CMD))) {
                 ORTE_ERROR_LOG(rc);
                 return;
             }
-            /* pack the jobid */
+            /* pack only the data for this proc - have to start with the jobid
+             * so the receiver can unpack it correctly
+             */
             if (ORTE_SUCCESS != (rc = opal_dss.pack(alert, &proc->jobid, 1, ORTE_JOBID))) {
                 ORTE_ERROR_LOG(rc);
                 return;
             }
-            /* pack all the local child vpids */
-            for (i=0; i < orte_local_children->size; i++) {
-                if (NULL == (ptr = (orte_proc_t*)opal_pointer_array_get_item(orte_local_children, i))) {
-                    continue;
-                }
-                if (ptr->name.jobid == proc->jobid) {
-                    if (ORTE_SUCCESS != (rc = opal_dss.pack(alert, &ptr->name.vpid, 1, ORTE_VPID))) {
-                        ORTE_ERROR_LOG(rc);
-                        return;
-                    }
-                }
-            }
-            /* pack an invalid marker */
-            if (ORTE_SUCCESS != (rc = opal_dss.pack(alert, &null, 1, ORTE_VPID))) {
+            child->state = state;
+            /* now pack the child's info */
+            if (ORTE_SUCCESS != (rc = pack_state_for_proc(alert, child))) {
                 ORTE_ERROR_LOG(rc);
                 return;
             }
-            /* add in contact info for all procs in the job */
-            if (ORTE_SUCCESS != (rc = pack_child_contact_info(proc->jobid, alert))) {
-                ORTE_ERROR_LOG(rc);
-                OBJ_DESTRUCT(&alert);
-                return;
-            }
+            OPAL_OUTPUT_VERBOSE((5, orte_errmgr_base_framework.framework_output,
+                                 "%s errmgr:default_orted reporting proc %s aborted to HNP (local procs = %d)",
+                                 ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
+                                 ORTE_NAME_PRINT(&child->name),
+                                 jdata->num_local_procs));
             /* send it */
-            if (0 > (rc = orte_rml.send_buffer_nb(ORTE_PROC_MY_HNP, alert,
+            if (0 > (rc = orte_rml.send_buffer_nb(orte_mgmt_conduit,
+                                                  ORTE_PROC_MY_HNP, alert,
                                                   ORTE_RML_TAG_PLM,
                                                   orte_rml_send_callback, NULL))) {
                 ORTE_ERROR_LOG(rc);
             }
-        }        
-        return;
+            /* mark that we notified the HNP for this job so we don't do it again */
+            orte_set_attribute(&jdata->attributes, ORTE_JOB_FAIL_NOTIFIED, ORTE_ATTR_LOCAL, NULL, OPAL_BOOL);
+        }
+        /* if the proc has terminated, notify the state machine */
+        if (ORTE_FLAG_TEST(child, ORTE_PROC_FLAG_IOF_COMPLETE) &&
+            ORTE_FLAG_TEST(child, ORTE_PROC_FLAG_WAITPID) &&
+            !ORTE_FLAG_TEST(child, ORTE_PROC_FLAG_RECORDED)) {
+            ORTE_ACTIVATE_PROC_STATE(proc, ORTE_PROC_STATE_TERMINATED);
+        }
+        goto cleanup;
     }
 
     /* only other state is terminated - see if anyone is left alive */
@@ -610,10 +691,8 @@ static void proc_errors(int fd, short args, void *cbdata)
                              "%s errmgr:default_orted reporting all procs in %s terminated",
                              ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
                              ORTE_JOBID_PRINT(jdata->jobid)));
-        
-        /* remove all of this job's children from the global list - do not lock
-         * the thread as we are already locked
-         */
+
+        /* remove all of this job's children from the global list */
         for (i=0; i < orte_local_children->size; i++) {
             if (NULL == (ptr = (orte_proc_t*)opal_pointer_array_get_item(orte_local_children, i))) {
                 continue;
@@ -628,11 +707,11 @@ static void proc_errors(int fd, short args, void *cbdata)
         orte_session_dir_cleanup(jdata->jobid);
 
         /* remove this job from our local job data since it is complete */
-        opal_pointer_array_set_item(orte_job_data, ORTE_LOCAL_JOBID(jdata->jobid), NULL);
         OBJ_RELEASE(jdata);
 
         /* send it */
-        if (0 > (rc = orte_rml.send_buffer_nb(ORTE_PROC_MY_HNP, alert,
+        if (0 > (rc = orte_rml.send_buffer_nb(orte_mgmt_conduit,
+                                              ORTE_PROC_MY_HNP, alert,
                                               ORTE_RML_TAG_PLM,
                                               orte_rml_send_callback, NULL))) {
             ORTE_ERROR_LOG(rc);
@@ -640,30 +719,10 @@ static void proc_errors(int fd, short args, void *cbdata)
         return;
     }
 
- cleanup:
+  cleanup:
     OBJ_RELEASE(caddy);
 }
 
-static int predicted_fault(opal_list_t *proc_list,
-                           opal_list_t *node_list,
-                           opal_list_t *suggested_map)
-{
-    return ORTE_ERR_NOT_IMPLEMENTED;
-}
-
-static int suggest_map_targets(orte_proc_t *proc,
-                               orte_node_t *oldnode,
-                               opal_list_t *node_list)
-{
-    return ORTE_ERR_NOT_IMPLEMENTED;
-}
-
-static int ft_event(int state)
-{
-    return ORTE_SUCCESS;
-}
-
-
 /*****************
  * Local Functions
  *****************/
@@ -678,7 +737,7 @@ static bool any_live_children(orte_jobid_t job)
         }
         /* is this child part of the specified job? */
         if ((job == child->name.jobid || ORTE_JOBID_WILDCARD == job) &&
-            child->alive) {
+            ORTE_FLAG_TEST(child, ORTE_PROC_FLAG_ALIVE)) {
             return true;
         }
     }
@@ -748,70 +807,6 @@ static int pack_state_update(opal_buffer_t *alert, orte_job_t *jobdat)
     return ORTE_SUCCESS;
 }
 
-static bool all_children_registered(orte_jobid_t job)
-{
-    int i;
-    orte_proc_t *child;
-
-    for (i=0; i < orte_local_children->size; i++) {
-        if (NULL == (child = (orte_proc_t*)opal_pointer_array_get_item(orte_local_children, i))) {
-            continue;
-        }
-        /* is this child part of the specified job? */
-        if (job == child->name.jobid || ORTE_JOBID_WILDCARD == job) {
-            /* if this child has terminated, we consider it as having
-             * registered for the purposes of this function. If it never
-             * did register, then we will send a NULL rml_uri back to
-             * the HNP, which will then know that the proc did not register.
-             * If other procs did register, then the HNP can declare an
-             * abnormal termination
-             */
-            if (ORTE_PROC_STATE_UNTERMINATED < child->state) {
-                /* this proc has terminated somehow - consider it
-                 * as registered for now
-                 */
-                continue;
-            }
-            /* if this child has *not* registered yet, return false */
-            if (!child->registered) {
-                return false;
-            }
-        }
-    }
-
-    /* if we get here, then everyone in the job has registered */
-    return true;
-
-}
-
-static int pack_child_contact_info(orte_jobid_t job, opal_buffer_t *buf)
-{
-    orte_proc_t *child;
-    int rc, i;
-
-    for (i=0; i < orte_local_children->size; i++) {
-        if (NULL == (child = (orte_proc_t*)opal_pointer_array_get_item(orte_local_children, i))) {
-            continue;
-        }
-        /* is this child part of the specified job? */
-        if (job == child->name.jobid || ORTE_JOBID_WILDCARD == job) {
-            /* pack the child's vpid - must be done in case rml_uri is NULL */
-            if (ORTE_SUCCESS != (rc = opal_dss.pack(buf, &(child->name.vpid), 1, ORTE_VPID))) {
-                ORTE_ERROR_LOG(rc);
-                return rc;
-            }            
-            /* pack the contact info */
-            if (ORTE_SUCCESS != (rc = opal_dss.pack(buf, &child->rml_uri, 1, OPAL_STRING))) {
-                ORTE_ERROR_LOG(rc);
-                return rc;
-            }
-        }
-    }
-
-    return ORTE_SUCCESS;
-
-}
-
 static void failed_start(orte_job_t *jobdat)
 {
     int i;
@@ -831,9 +826,9 @@ static void failed_start(orte_job_t *jobdat)
                  * is complete or else we will hang waiting for
                  * pipes to close that were never opened
                  */
-                child->iof_complete = true;
+                ORTE_FLAG_SET(child, ORTE_PROC_FLAG_IOF_COMPLETE);
                 /* ditto for waitpid */
-                child->waitpid_recvd = true;
+                ORTE_FLAG_SET(child, ORTE_PROC_FLAG_WAITPID);
             }
         }
     }
@@ -844,37 +839,13 @@ static void failed_start(orte_job_t *jobdat)
     return;
 }
 
-static void update_local_children(orte_job_t *jobdat, orte_job_state_t jobstate, orte_proc_state_t state)
-{
-    int i;
-    orte_proc_t *child;
-
-    /* update job state */
-    jobdat->state = jobstate;
-    /* update children */
-    for (i=0; i < orte_local_children->size; i++) {
-        if (NULL == (child = (orte_proc_t*)opal_pointer_array_get_item(orte_local_children, i))) {
-            continue;
-        }
-        /* is this child part of the specified job? */
-        if (jobdat->jobid == child->name.jobid) {
-            child->state = state;
-        }
-    }
-}
-
 static void killprocs(orte_jobid_t job, orte_vpid_t vpid)
 {
     opal_pointer_array_t cmd;
     orte_proc_t proc;
     int rc;
 
-    /* stop local sensors for this job */
-    if (ORTE_VPID_WILDCARD == vpid) {
-        orte_sensor.stop(job);
-    }
-
-    if (ORTE_JOBID_WILDCARD == job 
+    if (ORTE_JOBID_WILDCARD == job
         && ORTE_VPID_WILDCARD == vpid) {
         if (ORTE_SUCCESS != (rc = orte_odls.kill_local_procs(NULL))) {
             ORTE_ERROR_LOG(rc);
