@@ -5,15 +5,16 @@
  * Copyright (c) 2004-2011 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
- * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
+ * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart,
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
  * Copyright (c) 2011-2012 Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2013-2017 Intel, Inc. All rights reserved.
  * $COPYRIGHT$
- * 
+ *
  * Additional copyrights may follow
- * 
+ *
  * $HEADER$
  *
  */
@@ -35,8 +36,9 @@
 #include "opal/runtime/opal.h"
 
 #include "orte/util/show_help.h"
-#include "opal/mca/mca.h"
+#include "orte/mca/mca.h"
 #include "opal/mca/base/base.h"
+#include "opal/mca/pmix/pmix.h"
 #include "opal/util/output.h"
 #include "opal/util/malloc.h"
 #include "opal/util/argv.h"
@@ -61,7 +63,6 @@
 #include "orte/util/proc_info.h"
 #include "orte/util/session_dir.h"
 #include "orte/util/name_fns.h"
-#include "orte/util/nidmap.h"
 #include "orte/util/regex.h"
 
 #include "orte/runtime/runtime.h"
@@ -97,74 +98,27 @@ static int rte_init(void)
 {
     int ret;
     char *error = NULL;
-    char **hosts = NULL;
 
     /* run the prolog */
     if (ORTE_SUCCESS != (ret = orte_ess_base_std_prolog())) {
         error = "orte_ess_base_std_prolog";
         goto error;
     }
-    
+
     /* Start by getting a unique name from the enviro */
     env_set_name();
 
     /* if I am a daemon, complete my setup using the
      * default procedure
      */
-    if (ORTE_PROC_IS_DAEMON) {
-        if (NULL != orte_node_regex) {
-            /* extract the nodes */
-            if (ORTE_SUCCESS != (ret = orte_regex_extract_node_names(orte_node_regex, &hosts))) {
-                error = "orte_regex_extract_node_names";
-                goto error;
-            }
-        }
-        if (ORTE_SUCCESS != (ret = orte_ess_base_orted_setup(hosts))) {
-            ORTE_ERROR_LOG(ret);
-            error = "orte_ess_base_orted_setup";
-            goto error;
-        }
-        opal_argv_free(hosts);
-        return ORTE_SUCCESS;
-    }
-    
-    if (ORTE_PROC_IS_TOOL) {
-        /* otherwise, if I am a tool proc, use that procedure */
-        if (ORTE_SUCCESS != (ret = orte_ess_base_tool_setup())) {
-            ORTE_ERROR_LOG(ret);
-            error = "orte_ess_base_tool_setup";
-            goto error;
-        }
-        /* as a tool, I don't need a nidmap - so just return now */
-        return ORTE_SUCCESS;
-        
-    }
-    
-    /* otherwise, I must be an application process - use
-     * the default procedure to finish my setup
-     */
-    if (ORTE_SUCCESS != (ret = orte_ess_base_app_setup(true))) {
+    if (ORTE_SUCCESS != (ret = orte_ess_base_orted_setup())) {
         ORTE_ERROR_LOG(ret);
-        error = "orte_ess_base_app_setup";
+        error = "orte_ess_base_orted_setup";
         goto error;
     }
-    
-    /* if data was provided, update the database */
-    if (ORTE_SUCCESS != (ret = orte_util_nidmap_init(orte_process_info.sync_buf))) {
-        ORTE_ERROR_LOG(ret);
-        error = "orte_util_nidmap_init";
-        goto error;
-    }
-    
-    /* setup process binding */
-    if (ORTE_SUCCESS != (ret = orte_ess_base_proc_binding())) {
-        error = "proc_binding";
-        goto error;
-    }
-
     return ORTE_SUCCESS;
 
-error:
+ error:
     if (ORTE_ERR_SILENT != ret && !orte_report_silent_errors) {
         orte_show_help("help-orte-runtime.txt",
                        "orte_init:startup:internal-failure",
@@ -177,32 +131,11 @@ error:
 static int rte_finalize(void)
 {
     int ret;
-    
-    /* if I am a daemon, finalize using the default procedure */
-    if (ORTE_PROC_IS_DAEMON) {
-        if (ORTE_SUCCESS != (ret = orte_ess_base_orted_finalize())) {
-            ORTE_ERROR_LOG(ret);
-        }
-    } else if (ORTE_PROC_IS_TOOL) {
-        /* otherwise, if I am a tool proc, use that procedure */
-        if (ORTE_SUCCESS != (ret = orte_ess_base_tool_finalize())) {
-            ORTE_ERROR_LOG(ret);
-        }
-        /* as a tool, I didn't create a nidmap - so just return now */
-        return ret;
-    } else {
-        /* otherwise, I must be an application process
-         * use the default procedure to finish
-         */
-        if (ORTE_SUCCESS != (ret = orte_ess_base_app_finalize())) {
-            ORTE_ERROR_LOG(ret);
-        }
+
+    if (ORTE_SUCCESS != (ret = orte_ess_base_orted_finalize())) {
+        ORTE_ERROR_LOG(ret);
     }
-    
-    /* deconstruct the nidmap and jobmap arrays */
-    orte_util_nidmap_finalize();
-    
-    return ret;    
+    return ret;
 }
 
 static int env_set_name(void)
@@ -210,7 +143,7 @@ static int env_set_name(void)
     int rc;
     orte_jobid_t jobid;
     orte_vpid_t vpid;
-    
+
     if (NULL == orte_ess_base_jobid) {
         ORTE_ERROR_LOG(ORTE_ERR_NOT_FOUND);
         return ORTE_ERR_NOT_FOUND;
@@ -231,10 +164,10 @@ static int env_set_name(void)
 
     ORTE_PROC_MY_NAME->jobid = jobid;
     ORTE_PROC_MY_NAME->vpid = vpid;
-    
+
     OPAL_OUTPUT_VERBOSE((1, orte_ess_base_framework.framework_output,
                          "ess:env set name to %s", ORTE_NAME_PRINT(ORTE_PROC_MY_NAME)));
-    
+
     /* get the non-name common environmental variables */
     if (ORTE_SUCCESS != (rc = orte_ess_env_get())) {
         ORTE_ERROR_LOG(rc);
@@ -257,8 +190,7 @@ static int rte_ft_event(int state)
          */
         if( ORTE_SUCCESS != (ret = orte_snapc.ft_event(OPAL_CRS_CHECKPOINT))) {
             ORTE_ERROR_LOG(ret);
-            exit_status = ret;
-            goto cleanup;
+            return ret;
         }
 
         /*
@@ -266,8 +198,7 @@ static int rte_ft_event(int state)
          */
         if( ORTE_SUCCESS != (ret = orte_routed.ft_event(OPAL_CRS_CHECKPOINT))) {
             ORTE_ERROR_LOG(ret);
-            exit_status = ret;
-            goto cleanup;
+            return ret;
         }
 
         /*
@@ -275,8 +206,7 @@ static int rte_ft_event(int state)
          */
         if( ORTE_SUCCESS != (ret = orte_rml.ft_event(OPAL_CRS_CHECKPOINT))) {
             ORTE_ERROR_LOG(ret);
-            exit_status = ret;
-            goto cleanup;
+            return ret;
         }
     }
     /******** Continue Recovery ********/
@@ -290,8 +220,7 @@ static int rte_ft_event(int state)
          */
         if( ORTE_SUCCESS != (ret = orte_rml.ft_event(OPAL_CRS_CONTINUE))) {
             ORTE_ERROR_LOG(ret);
-            exit_status = ret;
-            goto cleanup;
+            return ret;
         }
 
         /*
@@ -299,8 +228,7 @@ static int rte_ft_event(int state)
          */
         if( ORTE_SUCCESS != (ret = orte_routed.ft_event(OPAL_CRS_CONTINUE))) {
             ORTE_ERROR_LOG(ret);
-            exit_status = ret;
-            goto cleanup;
+            return ret;
         }
 
         /*
@@ -308,20 +236,15 @@ static int rte_ft_event(int state)
          */
         if( ORTE_SUCCESS != (ret = orte_snapc.ft_event(OPAL_CRS_CONTINUE))) {
             ORTE_ERROR_LOG(ret);
-            exit_status = ret;
-            goto cleanup;
+            return ret;
         }
 
-        if( orte_cr_continue_like_restart ) {
+        if (opal_cr_continue_like_restart) {
             /*
              * Barrier to make all processes have been successfully restarted before
              * we try to remove some restart only files.
              */
-            if (ORTE_SUCCESS != (ret = orte_grpcomm.barrier())) {
-                opal_output(0, "ess:env: ft_event(%2d): Failed in orte_grpcomm.barrier (%d)",
-                            state, ret);
-                return ret;
-            }
+            opal_pmix.fence(NULL, 0);
 
             if( orte_cr_flush_restart_files ) {
                 OPAL_OUTPUT_VERBOSE((1, orte_ess_base_framework.framework_output,
@@ -342,11 +265,6 @@ static int rte_ft_event(int state)
          * This should follow the ess init() function
          */
 
-        /*
-         * Clear nidmap and jmap
-         */
-        orte_util_nidmap_finalize();
-
         /*
          * - Reset Contact information
          */
@@ -359,8 +277,7 @@ static int rte_ft_event(int state)
          */
         if( ORTE_SUCCESS != (ret = orte_rml.ft_event(OPAL_CRS_RESTART))) {
             ORTE_ERROR_LOG(ret);
-            exit_status = ret;
-            goto cleanup;
+            return ret;
         }
 
         /*
@@ -371,29 +288,12 @@ static int rte_ft_event(int state)
         orte_process_info.proc_type = ORTE_PROC_TOOL;
         if (ORTE_SUCCESS != (ret = orte_routed.finalize()) ) {
             ORTE_ERROR_LOG(ret);
-            exit_status = ret;
-            goto cleanup;
+            return ret;
         }
         orte_process_info.proc_type = svtype;
         if (ORTE_SUCCESS != (ret = orte_routed.initialize()) ) {
             ORTE_ERROR_LOG(ret);
-            exit_status = ret;
-            goto cleanup;
-        }
-
-        /*
-         * Group Comm - Clean out stale data
-         */
-        orte_grpcomm.finalize();
-        if (ORTE_SUCCESS != (ret = orte_grpcomm.init())) {
-            ORTE_ERROR_LOG(ret);
-            exit_status = ret;
-            goto cleanup;
-        }
-        if (ORTE_SUCCESS != (ret = orte_grpcomm.purge_proc_attrs())) {
-            ORTE_ERROR_LOG(ret);
-            exit_status = ret;
-            goto cleanup;
+            return ret;
         }
 
         /*
@@ -401,14 +301,12 @@ static int rte_ft_event(int state)
          */
         if (ORTE_SUCCESS != (ret = orte_plm.finalize())) {
             ORTE_ERROR_LOG(ret);
-            exit_status = ret;
-            goto cleanup;
+            return ret;
         }
 
         if (ORTE_SUCCESS != (ret = orte_plm.init())) {
             ORTE_ERROR_LOG(ret);
-            exit_status = ret;
-            goto cleanup;
+            return ret;
         }
 
         /*
@@ -416,8 +314,7 @@ static int rte_ft_event(int state)
          */
         if (ORTE_SUCCESS != (ret = orte_rml.enable_comm())) {
             ORTE_ERROR_LOG(ret);
-            exit_status = ret;
-            goto cleanup;
+            return ret;
         }
 
         /*
@@ -425,26 +322,15 @@ static int rte_ft_event(int state)
          */
         if( ORTE_SUCCESS != (ret = orte_routed.ft_event(OPAL_CRS_RESTART))) {
             ORTE_ERROR_LOG(ret);
-            exit_status = ret;
-            goto cleanup;
-        }
-
-        /* if one was provided, build my nidmap */
-        if (ORTE_SUCCESS != (ret = orte_util_nidmap_init(orte_process_info.sync_buf))) {
-            ORTE_ERROR_LOG(ret);
-            exit_status = ret;
-            goto cleanup;
+            return ret;
         }
 
         /*
          * Barrier to make all processes have been successfully restarted before
          * we try to remove some restart only files.
          */
-        if (ORTE_SUCCESS != (ret = orte_grpcomm.barrier())) {
-            opal_output(0, "ess:env ft_event(%2d): Failed in orte_grpcomm.barrier (%d)",
-                        state, ret);
-            return ret;
-        }
+        opal_pmix.fence(NULL, 0);
+
         if( orte_cr_flush_restart_files ) {
             OPAL_OUTPUT_VERBOSE((1, orte_ess_base_framework.framework_output,
                                  "ess:env ft_event(%2d): %s "
@@ -465,7 +351,7 @@ static int rte_ft_event(int state)
                                                         ORTE_PROC_MY_NAME))) {
                 exit_status = ret;
             }
-            
+
             opal_output_set_output_file_info(orte_process_info.proc_session_dir,
                                              "output-", NULL, NULL);
         }
@@ -475,8 +361,7 @@ static int rte_ft_event(int state)
          */
         if( ORTE_SUCCESS != (ret = orte_snapc.ft_event(OPAL_CRS_RESTART))) {
             ORTE_ERROR_LOG(ret);
-            exit_status = ret;
-            goto cleanup;
+            return ret;
         }
     }
     else if (OPAL_CRS_TERM == state ) {
@@ -486,8 +371,6 @@ static int rte_ft_event(int state)
         /* Error state = Nothing */
     }
 
- cleanup:
-
     return exit_status;
 }
 #endif
