@@ -11,9 +11,11 @@
  *                         All rights reserved.
  * Copyright (c) 2013      Los Alamos National Security, LLC.  All rights reserved.
  * Copyright (c) 2013      Cisco Systems, Inc.  All rights reserved.
- * Copyright (c) 2015      Intel, Inc. All rights reserved.
- * Copyright (c) 2015      Research Organization for Information Science
+ * Copyright (c) 2015-2017 Intel, Inc.  All rights reserved.
+ * Copyright (c) 2015-2017 Research Organization for Information Science
  *                         and Technology (RIST). All rights reserved.
+ * Copyright (c) 2017      IBM Corporation.  All rights reserved.
+ * Copyright (c) 2017      Mellanox Technologies. All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -37,6 +39,7 @@
 #include "orte/util/proc_info.h"
 #include "orte/runtime/orte_globals.h"
 #include "orte/util/name_fns.h"
+#include "orte/mca/rml/rml.h"
 
 #include "orte/mca/iof/iof.h"
 #include "orte/mca/iof/base/base.h"
@@ -52,132 +55,6 @@
 orte_iof_base_module_t orte_iof = {0};
 
 
-/* class instances */
-static void orte_iof_job_construct(orte_iof_job_t *ptr)
-{
-    ptr->jdata = NULL;
-    OBJ_CONSTRUCT(&ptr->xoff, opal_bitmap_t);
-}
-static void orte_iof_job_destruct(orte_iof_job_t *ptr)
-{
-    if (NULL != ptr->jdata) {
-        OBJ_RELEASE(ptr->jdata);
-    }
-    OBJ_DESTRUCT(&ptr->xoff);
-}
-OBJ_CLASS_INSTANCE(orte_iof_job_t,
-                   opal_object_t,
-                   orte_iof_job_construct,
-                   orte_iof_job_destruct);
-
-static void orte_iof_base_proc_construct(orte_iof_proc_t* ptr)
-{
-    ptr->revstdout = NULL;
-    ptr->revstderr = NULL;
-    ptr->revstddiag = NULL;
-    ptr->sink = NULL;
-}
-static void orte_iof_base_proc_destruct(orte_iof_proc_t* ptr)
-{
-    if (NULL != ptr->revstdout) {
-        OBJ_RELEASE(ptr->revstdout);
-    }
-    if (NULL != ptr->revstderr) {
-        OBJ_RELEASE(ptr->revstderr);
-    }
-    if (NULL != ptr->revstddiag) {
-        OBJ_RELEASE(ptr->revstddiag);
-    }
-}
-OBJ_CLASS_INSTANCE(orte_iof_proc_t,
-                   opal_list_item_t,
-                   orte_iof_base_proc_construct,
-                   orte_iof_base_proc_destruct);
-
-
-static void orte_iof_base_sink_construct(orte_iof_sink_t* ptr)
-{
-    ptr->daemon.jobid = ORTE_JOBID_INVALID;
-    ptr->daemon.vpid = ORTE_VPID_INVALID;
-    ptr->wev = OBJ_NEW(orte_iof_write_event_t);
-    ptr->xoff = false;
-    ptr->exclusive = false;
-}
-static void orte_iof_base_sink_destruct(orte_iof_sink_t* ptr)
-{
-    OPAL_OUTPUT_VERBOSE((20, orte_iof_base_framework.framework_output,
-                         "%s iof: closing sink for process %s",
-                         ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
-                         ORTE_NAME_PRINT(&ptr->name)));
-    if (NULL != ptr->wev) {
-        OBJ_RELEASE(ptr->wev);
-    }
-}
-OBJ_CLASS_INSTANCE(orte_iof_sink_t,
-                   opal_list_item_t,
-                   orte_iof_base_sink_construct,
-                   orte_iof_base_sink_destruct);
-
-
-static void orte_iof_base_read_event_construct(orte_iof_read_event_t* rev)
-{
-    rev->fd = -1;
-    rev->active = false;
-    rev->ev = opal_event_alloc();
-}
-static void orte_iof_base_read_event_destruct(orte_iof_read_event_t* rev)
-{
-    opal_event_free(rev->ev);
-    if (0 <= rev->fd) {
-        OPAL_OUTPUT_VERBOSE((20, orte_iof_base_framework.framework_output,
-                             "%s iof: closing fd %d for process %s",
-                             ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
-                             rev->fd, ORTE_NAME_PRINT(&rev->name)));
-        close(rev->fd);
-        rev->fd = -1;
-    }
-}
-OBJ_CLASS_INSTANCE(orte_iof_read_event_t,
-                   opal_object_t,
-                   orte_iof_base_read_event_construct,
-                   orte_iof_base_read_event_destruct);
-
-static void orte_iof_base_write_event_construct(orte_iof_write_event_t* wev)
-{
-    wev->pending = false;
-    wev->fd = -1;
-    OBJ_CONSTRUCT(&wev->outputs, opal_list_t);
-    wev->ev = opal_event_alloc();
-}
-static void orte_iof_base_write_event_destruct(orte_iof_write_event_t* wev)
-{
-    opal_event_free(wev->ev);
-    if (ORTE_PROC_IS_HNP && NULL != orte_xml_fp) {
-        int xmlfd = fileno(orte_xml_fp);
-        if (xmlfd == wev->fd) {
-            /* don't close this one - will get it later */
-            OBJ_DESTRUCT(&wev->outputs);
-            return;
-        }
-    }
-
-    if (2 < wev->fd) {
-        OPAL_OUTPUT_VERBOSE((20, orte_iof_base_framework.framework_output,
-                             "%s iof: closing fd %d for write event",
-                             ORTE_NAME_PRINT(ORTE_PROC_MY_NAME), wev->fd));
-        close(wev->fd);
-    }
-    OBJ_DESTRUCT(&wev->outputs);
-}
-OBJ_CLASS_INSTANCE(orte_iof_write_event_t,
-                   opal_list_item_t,
-                   orte_iof_base_write_event_construct,
-                   orte_iof_base_write_event_destruct);
-
-OBJ_CLASS_INSTANCE(orte_iof_write_output_t,
-                   opal_list_item_t,
-                   NULL, NULL);
-
 /*
  * Global variables
  */
@@ -204,6 +81,15 @@ static int orte_iof_base_register(mca_base_register_flag_t flags)
                                  MCA_BASE_VAR_SCOPE_READONLY,
                                  &orte_iof_base.input_files);
 
+    /* Redirect application stderr to stdout (at source) */
+    orte_iof_base.redirect_app_stderr_to_stdout = false;
+    (void) mca_base_var_register("orte", "iof","base", "redirect_app_stderr_to_stdout",
+                                 "Redirect application stderr to stdout at source (default: false)",
+                                 MCA_BASE_VAR_TYPE_BOOL, NULL, 0, 0,
+                                 OPAL_INFO_LVL_9,
+                                 MCA_BASE_VAR_SCOPE_READONLY,
+                                 &orte_iof_base.redirect_app_stderr_to_stdout);
+
     return ORTE_SUCCESS;
 }
 
@@ -214,6 +100,14 @@ static int orte_iof_base_close(void)
         orte_iof.finalize();
     }
 
+    if (!ORTE_PROC_IS_DAEMON) {
+        if (NULL != orte_iof_base.iof_write_stdout) {
+            OBJ_RELEASE(orte_iof_base.iof_write_stdout);
+        }
+        if (!orte_xml_output && NULL != orte_iof_base.iof_write_stderr) {
+            OBJ_RELEASE(orte_iof_base.iof_write_stderr);
+        }
+    }
     return mca_base_framework_components_close(&orte_iof_base_framework, NULL);
 }
 
@@ -250,8 +144,7 @@ static int orte_iof_base_open(mca_base_open_flag_t flags)
     }
 
     /* daemons do not need to do this as they do not write out stdout/err */
-    if (!ORTE_PROC_IS_DAEMON ||
-        (ORTE_PROC_IS_DAEMON && ORTE_PROC_IS_CM)) {
+    if (!ORTE_PROC_IS_DAEMON) {
         if (orte_xml_output) {
             if (NULL != orte_xml_fp) {
                 /* user wants all xml-formatted output sent to file */
@@ -261,17 +154,17 @@ static int orte_iof_base_open(mca_base_open_flag_t flags)
             }
             /* setup the stdout event */
             ORTE_IOF_SINK_DEFINE(&orte_iof_base.iof_write_stdout, ORTE_PROC_MY_NAME,
-                                 xmlfd, ORTE_IOF_STDOUT, orte_iof_base_write_handler, NULL);
+                                 xmlfd, ORTE_IOF_STDOUT, orte_iof_base_write_handler);
             /* don't create a stderr event - all output will go to
              * the stdout channel
              */
         } else {
             /* setup the stdout event */
             ORTE_IOF_SINK_DEFINE(&orte_iof_base.iof_write_stdout, ORTE_PROC_MY_NAME,
-                                 1, ORTE_IOF_STDOUT, orte_iof_base_write_handler, NULL);
+                                 1, ORTE_IOF_STDOUT, orte_iof_base_write_handler);
             /* setup the stderr event */
             ORTE_IOF_SINK_DEFINE(&orte_iof_base.iof_write_stderr, ORTE_PROC_MY_NAME,
-                                 2, ORTE_IOF_STDERR, orte_iof_base_write_handler, NULL);
+                                 2, ORTE_IOF_STDERR, orte_iof_base_write_handler);
         }
 
         /* do NOT set these file descriptors to non-blocking. If we do so,
@@ -294,3 +187,151 @@ MCA_BASE_FRAMEWORK_DECLARE(orte, iof, "ORTE I/O Forwarding",
                            orte_iof_base_register, orte_iof_base_open, orte_iof_base_close,
                            mca_iof_base_static_components, 0);
 
+
+/* class instances */
+static void orte_iof_job_construct(orte_iof_job_t *ptr)
+{
+    ptr->jdata = NULL;
+    OBJ_CONSTRUCT(&ptr->xoff, opal_bitmap_t);
+}
+static void orte_iof_job_destruct(orte_iof_job_t *ptr)
+{
+    if (NULL != ptr->jdata) {
+        OBJ_RELEASE(ptr->jdata);
+    }
+    OBJ_DESTRUCT(&ptr->xoff);
+}
+OBJ_CLASS_INSTANCE(orte_iof_job_t,
+                   opal_object_t,
+                   orte_iof_job_construct,
+                   orte_iof_job_destruct);
+
+static void orte_iof_base_proc_construct(orte_iof_proc_t* ptr)
+{
+    ptr->stdinev = NULL;
+    ptr->revstdout = NULL;
+    ptr->revstderr = NULL;
+    ptr->revstddiag = NULL;
+    ptr->subscribers = NULL;
+    ptr->copy = true;
+}
+static void orte_iof_base_proc_destruct(orte_iof_proc_t* ptr)
+{
+    if (NULL != ptr->stdinev) {
+        OBJ_RELEASE(ptr->stdinev);
+    }
+    if (NULL != ptr->revstdout) {
+        OBJ_RELEASE(ptr->revstdout);
+    }
+    if (NULL != ptr->revstderr) {
+        OBJ_RELEASE(ptr->revstderr);
+    }
+    if (NULL != ptr->revstddiag) {
+        OBJ_RELEASE(ptr->revstddiag);
+    }
+    if (NULL != ptr->subscribers) {
+        OPAL_LIST_RELEASE(ptr->subscribers);
+    }
+}
+OBJ_CLASS_INSTANCE(orte_iof_proc_t,
+                   opal_list_item_t,
+                   orte_iof_base_proc_construct,
+                   orte_iof_base_proc_destruct);
+
+
+static void orte_iof_base_sink_construct(orte_iof_sink_t* ptr)
+{
+    ptr->daemon.jobid = ORTE_JOBID_INVALID;
+    ptr->daemon.vpid = ORTE_VPID_INVALID;
+    ptr->wev = OBJ_NEW(orte_iof_write_event_t);
+    ptr->xoff = false;
+    ptr->exclusive = false;
+}
+static void orte_iof_base_sink_destruct(orte_iof_sink_t* ptr)
+{
+    if (NULL != ptr->wev && 0 <= ptr->wev->fd) {
+        OPAL_OUTPUT_VERBOSE((20, orte_iof_base_framework.framework_output,
+                             "%s iof: closing sink for process %s on fd %d",
+                             ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
+                             ORTE_NAME_PRINT(&ptr->name), ptr->wev->fd));
+        OBJ_RELEASE(ptr->wev);
+    }
+}
+OBJ_CLASS_INSTANCE(orte_iof_sink_t,
+                   opal_list_item_t,
+                   orte_iof_base_sink_construct,
+                   orte_iof_base_sink_destruct);
+
+
+static void orte_iof_base_read_event_construct(orte_iof_read_event_t* rev)
+{
+    rev->proc = NULL;
+    rev->fd = -1;
+    rev->active = false;
+    rev->ev = opal_event_alloc();
+    rev->sink = NULL;
+    rev->tv.tv_sec = 0;
+    rev->tv.tv_usec = 0;
+}
+static void orte_iof_base_read_event_destruct(orte_iof_read_event_t* rev)
+{
+    orte_iof_proc_t *proct = (orte_iof_proc_t*)rev->proc;
+
+    opal_event_free(rev->ev);
+    if (0 <= rev->fd) {
+        OPAL_OUTPUT_VERBOSE((20, orte_iof_base_framework.framework_output,
+                             "%s iof: closing fd %d for process %s",
+                             ORTE_NAME_PRINT(ORTE_PROC_MY_NAME), rev->fd,
+                             (NULL == proct) ? "UNKNOWN" : ORTE_NAME_PRINT(&proct->name)));
+        close(rev->fd);
+        rev->fd = -1;
+    }
+    if (NULL != rev->sink) {
+        OBJ_RELEASE(rev->sink);
+    }
+    if (NULL != proct) {
+        OBJ_RELEASE(proct);
+    }
+}
+OBJ_CLASS_INSTANCE(orte_iof_read_event_t,
+                   opal_object_t,
+                   orte_iof_base_read_event_construct,
+                   orte_iof_base_read_event_destruct);
+
+static void orte_iof_base_write_event_construct(orte_iof_write_event_t* wev)
+{
+    wev->pending = false;
+    wev->always_writable = false;
+    wev->fd = -1;
+    OBJ_CONSTRUCT(&wev->outputs, opal_list_t);
+    wev->ev = opal_event_alloc();
+    wev->tv.tv_sec = 0;
+    wev->tv.tv_usec = 0;
+}
+static void orte_iof_base_write_event_destruct(orte_iof_write_event_t* wev)
+{
+    opal_event_free(wev->ev);
+    if (ORTE_PROC_IS_HNP && NULL != orte_xml_fp) {
+        int xmlfd = fileno(orte_xml_fp);
+        if (xmlfd == wev->fd) {
+            /* don't close this one - will get it later */
+            OBJ_DESTRUCT(&wev->outputs);
+            return;
+        }
+    }
+    if (2 < wev->fd) {
+        OPAL_OUTPUT_VERBOSE((20, orte_iof_base_framework.framework_output,
+                             "%s iof: closing fd %d for write event",
+                             ORTE_NAME_PRINT(ORTE_PROC_MY_NAME), wev->fd));
+        close(wev->fd);
+    }
+    OBJ_DESTRUCT(&wev->outputs);
+}
+OBJ_CLASS_INSTANCE(orte_iof_write_event_t,
+                   opal_list_item_t,
+                   orte_iof_base_write_event_construct,
+                   orte_iof_base_write_event_destruct);
+
+OBJ_CLASS_INSTANCE(orte_iof_write_output_t,
+                   opal_list_item_t,
+                   NULL, NULL);
