@@ -5,17 +5,18 @@
  * Copyright (c) 2004-2006 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
- * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
+ * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart,
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
- * Copyright (c) 2011 Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2011      Cisco Systems, Inc.  All rights reserved.
  * Copyright (c) 2011      Los Alamos National Security, LLC.  All rights
- *                         reserved. 
+ *                         reserved.
+ * Copyright (c) 2016-2017 Intel, Inc. All rights reserved.
  * $COPYRIGHT$
- * 
+ *
  * Additional copyrights may follow
- * 
+ *
  * $HEADER$
  */
 /** @file:
@@ -35,7 +36,7 @@
 #include "opal/class/opal_bitmap.h"
 #include "opal/dss/dss_types.h"
 
-#include "orte/mca/grpcomm/grpcomm_types.h"
+#include "orte/mca/iof/base/iof_base_setup.h"
 #include "orte/mca/rml/rml_types.h"
 #include "orte/runtime/orte_globals.h"
 
@@ -56,11 +57,16 @@ typedef struct {
     opal_list_t xterm_ranks;
     /* the xterm cmd to be used */
     char **xtermcmd;
+    /* thread pool */
+    int num_threads;
+    opal_event_base_t **ev_bases;   // event base array for progress threads
+    char** ev_threads;              // event progress thread names
+    int next_base;                  // counter to load-level thread use
+    bool signal_direct_children_only;
 } orte_odls_globals_t;
 
 ORTE_DECLSPEC extern orte_odls_globals_t orte_odls_globals;
 
-
 /*
  * Default functions that are common to most environments - can
  * be overridden by specific environments if they need something
@@ -74,11 +80,27 @@ ORTE_DECLSPEC int
 orte_odls_base_default_construct_child_list(opal_buffer_t *data,
                                             orte_jobid_t *job);
 
+ORTE_DECLSPEC void orte_odls_base_spawn_proc(int fd, short sd, void *cbdata);
+
 /* define a function that will fork a local proc */
-typedef int (*orte_odls_base_fork_local_proc_fn_t)(orte_app_context_t *context,
-                                                   orte_proc_t *child,
-                                                   char **environ_copy,
-                                                   orte_job_t *jdata);
+typedef int (*orte_odls_base_fork_local_proc_fn_t)(void *cd);
+
+/* define an object for fork/exec the local proc */
+typedef struct {
+    opal_object_t super;
+    opal_event_t ev;
+    char *cmd;
+    char *wdir;
+    char **argv;
+    char **env;
+    orte_job_t *jdata;
+    orte_app_context_t *app;
+    orte_proc_t *child;
+    bool index_argv;
+    orte_iof_base_io_conf_t opts;
+    orte_odls_base_fork_local_proc_fn_t fork_local;
+} orte_odls_spawn_caddy_t;
+OBJ_CLASS_DECLARATION(orte_odls_spawn_caddy_t);
 
 /* define an object for starting local launch */
 typedef struct {
@@ -104,12 +126,7 @@ OBJ_CLASS_DECLARATION(orte_odls_launch_local_t);
 
 ORTE_DECLSPEC void orte_odls_base_default_launch_local(int fd, short sd, void *cbdata);
 
-ORTE_DECLSPEC int
-orte_odls_base_default_deliver_message(orte_jobid_t job, opal_buffer_t *buffer, orte_rml_tag_t tag);
-
-ORTE_DECLSPEC void odls_base_default_wait_local_proc(pid_t pid, int status, void* cbdata);
-
-ORTE_DECLSPEC void orte_odls_base_default_report_abort(orte_process_name_t *proc);
+ORTE_DECLSPEC void odls_base_default_wait_local_proc(orte_proc_t *proc, void* cbdata);
 
 /* define a function type to signal a local proc */
 typedef int (*orte_odls_base_signal_local_fn_t)(pid_t pid, int signum);
@@ -126,12 +143,7 @@ typedef bool (*orte_odls_base_child_died_fn_t)(orte_proc_t *child);
 
 ORTE_DECLSPEC int
 orte_odls_base_default_kill_local_procs(opal_pointer_array_t *procs,
-                                        orte_odls_base_kill_local_fn_t kill_local,
-                                        orte_odls_base_child_died_fn_t child_died);
-
-ORTE_DECLSPEC int orte_odls_base_default_require_sync(orte_process_name_t *proc,
-                                                      opal_buffer_t *buffer,
-                                                      bool drop_nidmap);
+                                        orte_odls_base_kill_local_fn_t kill_local);
 
 ORTE_DECLSPEC int orte_odls_base_default_restart_proc(orte_proc_t *child,
                                                       orte_odls_base_fork_local_proc_fn_t fork_local);
