@@ -9,10 +9,12 @@
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
- * Copyright (c) 2011      Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2011-2014 Cisco Systems, Inc.  All rights reserved.
  * Copyright (c) 2011-2012 Los Alamos National Security, LLC.
  *                         All rights reserved.
- * Copyright (c) 2013-2014 Intel, Inc. All rights reserved
+ * Copyright (c) 2013-2017 Intel, Inc.  All rights reserved.
+ * Copyright (c) 2015      Research Organization for Information Science
+ *                         and Technology (RIST). All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -31,7 +33,7 @@
 
 #include "opal/util/if.h"
 #include "opal/util/output.h"
-#include "opal/mca/mca.h"
+#include "orte/mca/mca.h"
 #include "opal/mca/base/base.h"
 #include "opal/mca/hwloc/base/base.h"
 #include "opal/threads/tsd.h"
@@ -56,6 +58,7 @@ static void reset_usage(orte_node_t *node, orte_jobid_t jobid)
     int j;
     orte_proc_t *proc;
     opal_hwloc_obj_data_t *data=NULL;
+    hwloc_obj_t bound;
 
     opal_output_verbose(10, orte_rmaps_base_framework.framework_output,
                         "%s reset_usage: node %s has %d procs on it",
@@ -63,7 +66,7 @@ static void reset_usage(orte_node_t *node, orte_jobid_t jobid)
                         node->name, node->num_procs);
 
     /* start by clearing any existing info */
-    opal_hwloc_base_clear_usage(node->topology);
+    opal_hwloc_base_clear_usage(node->topology->topo);
 
     /* cycle thru the procs on the node and record
      * their usage in the topology
@@ -80,7 +83,9 @@ static void reset_usage(orte_node_t *node, orte_jobid_t jobid)
                                 ORTE_NAME_PRINT(&proc->name));
             continue;
         }
-        if (NULL == proc->bind_location) {
+        bound = NULL;
+        if (!orte_get_attribute(&proc->attributes, ORTE_PROC_HWLOC_BOUND, (void**)&bound, OPAL_PTR) ||
+            NULL == bound) {
             /* this proc isn't bound - ignore it */
             opal_output_verbose(10, orte_rmaps_base_framework.framework_output,
                                 "%s reset_usage: proc %s has no bind location",
@@ -88,10 +93,10 @@ static void reset_usage(orte_node_t *node, orte_jobid_t jobid)
                                 ORTE_NAME_PRINT(&proc->name));
             continue;
         }
-        data = (opal_hwloc_obj_data_t*)proc->bind_location->userdata;
+        data = (opal_hwloc_obj_data_t*)bound->userdata;
         if (NULL == data) {
             data = OBJ_NEW(opal_hwloc_obj_data_t);
-            proc->bind_location->userdata = data;
+            bound->userdata = data;
         }
         data->num_bound++;
         opal_output_verbose(10, orte_rmaps_base_framework.framework_output,
@@ -101,6 +106,20 @@ static void reset_usage(orte_node_t *node, orte_jobid_t jobid)
     }
 }
 
+static void unbind_procs(orte_job_t *jdata)
+{
+    int j;
+    orte_proc_t *proc;
+
+    for (j=0; j < jdata->procs->size; j++) {
+        if (NULL == (proc = (orte_proc_t*)opal_pointer_array_get_item(jdata->procs, j))) {
+            continue;
+        }
+        orte_remove_attribute(&proc->attributes, ORTE_PROC_HWLOC_BOUND);
+        orte_remove_attribute(&proc->attributes, ORTE_PROC_CPU_BITMAP);
+    }
+}
+
 static int bind_upwards(orte_job_t *jdata,
                         orte_node_t *node,
                         hwloc_obj_type_t target,
@@ -117,6 +136,8 @@ static int bind_upwards(orte_job_t *jdata,
     hwloc_cpuset_t cpus;
     unsigned int idx, ncpus;
     opal_hwloc_obj_data_t *data;
+    hwloc_obj_t locale;
+    char *cpu_bitmap;
 
     opal_output_verbose(5, orte_rmaps_base_framework.framework_output,
                         "mca:rmaps: bind upwards for job %s with bindings %s",
@@ -135,23 +156,16 @@ static int bind_upwards(orte_job_t *jdata,
         if (proc->name.jobid != jdata->jobid) {
             continue;
         }
-        /* ignore procs that have already been bound - should
-         * never happen, but safer
-         */
-        if (NULL != proc->cpu_bitmap) {
-            continue;
-        }
         /* bozo check */
-        if (NULL == proc->locale) {
-            opal_output_verbose(5, orte_rmaps_base_framework.framework_output,
-                                "BIND UPWARDS: LOCALE FOR PROC %s IS NULL",
-                                ORTE_NAME_PRINT(&proc->name));
+        if (!orte_get_attribute(&proc->attributes, ORTE_PROC_HWLOC_LOCALE, (void**)&locale, OPAL_PTR)) {
+            orte_show_help("help-orte-rmaps-base.txt", "rmaps:no-locale", true, ORTE_NAME_PRINT(&proc->name));
             return ORTE_ERR_SILENT;
         }
         /* starting at the locale, move up thru the parents
          * to find the target object type
          */
-        for (obj = proc->locale->parent; NULL != obj; obj = obj->parent) {
+        cpu_bitmap = NULL;
+        for (obj = locale->parent; NULL != obj; obj = obj->parent) {
             opal_output_verbose(5, orte_rmaps_base_framework.framework_output,
                                 "%s bind:upward target %s type %s",
                                 ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
@@ -162,7 +176,7 @@ static int bind_upwards(orte_job_t *jdata,
                     continue;
                 }
                 /* get its index */
-                if (UINT_MAX == (idx = opal_hwloc_base_get_obj_idx(node->topology, obj, OPAL_HWLOC_AVAILABLE))) {
+                if (UINT_MAX == (idx = opal_hwloc_base_get_obj_idx(node->topology->topo, obj, OPAL_HWLOC_AVAILABLE))) {
                     ORTE_ERROR_LOG(ORTE_ERR_BAD_PARAM);
                     return ORTE_ERR_SILENT;
                 }
@@ -170,7 +184,7 @@ static int bind_upwards(orte_job_t *jdata,
                 data = (opal_hwloc_obj_data_t*)obj->userdata;
                 data->num_bound++;
                 /* get the number of cpus under this location */
-                if (0 == (ncpus = opal_hwloc_base_get_npus(node->topology, obj))) {
+                if (0 == (ncpus = opal_hwloc_base_get_npus(node->topology->topo, obj))) {
                     orte_show_help("help-orte-rmaps-base.txt", "rmaps:no-available-cpus", true, node->name);
                     return ORTE_ERR_SILENT;
                 }
@@ -178,29 +192,40 @@ static int bind_upwards(orte_job_t *jdata,
                  * and it wasn't a default binding policy (i.e., the user requested it)
                  */
                 if (ncpus < data->num_bound &&
-                    !OPAL_BIND_OVERLOAD_ALLOWED(jdata->map->binding) &&
-                    OPAL_BINDING_POLICY_IS_SET(jdata->map->binding)) {
-                    orte_show_help("help-orte-rmaps-base.txt", "rmaps:binding-overload", true,
-                                   opal_hwloc_base_print_binding(map->binding), node->name,
-                                   data->num_bound, ncpus);
-                    return ORTE_ERR_SILENT;
+                    !OPAL_BIND_OVERLOAD_ALLOWED(jdata->map->binding)) {
+                    if (OPAL_BINDING_POLICY_IS_SET(jdata->map->binding)) {
+                        /* if the user specified a binding policy, then we cannot meet
+                         * it since overload isn't allowed, so error out - have the
+                         * message indicate that setting overload allowed will remove
+                         * this restriction */
+                        orte_show_help("help-orte-rmaps-base.txt", "rmaps:binding-overload", true,
+                                       opal_hwloc_base_print_binding(map->binding), node->name,
+                                       data->num_bound, ncpus);
+                        return ORTE_ERR_SILENT;
+                    } else {
+                        /* if we have the default binding policy, then just don't bind */
+                        OPAL_SET_BINDING_POLICY(map->binding, OPAL_BIND_TO_NONE);
+                        unbind_procs(jdata);
+                        return ORTE_SUCCESS;
+                    }
                 }
                 /* bind it here */
-                cpus = opal_hwloc_base_get_available_cpus(node->topology, obj);
-                hwloc_bitmap_list_asprintf(&proc->cpu_bitmap, cpus);
+                cpus = opal_hwloc_base_get_available_cpus(node->topology->topo, obj);
+                hwloc_bitmap_list_asprintf(&cpu_bitmap, cpus);
+                orte_set_attribute(&proc->attributes, ORTE_PROC_CPU_BITMAP, ORTE_ATTR_GLOBAL, cpu_bitmap, OPAL_STRING);
                 /* record the location */
-                proc->bind_location = obj;
+                orte_set_attribute(&proc->attributes, ORTE_PROC_HWLOC_BOUND, ORTE_ATTR_LOCAL, obj, OPAL_PTR);
                 opal_output_verbose(5, orte_rmaps_base_framework.framework_output,
                                     "%s BOUND PROC %s TO %s[%s:%u] on node %s",
                                     ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
                                     ORTE_NAME_PRINT(&proc->name),
-                                    proc->cpu_bitmap,
+                                    cpu_bitmap,
                                     hwloc_obj_type_string(target),
                                     idx, node->name);
                 break;
             }
         }
-        if (NULL == proc->cpu_bitmap && OPAL_BINDING_REQUIRED(jdata->map->binding)) {
+        if (NULL == cpu_bitmap && OPAL_BINDING_REQUIRED(jdata->map->binding)) {
             /* didn't find anyone to bind to - this is an error
              * unless the user specified if-supported
              */
@@ -208,6 +233,9 @@ static int bind_upwards(orte_job_t *jdata,
                            opal_hwloc_base_print_binding(map->binding), node->name);
             return ORTE_ERR_SILENT;
         }
+        if (NULL != cpu_bitmap) {
+            free(cpu_bitmap);
+        }
     }
 
     return ORTE_SUCCESS;
@@ -227,6 +255,8 @@ static int bind_downwards(orte_job_t *jdata,
     opal_hwloc_obj_data_t *data;
     int total_cpus;
     hwloc_cpuset_t totalcpuset;
+    hwloc_obj_t locale;
+    char *cpu_bitmap;
 
     opal_output_verbose(5, orte_rmaps_base_framework.framework_output,
                         "mca:rmaps: bind downward for job %s with bindings %s",
@@ -245,18 +275,19 @@ static int bind_downwards(orte_job_t *jdata,
         if (proc->name.jobid != jdata->jobid) {
             continue;
         }
-        /* ignore procs that have already been bound - should
-         * never happen, but safer
-         */
-        if (NULL != proc->cpu_bitmap) {
-            continue;
+        /* bozo check */
+        locale = NULL;
+        if (!orte_get_attribute(&proc->attributes, ORTE_PROC_HWLOC_LOCALE, (void**)&locale, OPAL_PTR) ||
+            NULL == locale) {
+            orte_show_help("help-orte-rmaps-base.txt", "rmaps:no-locale", true, ORTE_NAME_PRINT(&proc->name));
+            hwloc_bitmap_free(totalcpuset);
+            return ORTE_ERR_SILENT;
         }
         /* we don't know if the target is a direct child of this locale,
          * or if it is some depth below it, so we have to conduct a bit
          * of a search. Let hwloc find the min usage one for us.
          */
-        trg_obj = opal_hwloc_base_find_min_bound_target_under_obj(node->topology,
-                                                                  proc->locale,
+        trg_obj = opal_hwloc_base_find_min_bound_target_under_obj(node->topology->topo, locale,
                                                                   target, cache_level);
         if (NULL == trg_obj) {
             /* there aren't any such targets under this object */
@@ -265,7 +296,7 @@ static int bind_downwards(orte_job_t *jdata,
             return ORTE_ERR_SILENT;
         }
         /* record the location */
-        proc->bind_location = trg_obj;
+        orte_set_attribute(&proc->attributes, ORTE_PROC_HWLOC_BOUND, ORTE_ATTR_LOCAL, trg_obj, OPAL_PTR);
         /* start with a clean slate */
         hwloc_bitmap_zero(totalcpuset);
         total_cpus = 0;
@@ -279,7 +310,7 @@ static int bind_downwards(orte_job_t *jdata,
             }
             trg_obj = nxt_obj;
             /* get the number of cpus under this location */
-            ncpus = opal_hwloc_base_get_npus(node->topology, trg_obj);
+            ncpus = opal_hwloc_base_get_npus(node->topology->topo, trg_obj);
             opal_output_verbose(5, orte_rmaps_base_framework.framework_output,
                                 "%s GOT %d CPUS",
                                 ORTE_NAME_PRINT(ORTE_PROC_MY_NAME), ncpus);
@@ -295,41 +326,50 @@ static int bind_downwards(orte_job_t *jdata,
             if (ncpus < data->num_bound &&
                 !OPAL_BIND_OVERLOAD_ALLOWED(jdata->map->binding)) {
                 if (OPAL_BINDING_POLICY_IS_SET(jdata->map->binding)) {
+                    /* if the user specified a binding policy, then we cannot meet
+                     * it since overload isn't allowed, so error out - have the
+                     * message indicate that setting overload allowed will remove
+                     * this restriction */
                     orte_show_help("help-orte-rmaps-base.txt", "rmaps:binding-overload", true,
                                    opal_hwloc_base_print_binding(map->binding), node->name,
                                    data->num_bound, ncpus);
                     hwloc_bitmap_free(totalcpuset);
                     return ORTE_ERR_SILENT;
                 } else {
-                    /* if this is the default binding policy, then just don't
-                     * bind this proc
-                     */
-                    data->num_bound--;  // maintain count
-                    /* show the proc as not bound */
-                    proc->bind_location = NULL;
+                    /* if we have the default binding policy, then just don't bind */
+                    OPAL_SET_BINDING_POLICY(map->binding, OPAL_BIND_TO_NONE);
+                    unbind_procs(jdata);
                     hwloc_bitmap_zero(totalcpuset);
-                    break;
+                    return ORTE_SUCCESS;
                 }
             }
             /* bind the proc here */
-            cpus = opal_hwloc_base_get_available_cpus(node->topology, trg_obj);
+            cpus = opal_hwloc_base_get_available_cpus(node->topology->topo, trg_obj);
             hwloc_bitmap_or(totalcpuset, totalcpuset, cpus);
             /* track total #cpus */
             total_cpus += ncpus;
             /* move to the next location, in case we need it */
             nxt_obj = trg_obj->next_cousin;
         } while (total_cpus < orte_rmaps_base.cpus_per_rank);
-        hwloc_bitmap_list_asprintf(&proc->cpu_bitmap, totalcpuset);
+        hwloc_bitmap_list_asprintf(&cpu_bitmap, totalcpuset);
+        opal_output_verbose(5, orte_rmaps_base_framework.framework_output,
+                            "%s PROC %s BITMAP %s",
+                            ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
+                            ORTE_NAME_PRINT(&proc->name), cpu_bitmap);
+        orte_set_attribute(&proc->attributes, ORTE_PROC_CPU_BITMAP, ORTE_ATTR_GLOBAL, cpu_bitmap, OPAL_STRING);
+        if (NULL != cpu_bitmap) {
+            free(cpu_bitmap);
+        }
         if (4 < opal_output_get_verbosity(orte_rmaps_base_framework.framework_output)) {
             char tmp1[1024], tmp2[1024];
             if (OPAL_ERR_NOT_BOUND == opal_hwloc_base_cset2str(tmp1, sizeof(tmp1),
-                                                               node->topology, totalcpuset)) {
+                                                               node->topology->topo, totalcpuset)) {
                 opal_output(orte_rmaps_base_framework.framework_output,
                             "%s PROC %s ON %s IS NOT BOUND",
                             ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
                             ORTE_NAME_PRINT(&proc->name), node->name);
             } else {
-                opal_hwloc_base_cset2mapstr(tmp2, sizeof(tmp2), node->topology, totalcpuset);
+                opal_hwloc_base_cset2mapstr(tmp2, sizeof(tmp2), node->topology->topo, totalcpuset);
                 opal_output(orte_rmaps_base_framework.framework_output,
                             "%s BOUND PROC %s[%s] TO %s: %s",
                             ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
@@ -339,7 +379,7 @@ static int bind_downwards(orte_job_t *jdata,
         }
     }
     hwloc_bitmap_free(totalcpuset);
-    
+
     return ORTE_SUCCESS;
 }
 
@@ -359,6 +399,9 @@ static int bind_in_place(orte_job_t *jdata,
     unsigned int idx, ncpus;
     struct hwloc_topology_support *support;
     opal_hwloc_obj_data_t *data;
+    hwloc_obj_t locale, sib;
+    char *cpu_bitmap;
+    bool found;
 
     opal_output_verbose(5, orte_rmaps_base_framework.framework_output,
                         "mca:rmaps: bind in place for job %s with bindings %s",
@@ -371,11 +414,14 @@ static int bind_in_place(orte_job_t *jdata,
         if (NULL == (node = (orte_node_t*)opal_pointer_array_get_item(map->nodes, i))) {
             continue;
         }
+        if (!orte_no_vm && (int)ORTE_PROC_MY_NAME->vpid != node->index) {
+            continue;
+        }
         if (!orte_do_not_launch) {
             /* if we don't want to launch, then we are just testing the system,
              * so ignore questions about support capabilities
              */
-            support = (struct hwloc_topology_support*)hwloc_topology_get_support(node->topology);
+            support = (struct hwloc_topology_support*)hwloc_topology_get_support(node->topology->topo);
             /* check if topology supports cpubind - have to be careful here
              * as Linux doesn't currently support thread-level binding. This
              * may change in the future, though, and it isn't clear how hwloc
@@ -418,7 +464,7 @@ static int bind_in_place(orte_job_t *jdata,
          * on this node, just silently skip it - we will not bind
          */
         if (!OPAL_BINDING_POLICY_IS_SET(map->binding) &&
-            HWLOC_TYPE_DEPTH_UNKNOWN == hwloc_get_type_depth(node->topology, HWLOC_OBJ_CORE)) {
+            HWLOC_TYPE_DEPTH_UNKNOWN == hwloc_get_type_depth(node->topology->topo, HWLOC_OBJ_CORE)) {
             opal_output_verbose(5, orte_rmaps_base_framework.framework_output,
                                 "Unable to bind-to core by default on node %s as no cores detected",
                                 node->name);
@@ -440,51 +486,100 @@ static int bind_in_place(orte_job_t *jdata,
             if (proc->name.jobid != jdata->jobid) {
                 continue;
             }
-            /* ignore procs that have already been bound - should
-             * never happen, but safer
-             */
-            if (NULL != proc->cpu_bitmap) {
-                continue;
-            }
-            /* get the index of this location */
-            if (UINT_MAX == (idx = opal_hwloc_base_get_obj_idx(node->topology, proc->locale, OPAL_HWLOC_AVAILABLE))) {
+            /* bozo check */
+            locale = NULL;
+            if (!orte_get_attribute(&proc->attributes, ORTE_PROC_HWLOC_LOCALE, (void**)&locale, OPAL_PTR)) {
+                orte_show_help("help-orte-rmaps-base.txt", "rmaps:no-locale", true, ORTE_NAME_PRINT(&proc->name));
                 return ORTE_ERR_SILENT;
             }
-            /* track the number bound */
-            data = (opal_hwloc_obj_data_t*)proc->locale->userdata;
-            data->num_bound++;
-             opal_output_verbose(5, orte_rmaps_base_framework.framework_output,
-                                "BINDING PROC %s TO %s NUMBER %u",
-                                ORTE_NAME_PRINT(&proc->name),
-                                hwloc_obj_type_string(proc->locale->type), idx);
+            /* get the index of this location */
+            if (UINT_MAX == (idx = opal_hwloc_base_get_obj_idx(node->topology->topo, locale, OPAL_HWLOC_AVAILABLE))) {
+                ORTE_ERROR_LOG(ORTE_ERR_BAD_PARAM);
+                return ORTE_ERR_SILENT;
+            }
+            data = (opal_hwloc_obj_data_t*)locale->userdata;
             /* get the number of cpus under this location */
-            if (0 == (ncpus = opal_hwloc_base_get_npus(node->topology, proc->locale))) {
+            if (0 == (ncpus = opal_hwloc_base_get_npus(node->topology->topo, locale))) {
                 orte_show_help("help-orte-rmaps-base.txt", "rmaps:no-available-cpus", true, node->name);
                 return ORTE_ERR_SILENT;
             }
-            /* error out if adding a proc would cause overload and that wasn't allowed,
-             * and it wasn't a default binding policy (i.e., the user requested it)
-             */
-            if (ncpus < data->num_bound &&
-                !OPAL_BIND_OVERLOAD_ALLOWED(jdata->map->binding) &&
-                OPAL_BINDING_POLICY_IS_SET(jdata->map->binding)) {
-                orte_show_help("help-orte-rmaps-base.txt", "rmaps:binding-overload", true,
-                               opal_hwloc_base_print_binding(map->binding), node->name,
-                               data->num_bound, ncpus);
-                return ORTE_ERR_SILENT;
+            /* if we don't have enough cpus to support this additional proc, try
+             * shifting the location to a cousin that can support it - the important
+             * thing is that we maintain the same level in the topology */
+            if (ncpus < (data->num_bound+1)) {
+                opal_output_verbose(5, orte_rmaps_base_framework.framework_output,
+                                    "%s bind_in_place: searching right",
+                                    ORTE_NAME_PRINT(ORTE_PROC_MY_NAME));
+                sib = locale;
+                found = false;
+                while (NULL != (sib = sib->next_cousin)) {
+                    data = (opal_hwloc_obj_data_t*)sib->userdata;
+                    ncpus = opal_hwloc_base_get_npus(node->topology->topo, sib);
+                    if (data->num_bound < ncpus) {
+                        found = true;
+                        locale = sib;
+                        break;
+                    }
+                }
+                if (!found) {
+                    /* try the other direction */
+                    opal_output_verbose(5, orte_rmaps_base_framework.framework_output,
+                                        "%s bind_in_place: searching left",
+                                        ORTE_NAME_PRINT(ORTE_PROC_MY_NAME));
+                    sib = locale;
+                    while (NULL != (sib = sib->prev_cousin)) {
+                        data = (opal_hwloc_obj_data_t*)sib->userdata;
+                        ncpus = opal_hwloc_base_get_npus(node->topology->topo, sib);
+                        if (data->num_bound < ncpus) {
+                            found = true;
+                            locale = sib;
+                            break;
+                        }
+                    }
+                }
+                if (!found) {
+                    /* no place to put this - see if overload is allowed */
+                    if (!OPAL_BIND_OVERLOAD_ALLOWED(jdata->map->binding)) {
+                        if (OPAL_BINDING_POLICY_IS_SET(jdata->map->binding)) {
+                            /* if the user specified a binding policy, then we cannot meet
+                             * it since overload isn't allowed, so error out - have the
+                             * message indicate that setting overload allowed will remove
+                             * this restriction */
+                            orte_show_help("help-orte-rmaps-base.txt", "rmaps:binding-overload", true,
+                                           opal_hwloc_base_print_binding(map->binding), node->name,
+                                           data->num_bound, ncpus);
+                            return ORTE_ERR_SILENT;
+                        } else {
+                            /* if we have the default binding policy, then just don't bind */
+                            OPAL_SET_BINDING_POLICY(map->binding, OPAL_BIND_TO_NONE);
+                            unbind_procs(jdata);
+                            return ORTE_SUCCESS;
+                        }
+                    }
+                }
             }
+            /* track the number bound */
+            data = (opal_hwloc_obj_data_t*)locale->userdata;  // just in case it changed
+            data->num_bound++;
+            opal_output_verbose(5, orte_rmaps_base_framework.framework_output,
+                                "BINDING PROC %s TO %s NUMBER %u",
+                                ORTE_NAME_PRINT(&proc->name),
+                                hwloc_obj_type_string(locale->type), idx);
             /* bind the proc here */
-            cpus = opal_hwloc_base_get_available_cpus(node->topology, proc->locale);
-            hwloc_bitmap_list_asprintf(&proc->cpu_bitmap, cpus);
-            /* record the location */
-            proc->bind_location = proc->locale;
+            cpus = opal_hwloc_base_get_available_cpus(node->topology->topo, locale);
+            hwloc_bitmap_list_asprintf(&cpu_bitmap, cpus);
+            orte_set_attribute(&proc->attributes, ORTE_PROC_CPU_BITMAP, ORTE_ATTR_GLOBAL, cpu_bitmap, OPAL_STRING);
+            /* update the location, in case it changed */
+            orte_set_attribute(&proc->attributes, ORTE_PROC_HWLOC_BOUND, ORTE_ATTR_LOCAL, locale, OPAL_PTR);
             opal_output_verbose(5, orte_rmaps_base_framework.framework_output,
                                 "%s BOUND PROC %s TO %s[%s:%u] on node %s",
                                 ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
                                 ORTE_NAME_PRINT(&proc->name),
-                                proc->cpu_bitmap,
-                                hwloc_obj_type_string(proc->locale->type),
+                                cpu_bitmap, hwloc_obj_type_string(locale->type),
                                 idx, node->name);
+            if (NULL != cpu_bitmap) {
+                free(cpu_bitmap);
+            }
         }
     }
 
@@ -493,7 +588,7 @@ static int bind_in_place(orte_job_t *jdata,
 
 static int bind_to_cpuset(orte_job_t *jdata)
 {
-    /* bind each process to opal_hwloc_base_cpu_set */
+    /* bind each process to opal_hwloc_base_cpu_list */
     int i, j;
     orte_job_map_t *map;
     orte_node_t *node;
@@ -501,11 +596,12 @@ static int bind_to_cpuset(orte_job_t *jdata)
     struct hwloc_topology_support *support;
     opal_hwloc_topo_data_t *sum;
     hwloc_obj_t root;
+    char *cpu_bitmap;
 
     opal_output_verbose(5, orte_rmaps_base_framework.framework_output,
-                        "mca:rmaps: bind job %s to cpuset %s",
+                        "mca:rmaps: bind job %s to cpus %s",
                         ORTE_JOBID_PRINT(jdata->jobid),
-                        opal_hwloc_base_cpu_set);
+                        opal_hwloc_base_cpu_list);
     /* initialize */
     map = jdata->map;
 
@@ -513,11 +609,14 @@ static int bind_to_cpuset(orte_job_t *jdata)
         if (NULL == (node = (orte_node_t*)opal_pointer_array_get_item(map->nodes, i))) {
             continue;
         }
+        if (!orte_no_vm && (int)ORTE_PROC_MY_NAME->vpid != node->index) {
+            continue;
+        }
         if (!orte_do_not_launch) {
             /* if we don't want to launch, then we are just testing the system,
              * so ignore questions about support capabilities
              */
-            support = (struct hwloc_topology_support*)hwloc_topology_get_support(node->topology);
+            support = (struct hwloc_topology_support*)hwloc_topology_get_support(node->topology->topo);
             /* check if topology supports cpubind - have to be careful here
              * as Linux doesn't currently support thread-level binding. This
              * may change in the future, though, and it isn't clear how hwloc
@@ -550,7 +649,7 @@ static int bind_to_cpuset(orte_job_t *jdata)
                 }
             }
         }
-        root = hwloc_get_root_obj(node->topology);
+        root = hwloc_get_root_obj(node->topology->topo);
         if (NULL == root->userdata) {
             /* something went wrong */
             ORTE_ERROR_LOG(ORTE_ERR_NOT_FOUND);
@@ -570,13 +669,11 @@ static int bind_to_cpuset(orte_job_t *jdata)
             if (proc->name.jobid != jdata->jobid) {
                 continue;
             }
-            /* ignore procs that have already been bound - should
-             * never happen, but safer
-             */
-            if (NULL != proc->cpu_bitmap) {
-                continue;
+            hwloc_bitmap_list_asprintf(&cpu_bitmap, sum->available);
+            orte_set_attribute(&proc->attributes, ORTE_PROC_CPU_BITMAP, ORTE_ATTR_GLOBAL, cpu_bitmap, OPAL_STRING);
+            if (NULL != cpu_bitmap) {
+                free(cpu_bitmap);
             }
-            hwloc_bitmap_list_asprintf(&proc->cpu_bitmap, sum->available);
         }
     }
     return ORTE_SUCCESS;
@@ -592,13 +689,12 @@ int orte_rmaps_base_compute_bindings(orte_job_t *jdata)
     int i, rc;
     struct hwloc_topology_support *support;
     bool force_down = false;
-    hwloc_cpuset_t totalcpuset;
     int bind_depth, map_depth;
 
     opal_output_verbose(5, orte_rmaps_base_framework.framework_output,
-                        "mca:rmaps: compute bindings for job %s with policy %s",
+                        "mca:rmaps: compute bindings for job %s with policy %s[%x]",
                         ORTE_JOBID_PRINT(jdata->jobid),
-                        opal_hwloc_base_print_binding(jdata->map->binding));
+                        opal_hwloc_base_print_binding(jdata->map->binding), jdata->map->binding);
 
     map = ORTE_GET_MAPPING_POLICY(jdata->map->mapping);
     bind = OPAL_GET_BINDING_POLICY(jdata->map->binding);
@@ -662,6 +758,7 @@ int orte_rmaps_base_compute_bindings(orte_job_t *jdata)
     switch (map) {
     case ORTE_MAPPING_BYNODE:
     case ORTE_MAPPING_BYSLOT:
+    case ORTE_MAPPING_SEQ:
         hwm = HWLOC_OBJ_MACHINE;
         break;
     case ORTE_MAPPING_BYDIST:
@@ -744,19 +841,24 @@ int orte_rmaps_base_compute_bindings(orte_job_t *jdata)
      * basis because different nodes could potentially have different
      * topologies, with different relative depths for the two levels
      */
- execute:
+  execute:
     /* initialize */
-    totalcpuset = hwloc_bitmap_alloc();
+    opal_output_verbose(5, orte_rmaps_base_framework.framework_output,
+                        "mca:rmaps: computing bindings for job %s",
+                        ORTE_JOBID_PRINT(jdata->jobid));
 
     for (i=0; i < jdata->map->nodes->size; i++) {
         if (NULL == (node = (orte_node_t*)opal_pointer_array_get_item(jdata->map->nodes, i))) {
             continue;
         }
+        if (!orte_no_vm && (int)ORTE_PROC_MY_NAME->vpid != node->index) {
+            continue;
+        }
         if (!orte_do_not_launch) {
             /* if we don't want to launch, then we are just testing the system,
              * so ignore questions about support capabilities
              */
-            support = (struct hwloc_topology_support*)hwloc_topology_get_support(node->topology);
+            support = (struct hwloc_topology_support*)hwloc_topology_get_support(node->topology->topo);
             /* check if topology supports cpubind - have to be careful here
              * as Linux doesn't currently support thread-level binding. This
              * may change in the future, though, and it isn't clear how hwloc
@@ -770,7 +872,6 @@ int orte_rmaps_base_compute_bindings(orte_job_t *jdata)
                     continue;
                 }
                 orte_show_help("help-orte-rmaps-base.txt", "rmaps:cpubind-not-supported", true, node->name);
-                hwloc_bitmap_free(totalcpuset);
                 return ORTE_ERR_SILENT;
             }
             /* check if topology supports membind - have to be careful here
@@ -789,7 +890,6 @@ int orte_rmaps_base_compute_bindings(orte_job_t *jdata)
                     membind_warned = true;
                 } else if (OPAL_HWLOC_BASE_MBFA_ERROR == opal_hwloc_base_mbfa) {
                     orte_show_help("help-orte-rmaps-base.txt", "rmaps:membind-not-supported-fatal", true, node->name);
-                    hwloc_bitmap_free(totalcpuset);
                     return ORTE_ERR_SILENT;
                 }
             }
@@ -801,7 +901,7 @@ int orte_rmaps_base_compute_bindings(orte_job_t *jdata)
          * on this node, just silently skip it - we will not bind
          */
         if (!OPAL_BINDING_POLICY_IS_SET(jdata->map->binding) &&
-            HWLOC_TYPE_DEPTH_UNKNOWN == hwloc_get_type_depth(node->topology, HWLOC_OBJ_CORE)) {
+            HWLOC_TYPE_DEPTH_UNKNOWN == hwloc_get_type_depth(node->topology->topo, HWLOC_OBJ_CORE)) {
             opal_output_verbose(5, orte_rmaps_base_framework.framework_output,
                                 "Unable to bind-to core by default on node %s as no cores detected",
                                 node->name);
@@ -825,9 +925,9 @@ int orte_rmaps_base_compute_bindings(orte_job_t *jdata)
                 /* must use a unique function because blasted hwloc
                  * just doesn't deal with caches very well...sigh
                  */
-                bind_depth = hwloc_get_cache_type_depth(node->topology, clvl, -1);
+                bind_depth = hwloc_get_cache_type_depth(node->topology->topo, clvl, (hwloc_obj_cache_type_t)-1);
             } else {
-                bind_depth = hwloc_get_type_depth(node->topology, hwb);
+                bind_depth = hwloc_get_type_depth(node->topology->topo, hwb);
             }
             if (0 > bind_depth) {
                 /* didn't find such an object */
@@ -839,9 +939,9 @@ int orte_rmaps_base_compute_bindings(orte_job_t *jdata)
                 /* must use a unique function because blasted hwloc
                  * just doesn't deal with caches very well...sigh
                  */
-                map_depth = hwloc_get_cache_type_depth(node->topology, clvm, -1);
+                map_depth = hwloc_get_cache_type_depth(node->topology->topo, clvm, (hwloc_obj_cache_type_t)-1);
             } else {
-                map_depth = hwloc_get_type_depth(node->topology, hwm);
+                map_depth = hwloc_get_type_depth(node->topology->topo, hwm);
             }
             if (0 > map_depth) {
                 /* didn't find such an object */
