@@ -5,29 +5,29 @@
  * Copyright (c) 2004-2011 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
- * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
+ * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart,
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
- * Copyright (c) 2006-2014 Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2006-2015 Cisco Systems, Inc.  All rights reserved.
  * Copyright (c) 2011-2013 Los Alamos National Security, LLC.
  *                         All rights reserved.
- * Copyright (c) 2014      Intel, Inc. All rights reserved.
+ * Copyright (c) 2014-2017 Intel, Inc.  All rights reserved.
+ * Copyright (c) 2014-2015 Research Organization for Information Science
+ *                         and Technology (RIST). All rights reserved.
  * $COPYRIGHT$
- * 
+ *
  * Additional copyrights may follow
- * 
+ *
  * $HEADER$
  */
 
 #include "orte_config.h"
 #include "orte/constants.h"
 
-#ifdef HAVE_STRING_H
 #include <string.h>
-#endif
 
-#include "opal/mca/mca.h"
+#include "orte/mca/mca.h"
 #include "opal/util/argv.h"
 #include "opal/util/output.h"
 #include "opal/mca/base/base.h"
@@ -49,7 +49,7 @@
 /*
  * Global variables
  */
-orte_rmaps_base_t orte_rmaps_base;
+orte_rmaps_base_t orte_rmaps_base = {{{0}}};
 bool orte_rmaps_base_pernode = false;
 int orte_rmaps_base_n_pernode = 0;
 int orte_rmaps_base_n_persocket = 0;
@@ -68,6 +68,7 @@ static bool rmaps_base_oversubscribe = false;
 static bool rmaps_base_display_devel_map = false;
 static bool rmaps_base_display_diffable_map = false;
 static char *rmaps_base_topo_file = NULL;
+static char *rmaps_dist_device = NULL;
 
 static int orte_rmaps_base_register(mca_base_register_flag_t flags)
 {
@@ -108,11 +109,7 @@ static int orte_rmaps_base_register(mca_base_register_flag_t flags)
     /* define default mapping policy */
     rmaps_base_mapping_policy = NULL;
     var_id = mca_base_var_register("orte", "rmaps", "base", "mapping_policy",
-#if OPAL_HAVE_HWLOC
                                    "Mapping Policy [slot | hwthread | core (default:np<=2) | l1cache | l2cache | l3cache | socket (default:np>2) | numa | board | node | seq | dist | ppr], with allowed modifiers :PE=y,SPAN,OVERSUBSCRIBE,NOOVERSUBSCRIBE",
-#else
-                                   "Mapping Policy [slot (default) | node], with allowed modifiers :SPAN,OVERSUBSCRIBE,NOOVERSUBSCRIBE",
-#endif
                                    MCA_BASE_VAR_TYPE_STRING, NULL, 0, 0,
                                    OPAL_INFO_LVL_9,
                                    MCA_BASE_VAR_SCOPE_READONLY,
@@ -123,11 +120,7 @@ static int orte_rmaps_base_register(mca_base_register_flag_t flags)
     /* define default ranking policy */
     rmaps_base_ranking_policy = NULL;
     (void) mca_base_var_register("orte", "rmaps", "base", "ranking_policy",
-#if OPAL_HAVE_HWLOC
                                            "Ranking Policy [slot (default:np<=2) | hwthread | core | l1cache | l2cache | l3cache | socket (default:np>2) | numa | board | node], with modifier :SPAN or :FILL",
-#else
-                                           "Ranking Policy [slot (default) | node]",
-#endif
                                 MCA_BASE_VAR_TYPE_STRING, NULL, 0, 0,
                                 OPAL_INFO_LVL_9,
                                 MCA_BASE_VAR_SCOPE_READONLY,
@@ -156,15 +149,21 @@ static int orte_rmaps_base_register(mca_base_register_flag_t flags)
                                  MCA_BASE_VAR_SCOPE_READONLY, &rmaps_base_bynode);
 
     /* #cpus/rank to use */
-    orte_rmaps_base.cpus_per_rank = 1;
-#if OPAL_HAVE_HWLOC
+    orte_rmaps_base.cpus_per_rank = 0;
     var_id = mca_base_var_register("orte", "rmaps", "base", "cpus_per_proc",
                                    "Number of cpus to use for each rank [1-2**15 (default=1)]",
                                    MCA_BASE_VAR_TYPE_INT, NULL, 0, 0,
                                    OPAL_INFO_LVL_9,
                                    MCA_BASE_VAR_SCOPE_READONLY, &orte_rmaps_base.cpus_per_rank);
     mca_base_var_register_synonym(var_id, "orte", "rmaps", "base", "cpus_per_rank", 0);
-#endif
+
+    rmaps_dist_device = NULL;
+    var_id = mca_base_var_register("orte", "rmaps", NULL, "dist_device",
+                                   "If specified, map processes near to this device. Any device name that is identified by the lstopo hwloc utility as Net or OpenFabrics (for example eth0, mlx4_0, etc) or special name as auto ",
+                                   MCA_BASE_VAR_TYPE_STRING, NULL, 0, 0,
+                                   OPAL_INFO_LVL_9,
+                                   MCA_BASE_VAR_SCOPE_READONLY,
+                                   &rmaps_dist_device);
 
     rmaps_base_no_schedule_local = false;
     (void) mca_base_var_register("orte", "rmaps", "base", "no_schedule_local",
@@ -254,8 +253,8 @@ static int orte_rmaps_base_open(mca_base_open_flag_t flags)
     orte_rmaps_base.slot_list = NULL;
     orte_rmaps_base.mapping = 0;
     orte_rmaps_base.ranking = 0;
+    orte_rmaps_base.device = NULL;
 
-#if OPAL_HAVE_HWLOC
     /* if a topology file was given, then set our topology
      * from it. Even though our actual topology may differ,
      * mpirun only needs to see the compute node topology
@@ -267,7 +266,6 @@ static int orte_rmaps_base_open(mca_base_open_flag_t flags)
             return ORTE_ERR_SILENT;
         }
     }
-#endif
 
     /* check for violations that has to be detected before we parse the mapping option */
     if (NULL != orte_rmaps_base.ppr) {
@@ -282,7 +280,7 @@ static int orte_rmaps_base_open(mca_base_open_flag_t flags)
             return ORTE_ERR_SILENT;
         }
     }
-    if (1 < orte_rmaps_base.cpus_per_rank) {
+    if (0 < orte_rmaps_base.cpus_per_rank) {
         orte_show_help("help-orte-rmaps-base.txt", "deprecated", true,
                        "--cpus-per-proc, -cpus-per-proc, --cpus-per-rank, -cpus-per-rank",
                        "--map-by <obj>:PE=N, default <obj>=NUMA",
@@ -378,9 +376,8 @@ static int orte_rmaps_base_open(mca_base_open_flag_t flags)
         ORTE_SET_RANKING_DIRECTIVE(orte_rmaps_base.ranking, ORTE_RANKING_GIVEN);
     }
 
-    if (1 < orte_rmaps_base.cpus_per_rank) {
-#if OPAL_HAVE_HWLOC
-        /* if we were asked for multiple cpus/proc, then we have to
+    if (0 < orte_rmaps_base.cpus_per_rank) {
+        /* if we were asked for cpus/proc, then we have to
          * bind to those cpus - any other binding policy is an
          * error
          */
@@ -406,26 +403,28 @@ static int orte_rmaps_base_open(mca_base_open_flag_t flags)
             if (opal_hwloc_use_hwthreads_as_cpus) {
                 OPAL_SET_BINDING_POLICY(opal_hwloc_binding_policy, OPAL_BIND_TO_HWTHREAD);
             } else {
+                opal_output(0, "SETTING BINDING TO CORE");
                 OPAL_SET_BINDING_POLICY(opal_hwloc_binding_policy, OPAL_BIND_TO_CORE);
             }
         }
-        /* we also need to ensure we are mapping to a high-enough level to have
-         * multiple cpus beneath it - by default, we'll go to the NUMA level */
-        if (ORTE_MAPPING_GIVEN & ORTE_GET_MAPPING_DIRECTIVE(orte_rmaps_base.mapping)) {
-            if (ORTE_GET_MAPPING_POLICY(orte_rmaps_base.mapping) == ORTE_MAPPING_BYHWTHREAD ||
-              (ORTE_GET_MAPPING_POLICY(orte_rmaps_base.mapping) == ORTE_MAPPING_BYCORE &&
-              !opal_hwloc_use_hwthreads_as_cpus)) {
-                orte_show_help("help-orte-rmaps-base.txt", "mapping-too-low-init", true);
-                return ORTE_ERR_SILENT;
+        if (1 < orte_rmaps_base.cpus_per_rank) {
+            /* we need to ensure we are mapping to a high-enough level to have
+             * multiple cpus beneath it - by default, we'll go to the NUMA level */
+            if (ORTE_MAPPING_GIVEN & ORTE_GET_MAPPING_DIRECTIVE(orte_rmaps_base.mapping)) {
+                if (ORTE_GET_MAPPING_POLICY(orte_rmaps_base.mapping) == ORTE_MAPPING_BYHWTHREAD ||
+                  (ORTE_GET_MAPPING_POLICY(orte_rmaps_base.mapping) == ORTE_MAPPING_BYCORE &&
+                  !opal_hwloc_use_hwthreads_as_cpus)) {
+                    orte_show_help("help-orte-rmaps-base.txt", "mapping-too-low-init", true);
+                    return ORTE_ERR_SILENT;
+                }
+            } else {
+                opal_output_verbose(5, orte_rmaps_base_framework.framework_output,
+                                    "%s rmaps:base pe/rank set - setting mapping to BYNUMA",
+                                    ORTE_NAME_PRINT(ORTE_PROC_MY_NAME));
+                ORTE_SET_MAPPING_POLICY(orte_rmaps_base.mapping, ORTE_MAPPING_BYNUMA);
+                ORTE_SET_MAPPING_DIRECTIVE(orte_rmaps_base.mapping, ORTE_MAPPING_GIVEN);
             }
-        } else {
-            opal_output_verbose(5, orte_rmaps_base_framework.framework_output,
-                                "%s rmaps:base pe/rank set - setting mapping to BYNUMA",
-                                ORTE_NAME_PRINT(ORTE_PROC_MY_NAME));
-            ORTE_SET_MAPPING_POLICY(orte_rmaps_base.mapping, ORTE_MAPPING_BYNUMA);
-            ORTE_SET_MAPPING_DIRECTIVE(orte_rmaps_base.mapping, ORTE_MAPPING_GIVEN);
         }
-#endif
     }
 
     if (orte_rmaps_base_pernode) {
@@ -505,10 +504,8 @@ static int orte_rmaps_base_open(mca_base_open_flag_t flags)
         }
         ORTE_UNSET_MAPPING_DIRECTIVE(orte_rmaps_base.mapping, ORTE_MAPPING_NO_OVERSUBSCRIBE);
         ORTE_SET_MAPPING_DIRECTIVE(orte_rmaps_base.mapping, ORTE_MAPPING_SUBSCRIBE_GIVEN);
-#if OPAL_HAVE_HWLOC
         /* also set the overload allowed flag */
         opal_hwloc_binding_policy |= OPAL_BIND_ALLOW_OVERLOAD;
-#endif
     }
 
     /* should we display a detailed (developer-quality) version of the map after determining it? */
@@ -516,7 +513,7 @@ static int orte_rmaps_base_open(mca_base_open_flag_t flags)
         orte_rmaps_base.display_map = true;
         orte_devel_level_output = true;
     }
-   
+
     /* should we display a diffable report of proc locations after determining it? */
     if (rmaps_base_display_diffable_map) {
         orte_rmaps_base.display_map = true;
@@ -572,13 +569,10 @@ static int check_modifiers(char *ck, orte_mapping_policy_t *tmp)
             if (NULL == (ptr = strchr(ck2[i], '='))) {
                 /* missing the value */
                 orte_show_help("help-orte-rmaps-base.txt", "missing-value", true, "pe", ck2[i]);
+                opal_argv_free(ck2);
                 return ORTE_ERR_SILENT;
             }
             ptr++;
-            if (NULL == ptr) {
-                /* still missing the value */
-                return ORTE_ERR_SILENT;
-            }
             orte_rmaps_base.cpus_per_rank = strtol(ptr, NULL, 10);
             opal_output_verbose(5, orte_rmaps_base_framework.framework_output,
                                 "%s rmaps:base setting pe/rank to %d",
@@ -610,22 +604,24 @@ int orte_rmaps_base_set_mapping_policy(orte_mapping_policy_t *policy,
                                        char **device, char *inspec)
 {
     char *ck;
-#if OPAL_HAVE_HWLOC
     char *ptr;
-#endif
     orte_mapping_policy_t tmp;
     int rc;
     size_t len;
     char *spec;
+    char *pch;
 
     /* set defaults */
     tmp = 0;
-    *device = NULL;
+    if (NULL != device) {
+        *device = NULL;
+    }
 
     opal_output_verbose(5, orte_rmaps_base_framework.framework_output,
-                        "%s rmaps:base set policy with %s",
+                        "%s rmaps:base set policy with %s device %s",
                         ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
-                        (NULL == inspec) ? "NULL" : inspec);
+                        (NULL == inspec) ? "NULL" : inspec,
+                        (NULL == device) ? "NULL" : "NONNULL");
 
     if (NULL == inspec) {
         ORTE_SET_MAPPING_POLICY(tmp, ORTE_MAPPING_BYSOCKET);
@@ -659,31 +655,7 @@ int orte_rmaps_base_set_mapping_policy(orte_mapping_policy_t *policy,
             /* if the policy is "dist", then we set the policy to that value
              * and save the second argument as the device
              */
-#if OPAL_HAVE_HWLOC
-            if (0 == strncasecmp(spec, "dist", strlen(spec))) {
-                ORTE_SET_MAPPING_POLICY(tmp, ORTE_MAPPING_BYDIST);
-                /* the first argument after the colon *must* be the
-                 * device we are mapping near - however, other modifiers
-                 * could have been provided, so check for them, okay if
-                 * none found
-                 */
-                if (NULL != (ptr = strchr(ck, ','))) {
-                    *ptr = '\0';
-                    ptr++; // move past the comma
-                    /* check the remaining string for modifiers - may be none, so
-                     * don't emit an error message if the modifier isn't recognized
-                     */
-                    if (ORTE_ERR_SILENT == (rc = check_modifiers(ptr, &tmp)) &&
-                        ORTE_ERR_BAD_PARAM != rc) {
-                        free(spec);
-                        return ORTE_ERR_SILENT;
-                    }
-                }
-                *device = strdup(ck);
-                ORTE_SET_MAPPING_DIRECTIVE(tmp, ORTE_MAPPING_GIVEN);
-                free(spec);
-                goto setpolicy;
-            } else if (0 == strncasecmp(spec, "ppr", strlen(spec))) {
+            if (0 == strncasecmp(spec, "ppr", strlen(spec))) {
                 /* we have to allow additional modifiers here - e.g., specifying
                  * #pe's/proc or oversubscribe - so check for modifiers
                  */
@@ -716,7 +688,6 @@ int orte_rmaps_base_set_mapping_policy(orte_mapping_policy_t *policy,
                 free(spec);
                 goto setpolicy;
             }
-#endif
             if (ORTE_SUCCESS != (rc = check_modifiers(ck, &tmp)) &&
                 ORTE_ERR_TAKE_NEXT_OPTION != rc) {
                 if (ORTE_ERR_BAD_PARAM == rc) {
@@ -733,7 +704,6 @@ int orte_rmaps_base_set_mapping_policy(orte_mapping_policy_t *policy,
             ORTE_SET_MAPPING_POLICY(tmp, ORTE_MAPPING_BYNODE);
         } else if (0 == strncasecmp(spec, "seq", len)) {
             ORTE_SET_MAPPING_POLICY(tmp, ORTE_MAPPING_SEQ);
-#if OPAL_HAVE_HWLOC
         } else if (0 == strncasecmp(spec, "core", len)) {
             ORTE_SET_MAPPING_POLICY(tmp, ORTE_MAPPING_BYCORE);
         } else if (0 == strncasecmp(spec, "l1cache", len)) {
@@ -753,8 +723,21 @@ int orte_rmaps_base_set_mapping_policy(orte_mapping_policy_t *policy,
             /* if we are mapping processes to individual hwthreads, then
              * we need to treat those hwthreads as separate cpus
              */
-            opal_hwloc_use_hwthreads_as_cpus = true;            
-#endif
+            opal_hwloc_use_hwthreads_as_cpus = true;
+        } else if (0 == strncasecmp(spec, "dist", len)) {
+            if (NULL != rmaps_dist_device) {
+                if (NULL != (pch = strchr(rmaps_dist_device, ':'))) {
+                    *pch = '\0';
+                }
+                if (NULL != device) {
+                    *device = strdup(rmaps_dist_device);
+                }
+                ORTE_SET_MAPPING_POLICY(tmp, ORTE_MAPPING_BYDIST);
+            } else {
+                orte_show_help("help-orte-rmaps-base.txt", "device-not-specified", true);
+                free(spec);
+                return ORTE_ERR_SILENT;
+            }
         } else {
             orte_show_help("help-orte-rmaps-base.txt", "unrecognized-policy", true, "mapping", spec);
             free(spec);
@@ -795,7 +778,6 @@ int orte_rmaps_base_set_ranking_policy(orte_ranking_policy_t *policy,
             case ORTE_MAPPING_BYNODE:
                 ORTE_SET_RANKING_POLICY(tmp, ORTE_RANK_BY_NODE);
                 break;
-#if OPAL_HAVE_HWLOC
             case ORTE_MAPPING_BYCORE:
                 ORTE_SET_RANKING_POLICY(tmp, ORTE_RANK_BY_CORE);
                 break;
@@ -820,7 +802,6 @@ int orte_rmaps_base_set_ranking_policy(orte_ranking_policy_t *policy,
             case ORTE_MAPPING_BYHWTHREAD:
                 ORTE_SET_RANKING_POLICY(tmp, ORTE_RANK_BY_HWTHREAD);
                 break;
-#endif
             default:
                 /* anything not tied to a specific hw obj can rank by slot */
                 ORTE_SET_RANKING_POLICY(tmp, ORTE_RANK_BY_SLOT);
@@ -845,7 +826,7 @@ int orte_rmaps_base_set_ranking_policy(orte_ranking_policy_t *policy,
                 ORTE_SET_RANKING_DIRECTIVE(tmp, ORTE_RANKING_FILL);
             } else {
                 /* unrecognized modifier */
-                orte_show_help("help-orte-rmaps-base.txt", "unrecognized-modifier", true, "ranking", ck[1]);
+                orte_show_help("help-orte-rmaps-base.txt", "unrecognized-modifier", true, ck[1]);
                 opal_argv_free(ck);
                 return ORTE_ERR_SILENT;
             }
@@ -855,7 +836,6 @@ int orte_rmaps_base_set_ranking_policy(orte_ranking_policy_t *policy,
             ORTE_SET_RANKING_POLICY(tmp, ORTE_RANK_BY_SLOT);
         } else if (0 == strncasecmp(ck[0], "node", len)) {
             ORTE_SET_RANKING_POLICY(tmp, ORTE_RANK_BY_NODE);
-#if OPAL_HAVE_HWLOC
         } else if (0 == strncasecmp(ck[0], "hwthread", len)) {
             ORTE_SET_RANKING_POLICY(tmp, ORTE_RANK_BY_HWTHREAD);
         } else if (0 == strncasecmp(ck[0], "core", len)) {
@@ -872,9 +852,9 @@ int orte_rmaps_base_set_ranking_policy(orte_ranking_policy_t *policy,
             ORTE_SET_RANKING_POLICY(tmp, ORTE_RANK_BY_NUMA);
         } else if (0 == strncasecmp(ck[0], "board", len)) {
             ORTE_SET_RANKING_POLICY(tmp, ORTE_RANK_BY_BOARD);
-#endif
         } else {
             orte_show_help("help-orte-rmaps-base.txt", "unrecognized-policy", true, "ranking", rmaps_base_ranking_policy);
+            opal_argv_free(ck);
             return ORTE_ERR_SILENT;
         }
         opal_argv_free(ck);
