@@ -12,7 +12,9 @@
  *                         All rights reserved.
  * Copyright (c) 2007-2014 Los Alamos National Security, LLC.  All rights
  *                         reserved.
- * Copyright (c) 2014-2016 Intel, Inc.  All rights reserved.
+ * Copyright (c) 2014-2017 Intel, Inc.  All rights reserved.
+ * Copyright (c) 2016      Research Organization for Information Science
+ *                         and Technology (RIST). All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -25,7 +27,7 @@
  *
  * RML Framework maintenence interface
  *
- * Interface for starting / stopping / controlling the RML framework,
+ * Interface for starting / stopping / controlling the RML framework,307
  * as well as support for modifying RML datatypes.
  *
  * @note The only RML datatype exposed to the user is the RML tag.
@@ -43,14 +45,15 @@
 #include "opal/dss/dss_types.h"
 #include "orte/mca/mca.h"
 #include "opal/util/timings.h"
+#include "opal/class/opal_pointer_array.h"
 
 #include "orte/runtime/orte_globals.h"
+#include "orte/mca/routed/routed.h"
 
 #include "orte/mca/rml/rml.h"
 
-BEGIN_C_DECLS
 
-OPAL_TIMING_DECLARE_EXT(ORTE_DECLSPEC, tm_rml)
+BEGIN_C_DECLS
 
 /*
  * MCA Framework
@@ -80,8 +83,21 @@ ORTE_DECLSPEC void orte_rml_base_comm_start(void);
 ORTE_DECLSPEC void orte_rml_base_comm_stop(void);
 
 
+/*
+ *  globals that might be needed
+ */
+/* adding element to hold the active modules and components */
+typedef struct {
+    opal_list_item_t super;
+    int pri;
+    orte_rml_component_t *component;
+} orte_rml_base_active_t;
+OBJ_CLASS_DECLARATION(orte_rml_base_active_t);
+
 /* a global struct containing framework-level values */
 typedef struct {
+    opal_list_t actives;  /* list to hold the active components */
+    opal_pointer_array_t conduits;  /* array to hold the open conduits */
     opal_list_t posted_recvs;
     opal_list_t unmatched_msgs;
     int max_retries;
@@ -92,31 +108,6 @@ typedef struct {
 ORTE_DECLSPEC extern orte_rml_base_t orte_rml_base;
 
 
-/**
- * List of components that are available to the RML
- *
- * List of components that are currently available to the RML
- * framework.  Useable between calls to orte_rml_base_open() and
- * orte_rml_base_close().
- *
- * @note This list should not be used by code outside the RML base.
- */
-ORTE_DECLSPEC extern opal_list_t orte_rml_base_components;
-
-
-/**
- * Component structure for the selected RML component
- *
- * Component structure pointer for the currently selected RML
- * component.  Useable between calls to orte_rml_base_select() and
- * orte_rml_base_close().
- *
- * @note This pointer should not be used outside the RML base.  It is
- * available outside the RML base only for the F/T component.
- */
-ORTE_DECLSPEC extern orte_rml_component_t *orte_rml_component;
-
-
 /* structure to send RML messages - used internally */
 typedef struct {
     opal_list_item_t super;
@@ -138,10 +129,14 @@ typedef struct {
     int count;
     /* pointer to the user's buffer */
     opal_buffer_t *buffer;
+    /* msg seq number */
+    uint32_t seq_num;
     /* pointer to raw data for cross-transport
      * transfers
      */
     char *data;
+    /* routed module to be used */
+    char *routed;
 } orte_rml_send_t;
 OBJ_CLASS_DECLARATION(orte_rml_send_t);
 
@@ -149,7 +144,7 @@ OBJ_CLASS_DECLARATION(orte_rml_send_t);
 typedef struct {
     opal_object_t super;
     opal_event_t ev;
-    orte_rml_send_t post;
+    orte_rml_send_t send;
 } orte_rml_send_request_t;
 OBJ_CLASS_DECLARATION(orte_rml_send_request_t);
 
@@ -159,6 +154,7 @@ typedef struct {
     opal_event_t ev;
     orte_process_name_t sender;  // sender
     orte_rml_tag_t tag;          // targeted tag
+    uint32_t seq_num;             //sequence number
     struct iovec iov;            // the recvd data
 } orte_rml_recv_t;
 OBJ_CLASS_DECLARATION(orte_rml_recv_t);
@@ -186,17 +182,34 @@ typedef struct {
 } orte_rml_recv_request_t;
 OBJ_CLASS_DECLARATION(orte_rml_recv_request_t);
 
-#define ORTE_RML_POST_MESSAGE(p, t, b, l)                               \
+/* define a structure for sending a message to myself */
+typedef struct {
+    opal_object_t object;
+    opal_event_t ev;
+    orte_rml_tag_t tag;
+    struct iovec* iov;
+    int count;
+    opal_buffer_t *buffer;
+    union {
+        orte_rml_callback_fn_t        iov;
+        orte_rml_buffer_callback_fn_t buffer;
+    } cbfunc;
+    void *cbdata;
+} orte_self_send_xfer_t;
+OBJ_CLASS_DECLARATION(orte_self_send_xfer_t);
+
+#define ORTE_RML_POST_MESSAGE(p, t, s, b, l)                            \
     do {                                                                \
         orte_rml_recv_t *msg;                                           \
         opal_output_verbose(5, orte_rml_base_framework.framework_output, \
-                            "%s Message posted at %s:%d",               \
+                            "%s Message posted at %s:%d for tag %d",    \
                             ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),         \
-                            __FILE__, __LINE__);                        \
+                            __FILE__, __LINE__, (t));                   \
         msg = OBJ_NEW(orte_rml_recv_t);                                 \
         msg->sender.jobid = (p)->jobid;                                 \
         msg->sender.vpid = (p)->vpid;                                   \
         msg->tag = (t);                                                 \
+        msg->seq_num = (s);                                             \
         msg->iov.iov_base = (IOVBASE_TYPE*)(b);                         \
         msg->iov.iov_len = (l);                                         \
         /* setup the event */                                           \
@@ -227,31 +240,68 @@ OBJ_CLASS_DECLARATION(orte_rml_recv_request_t);
         if (NULL != (m)->iov) {                                         \
             if (NULL != (m)->cbfunc.iov) {                              \
                 (m)->cbfunc.iov((m)->status,                            \
-                                &((m)->dst),                            \
-                                (m)->iov, (m)->count,                   \
-                                (m)->tag, (m)->cbdata);                 \
+                            &((m)->dst),                                \
+                            (m)->iov, (m)->count,                       \
+                            (m)->tag, (m)->cbdata);                     \
             }                                                           \
-        } else if (NULL != (m)->cbfunc.buffer) {                        \
+         } else if (NULL != (m)->cbfunc.buffer) {                       \
             /* non-blocking buffer send */                              \
-            (m)->cbfunc.buffer((m)->status, &((m)->origin),             \
-                               (m)->buffer,                             \
-                               (m)->tag, (m)->cbdata);                  \
-        }                                                               \
+            (m)->cbfunc.buffer((m)->status, &((m)->dst),                \
+                           (m)->buffer,                                 \
+                           (m)->tag, (m)->cbdata);                      \
+         }                                                              \
         OBJ_RELEASE(m);                                                 \
     }while(0);
 
-/*
- * This is the base priority for a RML wrapper component
- * If there exists more than one wrapper, then the one with
- * the lowest priority wins.
- */
-#define RML_SELECT_WRAPPER_PRIORITY -128
-
+#define ORTE_RML_INVALID_CHANNEL_NUM  UINT32_MAX
 /* common implementations */
 ORTE_DECLSPEC void orte_rml_base_post_recv(int sd, short args, void *cbdata);
 ORTE_DECLSPEC void orte_rml_base_process_msg(int fd, short flags, void *cbdata);
-ORTE_DECLSPEC void orte_rml_base_process_error(int fd, short flags, void *cbdata);
+
+
+/* Stub API interfaces to cycle through active plugins */
+char* orte_rml_API_get_contact_info(void);
+void orte_rml_API_set_contact_info(const char *contact_info);
+
+int orte_rml_API_ping(orte_rml_conduit_t conduit_id,
+                      const char* contact_info,
+                      const struct timeval* tv);
+
+int orte_rml_API_send_nb(orte_rml_conduit_t conduit_id,
+                         orte_process_name_t* peer, struct iovec* msg,
+                         int count, orte_rml_tag_t tag,
+                         orte_rml_callback_fn_t cbfunc, void* cbdata);
+
+int orte_rml_API_send_buffer_nb(orte_rml_conduit_t conduit_id,
+                                orte_process_name_t* peer,
+                                struct opal_buffer_t* buffer,
+                                orte_rml_tag_t tag,
+                                orte_rml_buffer_callback_fn_t cbfunc,
+                                void* cbdata);
+
+void orte_rml_API_recv_nb(orte_process_name_t* peer,
+                          orte_rml_tag_t tag,
+                          bool persistent,
+                          orte_rml_callback_fn_t cbfunc,
+                          void* cbdata);
+void orte_rml_API_recv_buffer_nb(orte_process_name_t* peer,
+                                 orte_rml_tag_t tag,
+                                 bool persistent,
+                                 orte_rml_buffer_callback_fn_t cbfunc,
+                                 void* cbdata);
+
+void orte_rml_API_recv_cancel(orte_process_name_t* peer, orte_rml_tag_t tag);
+
+void orte_rml_API_purge(orte_process_name_t *peer);
+
+int orte_rml_API_query_transports(opal_list_t *providers);
+
+orte_rml_conduit_t orte_rml_API_open_conduit(opal_list_t *attributes);
+
+void orte_rml_API_close_conduit(orte_rml_conduit_t id);
+
+char* orte_rml_API_get_routed(orte_rml_conduit_t id);
 
 END_C_DECLS
 
-#endif /* MCA_RML_BASE_H */
+#endif /* MCA_RML_BASE_H  */
