@@ -5,19 +5,22 @@
  * Copyright (c) 2004-2008 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
- * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
+ * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart,
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
  * Copyright (c) 2007-2011 Cisco Systems, Inc.  All rights reserved.
- * Copyright (c) 2009-2010 Oracle and/or its affiliates.  All rights reserved. 
+ * Copyright (c) 2009-2010 Oracle and/or its affiliates.  All rights reserved.
  * Copyright (c) 2012-2013 Los Alamos National Security, LLC.
  *                         All rights reserved
- * Copyright (c) 2013      Intel, Inc. All rights reserved
+ * Copyright (c) 2013-2017 Intel, Inc.  All rights reserved.
+ * Copyright (c) 2014      Research Organization for Information Science
+ *                         and Technology (RIST). All rights reserved.
+ * Copyright (c) 2017      IBM Corporation.  All rights reserved.
  * $COPYRIGHT$
- * 
+ *
  * Additional copyrights may follow
- * 
+ *
  * $HEADER$
  */
 
@@ -43,12 +46,13 @@
 
 static bool passed_thru = false;
 static int orte_progress_thread_debug_level = -1;
-static char *orte_timing_file = NULL;
 static char *orte_xml_file = NULL;
 static char *orte_fork_agent_string = NULL;
 static char *orte_tmpdir_base = NULL;
 static char *orte_local_tmpdir_base = NULL;
 static char *orte_remote_tmpdir_base = NULL;
+static char *orte_top_session_dir = NULL;
+static char *orte_jobfam_session_dir = NULL;
 
 int orte_register_params(void)
 {
@@ -62,7 +66,7 @@ int orte_register_params(void)
         return ORTE_SUCCESS;
     }
     passed_thru = true;
-    
+
     /* get a clean output channel too - need to do this here because
      * we use it below, and orterun and some other tools call this
      * function prior to calling orte_init
@@ -71,14 +75,14 @@ int orte_register_params(void)
     lds.lds_want_stdout = true;
     orte_clean_output = opal_output_open(&lds);
     OBJ_DESTRUCT(&lds);
-    
+
     orte_help_want_aggregate = true;
     (void) mca_base_var_register ("orte", "orte", "base", "help_aggregate",
                                   "If orte_base_help_aggregate is true, duplicate help messages will be aggregated rather than displayed individually.  This can be helpful for parallel jobs that experience multiple identical failures; rather than print out the same help/failure message N times, display it once with a count of how many processes sent the same message.",
                                   MCA_BASE_VAR_TYPE_BOOL, NULL, 0, MCA_BASE_VAR_FLAG_SETTABLE,
                                   OPAL_INFO_LVL_9, MCA_BASE_VAR_SCOPE_ALL_EQ,
                                   &orte_help_want_aggregate);
-    
+
     /* LOOK FOR A TMP DIRECTORY BASE */
     /* Several options are provided to cover a range of possibilities:
      *
@@ -115,17 +119,17 @@ int orte_register_params(void)
      */
     if (NULL != orte_tmpdir_base &&
         (NULL != orte_local_tmpdir_base || NULL != orte_remote_tmpdir_base)) {
-            opal_output(orte_clean_output,
-                        "------------------------------------------------------------------\n"
-                        "The MCA param orte_tmpdir_base was specified, which sets the base\n"
-                        "of the temporary directory tree for all procs. However, values for\n"
-                        "the local and/or remote tmpdir base were also given. This can lead\n"
-                        "to confusion and is therefore not allowed. Please specify either a\n"
-                        "global tmpdir base OR a local/remote tmpdir base value\n"
-                        "------------------------------------------------------------------");
-            exit(1);
+        opal_output(orte_clean_output,
+                    "------------------------------------------------------------------\n"
+                    "The MCA param orte_tmpdir_base was specified, which sets the base\n"
+                    "of the temporary directory tree for all procs. However, values for\n"
+                    "the local and/or remote tmpdir base were also given. This can lead\n"
+                    "to confusion and is therefore not allowed. Please specify either a\n"
+                    "global tmpdir base OR a local/remote tmpdir base value\n"
+                    "------------------------------------------------------------------");
+        exit(1);
     }
-     
+
     if (NULL != orte_tmpdir_base) {
         if (NULL != orte_process_info.tmpdir_base) {
             free(orte_process_info.tmpdir_base);
@@ -149,6 +153,34 @@ int orte_register_params(void)
         orte_process_info.tmpdir_base = strdup (orte_remote_tmpdir_base);
     }
 
+    orte_top_session_dir = NULL;
+    (void) mca_base_var_register ("orte", "orte", NULL, "top_session_dir",
+                                  "Top of the session directory tree for applications",
+                                  MCA_BASE_VAR_TYPE_STRING, NULL, 0, 0,
+                                  OPAL_INFO_LVL_9, MCA_BASE_VAR_SCOPE_ALL_EQ,
+                                  &orte_top_session_dir);
+
+    if (NULL != orte_top_session_dir) {
+         if (NULL != orte_process_info.top_session_dir) {
+            free(orte_process_info.top_session_dir);
+        }
+        orte_process_info.top_session_dir = strdup(orte_top_session_dir);
+    }
+
+    orte_jobfam_session_dir = NULL;
+    (void) mca_base_var_register ("orte", "orte", NULL, "jobfam_session_dir",
+                                  "The jobfamily session directory for applications",
+                                  MCA_BASE_VAR_TYPE_STRING, NULL, 0, 0,
+                                  OPAL_INFO_LVL_9, MCA_BASE_VAR_SCOPE_ALL_EQ,
+                                  &orte_jobfam_session_dir);
+
+    if (NULL != orte_jobfam_session_dir) {
+        if (NULL != orte_process_info.jobfam_session_dir) {
+            free(orte_process_info.jobfam_session_dir);
+        }
+        orte_process_info.jobfam_session_dir = strdup(orte_jobfam_session_dir);
+    }
+
     orte_prohibited_session_dirs = NULL;
     (void) mca_base_var_register ("orte", "orte", NULL, "no_session_dirs",
                                   "Prohibited locations for session directories (multiple locations separated by ',', default=NULL)",
@@ -162,7 +194,7 @@ int orte_register_params(void)
                                   MCA_BASE_VAR_TYPE_BOOL, NULL, 0, 0,
                                   OPAL_INFO_LVL_9, MCA_BASE_VAR_SCOPE_ALL,
                                   &orte_create_session_dirs);
-    
+
     orte_execute_quiet = false;
     (void) mca_base_var_register ("orte", "orte", NULL, "execute_quiet",
                                   "Do not output error and help messages",
@@ -240,7 +272,7 @@ int orte_register_params(void)
                                   MCA_BASE_VAR_TYPE_BOOL, NULL, 0, 0,
                                   OPAL_INFO_LVL_9, MCA_BASE_VAR_SCOPE_ALL,
                                   &orte_leave_session_attached);
-    
+
     /* if any debug level is set, ensure we output debug level dumps */
     if (orte_debug_flag || orte_debug_daemons_flag || orte_leave_session_attached) {
         orte_devel_level_output = true;
@@ -275,7 +307,7 @@ int orte_register_params(void)
                                   "Test debugger colaunch after debugger attachment",
                                   MCA_BASE_VAR_TYPE_BOOL, NULL, 0, 0,
                                   OPAL_INFO_LVL_9, MCA_BASE_VAR_SCOPE_READONLY,
-                                  &orte_debugger_test_daemon);
+                                  &orte_debugger_test_attach);
 
     orte_debugger_check_rate = 0;
     (void) mca_base_var_register ("orte", "orte", NULL, "debugger_check_rate",
@@ -318,48 +350,7 @@ int orte_register_params(void)
                                   MCA_BASE_VAR_TYPE_INT, NULL, 0, 0,
                                   OPAL_INFO_LVL_9, MCA_BASE_VAR_SCOPE_READONLY,
                                   &orte_startup_timeout);
- 
-    /* check for timing requests */
-    orte_timing_details = false;
-    (void) mca_base_var_register ("orte", "orte", NULL, "timing_details",
-                                  "Request that detailed timing data by reported",
-                                  MCA_BASE_VAR_TYPE_BOOL, NULL, 0, 0,
-                                  OPAL_INFO_LVL_9, MCA_BASE_VAR_SCOPE_READONLY,
-                                  &orte_timing_details);
 
-    /* ensure the timing flag is set too */
-    orte_timing = orte_timing_details;
-
-    (void) mca_base_var_register ("orte", "orte", NULL, "timing",
-                                  "Request that critical timing loops be measured",
-                                  MCA_BASE_VAR_TYPE_BOOL, NULL, 0, 0,
-                                  OPAL_INFO_LVL_9, MCA_BASE_VAR_SCOPE_READONLY,
-                                  &orte_timing);
-
-    
-    if (ORTE_PROC_IS_HNP) {
-        orte_timing_file = NULL;
-        (void) mca_base_var_register ("orte", "orte", NULL, "timing_file",
-                                      "Name of the file where timing data is to be written (relative or absolute path)",
-                                      MCA_BASE_VAR_TYPE_STRING, NULL, 0, 0,
-                                      OPAL_INFO_LVL_9, MCA_BASE_VAR_SCOPE_READONLY,
-                                      &orte_timing_file);
-        if (orte_timing && NULL == orte_timing_file) {
-            /* send the timing output to stdout */
-            orte_timing_output = stdout;
-        } else if (NULL != orte_timing_file) {
-            /* make sure the timing flag is set */
-            orte_timing = true;
-            /* send the output to the indicated file */
-            orte_timing_output = fopen(orte_timing_file,  "w");
-            if (NULL == orte_timing_output) {
-                /* couldn't be opened */
-                opal_output(0, "File %s could not be opened", orte_timing_file);
-                orte_timing_output = stderr;
-            }
-        }        
-    }
-    
     /* User-level debugger info string */
     orte_base_user_debugger = "totalview @mpirun@ -a @mpirun_args@ : ddt -n @np@ -start @executable@ @executable_argv@ @single_app@ : fxp @mpirun@ -a @mpirun_args@";
     (void) mca_base_var_register ("orte", "orte", NULL, "base_user_debugger",
@@ -373,12 +364,12 @@ int orte_register_params(void)
                                 "Max time to wait [in secs] before aborting an ORTE operation (default: 1sec)",
                                 false, false, 1, &value);
     orte_max_timeout = 1000000.0 * value;  /* convert to usec */
-    
+
     mca_base_param_reg_int_name("orte", "timeout_step",
                                 "Time to wait [in usecs/proc] before aborting an ORTE operation (default: 1000 usec/proc)",
                                 false, false, 1000, &orte_timeout_usec_per_proc);
 #endif
-    
+
     /* default hostfile */
     orte_default_hostfile = NULL;
     (void) mca_base_var_register ("orte", "orte", NULL, "default_hostfile",
@@ -386,7 +377,7 @@ int orte_register_params(void)
                                   MCA_BASE_VAR_TYPE_STRING, NULL, 0, 0,
                                   OPAL_INFO_LVL_9, MCA_BASE_VAR_SCOPE_READONLY,
                                   &orte_default_hostfile);
-                                  
+
     if (NULL == orte_default_hostfile) {
         /* nothing was given, so define the default */
         asprintf(&orte_default_hostfile, "%s/openmpi-default-hostfile", opal_install_dirs.sysconfdir);
@@ -401,7 +392,20 @@ int orte_register_params(void)
         /* flag that it was given */
         orte_default_hostfile_given = true;
     }
-    
+
+    /* default dash-host */
+    orte_default_dash_host = NULL;
+    (void) mca_base_var_register ("orte", "orte", NULL, "default_dash_host",
+                                  "Default -host setting (specify \"none\" to ignore environmental or default MCA param setting)",
+                                  MCA_BASE_VAR_TYPE_STRING, NULL, 0, 0,
+                                  OPAL_INFO_LVL_9, MCA_BASE_VAR_SCOPE_READONLY,
+                                  &orte_default_dash_host);
+    if (NULL != orte_default_dash_host &&
+        0 == strcmp(orte_default_dash_host, "none")) {
+        free(orte_default_dash_host);
+        orte_default_dash_host = NULL;
+    }
+
     /* regex of nodes in system */
     orte_node_regex = NULL;
     (void) mca_base_var_register ("orte", "orte", NULL, "node_regex",
@@ -417,7 +421,7 @@ int orte_register_params(void)
                                   MCA_BASE_VAR_TYPE_BOOL, NULL, 0, 0,
                                   OPAL_INFO_LVL_9, MCA_BASE_VAR_SCOPE_READONLY,
                                   &orte_keep_fqdn_hostnames);
-    
+
     /* whether or not to retain aliases of hostnames */
     orte_retain_aliases = false;
     (void) mca_base_var_register ("orte", "orte", NULL, "retain_aliases",
@@ -426,6 +430,13 @@ int orte_register_params(void)
                                   OPAL_INFO_LVL_9, MCA_BASE_VAR_SCOPE_READONLY,
                                   &orte_retain_aliases);
 
+    orte_hostname_cutoff = 1000;
+    (void) mca_base_var_register ("orte", "orte", NULL, "hostname_cutoff",
+                                  "Pass hostnames to all procs when #nodes is less than cutoff [default:1000]",
+                                  MCA_BASE_VAR_TYPE_INT, NULL, 0, 0,
+                                  OPAL_INFO_LVL_3, MCA_BASE_VAR_SCOPE_READONLY,
+                                  &orte_hostname_cutoff);
+
     /* which alias to use in MPIR_proctab */
     orte_use_hostname_alias = 1;
     (void) mca_base_var_register ("orte", "orte", NULL, "hostname_alias_index",
@@ -452,7 +463,7 @@ int orte_register_params(void)
     if (orte_xml_output) {
         orte_tag_output = true;
     }
-    
+
 
     orte_xml_file = NULL;
     (void) mca_base_var_register ("orte", "orte", NULL, "xml_file",
@@ -476,7 +487,7 @@ int orte_register_params(void)
         /* default to stdout */
         orte_xml_fp = stdout;
     }
-        
+
     /* whether to timestamp output */
     orte_timestamp_output = false;
     (void) mca_base_var_register ("orte", "orte", NULL, "timestamp_output",
@@ -484,7 +495,7 @@ int orte_register_params(void)
                                   MCA_BASE_VAR_TYPE_BOOL, NULL, 0, 0,
                                   OPAL_INFO_LVL_9, MCA_BASE_VAR_SCOPE_READONLY,
                                   &orte_timestamp_output);
-    
+
     /* redirect output into files */
     orte_output_filename = NULL;
     (void) mca_base_var_register ("orte", "orte", NULL, "output_filename",
@@ -500,23 +511,6 @@ int orte_register_params(void)
                                   OPAL_INFO_LVL_9, MCA_BASE_VAR_SCOPE_READONLY,
                                   &orte_show_resolved_nodenames);
 
-#if 0    
-    /* XXX -- option doesn't appear to do anything */
-    mca_base_param_reg_int_name("orte", "hetero_apps",
-                                "Indicates that multiple app_contexts are being provided that are a mix of 32/64 bit binaries (default: false)",
-                                false, false, (int) false, &value);
-    orte_hetero_apps = OPAL_INT_TO_BOOL(value);
-#endif
-
-#if OPAL_HAVE_HWLOC
-    orte_hetero_nodes = false;
-    (void) mca_base_var_register ("orte", "orte", NULL, "hetero_nodes",
-                                  "Nodes in cluster may differ in topology, so send the topology back from each node [Default = false]",
-                                  MCA_BASE_VAR_TYPE_BOOL, NULL, 0, 0,
-                                  OPAL_INFO_LVL_9, MCA_BASE_VAR_SCOPE_READONLY,
-                                  &orte_hetero_nodes);
-#endif
-
     /* allow specification of the launch agent */
     orte_launch_agent = "orted";
     (void) mca_base_var_register ("orte", "orte", NULL, "launch_agent",
@@ -532,17 +526,17 @@ int orte_register_params(void)
                                   OPAL_INFO_LVL_9, MCA_BASE_VAR_SCOPE_READONLY,
                                   &orte_fork_agent_string);
 
-   if (NULL != orte_fork_agent_string) {
+    if (NULL != orte_fork_agent_string) {
         orte_fork_agent = opal_argv_split(orte_fork_agent_string, ' ');
     }
 
     /* whether or not to require RM allocation */
-   orte_allocation_required = false;
-   (void) mca_base_var_register ("orte", "orte", NULL, "allocation_required",
-                                 "Whether or not an allocation by a resource manager is required [default: no]",
-                                 MCA_BASE_VAR_TYPE_BOOL, NULL, 0, 0,
-                                 OPAL_INFO_LVL_9, MCA_BASE_VAR_SCOPE_READONLY,
-                                 &orte_allocation_required);
+    orte_allocation_required = false;
+    (void) mca_base_var_register ("orte", "orte", NULL, "allocation_required",
+                                  "Whether or not an allocation by a resource manager is required [default: no]",
+                                  MCA_BASE_VAR_TYPE_BOOL, NULL, 0, 0,
+                                  OPAL_INFO_LVL_9, MCA_BASE_VAR_SCOPE_READONLY,
+                                  &orte_allocation_required);
 
     /* whether or not to map stddiag to stderr */
     orte_map_stddiag_to_stderr = false;
@@ -552,13 +546,25 @@ int orte_register_params(void)
                                   OPAL_INFO_LVL_9, MCA_BASE_VAR_SCOPE_READONLY,
                                   &orte_map_stddiag_to_stderr);
 
-   /* generate new terminal windows to display output from specified ranks */
-   orte_xterm = NULL;
-   (void) mca_base_var_register ("orte", "orte", NULL, "xterm",
-                                 "Create a new xterm window and display output from the specified ranks there [default: none]",
-                                 MCA_BASE_VAR_TYPE_STRING, NULL, 0, 0,
-                                 OPAL_INFO_LVL_9, MCA_BASE_VAR_SCOPE_READONLY,
-                                 &orte_xterm);
+    /* whether or not to map stddiag to stderr */
+    orte_map_stddiag_to_stdout = false;
+    (void) mca_base_var_register ("orte", "orte", NULL, "map_stddiag_to_stdout",
+                                  "Map output from opal_output to stdout of the local process [default: no]",
+                                  MCA_BASE_VAR_TYPE_BOOL, NULL, 0, 0,
+                                  OPAL_INFO_LVL_9, MCA_BASE_VAR_SCOPE_READONLY,
+                                  &orte_map_stddiag_to_stdout);
+    if( orte_map_stddiag_to_stderr && orte_map_stddiag_to_stdout ) {
+        opal_output(0, "The options \"orte_map_stddiag_to_stderr\" and \"orte_map_stddiag_to_stdout\" are mutually exclusive. They cannot both be set to true.");
+        return ORTE_ERROR;
+    }
+
+    /* generate new terminal windows to display output from specified ranks */
+    orte_xterm = NULL;
+    (void) mca_base_var_register ("orte", "orte", NULL, "xterm",
+                                  "Create a new xterm window and display output from the specified ranks there [default: none]",
+                                  MCA_BASE_VAR_TYPE_STRING, NULL, 0, 0,
+                                  OPAL_INFO_LVL_9, MCA_BASE_VAR_SCOPE_READONLY,
+                                  &orte_xterm);
     if (NULL != orte_xterm) {
         /* if an xterm request is given, we have to leave any ssh
          * sessions attached so the xterm window manager can get
@@ -572,14 +578,6 @@ int orte_register_params(void)
         orte_map_stddiag_to_stderr = true;
     }
 
-    /* whether or not to forward SIGTSTP and SIGCONT signals */
-    orte_forward_job_control = false;
-    (void) mca_base_var_register ("orte", "orte", NULL, "forward_job_control",
-                                  "Forward SIGTSTP (after converting to SIGSTOP) and SIGCONT signals to the application procs [default: no]",
-                                  MCA_BASE_VAR_TYPE_BOOL, NULL, 0, 0,
-                                  OPAL_INFO_LVL_9, MCA_BASE_VAR_SCOPE_READONLY,
-                                  &orte_forward_job_control);
-    
     /* whether or not to report launch progress */
     orte_report_launch_progress = false;
     (void) mca_base_var_register ("orte", "orte", NULL, "report_launch_progress",
@@ -587,7 +585,7 @@ int orte_register_params(void)
                                   MCA_BASE_VAR_TYPE_BOOL, NULL, 0, 0,
                                   OPAL_INFO_LVL_9, MCA_BASE_VAR_SCOPE_READONLY,
                                   &orte_report_launch_progress);
-    
+
     /* cluster hardware info detected by orte only */
     orte_local_cpu_type = NULL;
     (void) mca_base_var_register ("orte", "orte", NULL, "cpu_type",
@@ -613,7 +611,7 @@ int orte_register_params(void)
     if (NULL != orte_report_events_uri) {
         orte_report_events = true;
     }
-    
+
     /* barrier control */
     orte_do_not_barrier = false;
     (void) mca_base_var_register ("orte", "orte", NULL, "do_not_barrier",
@@ -635,7 +633,7 @@ int orte_register_params(void)
                                   MCA_BASE_VAR_TYPE_INT, NULL, 0, 0,
                                   OPAL_INFO_LVL_9, MCA_BASE_VAR_SCOPE_READONLY,
                                   &orte_max_restarts);
-    
+
     if (!orte_enable_recovery && orte_max_restarts != 0) {
         if (ORTE_PROC_IS_HNP) {
             opal_output(orte_clean_output,
@@ -647,11 +645,11 @@ int orte_register_params(void)
                         "this warning in the future, please set the orte_enable_recovery\n"
                         "param to non-zero.\n"
                         "------------------------------------------------------------------",
-                        orte_max_restarts);            
+                        orte_max_restarts);
         }
         orte_enable_recovery = true;
     }
-    
+
     orte_abort_non_zero_exit = true;
     (void) mca_base_var_register ("orte", "orte", NULL, "abort_on_non_zero_status",
                                   "Abort the job if any process returns a non-zero exit status - no restart in such cases",
@@ -666,13 +664,6 @@ int orte_register_params(void)
                                   OPAL_INFO_LVL_9, MCA_BASE_VAR_SCOPE_READONLY,
                                   &orte_allowed_exit_without_sync);
 
-    orte_staged_execution = false;
-    (void) mca_base_var_register ("orte", "orte", NULL, "staged_execution",
-                                  "Staged execution is being used",
-                                  MCA_BASE_VAR_TYPE_BOOL, NULL, 0, 0,
-                                  OPAL_INFO_LVL_9, MCA_BASE_VAR_SCOPE_READONLY,
-                                  &orte_staged_execution);
-
     orte_report_child_jobs_separately = false;
     (void) mca_base_var_register ("orte", "orte", NULL, "report_child_jobs_separately",
                                   "Return the exit status of the primary job only",
@@ -680,7 +671,7 @@ int orte_register_params(void)
                                   OPAL_INFO_LVL_9, MCA_BASE_VAR_SCOPE_READONLY,
                                   &orte_report_child_jobs_separately);
 
-    
+
 #if 0
     /* XXX -- unused parameter */
     mca_base_param_reg_int_name("orte", "child_time_to_exit",
@@ -698,12 +689,14 @@ int orte_register_params(void)
                                   OPAL_INFO_LVL_9, MCA_BASE_VAR_SCOPE_READONLY,
                                   &orte_stat_history_size);
 
-    orte_forward_envars = NULL;
-    (void) mca_base_var_register ("orte", "orte", NULL, "forward_envars",
-                                  "Comma-delimited environmental variables to forward, can include value to set",
-                                  MCA_BASE_VAR_TYPE_STRING, NULL, 0, 0,
-                                  OPAL_INFO_LVL_9, MCA_BASE_VAR_SCOPE_READONLY,
-                                  &orte_forward_envars);
+    orte_no_vm = false;
+    id = mca_base_var_register ("orte", "orte", NULL, "no_vm",
+                                "Do not build the VM at start to detect topologies",
+                                MCA_BASE_VAR_TYPE_BOOL, NULL, 0, 0,
+                                OPAL_INFO_LVL_9, MCA_BASE_VAR_SCOPE_READONLY,
+                                &orte_no_vm);
+    /* register a synonym for old name */
+    mca_base_var_register_synonym (id, "orte", "state", "novm", "select", MCA_BASE_VAR_SYN_FLAG_DEPRECATED);
 
     orte_max_vm_size = -1;
     (void) mca_base_var_register ("orte", "orte", NULL, "max_vm_size",
@@ -712,11 +705,10 @@ int orte_register_params(void)
                                   OPAL_INFO_LVL_9, MCA_BASE_VAR_SCOPE_READONLY,
                                   &orte_max_vm_size);
 
-#if OPAL_HAVE_HWLOC
     if (opal_hwloc_use_hwthreads_as_cpus) {
-        orte_set_slots = strdup("hwthreads");
+        orte_set_slots = "hwthreads";
     } else {
-        orte_set_slots = strdup("cores");
+        orte_set_slots = "cores";
     }
     (void) mca_base_var_register ("orte", "orte", NULL, "set_default_slots",
                                   "Set the number of slots on nodes that lack such info to the"
@@ -726,9 +718,6 @@ int orte_register_params(void)
                                   MCA_BASE_VAR_TYPE_STRING, NULL, 0, 0,
                                   OPAL_INFO_LVL_9, MCA_BASE_VAR_SCOPE_READONLY,
                                   &orte_set_slots);
-#else
-    orte_set_slots = NULL;
-#endif
 
     /* should we display the allocation after determining it? */
     orte_display_allocation = false;
@@ -772,6 +761,41 @@ int orte_register_params(void)
                                   OPAL_INFO_LVL_5, MCA_BASE_VAR_SCOPE_READONLY,
                                   &orte_daemon_cores);
 
+    /* get the conduit params */
+    orte_coll_transport = "fabric,ethernet";
+    (void) mca_base_var_register("orte", "orte", "coll", "transports",
+                                 "Comma-separated list of transports to use for ORTE collectives",
+                                 MCA_BASE_VAR_TYPE_STRING, NULL, 0, 0, OPAL_INFO_LVL_9,
+                                 MCA_BASE_VAR_SCOPE_READONLY, &orte_coll_transport);
+
+    orte_mgmt_transport = "oob";
+    (void) mca_base_var_register("orte", "orte", "mgmt", "transports",
+                                 "Comma-separated list of transports to use for ORTE management messages",
+                                 MCA_BASE_VAR_TYPE_STRING, NULL, 0, 0, OPAL_INFO_LVL_9,
+                                 MCA_BASE_VAR_SCOPE_READONLY, &orte_mgmt_transport);
+
+    /* Amount of time to wait for a stack trace to return from the daemons */
+    orte_stack_trace_wait_timeout = 30;
+    (void) mca_base_var_register ("orte", "orte", NULL, "timeout_for_stack_trace",
+                                  "Seconds to wait for stack traces to return before terminating "
+                                  "the job (<= 0 wait forever)",
+                                  MCA_BASE_VAR_TYPE_INT, NULL, 0, 0,
+                                  OPAL_INFO_LVL_9, MCA_BASE_VAR_SCOPE_READONLY,
+                                  &orte_stack_trace_wait_timeout);
+
+    (void) mca_base_var_register ("orte", "orte", NULL, "fwd_mpirun_port",
+                                  "Forward the port used by mpirun so all daemons will use it",
+                                  MCA_BASE_VAR_TYPE_BOOL, NULL, 0, 0,
+                                  OPAL_INFO_LVL_9, MCA_BASE_VAR_SCOPE_READONLY,
+                                  &orte_fwd_mpirun_port);
+
+    /* register the URI of the UNIVERSAL data server */
+    orte_data_server_uri = NULL;
+    (void) mca_base_var_register ("orte", "pmix", NULL, "server_uri",
+                                  "URI of a session-level keyval server for publish/lookup operations",
+                                  MCA_BASE_VAR_TYPE_STRING, NULL, 0, 0,
+                                  OPAL_INFO_LVL_3, MCA_BASE_VAR_SCOPE_ALL,
+                                  &orte_data_server_uri);
 
     return ORTE_SUCCESS;
 }
