@@ -6,7 +6,7 @@
  */
 
 #include <stdio.h>
-#include "opal/mca/hwloc/hwloc.h"
+#include "opal/mca/hwloc/base/base.h"
 #include "mpi.h"
 
 #include "orte/util/proc_info.h"
@@ -15,15 +15,17 @@ int main(int argc, char* argv[])
 {
     int rank, size, rc;
     hwloc_cpuset_t cpus;
-    char *bindings;
+    char *bindings = NULL;
 
     MPI_Init(&argc, &argv);
     MPI_Comm_rank(MPI_COMM_WORLD, &rank);
     MPI_Comm_size(MPI_COMM_WORLD, &size);
 
-    cpus = hwloc_bitmap_alloc();
-    rc = hwloc_get_cpubind(opal_hwloc_topology, cpus, HWLOC_CPUBIND_PROCESS);
-    hwloc_bitmap_list_asprintf(&bindings, cpus);
+    if (OPAL_SUCCESS == opal_hwloc_base_get_topology()) {
+        cpus = hwloc_bitmap_alloc();
+        rc = hwloc_get_cpubind(opal_hwloc_topology, cpus, HWLOC_CPUBIND_PROCESS);
+        hwloc_bitmap_list_asprintf(&bindings, cpus);
+    }
 
     printf("Hello, World, I am %d of %d [%d local peers]: get_cpubind: %d bitmap %s\n",
            rank, size, orte_process_info.num_local_peers, rc,
