@@ -1,3 +1,4 @@
+/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil -*- */
 /*
  * Copyright (c) 2004-2010 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
@@ -5,17 +6,18 @@
  * Copyright (c) 2004-2007 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
- * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
+ * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart,
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
- * Copyright (c) 2007      Los Alamos National Security, LLC.  All rights
- *                         reserved. 
+ * Copyright (c) 2007-2016 Los Alamos National Security, LLC.  All rights
+ *                         reserved.
  * Copyright (c) 2011-2012 Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2015      Intel, Inc. All rights reserved.
  * $COPYRIGHT$
- * 
+ *
  * Additional copyrights may follow
- * 
+ *
  * $HEADER$
  */
 
@@ -30,9 +32,7 @@
 
 #include <stdio.h>
 #include <errno.h>
-#ifdef HAVE_STDLIB_H
 #include <stdlib.h>
-#endif  /*  HAVE_STDLIB_H */
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif  /* HAVE_UNISTD_H */
@@ -48,9 +48,7 @@
 #ifdef HAVE_SYS_WAIT_H
 #include <sys/wait.h>
 #endif  /* HAVE_SYS_WAIT_H */
-#ifdef HAVE_STRING_H
 #include <string.h>
-#endif  /* HAVE_STRING_H */
 
 
 #include "opal/util/cmd_line.h"
@@ -147,63 +145,63 @@ orte_checkpoint_globals_t orte_checkpoint_globals;
 
 opal_cmd_line_init_t cmd_line_opts[] = {
     { NULL,
-      'h', NULL, "help", 
+      'h', NULL, "help",
       0,
       &orte_checkpoint_globals.help, OPAL_CMD_LINE_TYPE_BOOL,
       "This help message" },
 
     { NULL,
-      'v', NULL, "verbose", 
+      'v', NULL, "verbose",
       0,
       &orte_checkpoint_globals.verbose, OPAL_CMD_LINE_TYPE_BOOL,
       "Be Verbose" },
 
     { NULL,
-      'V', NULL, NULL, 
+      'V', NULL, NULL,
       1,
       &orte_checkpoint_globals.verbose_level, OPAL_CMD_LINE_TYPE_INT,
       "Set the verbosity level (For additional debugging information)" },
 
     { NULL,
-      '\0', NULL, "term", 
+      '\0', NULL, "term",
       0,
       &(orte_checkpoint_globals.term), OPAL_CMD_LINE_TYPE_BOOL,
       "Terminate the application after checkpoint (Cannot be used with --stop)" },
 
     { NULL,
-      '\0', NULL, "stop", 
+      '\0', NULL, "stop",
       0,
       &(orte_checkpoint_globals.stop), OPAL_CMD_LINE_TYPE_BOOL,
       "Send SIGSTOP to application just after checkpoint (checkpoint will not finish until SIGCONT is sent) (Cannot be used with --term)" },
 
     { NULL,
-      'w', NULL, "nowait", 
+      'w', NULL, "nowait",
       0,
       &orte_checkpoint_globals.nowait, OPAL_CMD_LINE_TYPE_BOOL,
       "Do not wait for the application to finish checkpointing before returning" },
 
     { NULL,
-      's', NULL, "status", 
+      's', NULL, "status",
       0,
       &orte_checkpoint_globals.status, OPAL_CMD_LINE_TYPE_BOOL,
       "Display status messages describing the progression of the checkpoint" },
 
     { "hnp-jobid",
-      '\0', NULL, "hnp-jobid", 
+      '\0', NULL, "hnp-jobid",
       1,
       &orte_checkpoint_globals.req_hnp, OPAL_CMD_LINE_TYPE_INT,
       "This should be the jobid of the HNP whose applications you wish "
       "to checkpoint." },
 
     { "hnp-pid",
-      '\0', NULL, "hnp-pid", 
+      '\0', NULL, "hnp-pid",
       1,
       &orte_checkpoint_globals.pid, OPAL_CMD_LINE_TYPE_INT,
       "This should be the pid of the mpirun whose applications you wish "
       "to checkpoint." },
 
     { NULL,
-      'l', NULL, "list", 
+      'l', NULL, "list",
       0,
       &orte_checkpoint_globals.list_only, OPAL_CMD_LINE_TYPE_BOOL,
       "Display a list of checkpoint files available on this machine" },
@@ -216,13 +214,13 @@ opal_cmd_line_init_t cmd_line_opts[] = {
       "Enable C/R Enhanced Debugging" },
 
     { NULL,
-      '\0', "attach", "attach", 
+      '\0', "attach", "attach",
       0,
       &(orte_checkpoint_globals.attach_debugger), OPAL_CMD_LINE_TYPE_BOOL,
       "Wait for the debugger to attach directly after taking the checkpoint." },
 
     { NULL,
-      '\0', "detach", "detach", 
+      '\0', "detach", "detach",
       0,
       &(orte_checkpoint_globals.detach_debugger), OPAL_CMD_LINE_TYPE_BOOL,
       "Do not wait for the debugger to reattach after taking the checkpoint." },
@@ -291,7 +289,7 @@ main(int argc, char *argv[])
         opal_output_verbose(10, orte_checkpoint_globals.output,
                             "\t Connected to Mpirun %s",
                             ORTE_NAME_PRINT(&orterun_hnp->name));
-         
+
         if(orte_checkpoint_globals.options->term) {
             opal_output_verbose(10, orte_checkpoint_globals.output,
                                 "\t Terminating after checkpoint\n");
@@ -387,12 +385,12 @@ static int parse_args(int argc, char *argv[]) {
         goto cleanup;
     }
 #endif
-    
+
     /* Parse the command line options */
     opal_cmd_line_create(&cmd_line, cmd_line_opts);
     mca_base_open();
     mca_base_cmd_line_setup(&cmd_line);
-    ret = opal_cmd_line_parse(&cmd_line, true, argc, argv);
+    ret = opal_cmd_line_parse(&cmd_line, true, false, argc, argv);
 
     if (OPAL_SUCCESS != ret) {
         if (OPAL_ERR_SILENT != ret) {
@@ -417,10 +415,10 @@ static int parse_args(int argc, char *argv[]) {
         exit(0);
     }
 
-    /** 
-     * Put all of the MCA arguments in the environment 
+    /**
+     * Put all of the MCA arguments in the environment
      */
-    mca_base_cmd_line_process_args(&cmd_line, &app_env, &global_env);
+    mca_base_cmd_line_process_args(argc, &app_env, &global_env);
 
     len = opal_argv_count(app_env);
     for(i = 0; i < len; ++i) {
@@ -474,10 +472,10 @@ static int parse_args(int argc, char *argv[]) {
     }
 
     /*
-     * If the user did not supply an hnp jobid, then they must 
+     * If the user did not supply an hnp jobid, then they must
      *  supply the PID of MPIRUN
      */
-    if(0 >= argc && 
+    if(0 >= argc &&
        ORTE_JOBID_INVALID != orte_checkpoint_globals.req_hnp) {
         exit_status = ORTE_SUCCESS;
         goto cleanup;
@@ -490,7 +488,7 @@ static int parse_args(int argc, char *argv[]) {
         exit_status = ORTE_ERROR;
         goto cleanup;
     }
-    
+
     /*
      * JJH: No wait is currently not implemented or tested
      */
@@ -521,7 +519,7 @@ static int find_hnp(void) {
     opal_list_t hnp_list;
     opal_list_item_t *item;
     orte_hnp_contact_t *hnpcandidate;
-    
+
     /* get the list of local hnp's available to us and setup
      * contact info for them into the RML
      */
@@ -535,7 +533,7 @@ static int find_hnp(void) {
         exit_status = ret;
         goto cleanup;
     }
-    
+
     /* search the list for the desired hnp */
     while (NULL != (item = opal_list_remove_first(&hnp_list))) {
         hnpcandidate = (orte_hnp_contact_t*)item;
@@ -602,7 +600,11 @@ static int ckpt_init(int argc, char *argv[]) {
                 true, &environ);
     free(tmp_env_var);
     tmp_env_var = NULL;
-    
+
+    /* we are never allowed to operate as a distributed tool,
+     * so insist on the ess/tool component */
+    opal_setenv("OMPI_MCA_ess", "tool", true, &environ);
+
     /***************************
      * We need all of OPAL and the TOOLS portion of ORTE - this
      * sets us up so we can talk to any HNP over the wire
@@ -646,47 +648,29 @@ static int ckpt_finalize(void) {
     if (ORTE_SUCCESS != (ret = orte_finalize())) {
         exit_status = ret;
     }
-    
+
     return exit_status;
 }
 
 static int start_listener(void)
 {
-    int ret, exit_status = ORTE_SUCCESS;
-
-    if (ORTE_SUCCESS != (ret = orte_rml.recv_buffer_nb(ORTE_NAME_WILDCARD,
-                                                       ORTE_RML_TAG_CKPT,
-                                                       ORTE_RML_PERSISTENT,
-                                                       hnp_receiver,
-                                                       NULL))) {
-        exit_status = ret;
-        goto cleanup;
-    }
+    orte_rml.recv_buffer_nb(ORTE_NAME_WILDCARD, ORTE_RML_TAG_CKPT,
+                            ORTE_RML_PERSISTENT, hnp_receiver, NULL);
 
     listener_started = true;
-
- cleanup:
-    return exit_status;
+    return ORTE_SUCCESS;
 }
 
 static int stop_listener(void)
 {
-    int ret, exit_status = ORTE_SUCCESS;
-
     if( !listener_started ) {
-        exit_status = ORTE_ERROR;
-        goto cleanup;
+        return ORTE_ERROR;
     }
 
-    if (ORTE_SUCCESS != (ret = orte_rml.recv_cancel(ORTE_NAME_WILDCARD,
-                                                    ORTE_RML_TAG_CKPT))) {
-        exit_status = ret;
-        goto cleanup;
-    }
+    orte_rml.recv_cancel(ORTE_NAME_WILDCARD, ORTE_RML_TAG_CKPT);
 
     listener_started = false;
- cleanup:
-    return exit_status;
+    return ORTE_SUCCESS;
 }
 
 static void hnp_receiver(int status,
@@ -710,7 +694,7 @@ static void hnp_receiver(int status,
         ORTE_ERROR_LOG(rc);
         return;
     }
-    
+
     switch (command) {
         case ORTE_SNAPC_GLOBAL_UPDATE_CMD:
             opal_output_verbose(10, orte_checkpoint_globals.output,
@@ -731,7 +715,7 @@ static void hnp_receiver(int status,
 static void process_ckpt_update_cmd(orte_process_name_t* sender,
                                     opal_buffer_t* buffer)
 {
-    int ret, exit_status = ORTE_SUCCESS;
+    int ret;
     orte_std_cntr_t count = 1;
     int ckpt_status = ORTE_SNAPC_CKPT_STATE_NONE;
 
@@ -743,8 +727,7 @@ static void process_ckpt_update_cmd(orte_process_name_t* sender,
      */
     count = 1;
     if ( ORTE_SUCCESS != (ret = opal_dss.unpack(buffer, &ckpt_status, &count, OPAL_INT)) ) {
-        exit_status = ret;
-        goto cleanup;
+        return;
     }
     orte_checkpoint_globals.ckpt_status = ckpt_status;
 
@@ -754,13 +737,11 @@ static void process_ckpt_update_cmd(orte_process_name_t* sender,
         ORTE_SNAPC_CKPT_STATE_ERROR       == orte_checkpoint_globals.ckpt_status ) {
         count = 1;
         if ( ORTE_SUCCESS != (ret = opal_dss.unpack(buffer, &global_snapshot_handle, &count, OPAL_STRING)) ) {
-            exit_status = ret;
-            goto cleanup;
+            return;
         }
         count = 1;
         if ( ORTE_SUCCESS != (ret = opal_dss.unpack(buffer, &global_sequence_num, &count, OPAL_INT)) ) {
-            exit_status = ret;
-            goto cleanup;
+            return;
         }
     }
 
@@ -768,20 +749,18 @@ static void process_ckpt_update_cmd(orte_process_name_t* sender,
      * If the job is not able to be checkpointed, then return
      */
     if( ORTE_SNAPC_CKPT_STATE_NO_CKPT == orte_checkpoint_globals.ckpt_status) {
-        opal_show_help("help-orte-checkpoint.txt", "non-ckptable", 
+        opal_show_help("help-orte-checkpoint.txt", "non-ckptable",
                        true,
                        orte_checkpoint_globals.pid);
         is_checkpoint_finished = true;
-        exit_status = ORTE_ERROR;
-        goto cleanup;
+        return;
     }
 
     if( ORTE_SNAPC_CKPT_STATE_ERROR == orte_checkpoint_globals.ckpt_status) {
         opal_show_help("help-orte-checkpoint.txt", "ckpt_failure", true,
                        orte_checkpoint_globals.pid, ORTE_ERROR);
         is_checkpoint_finished = true;
-        exit_status = ORTE_ERROR;
-        goto cleanup;
+        return;
     }
 
     /* Status progression */
@@ -791,14 +770,14 @@ static void process_ckpt_update_cmd(orte_process_name_t* sender,
 
     if( ORTE_SNAPC_CKPT_STATE_STOPPED == orte_checkpoint_globals.ckpt_status) {
         is_checkpoint_finished = true;
-        goto cleanup;
+        return;
     }
 
     /* Normal termination check */
     if( (ORTE_SNAPC_CKPT_STATE_RECOVERED   == orte_checkpoint_globals.ckpt_status && is_checkpoint_established) ||
         (ORTE_SNAPC_CKPT_STATE_ESTABLISHED == orte_checkpoint_globals.ckpt_status && is_checkpoint_recovered) ){
         is_checkpoint_finished = true;
-        goto cleanup;
+        return;
     }
     else if( ORTE_SNAPC_CKPT_STATE_RECOVERED  == orte_checkpoint_globals.ckpt_status ) {
         is_checkpoint_recovered = true;
@@ -806,9 +785,6 @@ static void process_ckpt_update_cmd(orte_process_name_t* sender,
     else if(ORTE_SNAPC_CKPT_STATE_ESTABLISHED == orte_checkpoint_globals.ckpt_status ) {
         is_checkpoint_established = true;
     }
-
- cleanup:
-    return;
 }
 
 static int notify_process_for_checkpoint(opal_crs_base_ckpt_options_t *options)
@@ -851,7 +827,9 @@ static int notify_process_for_checkpoint(opal_crs_base_ckpt_options_t *options)
         goto cleanup;
     }
 
-    if ( 0 > (ret = orte_rml.send_buffer(&(orterun_hnp->name), buffer, ORTE_RML_TAG_CKPT, 0)) ) {
+    if (ORTE_SUCCESS != (ret = orte_rml.send_buffer_nb(&(orterun_hnp->name), buffer,
+                                                       ORTE_RML_TAG_CKPT, orte_rml_send_callback,
+                                                       NULL))) {
         exit_status = ret;
         goto cleanup;
     }
@@ -861,11 +839,6 @@ static int notify_process_for_checkpoint(opal_crs_base_ckpt_options_t *options)
                         ORTE_JOBID_PRINT(jobid));
 
  cleanup:
-    if( NULL != buffer) {
-        OBJ_RELEASE(buffer);
-        buffer = NULL;
-    }
-
     if( ORTE_SUCCESS != exit_status ) {
         opal_show_help("help-orte-checkpoint.txt", "unable_to_connect", true,
                        orte_checkpoint_globals.pid);
@@ -906,12 +879,12 @@ static int pretty_print_status(void) {
 
     if( NULL != global_snapshot_handle ) {
         opal_output(0,
-                    "[%6.2f / %6.2f] %*s - %s\n", 
+                    "[%6.2f / %6.2f] %*s - %s\n",
                     (cur_time - timer_last), (cur_time - timer_start),
                     25, state_str, global_snapshot_handle);
     } else {
         opal_output(0,
-                    "[%6.2f / %6.2f] %*s - ...\n", 
+                    "[%6.2f / %6.2f] %*s - ...\n",
                     (cur_time - timer_last), (cur_time - timer_start),
                     25, state_str);
     }
@@ -939,7 +912,7 @@ static int pretty_print_reference(void)
     printf("Snapshot Ref.: %3d %s\n",
            global_sequence_num,
            global_snapshot_handle);
-    
+
     return ORTE_SUCCESS;
 }
 
@@ -948,7 +921,6 @@ static int list_all_snapshots(void) {
     opal_list_t *all_snapshots = NULL;
     opal_list_item_t* item = NULL;
     orte_sstore_base_global_snapshot_info_t *global_snapshot = NULL;
-    int s;
 
     all_snapshots = OBJ_NEW(opal_list_t);
 
@@ -979,8 +951,8 @@ static int list_all_snapshots(void) {
             goto cleanup;
         }
 
-        s = 0; /* Silence a compiler warning */
 #if OPAL_ENABLE_CRDEBUG == 1
+        int s;
         /* Pretty print the result - C/R Debug version */
         if( orte_checkpoint_globals.enable_crdebug ) {
             for(s = 0; s < global_snapshot->num_seqs; ++s) {
