@@ -1,11 +1,13 @@
 /*
  * Copyright (c) 2013      Mellanox Technologies, Inc.
  *                         All rights reserved.
- *
+ * Copyright (c) 2015-2016 Research Organization for Information Science
+ *                         and Technology (RIST). All rights reserved.
+ * Copyright (c) 2015      Cisco Systems, Inc.  All rights reserved.
  * $COPYRIGHT$
- * 
+ *
  * Additional copyrights may follow
- * 
+ *
  * $HEADER$
  */
 
@@ -14,9 +16,7 @@
 #ifdef HAVE_SYS_TIME_H
 #include <sys/time.h>
 #endif  /* HAVE_SYS_TIME_H */
-#ifdef HAVE_PTHREAD_H
 #include <pthread.h>
-#endif
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
@@ -46,9 +46,9 @@
 #include "orte/util/name_fns.h"
 
 #include "ompi/datatype/ompi_datatype.h"
-#include "ompi/mca/rcache/base/base.h"
-#include "ompi/mca/mpool/base/base.h"
-#include "ompi/mca/allocator/base/base.h"
+#include "opal/mca/rcache/base/base.h"
+#include "opal/mca/mpool/base/base.h"
+#include "opal/mca/allocator/base/base.h"
 #include "ompi/proc/proc.h"
 #include "ompi/runtime/mpiruntime.h"
 
@@ -61,6 +61,7 @@
 #include "oshmem/mca/atomic/base/base.h"
 #include "oshmem/mca/memheap/base/base.h"
 #include "oshmem/mca/sshmem/base/base.h"
+#include "oshmem/info/info.h"
 #include "oshmem/proc/proc.h"
 #include "oshmem/proc/proc_group_cache.h"
 #include "oshmem/op/op.h"
@@ -101,84 +102,10 @@ int oshmem_mpi_thread_provided = SHMEM_THREAD_SINGLE;
 long *preconnect_value = 0;
 int shmem_api_logger_output = -1;
 
-MPI_Comm oshmem_comm_world;
+MPI_Comm oshmem_comm_world = {0};
 
 opal_thread_t *oshmem_mpi_main_thread = NULL;
 
-/* Constants for the Fortran layer.  These values are referred to via
- common blocks in the Fortran equivalents.  See
- ompi/mpi/f77/constants.h for a more detailed explanation.
-
- The values are *NOT* initialized.  We do not use the values of
- these constants; only their addresses (because they're always
- passed by reference by Fortran).  
-
- Initializing upon instantiation these can reveal size and/or
- alignment differences between Fortran and C (!) which can cause
- warnings or errors upon linking (e.g., making static libraries with
- the intel 9.0 compilers on 64 bit platforms shows alignment
- differences between libmpi.a and the user's application, resulting
- in a linker warning).  FWIW, if you initialize these variables in
- functions (i.e., not at the instantiation in the global scope), the
- linker somehow "figures it all out" (w.r.t. different alignments
- between fortan common blocks and the corresponding C variables) and
- no linker warnings occur.
-
- Note that the rationale for the types of each of these variables is
- discussed in ompi/include/mpif-common.h.  Do not change the types
- without also modifying ompi/mpi/f77/constants.h and
- ompi/include/mpif-common.h.
- */
-
-#define INST(type, upper_case, lower_case, single_u, double_u)   \
-    type lower_case; \
-type upper_case; \
-type single_u;  \
-type double_u
-
-INST(int,
-     MPI_FORTRAN_BOTTOM,
-     mpi_fortran_bottom,
-     mpi_fortran_bottom_,
-     mpi_fortran_bottom__);
-INST(int,
-     MPI_FORTRAN_IN_PLACE,
-     mpi_fortran_in_place,
-     mpi_fortran_in_place_,
-     mpi_fortran_in_place__);
-INST(char *,
-     MPI_FORTRAN_ARGV_NULL,
-     mpi_fortran_argv_null,
-     mpi_fortran_argv_null_,
-     mpi_fortran_argv_null__);
-INST(double,
-     MPI_FORTRAN_ARGVS_NULL,
-     mpi_fortran_argvs_null,
-     mpi_fortran_argvs_null_,
-     mpi_fortran_argvs_null__);
-INST(int *,
-     MPI_FORTRAN_ERRCODES_IGNORE,
-     mpi_fortran_errcodes_ignore,
-     mpi_fortran_errcodes_ignore_,
-     mpi_fortran_errcodes_ignore__);
-INST(int *,
-     MPI_FORTRAN_STATUS_IGNORE,
-     mpi_fortran_status_ignore,
-     mpi_fortran_status_ignore_,
-     mpi_fortran_status_ignore__);
-INST(double,
-     MPI_FORTRAN_STATUSES_IGNORE,
-     mpi_fortran_statuses_ignore,
-     mpi_fortran_statuses_ignore_,
-     mpi_fortran_statuses_ignore__);
-
-/*
- * Hash tables for MPI_Type_create_f90* functions
- */
-opal_hash_table_t ompi_mpi_f90_integer_hashtable;
-opal_hash_table_t ompi_mpi_f90_real_hashtable;
-opal_hash_table_t ompi_mpi_f90_complex_hashtable;
-
 static int _shmem_init(int argc, char **argv, int requested, int *provided);
 
 #if OSHMEM_OPAL_THREAD_ENABLE
@@ -196,8 +123,8 @@ static void* shmem_opal_thread(void* argc)
 }
 #endif
 
-int oshmem_shmem_inglobalexit;
-int oshmem_shmem_globalexit_status;
+int oshmem_shmem_inglobalexit = 0;
+int oshmem_shmem_globalexit_status = -1;
 
 static void sighandler__SIGUSR1(int signum)
 {
@@ -220,35 +147,37 @@ int oshmem_shmem_init(int argc, char **argv, int requested, int *provided)
         if (!ompi_mpi_initialized && !ompi_mpi_finalized) {
             ret = ompi_mpi_init(argc, argv, requested, provided);
         }
-        MPI_Comm_dup(MPI_COMM_WORLD, &oshmem_comm_world);
 
-        if (OSHMEM_SUCCESS == ret) {
-            ret = _shmem_init(argc, argv, requested, provided);
+        if (OSHMEM_SUCCESS != ret) {
+            return ret;
         }
 
-        if (OSHMEM_SUCCESS == ret) {
-            oshmem_shmem_initialized = true;
+        PMPI_Comm_dup(MPI_COMM_WORLD, &oshmem_comm_world);
+        ret = _shmem_init(argc, argv, requested, provided);
 
-            if (OSHMEM_SUCCESS != shmem_lock_init()) {
-                SHMEM_API_ERROR( "shmem_lock_init() failed");
-                return OSHMEM_ERROR;
-            }
+        if (OSHMEM_SUCCESS != ret) {
+            return ret;
+        }
+        oshmem_shmem_initialized = true;
 
-            /* this is a collective op, implies barrier */
-            MCA_MEMHEAP_CALL(get_all_mkeys());
+        if (OSHMEM_SUCCESS != shmem_lock_init()) {
+            SHMEM_API_ERROR( "shmem_lock_init() failed");
+            return OSHMEM_ERROR;
+        }
 
-            oshmem_shmem_preconnect_all();
+        /* this is a collective op, implies barrier */
+        MCA_MEMHEAP_CALL(get_all_mkeys());
+
+        oshmem_shmem_preconnect_all();
 #if OSHMEM_OPAL_THREAD_ENABLE
-            pthread_t thread_id;
-            int perr;
-            perr = pthread_create(&thread_id, NULL, &shmem_opal_thread, NULL);
-            if (perr != 0)
-            {
-                SHMEM_API_ERROR("cannot creat opal thread for SHMEM");
-                return OSHMEM_ERROR;
-            }
-#endif
+        pthread_t thread_id;
+        int perr;
+        perr = pthread_create(&thread_id, NULL, &shmem_opal_thread, NULL);
+        if (0 != perr) {
+            SHMEM_API_ERROR("cannot create opal thread for SHMEM");
+            return OSHMEM_ERROR;
         }
+#endif
     }
 #ifdef SIGUSR1
     signal(SIGUSR1,sighandler__SIGUSR1);
@@ -264,7 +193,8 @@ int oshmem_shmem_preconnect_all(void)
     /* force qp creation and rkey exchange for memheap. Does not force exchange of static vars */
     if (oshmem_preconnect_all) {
         long val;
-        int nproc = 0;
+        int nproc;
+        int my_pe;
         int i;
 
         val = 0xdeadbeaf;
@@ -277,11 +207,12 @@ int oshmem_shmem_preconnect_all(void)
             SHMEM_API_ERROR("shmem_preconnect_all failed");
             return OSHMEM_ERR_OUT_OF_RESOURCE;
         }
-        nproc = _num_pes();
+
+        nproc = oshmem_num_procs();
+        my_pe = oshmem_my_proc_id();
         for (i = 0; i < nproc; i++) {
-            shmem_long_p(preconnect_value, val, i);
+            shmem_long_p(preconnect_value, val, (my_pe + i) % nproc);
         }
-        shmem_fence();
         shmem_barrier_all();
         SHMEM_API_VERBOSE(5, "Preconnected all PEs");
     }
@@ -304,16 +235,6 @@ static int _shmem_init(int argc, char **argv, int requested, int *provided)
     int ret = OSHMEM_SUCCESS;
     char *error = NULL;
 
-    if (OSHMEM_SUCCESS != (ret = oshmem_proc_init())) {
-        error = "oshmem_proc_init() failed";
-        goto error;
-    }
-
-    /* We need to do this anyway.
-     * This place requires to be reviewed and more elegant way is expected
-     */
-    ompi_proc_local_proc = (ompi_proc_t*) oshmem_proc_local_proc;
-
     /* Register the OSHMEM layer's MCA parameters */
     if (OSHMEM_SUCCESS != (ret = oshmem_shmem_register_params())) {
         error = "oshmem_info_register: oshmem_register_params failed";
@@ -326,6 +247,18 @@ static int _shmem_init(int argc, char **argv, int requested, int *provided)
     opal_output_set_verbosity(shmem_api_logger_output,
                               oshmem_shmem_api_verbose);
 
+    /* initialize info */
+    if (OSHMEM_SUCCESS != (ret = oshmem_info_init())) {
+        error = "oshmem_info_init() failed";
+        goto error;
+    }
+
+    /* initialize proc */
+    if (OSHMEM_SUCCESS != (ret = oshmem_proc_init())) {
+        error = "oshmem_proc_init() failed";
+        goto error;
+    }
+
     if (OSHMEM_SUCCESS != (ret = oshmem_group_cache_list_init())) {
         error = "oshmem_group_cache_list_init() failed";
         goto error;
@@ -346,17 +279,12 @@ static int _shmem_init(int argc, char **argv, int requested, int *provided)
         goto error;
     }
 
-    if (OSHMEM_SUCCESS
-            != (ret = mca_spml_base_select(OMPI_ENABLE_PROGRESS_THREADS,
-                                           OMPI_ENABLE_THREAD_MULTIPLE))) {
+    if (OSHMEM_SUCCESS != (ret = mca_spml_base_select(OPAL_ENABLE_PROGRESS_THREADS, 1))) {
         error = "mca_spml_base_select() failed";
         goto error;
     }
 
-    if (OSHMEM_SUCCESS
-            != (ret =
-                    mca_scoll_base_find_available(OMPI_ENABLE_PROGRESS_THREADS,
-                                                  OMPI_ENABLE_THREAD_MULTIPLE))) {
+    if (OSHMEM_SUCCESS != (ret = mca_scoll_base_find_available(OPAL_ENABLE_PROGRESS_THREADS, 1))) {
         error = "mca_scoll_base_find_available() failed";
         goto error;
     }
@@ -368,11 +296,8 @@ static int _shmem_init(int argc, char **argv, int requested, int *provided)
         goto error;
     }
 
-    /* identify the architectures of remote procs and setup
-     * their datatype convertors, if required
-     */
-    if (OSHMEM_SUCCESS != (ret = oshmem_proc_set_arch())) {
-        error = "oshmem_proc_set_arch failed";
+    if (OSHMEM_SUCCESS != (ret = oshmem_proc_group_init())) {
+        error = "oshmem_proc_group_init() failed";
         goto error;
     }
 
@@ -383,20 +308,6 @@ static int _shmem_init(int argc, char **argv, int requested, int *provided)
         goto error;
     }
 
-    /* There is issue with call add_proc twice so
-     * we need to use btl info got from PML add_procs() before call of SPML add_procs()
-     */
-    {
-        ompi_proc_t** procs = NULL;
-        size_t nprocs = 0;
-        procs = ompi_proc_world(&nprocs);
-        while (nprocs--) {
-            oshmem_group_all->proc_array[nprocs]->proc_endpoints[OMPI_PROC_ENDPOINT_TAG_BML] =
-                    procs[nprocs]->proc_endpoints[OMPI_PROC_ENDPOINT_TAG_BML];
-        }
-        free(procs);
-    }
-
     ret =
             MCA_SPML_CALL(add_procs(oshmem_group_all->proc_array, oshmem_group_all->proc_count));
     if (OSHMEM_SUCCESS != ret) {
@@ -429,10 +340,7 @@ static int _shmem_init(int argc, char **argv, int requested, int *provided)
         goto error;
     }
 
-    if (OSHMEM_SUCCESS
-            != (ret =
-                    mca_atomic_base_find_available(OMPI_ENABLE_PROGRESS_THREADS,
-                                                   OMPI_ENABLE_THREAD_MULTIPLE))) {
+    if (OSHMEM_SUCCESS != (ret = mca_atomic_base_find_available(OPAL_ENABLE_PROGRESS_THREADS, 1))) {
         error = "mca_atomic_base_find_available() failed";
         goto error;
     }
